//-----------------------------------------------------------------------------
// config.h
// Detector configuration
// Jiri Havel, DCGM FIT BUT, Brno
// $Id: config.h 189 2009-12-07 13:23:54Z ijuranek $
//-----------------------------------------------------------------------------
#ifndef _DETECT_CONFIG_H_
#define _DETECT_CONFIG_H_

#include <string>
#include <ostream>
#include <vector>

struct Config
{
    std::string engine;

    std::string classifier;
    double threshold;
    unsigned stages;
    bool disableWB;
    unsigned hstep, vstep;
    std::vector<std::string> arguments;

    double groupThreshold;
    unsigned groupMinCount;
    bool doNMS;

    unsigned smooth;
    double scaleBase;

    double rotationStart, rotationStep;
    unsigned rotationCount;

    std::string output;
    std::string detections;
    std::string timestamps;
    int fourcc;
    bool tag;
    bool show;
    bool fullscreen;
    bool drawAll;

    std::string database;
    std::string datapath;
    std::string dataset;
    std::string process;
    std::string description;

    bool verbose;

    unsigned start, frames, step;
    struct Input
    {
        enum Type
        {
            IMAGE,
            VIDEO,
            CAMERA,
            DATASET,
            SEQUENCE
        } type;
        int number;
        std::string name;
        Input(const Type t) : type(t) {}
        Input(const Type t, const int c) : type(t), number(c) {}
        Input(const Type t, const std::string & n) : type(t), name(n) {}
    };
    typedef std::vector<Input> Inputs;
    Inputs inputs;
};

namespace std {

std::ostream & operator<<(std::ostream & o, const Config::Input & i);
std::ostream & operator<<(std::ostream & o, const Config & c);

}

static inline std::string fourccToString(const int fourcc)
{
    return std::string((const char*)&fourcc, 4);
}

bool parseCommandFile(Config & config, const char * filename);
bool parseCommandLine(Config & config, int argc, char * argv[]);
void splitCommandString(const char *str, std::vector<std::string> & arguments);

#endif
