//-----------------------------------------------------------------------------
// input.h
// Input and output image stream classes
// Jiri Havel, DCGM FIT BUT, Brno
// $Id: input.h 157 2009-08-25 08:31:24Z ihavel $
//-----------------------------------------------------------------------------
#ifndef INPUT_H
#define INPUT_H

#include "config.h"

#include <abon.h>

// STL
#include <fstream>
#include <string>
#include <memory>

// OpenCV
#include <highgui.h>

// VPL database
//#include <vpldb.h>

//TODO : pidat dynamickou alokaci podle poteby
#define DETECTION_COUNT (1024*1024)

class ImageManager;

// Image - enhanced abonImageData
class Image : public abonImageData
{
public :
    IplImage *image, *grayscale;
private :
    abonDetection detection_array[DETECTION_COUNT];
public :
    void init();
    void init(IplImage *img);
    Image();
};

class ImageManager
{
private :
    struct EnhancedImage : public Image
    {
        EnhancedImage *next_free;
    };
    EnhancedImage *first_free;
public :

    Image *create();
    void release(Image *image);

    ImageManager();
    ~ImageManager() throw();
};

// Input - stream of images
class Input
{
public :
    //Read image from stream
    //Advance step images forward
    //step == 0 - read same image
    //step == 1 - read next image
    //retured images mustn't be freed
    //return NULL == end of stream
    virtual IplImage * getFrame(unsigned step) = 0;
    virtual Image *getImage(ImageManager &manager, unsigned step);
    Input();
    virtual ~Input() throw();
private :
    unsigned id;

    Input(const Input &);
    const Input & operator=(const Input &);
};

// Stream of one image
class ImageInput : public Input
{
public :
    virtual IplImage * getFrame(unsigned step);
    ImageInput(IplImage * i);
    virtual ~ImageInput() throw();
private :
    IplImage * image;
};

// Wrapper around highgui capture
class CaptureInput : public Input
{
public :
    virtual IplImage * getFrame(unsigned step);
    double getProperty(int id);
    CaptureInput(CvCapture *);
    virtual ~CaptureInput() throw();
private :
    CvCapture * capture;
};
/*
struct DatabaseInput : public Input
{
	virtual VplDbID getSequenceID()   const throw() = 0;
	virtual unsigned getFrameNumber() const throw() = 0;
	virtual ~DatabaseInput() throw() {}
};

class SequenceInput : public DatabaseInput
{
public :
	virtual IplImage * getFrame(unsigned step);
	virtual VplDbID getSequenceID()   const throw() { return sequence->id; }
	virtual unsigned getFrameNumber() const throw() { return frameNumber; }
	SequenceInput(VplDb *, VplDbSequence *);
	virtual ~SequenceInput() throw();
private :
	unsigned frameNumber;
	VplDb * database;
	VplDbSequence * sequence;
	VplDbFramesList * frames;
};

class DatasetInput : public DatabaseInput
{
public :
	virtual IplImage * getFrame(unsigned step);
	virtual VplDbID getSequenceID()   const throw() { return sequence->getSequenceID(); }
	virtual unsigned getFrameNumber() const throw() { return sequence->getFrameNumber(); }
	DatasetInput(VplDb *, CvSeq *);
	virtual ~DatasetInput() throw() {}
private :
	unsigned sequenceNumber;
	VplDb * database;
	CvSeq * sequences;
	std::auto_ptr<SequenceInput> sequence;
};
*/

// Output - stream of images and detections
class Output
{
public :
    virtual bool putImage(abonImageData *image) = 0;
    Output() {}
    virtual ~Output() throw() {}
private :
    Output(const Output &);
    const Output & operator=(const Output &);
};

// Output to multiple targets
class CompositeOutput : public Output
{
public :
    void add(Output *output);
    virtual bool putImage(abonImageData *image);
    virtual ~CompositeOutput() throw();
private :
    std::vector<Output*> outputs;
};

// Outputs detection into csv text file
class DetectionsTextOutput : public Output
{
public :
    virtual bool putImage(abonImageData *image);
    DetectionsTextOutput(const char *filename);
    virtual ~DetectionsTextOutput() throw();
private :
    std::ofstream out;
};

// Outputs timestamps into csv text file
class TimestampsTextOutput : public Output
{
public :
    virtual bool putImage(abonImageData *image);
    TimestampsTextOutput(const char *filename);
    virtual ~TimestampsTextOutput() throw();
private :
    std::ofstream out;
};

// Output to image file
class ImageOutput : public Output
{
public :
    virtual bool putImage(abonImageData *image);
    ImageOutput(const std::string & n) : name(n) {}
    virtual ~ImageOutput() throw() {}
private :
    std::string name;
};

// Output to video file
class VideoOutput : public Output
{
public :
    virtual bool putImage(abonImageData *image);
    VideoOutput(CvVideoWriter * w) : writer(w) {}
    virtual ~VideoOutput() throw();
private :
    CvVideoWriter *writer;
};

// Output to database process
//class ProcessOutput : public Output
//{
//public :
//    virtual bool putDetections(const TDetectionList &);
//    ProcessOutput(VplDb *db, VplDbID pid, const DatabaseInput * i)
//        : database(db), process(pid), input(i) {}
//    virtual ~ProcessOutput() throw() {}
//private :
//    VplDb *database;
//    VplDbID process;
//    const DatabaseInput * input;
//};

Input * inputCreate(const Config & config, unsigned i);
Output * outputCreate(const Config & config, unsigned i, Input * input);

#endif
