//-----------------------------------------------------------------------------
// io.h
// Input/output wrapper
// Jiri Havel, DCGM FIT BUT, Brno
// $Id: io.h 162 2009-08-31 07:49:22Z ihavel $
//-----------------------------------------------------------------------------
#ifndef _DETECT_INPUT_OUTPUT_H_
#define _DETECT_INPUT_OUTPUT_H_

#include "input.h"
#include "window.h"

// STL
#include <queue>

class InputOutput
{
private :
    const Config &config;
    abonDetector *detector;

    Timer *timer;
    double times[abontsNumTimeStamps];

    Window &window;

    bool quit;
    Input *input;
    Output *output;

    unsigned frames;
    Image *image;

    unsigned faces;

    ImageManager manager;

    //output ordering
    unsigned nextId;
    struct ImageIdComparator
    {
        bool operator()(const Image *const a, const Image *const b) const throw()
        {
            return a->imageID > b->imageID;
        }
    };
    std::priority_queue<Image*, std::vector<Image*>, ImageIdComparator> outputQueue;

    Image *getImage();
    abonErrorCode putImage(Image *image);//orders images and calls outputImage
    abonErrorCode outputImage(Image *image);

    //wrappers for get/put Image
    static abonImageData *ioGetImage(void *param);
    static abonErrorCode ioPutImage(void *param, abonImageData *image);
public :
    InputOutput(const Config &cfg, abonDetector *det, Timer *timer, Window &window);
    ~InputOutput() throw();

	std::pair<abonErrorCode, unsigned> runWithIO(Input *in, Output *out);
};

#endif
