// includes, system
#include <string.h>

#include <memory.h>
#include <stdio.h>

#include "GLee.h"
#include <GL/gl.h>
#include <GL/glut.h>

#include "lrd.h"

#include <cuda_gl_interop.h>

#define WEAK_PARAMS 4

// includes, kernels
#include "lrdkernel.cu"



cudaArray* g_cuaPredictors;
cudaArray* g_cuaWeakParam;


double GetTimeMy2()
{
#if _WIN32                                    /* toto jede na Windows */
  static int initialized = 0;
  static LARGE_INTEGER frequency;
  LARGE_INTEGER value;

  if (!initialized) {                         /* prvni volani */
    initialized = 1;
    if (QueryPerformanceFrequency(&frequency) == 0)
      exit(-1);
  }

  QueryPerformanceCounter(&value);
  return (double)value.QuadPart / (double)frequency.QuadPart;

#else                                         /* toto jede na Linux/Unixovych systemech */
  struct timeval tv;
  if (gettimeofday(&tv, NULL) == -1) {        /* vezmi cas */
    assert(0 && "gettimeofday does not work.");  /* osetri chyby */
    exit(-1);
  }
  return (double)tv.tv_sec + (double)tv.tv_usec/1000000.;  /* vrat cas v sekundach */
#endif
}



void FillPositionScaleArray(float _fS, float _fX, float _fY, float _fW, float _fH, unsigned int _uiDataW, tPositionScale *_paData){
  for(unsigned int fy=_fY; fy<unsigned int(_fY+_fH); fy++)
    for(unsigned int fx=_fX; fx<unsigned int(_fX+_fW); fx++){
      _paData[fy*_uiDataW+fx].uiStartX = _fX;
      _paData[fy*_uiDataW+fx].uiStartY = _fY;
      _paData[fy*_uiDataW+fx].fScale = _fS;
    }
}



void LoadClassifiersToGPU(t_sContextData *_psContextData)
{
  if(_psContextData->m_bClassifiersLoaded) return;

  printf("Loading Classifiers to GPU...");

  unsigned int uiWeakClasifiersCount = (unsigned int)_psContextData->m_vuiWeakParam.size();

  cudaMemcpyToSymbol("c_uiWeakClasifiersCount", &uiWeakClasifiersCount, sizeof(unsigned int), 0, cudaMemcpyHostToDevice);
  
  // Load TresholdValues
  cudaMemcpyToSymbol("c_uiTresholds", &_psContextData->m_vfTreshold[0], sizeof(float)*uiWeakClasifiersCount, 0, cudaMemcpyHostToDevice);
  
  // Load PredictorValues
  if(_psContextData->m_vfLRDPredictors.size()>0){
    tLRDPredictionValues.addressMode[0] = cudaAddressModeClamp;
    tLRDPredictionValues.addressMode[1] = cudaAddressModeClamp;
    tLRDPredictionValues.filterMode = cudaFilterModePoint;
    tLRDPredictionValues.normalized = false;    // access with normalized texture coordinates

    cudaMallocArray( &g_cuaPredictors, &tLRDPredictionValues.channelDesc, 17, uiWeakClasifiersCount); 
    cudaMemcpyToArray( g_cuaPredictors, 0, 0, &_psContextData->m_vfLRDPredictors[0], sizeof(float)*17*uiWeakClasifiersCount, cudaMemcpyHostToDevice);
    cudaBindTextureToArray( tLRDPredictionValues, g_cuaPredictors, tLRDPredictionValues.channelDesc);
  }else if(_psContextData->m_vfLRPPredictors.size()>0){
    tLRPPredictionValues.addressMode[0] = cudaAddressModeClamp;
    tLRPPredictionValues.addressMode[1] = cudaAddressModeClamp;
    tLRPPredictionValues.filterMode = cudaFilterModePoint;
    tLRPPredictionValues.normalized = false;    // access with normalized texture coordinates

    //cudaExtent cExtent;
    //cExtent.width = 10;   cExtent.height = 10;   cExtent.depth = uiWeakClasifiersCount;
    //cudaMalloc3DArray( &g_cuaPredictors, &tLRPPredictionValues.channelDesc, cExtent); 
    //cudaMemcpyToArray( g_cuaPredictors, 0, 0, &_psContextData->m_vfLRPPredictors[0], sizeof(float)*100*uiWeakClasifiersCount, cudaMemcpyHostToDevice);
    cudaMallocArray( &g_cuaPredictors, &tLRPPredictionValues.channelDesc, 100, uiWeakClasifiersCount); 
    cudaMemcpyToArray( g_cuaPredictors, 0, 0, &_psContextData->m_vfLRPPredictors[0], sizeof(float)*100*uiWeakClasifiersCount, cudaMemcpyHostToDevice);
    cudaBindTextureToArray( tLRPPredictionValues, g_cuaPredictors, tLRPPredictionValues.channelDesc);
  }else if(_psContextData->m_vfLBPPredictors.size()>0){
    tLBPPredictionValues.addressMode[0] = cudaAddressModeClamp;
    tLBPPredictionValues.addressMode[1] = cudaAddressModeClamp;
    tLBPPredictionValues.filterMode = cudaFilterModePoint;
    tLBPPredictionValues.normalized = false;    // access with normalized texture coordinates

    //cudaExtent cExtent;
    //cExtent.width = 10;   cExtent.height = 10;   cExtent.depth = uiWeakClasifiersCount;
    //cudaMalloc3DArray( &g_cuaPredictors, &tLRPPredictionValues.channelDesc, cExtent); 
    //cudaMemcpyToArray( g_cuaPredictors, 0, 0, &_psContextData->m_vfLRPPredictors[0], sizeof(float)*100*uiWeakClasifiersCount, cudaMemcpyHostToDevice);
    cudaMallocArray( &g_cuaPredictors, &tLBPPredictionValues.channelDesc, 256, uiWeakClasifiersCount); 
    cudaMemcpyToArray( g_cuaPredictors, 0, 0, &_psContextData->m_vfLBPPredictors[0], sizeof(float)*256*uiWeakClasifiersCount, cudaMemcpyHostToDevice);
    cudaBindTextureToArray( tLBPPredictionValues, g_cuaPredictors, tLBPPredictionValues.channelDesc);
  }

  // Load WeakParameters
  cudaMemcpyToSymbol("c_uiWeakParam", &_psContextData->m_vuiWeakParam[0], sizeof(int)*WEAK_PARAMS*uiWeakClasifiersCount, 0, cudaMemcpyHostToDevice);

  tWeakParam.addressMode[0] = cudaAddressModeClamp;
  tWeakParam.addressMode[1] = cudaAddressModeClamp;
  tWeakParam.filterMode = cudaFilterModePoint;
  tWeakParam.normalized = false;    // access with normalized texture coordinates

  cudaMallocArray( &g_cuaWeakParam, &tWeakParam.channelDesc, uiWeakClasifiersCount, 1 ); 
  cudaMemcpyToArray( g_cuaWeakParam, 0, 0, &_psContextData->m_vuiWeakParam[0], sizeof(int)*WEAK_PARAMS*uiWeakClasifiersCount, cudaMemcpyHostToDevice);
  cudaBindTextureToArray( tWeakParam, g_cuaWeakParam, tWeakParam.channelDesc);


  cudaThreadSynchronize();
  
  _psContextData->m_bClassifiersLoaded = true;

  printf("\t\tOK\n");
}

void UnloadClasifiersFromGPU()
{
  printf("Unloading Classifiers from GPU...");
  
  cudaFreeArray(g_cuaPredictors);
  cudaFreeArray(g_cuaWeakParam);

  printf("\t\tOK\n");
}







t_sTimeStamps::t_sTimeStamps()
{
  GetTimeFunc = &GetTimeMy2;

  for(int f=0; f<LRD_NUM_TIMESTAMPS; f++)
    g_pdTimeStamp[f]=0.0;
}















void t_sImageData::CudaRegisterPBO()
{
  cudaGLRegisterBufferObject(m_uiPBOId);
}

void t_sImageData::CudaUnRegisterPBO()
{
  cudaGLUnregisterBufferObject(m_uiPBOId);
}



void t_sImageData::PrepareTexture(unsigned int _uiImageW, unsigned int _uiImageH, unsigned int _uiImageWPitch, unsigned char _uiComponents, unsigned char *_Data)
{
  m_sTimeStamps.g_pdTimeStamp[LRD_TEX_PREP_S] = (*m_sTimeStamps.GetTimeFunc)();

  if(m_uiState==0){
    m_uiState++; 

    m_uiImageW = _uiImageW;
    m_uiImageH = _uiImageH;

    if(!glIsTexture(m_uiImageTextureId)){        
      glGenTextures(1, &m_uiImageTextureId);
      glBindTexture(GL_TEXTURE_2D, m_uiImageTextureId );
      glTexParameteri ( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
      glTexParameteri ( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
      glTexParameteri ( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST  );	// Set Filtering type
      glTexParameteri ( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST  );	// Set Filtering type
      glTexEnvf ( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );			//  Set Texture Evironment

      if(m_uiVerbose>0) printf("m_uiImageTextureId generated. Id=%i\n", m_uiImageTextureId);
    }

    glBindTexture(GL_TEXTURE_2D, m_uiImageTextureId );
    
    int iFormat = 0;
    switch(_uiComponents){
      case 1: iFormat = GL_LUMINANCE; break;
      case 4: iFormat = GL_BGRA;      break;
      default: break;
    }

    if(_uiImageW==_uiImageWPitch){
      glTexImage2D(GL_TEXTURE_2D, 0,  GL_LUMINANCE8, m_uiImageW, m_uiImageH, 0, iFormat, GL_UNSIGNED_BYTE, (void*)_Data);
    }else{
      unsigned char *data = (unsigned char*)malloc(_uiImageW*_uiImageH * sizeof(unsigned char));

      for(unsigned int f=0; f<_uiImageH; f++)
        memcpy(&data[f*_uiImageW], &_Data[f*_uiImageWPitch], _uiImageW*sizeof(unsigned char));
      
      glTexImage2D(GL_TEXTURE_2D, 0,  GL_LUMINANCE8, m_uiImageW, m_uiImageH, 0, iFormat, GL_UNSIGNED_BYTE, (void*)data);
      free(data);
      printf("WARNING: PrepareTexture memcpy!\n");
    }

  }


  m_sTimeStamps.g_pdTimeStamp[LRD_TEX_PREP_E] = (*m_sTimeStamps.GetTimeFunc)();
}


void t_sImageData::PyramidScaled(float _fScaleFactor)
{
  m_sTimeStamps.g_pdTimeStamp[LRD_PYRAMID_S] = (*m_sTimeStamps.GetTimeFunc)();

  if(m_uiState==1){
    m_uiState++;

    unsigned int uiScaleLevels = int(log(24.0f/float(m_uiImageH)) / log(_fScaleFactor));
    float fHalfScale = _fScaleFactor/(1.0f+_fScaleFactor);  // s/0.8+s = 1.0 => s+s*0.8 = 0.8
    unsigned int uiHalfLevels = unsigned int(logf(fHalfScale)/logf(_fScaleFactor)) + 1;

    float fHeightScale = 0.0f;
    for(unsigned int f=0; f<uiHalfLevels; f++){
      fHeightScale = 1.0f+_fScaleFactor*fHeightScale;
    }

    m_uiScaleTextureW = m_uiImageW;
    m_uiScaleTextureH = fHeightScale*m_uiImageH;

    if(m_ScaleTextureScales==NULL){
      m_ScaleTextureScales = (tPositionScale*)calloc(m_uiScaleTextureW*m_uiScaleTextureH, sizeof(tPositionScale));

      float x = 0.0f, y = 0.0f;
      float w = m_uiImageW, h = m_uiImageH;
      float s = 1.0f;
      
      for(unsigned int f=0; f<uiScaleLevels; f++){
        if(s>=fHalfScale){
          FillPositionScaleArray(s, x, y, w, h, m_uiScaleTextureW, m_ScaleTextureScales);
          y+=h;
        }else{
          FillPositionScaleArray(s, m_uiScaleTextureW-w, y-h, w, h, m_uiScaleTextureW, m_ScaleTextureScales);
          y-=h;
        }
        w*=_fScaleFactor;
        h*=_fScaleFactor;
        s*=_fScaleFactor;
      }

      
      if(m_uiVerbose>0) printf("g_ScaleTextureScales generated. Size=%i KiB\n", m_uiScaleTextureW*m_uiScaleTextureH*sizeof(tPositionScale)/1024);

/*
      FILE *stream;
      char sFName[255];
      sprintf( sFName,"ScaleTexture.%ix%ix8b.raw", m_uiScaleTextureW, m_uiScaleTextureH); // C4996
      if( fopen_s( &stream, sFName, "w+b" ) == 0 ){
        for(int f1=0; f1<m_uiScaleTextureH; f1++)
          for(int f2=0; f2<m_uiScaleTextureW; f2++){
            char data = 255*m_ScaleTextureScales[m_uiScaleTextureW*f1+f2].fScale;
            fwrite( &data, sizeof( char ), 1, stream );
          }
        printf("Created: %s\n", sFName);
        fclose( stream );
      }
*/

    }

    

    glViewport(0, 0, m_uiScaleTextureW, m_uiScaleTextureH);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluOrtho2D(0, m_uiScaleTextureW-1, 0, m_uiScaleTextureH-1);

    if(!glIsTexture(m_uiScaleTextureId)){        
        glGenTextures(1, &m_uiScaleTextureId);
        glBindTexture(GL_TEXTURE_2D, m_uiScaleTextureId);
        glTexImage2D(GL_TEXTURE_2D, 0, GL_LUMINANCE8, m_uiScaleTextureW, m_uiScaleTextureH, 0, GL_LUMINANCE, GL_UNSIGNED_BYTE, NULL);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);

        if(m_uiVerbose>0) printf("m_ScaleTextureScales (%ix%i) generated. Id=%i\n",m_uiScaleTextureW, m_uiScaleTextureH, m_uiScaleTextureId);
    }

    static unsigned int uiFrameBuffer = 0;
    if(!glIsFramebufferEXT(uiFrameBuffer)){
        glGenFramebuffersEXT(1, &uiFrameBuffer);
        glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, uiFrameBuffer);
        // attach texture to framebuffer
        glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, m_uiScaleTextureId, 0);
        glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
        glClear(GL_COLOR_BUFFER_BIT);
        glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);

        if(m_uiVerbose>0) printf("uiFrameBuffer generated. Id=%i\n",uiFrameBuffer);
    }

    if(!glIsBuffer(m_uiPBOId)){
      glGenBuffers(1, &m_uiPBOId);
      glBindBuffer(GL_PIXEL_PACK_BUFFER, m_uiPBOId);
      glBufferData(GL_PIXEL_PACK_BUFFER, m_uiScaleTextureW*m_uiScaleTextureH*sizeof(unsigned char), NULL, GL_DYNAMIC_COPY);
      glBindBuffer(GL_PIXEL_PACK_BUFFER, 0);
      CudaRegisterPBO();

      if(m_uiVerbose>0) printf("uiPBOId generated. Id=%i\n",m_uiPBOId);
    }

    
    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, uiFrameBuffer);
    glBindBuffer(GL_PIXEL_PACK_BUFFER, m_uiPBOId);

    glPushAttrib(GL_ENABLE_BIT);
    glDisable(GL_DEPTH_TEST);
    glDisable(GL_LIGHTING);
    glDisable(GL_BLEND);
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D, m_uiImageTextureId );
    glColor4f(1.0f, 1.0f, 1.0f, 1.0f);


    float x = 0.0f, y = 0.0f;
    float w = m_uiImageW, h = m_uiImageH;
    float s = 1.0f;
    glBegin(GL_QUADS);
    for(unsigned int f=0; f<uiScaleLevels; f++){
      if(s>=fHalfScale){
        //glBegin(GL_QUADS);
          glTexCoord2f( 0.0f,  0.0f);   glVertex2f( x,        y );
          glTexCoord2f( 1.0f,  0.0f);   glVertex2f( x+w-1.0f, y );
          glTexCoord2f( 1.0f,  1.0f);   glVertex2f( x+w-1.0f, y+h-1.0f );
          glTexCoord2f( 0.0f,  1.0f);   glVertex2f( x,        y+h-1.0f );
        //glEnd();

        y+=h;
      }else{
        //glBegin(GL_QUADS);
          glTexCoord2f( 0.0f,  0.0f);   glVertex2f( m_uiScaleTextureW-w+1.0f, y-h+1.0f );
          glTexCoord2f( 1.0f,  0.0f);   glVertex2f( m_uiScaleTextureW,        y-h+1.0f );
          glTexCoord2f( 1.0f,  1.0f);   glVertex2f( m_uiScaleTextureW,        y );
          glTexCoord2f( 0.0f,  1.0f);   glVertex2f( m_uiScaleTextureW-w+1.0f, y );
        //glEnd();

        y-=h;
      }
      w*=_fScaleFactor;
      h*=_fScaleFactor;
      s*=_fScaleFactor;
    }
    glEnd();
    

    glReadPixels(0, 0, m_uiScaleTextureW, m_uiScaleTextureH, GL_LUMINANCE, GL_UNSIGNED_BYTE, NULL);

    
    glBindBuffer(GL_PIXEL_PACK_BUFFER, 0);
    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
    
    glPopAttrib();
  }

  

  m_sTimeStamps.g_pdTimeStamp[LRD_PYRAMID_E] = (*m_sTimeStamps.GetTimeFunc)();
 
}

void t_sImageData::CudaLRD(t_eCudaClassifierType _eClassifierType) 
{

    m_sTimeStamps.g_pdTimeStamp[LRD_CUDA_S] = (*m_sTimeStamps.GetTimeFunc)();
    
    
    if(m_uiState==2){
      m_uiState++;

      if(m_iStream==0){
        if(m_uiVerbose>0) printf("Creating Stream : ");
        cudaStreamCreate(&m_iStream);
        if(m_uiVerbose>0) printf("%i\t\t\tOK\n", m_iStream);
      }

      static bool bPrintedScaledImageSize=false;
      if(!bPrintedScaledImageSize){
        printf("Scaled Image Size %ix%i\n",m_uiScaleTextureW, m_uiScaleTextureH);
        bPrintedScaledImageSize = true;
      }

      
      tex.normalized = false;
      tex.filterMode = cudaFilterModeLinear;
      tex.addressMode[0] = cudaAddressModeClamp;
      tex.addressMode[1] = cudaAddressModeClamp;
      tex.addressMode[2] = cudaAddressModeClamp;

      // set image
      unsigned char *puiCudaPBOData;
      cudaGLMapBufferObject((void**)&puiCudaPBOData, m_uiPBOId);

      if(m_d_cuaImage==NULL) 
        cudaMallocArray( &m_d_cuaImage, &tex.channelDesc, m_uiScaleTextureW, m_uiScaleTextureH); 
      cudaMemcpyToArrayAsync( m_d_cuaImage, 0, 0, puiCudaPBOData, sizeof(unsigned char)*m_uiScaleTextureW*m_uiScaleTextureH, cudaMemcpyDeviceToDevice, m_iStream);
      cudaBindTextureToArray( tex, m_d_cuaImage);

      // unmap buffer object sooner befor starting async kernel
      cudaGLUnmapBufferObject(m_uiPBOId);
      
      //cudaChannelFormatDesc channelDesc = cudaCreateChannelDesc<unsigned char>();
      //cudaBindTexture2D(NULL, tImage1, puiCudaPBOData, channelDesc, _psContextData->m_sImage0Data.m_uiScaleTextureW, _psContextData->m_sImage0Data.m_uiScaleTextureH, _psContextData->m_sImage0Data.m_uiScaleTextureW);



      // allocate device memory for results
      if(m_puiDetections==NULL)
        cudaHostAlloc( (void **)&m_puiDetections, MAX_DETECTIONS_COUNT * sizeof(uint2), cudaHostAllocWriteCombined | cudaHostAllocMapped); 
      if(m_d_pui2Detections==NULL)
        cudaHostGetDevicePointer( (void**)&m_d_pui2Detections, m_puiDetections, 0);

      
      if(m_pfDetectionsRet==NULL)
        cudaHostAlloc( (void **)&m_pfDetectionsRet, MAX_DETECTIONS_COUNT * sizeof(float), cudaHostAllocWriteCombined | cudaHostAllocMapped); 
      if(m_d_pfDetectionsRet==NULL)
        cudaHostGetDevicePointer( (void**)&m_d_pfDetectionsRet, m_pfDetectionsRet, 0);


      if(m_puiDetectionsCount==NULL)
        cudaHostAlloc( (void **)&m_puiDetectionsCount, sizeof(unsigned int), cudaHostAllocWriteCombined | cudaHostAllocMapped); 
      if(m_d_puiDetectionsCount==NULL) 
        cudaHostGetDevicePointer( (void**)&m_d_puiDetectionsCount, m_puiDetectionsCount, 0);
      *m_puiDetectionsCount = 0;    


#ifdef LRD_DEBUG
      static unsigned int* d_puiCutDetectionCount = NULL;
      if(d_puiCutDetectionCount==NULL){
        cudaMalloc( (void**)&d_puiCutDetectionCount, 3*sizeof(unsigned int) );
        cudaMemcpyToSymbol("c_puiCutDetectionCount", &d_puiCutDetectionCount, sizeof(unsigned int), 0, cudaMemcpyHostToDevice);
      }
      cudaMemset( (void*)d_puiCutDetectionCount, 0, 3*sizeof(unsigned int) );

      static unsigned int* d_puiDetectionOverflows = NULL;
      if(d_puiDetectionOverflows==NULL){
        cudaMalloc( (void**)&d_puiDetectionOverflows, sizeof(unsigned int) );
        cudaMemcpyToSymbol("c_puiDetectionOverflows", &d_puiDetectionOverflows, sizeof(unsigned int), 0, cudaMemcpyHostToDevice);
      }
      cudaMemset( (void*)d_puiDetectionOverflows, 0, sizeof(unsigned int) );
#endif
      
#ifdef LRD_WEAK_COUNT
      
      if(m_puiWeakCount==NULL)
        cudaHostAlloc( (void **)&m_puiWeakCount, m_uiScaleTextureW*m_uiScaleTextureH * sizeof(unsigned short), cudaHostAllocWriteCombined | cudaHostAllocMapped); 

      if(m_d_puiWeakCount==NULL){
        cudaHostGetDevicePointer( (void**)&m_d_puiWeakCount, m_puiWeakCount, 0);
        cudaMemcpyToSymbol("c_puiWeakCount", &m_d_puiWeakCount, sizeof(unsigned short*), (m_iStream-1)*sizeof(unsigned short*), cudaMemcpyHostToDevice);
      }
      
      memset(m_puiWeakCount, 0, m_uiScaleTextureW*m_uiScaleTextureH * sizeof(unsigned short));
#endif

      // setup execution parameters
      dim3  grid( (unsigned int)ceil((float)m_uiScaleTextureW/SCANW), (unsigned int)ceil((float)m_uiScaleTextureH/THREADS), 1);
      dim3  threads( 1, THREADS, 1);

      m_sTimeStamps.g_pdTimeStamp[LRD_CUDA_PRE] = (*m_sTimeStamps.GetTimeFunc)();
      
      // execute kernel
      if(_eClassifierType==CUDA_LRD){
        CudaKernelLRD<<< grid, threads, 0, m_iStream>>>( m_iStream-1, m_d_pui2Detections, m_d_pfDetectionsRet, m_d_puiDetectionsCount );
        //printf("LRD Kernel\n");
      }else if(_eClassifierType==CUDA_LRP){
        CudaKernelLRP<<< grid, threads, 0, m_iStream>>>( m_iStream-1, m_d_pui2Detections, m_d_pfDetectionsRet, m_d_puiDetectionsCount );
        //printf("LRP Kernel\n");
      }else if(_eClassifierType==CUDA_LBP){
        CudaKernelLBP<<< grid, threads, 0, m_iStream>>>( m_iStream-1, m_d_pui2Detections, m_d_pfDetectionsRet, m_d_puiDetectionsCount );
        //printf("LRP Kernel\n");
      }else{
        printf("Unknown Classifier Type\n");
      }
      //CUT_CHECK_ERROR("Kernel execution failed");
      
      m_sTimeStamps.g_pdTimeStamp[LRD_CUDA_RUN] = (*m_sTimeStamps.GetTimeFunc)();


      //printf("Det: %3i\n", *_psContextData->m_sImage0Data.m_puiDetectionsCount);
#ifdef LRD_DEBUG
      // get debug output info
      if(m_uiVerbose>0){
        unsigned int uiDetectionsCount1, uiDetectionsCount2, uiDetectionsCount3;
        cudaMemcpy( &uiDetectionsCount1, d_puiCutDetectionCount+0, sizeof(unsigned int), cudaMemcpyDeviceToHost);
        cudaMemcpy( &uiDetectionsCount2, d_puiCutDetectionCount+1, sizeof(unsigned int), cudaMemcpyDeviceToHost);
        cudaMemcpy( &uiDetectionsCount3, d_puiCutDetectionCount+2, sizeof(unsigned int), cudaMemcpyDeviceToHost);

        printf("GridDim: %i x %i\t", grid.x, grid.y);
        printf("Det: %i -> %6i -> %5i -> %4i -> %3i [det]\t", m_uiScaleTextureW*m_uiScaleTextureH, 
                                                              uiDetectionsCount1, 
                                                              uiDetectionsCount2, 
                                                              uiDetectionsCount3, 
                                                              *m_puiDetectionsCount);
        printf("%.1f -> %6.2f -> %6.2f -> %5.2f -> %5.3f [det/block]\t", float(m_uiScaleTextureW*m_uiScaleTextureH)/(grid.x*grid.y),
                                                                         float(uiDetectionsCount1)/(grid.x*grid.y),
                                                                         float(uiDetectionsCount2)/(grid.x*grid.y),
                                                                         float(uiDetectionsCount3)/(grid.x*grid.y),
                                                                         float(*m_puiDetectionsCount)/(grid.x*grid.y));
      }
      unsigned int uiDetectionsCountOverflows;
      cudaMemcpy( &uiDetectionsCountOverflows, d_puiDetectionOverflows, sizeof(unsigned int), cudaMemcpyDeviceToHost);
      printf("Overflows: %4i\n", uiDetectionsCountOverflows);
#endif

    }
    

    m_sTimeStamps.g_pdTimeStamp[LRD_CUDA_E] = (*m_sTimeStamps.GetTimeFunc)();

    // cleanup memory
    //cudaFreeArray(cuaImage);
    //cudaFree(d_odata);
}


void t_sImageData::GetData(vector<t_cDetection> *_vDetections)
{
  m_sTimeStamps.g_pdTimeStamp[LRD_GET_S] = (*m_sTimeStamps.GetTimeFunc)();
  
  _vDetections->clear();

  if(m_uiState==3){

    m_uiState=0;

    cudaStreamSynchronize(m_iStream);

    t_cDetection out;
    for(unsigned int f=0; f<*m_puiDetectionsCount;f++){
      // get LRD output
      unsigned int uiAbsX = m_puiDetections[2*f+0];
      unsigned int uiAbsY = m_puiDetections[2*f+1];
      
      // get scale and scale-position
      tPositionScale sPositionScale = m_ScaleTextureScales[ uiAbsY*m_uiScaleTextureW+uiAbsX ];
      
      // transform with scale and scale-position to oreginal picture
      out.x =  int(float(uiAbsX-sPositionScale.uiStartX)/sPositionScale.fScale);
      out.y =  int(float(uiAbsY-sPositionScale.uiStartY)/sPositionScale.fScale);
      out.dx = int(24.0f/sPositionScale.fScale);
      out.dy = int(24.0f/sPositionScale.fScale);
      out.angle = 0.0f;
      out.response = m_pfDetectionsRet[f];
      
      _vDetections->push_back(out);
    }



#ifdef LRD_WEAK_COUNT
    static unsigned int id = 0;
    FILE *stream;
    char  sFName[200];
    
    
    unsigned int a;
    unsigned int b;
    float d;
    unsigned int uiToatalWeakCount = 0;

    for(unsigned int f0=0; f0<m_uiScaleTextureH*m_uiScaleTextureW; f0++)
      uiToatalWeakCount+=m_puiWeakCount[f0];
    printf("%i TotalWeakCount = %i\n", id, uiToatalWeakCount);

/*
    sprintf( sFName,"WeakCountStat.%ix%i.csv", m_uiScaleTextureW, m_uiScaleTextureH);
    if( (id==0) && (fopen_s( &stream, sFName, "w+" )==0) ){
        a = 0;  b = 0;  d = 1.0f;
        while(a<=1000){
          fprintf(stream, "%i,", a);
          a=b+1;  b=b+int(d);  d*=1.1f;
        }
        fprintf(stream, "\n");

        a = 0;  b = 0;  d = 1.0f;
        while(a<=1000){
          fprintf(stream, "%i,", b);
          a=b+1;  b=b+int(d);  d*=1.1f;
        }
        fprintf(stream, "\n");
        printf("Created: %s\n", sFName);

        fclose( stream );
    }

    if( fopen_s( &stream, sFName, "a+" )==0 ){
      a = 0;  b = 0;  d = 1.0f;
      while(a<=1000){
        unsigned int uiCount = 0;
        
        for(unsigned int f0=0; f0<m_uiScaleTextureH*m_uiScaleTextureW; f0++)
          if( (a<=m_puiWeakCount[f0]) && (m_puiWeakCount[f0]<=b) ) uiCount++;

        fprintf(stream, "%i,", uiCount);
        a=b+1;  b=b+int(d);  d*=1.1f;
      }
      fprintf(stream, "\n");

      
      fclose( stream );
    }
*/
    

/*
    sprintf( sFName,"%i.%i.WeakCount.%ix%i.csv", id, m_iStream, m_uiScaleTextureW, m_uiScaleTextureH); // C4996
    if( fopen_s( &stream, sFName, "w+" ) == 0 ){
      
      for(unsigned int f0=0; f0<m_uiScaleTextureH; f0++){
        for(unsigned int f1=0; f1<m_uiScaleTextureW; f1++)
          fprintf(stream, "%i,", m_puiWeakCount[f0*m_uiScaleTextureW+f1]);
        fprintf(stream, "\n");
      }

      printf("Created: %s\n", sFName);
      fclose( stream );
    }
*/

/*
    for(unsigned int f0=0; f0<m_uiScaleTextureH*m_uiScaleTextureW; f0++)
      m_puiWeakCount[f0] *= 65;
    
    sprintf( sFName,"%i.%i.WeakCount.%ix%ix16b.raw", id, m_iStream, m_uiScaleTextureW, m_uiScaleTextureH); // C4996
    if( fopen_s( &stream, sFName, "w+b" ) == 0 ){
      fwrite( m_puiWeakCount, sizeof( unsigned short ), m_uiScaleTextureW*m_uiScaleTextureH, stream );
      printf("Created: %s\n", sFName);
      fclose( stream );
    }
*/

    id++;

#endif

  }

  m_sTimeStamps.g_pdTimeStamp[LRD_GET_E] = (*m_sTimeStamps.GetTimeFunc)();
}





















void t_sContextData::InitGL()
{
  if((!m_bUseGL) || m_bInitializedGL) return;

  if(m_uiVerbose>0) printf("Initializing GL...");

  int argc = 0;
  char *argv = (char*)calloc(1,sizeof(char));

  glutInitDisplayMode(GLUT_RGBA);
  glutInit(&argc, &argv);
  glutCreateWindow("");
  glutHideWindow();

  //SetVSync(false);

  glShadeModel(GL_SMOOTH);
  glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

  free(argv);

  m_bInitializedGL = true;

  if(m_uiVerbose>0) printf("\t\t\tOK\n");
}


void t_sContextData::InitCUDA()
{
  if(m_iInitializedDevice>=0) return;

  #ifdef _DEBUG
    printf("WARNING: DEBUG compilation\n");
  #endif

  printf("DEFINES %i %i %i %i\n", SCANW, CUT1, CUT2, CUT3);
  
  int deviceCount;
  cudaGetDeviceCount(&deviceCount);
  for (int device = 0; device < deviceCount; ++device) {
    cudaDeviceProp deviceProp;
    cudaGetDeviceProperties(&deviceProp, device);

    if(m_uiVerbose>0){
      printf("Device Properities\n", device);
      printf("\tId\t\t\t\t%i\n", device);
      printf("\tName\t\t\t\t%s\n", deviceProp.name);
      printf("\tCompute Cap\t\t\t%i.%i\n", deviceProp.major, deviceProp.minor);
      printf("\tGlobal Memory(total)\t\t%i MB\n", deviceProp.totalGlobalMem/(1024*1024));
      printf("\tConst Memory(total)\t\t%i B\n", deviceProp.totalConstMem);
      printf("\tShared Memoty(block)\t\t%i B\n", deviceProp.sharedMemPerBlock);
      printf("\tRegisters(block)\t\t%i\n", deviceProp.regsPerBlock);
      printf("\tMax Threads(block)\t\t%i\n", deviceProp.maxThreadsPerBlock);
      printf("\tMax Threads(dim)\t\t%i x %i x %i\n", deviceProp.maxThreadsDim[0],deviceProp.maxThreadsDim[1],deviceProp.maxThreadsDim[2]);
      printf("\tMax Grid Size\t\t\t%i x %i x %i\n", deviceProp.maxGridSize[0],deviceProp.maxGridSize[1],deviceProp.maxGridSize[2]);
      printf("\tWarp Size\t\t\t%i\n", deviceProp.warpSize);
      printf("\tMax Memory Pitch\t\t%i B\n", deviceProp.memPitch);
      printf("\tMultiProc Count\t\t\t%i\n", deviceProp.multiProcessorCount);
      printf("\tClock Rate\t\t\t%i MHz\n", deviceProp.clockRate/1000);
      printf("\tDevice Overlaping\t\t%s\n", deviceProp.deviceOverlap?"Yes":"No");
      printf("\tMap Host Memory\t\t\t%s\n", deviceProp.canMapHostMemory?"Yes":"No");
      printf("\tKrnl Exec Timeout Enabled\t%s\n", deviceProp.kernelExecTimeoutEnabled?"Yes":"No");
      printf("\tDevice Integrated\t\t%s\n", deviceProp.integrated?"Yes":"No");
      printf("\tCompute Mode\t\t\t%s\n\n", deviceProp.computeMode);
    }
  }

  if(m_uiVerbose>0) printf("Initializing CUDA...");
  if(m_iUseDevice<0) m_iUseDevice = 0;
  cudaSetDevice(m_iUseDevice);
  m_iInitializedDevice = m_iUseDevice;
  if(m_uiVerbose>0) printf("\t\t\tOK\n");

  if(m_uiVerbose>0) printf("Setting DeviceFlags to %i", cudaDeviceScheduleAuto | cudaDeviceMapHost);
  cudaSetDeviceFlags(cudaDeviceScheduleAuto | cudaDeviceMapHost);
  if(m_uiVerbose>0) printf("\t\tOK\n");
}


void t_sContextData::LoadConstants()
{
  if(m_bConstLoaded) return;

  if(m_uiVerbose>0) printf("Loading Constants to GPU...");
  
  // set constants
    
  //printf("c_uiW = %i\n", m_sImageData[0].m_uiScaleTextureW);
  cudaMemcpyToSymbol("c_uiW", &m_sImageData[0].m_uiScaleTextureW, sizeof(unsigned int), 0, cudaMemcpyHostToDevice);
  //printf("c_uiH = %i\n", m_sImageData[0].m_uiScaleTextureH);
  cudaMemcpyToSymbol("c_uiH", &m_sImageData[0].m_uiScaleTextureH, sizeof(unsigned int), 0, cudaMemcpyHostToDevice);

  cudaThreadSynchronize();

  m_bConstLoaded = true;

  if(m_uiVerbose>0) printf("\t\tOK\n");
}



void t_sContextData::ImageStart(unsigned int _uiStreamId, unsigned int _uiImageW, unsigned int _uiImageH, unsigned int _uiImageWPitch, unsigned char _uiComponents, unsigned char *_Data)
{
  InitGL();
  InitCUDA();
  LoadClassifiersToGPU(this);


  m_sImageData[_uiStreamId].m_sTimeStamps.g_pdTimeStamp[LRD_S] = (*m_sImageData[_uiStreamId].m_sTimeStamps.GetTimeFunc)();
  
  m_sImageData[_uiStreamId].PrepareTexture(_uiImageW, _uiImageH, _uiImageWPitch, _uiComponents, _Data);
  m_sImageData[_uiStreamId].PyramidScaled();
  
  LoadConstants();
}

void t_sContextData::ImageFinish(unsigned int _uiStreamId, vector<t_cDetection> *_vDetections)
{
  m_sImageData[_uiStreamId].CudaLRD(m_vfLRDPredictors.size()>0?CUDA_LRD:m_vfLRPPredictors.size()>0?CUDA_LRP:m_vfLBPPredictors.size()>0?CUDA_LBP:CUDA_UNKNOWN);
  m_sImageData[_uiStreamId].GetData(_vDetections);
  
  m_sImageData[_uiStreamId].m_sTimeStamps.g_pdTimeStamp[LRD_E] = (*m_sImageData[_uiStreamId].m_sTimeStamps.GetTimeFunc)();
}





void t_sContextData::SetVerbosity(unsigned int _uiVerbosity)
{
  m_uiVerbose = _uiVerbosity;
  for(int f=0; f<MAX_STREAMS; f++)
    m_sImageData[f].m_uiVerbose = _uiVerbosity;
}

t_sContextData::t_sContextData()
{
  m_iUseDevice = -1;
  m_iInitializedDevice = -1;

  m_bUseGL = true;
  m_bInitializedGL = false;

  m_bConstLoaded = false;
  m_bClassifiersLoaded = false;

  m_uiVerbose = 0;
}

t_sContextData::~t_sContextData()
{
  UnloadClasifiersFromGPU();
}
