#ifndef LRD_H
#define LRD_H


#include <vector>
#include <windows.h>


#include <builtin_types.h>
#include <cuda_texture_types.h>



using namespace std;

#define DEFAULTSCALE 0.84f //pow(2.0f, -0.25f)
#define MAX_STREAMS 3


enum t_eCudaClassifierType { CUDA_UNKNOWN=0, CUDA_LRD, CUDA_LRP, CUDA_LBP, CUDA_numClassifierTypes };


struct tPositionScale {
  unsigned int uiStartX;
  unsigned int uiStartY;
  float fScale;

  tPositionScale &operator=( const tPositionScale &a)   { uiStartX = a.uiStartX; uiStartY = a.uiStartY; fScale = a.fScale; return *this; };
};

struct tInt4 {
  int iPositionX;
  int iPositionY;
  int iBlockWH;
  int iBlockAB;

  tInt4 &operator=( const tInt4 &a)   { iPositionX = a.iPositionX; iPositionY = a.iPositionY; iBlockWH = a.iBlockWH; iBlockAB = a.iBlockAB; return *this; };

  tInt4(int _iPositionX, int _iPositionY, int _iBlockWH, int _iBlockAB){
    iPositionX = _iPositionX;
    iPositionY = _iPositionY;
    iBlockWH = _iBlockWH;
    iBlockAB = _iBlockAB;
  };
};

struct tFloat17 {
  float fParam[17];

  tFloat17 &operator=( const tFloat17 &a)   { memcpy((void*)(fParam), (void*)(a.fParam), sizeof(tFloat17)); return *this; };

  tFloat17(float _fParam[17])               { memcpy((void*)(fParam), (void*)(_fParam), sizeof(tFloat17)); };
};

struct tFloat100 {
  float fParam[100];

  tFloat100 &operator=( const tFloat100 &a)   { memcpy((void*)(fParam), (void*)(a.fParam), sizeof(tFloat100)); return *this; };

  tFloat100(float _fParam[100])               { memcpy((void*)(fParam), (void*)(_fParam), sizeof(tFloat100)); };
};

struct tFloat256 {
  float fParam[256];

  tFloat256 &operator=( const tFloat256 &a)   { memcpy((void*)(fParam), (void*)(a.fParam), sizeof(tFloat256)); return *this; };

  tFloat256(float _fParam[256])               { memcpy((void*)(fParam), (void*)(_fParam), sizeof(tFloat256)); };
};

enum t_eTimeStamps{
  LRD_S,
  LRD_TEX_PREP_S,
  LRD_TEX_PREP_E,
  LRD_PYRAMID_S,
  LRD_PYRAMID_E,
  LRD_CUDA_S,
  LRD_CUDA_PRE,
  LRD_CUDA_RUN,
  LRD_CUDA_E,
  LRD_GET_S,
  LRD_GET_E,
  LRD_E,
  LRD_NUM_TIMESTAMPS
};

struct t_sTimeStamps{
  double (*GetTimeFunc)();
  double g_pdTimeStamp[LRD_NUM_TIMESTAMPS];

  t_sTimeStamps();
};

struct t_cDetection{
  unsigned int x, y;
  unsigned int dx, dy;
  float response;
  float angle;
  

  t_cDetection &operator =( t_cDetection p)   { memcpy((void*)this, (void*)&p, sizeof(t_cDetection)); return *this; };
};


struct t_sImageData{
  unsigned int    m_uiVerbose;
  unsigned int    m_uiState;

  cudaStream_t    m_iStream;
  cudaArray      *m_d_cuaImage;
  uint2          *m_d_pui2Detections;
  float          *m_d_pfDetectionsRet;
  unsigned int   *m_d_puiDetectionsCount;
  
  unsigned short *m_puiWeakCount;
  unsigned short *m_d_puiWeakCount;

  //t_tImage        m_tImage;

  unsigned int    m_uiImageW;
  unsigned int    m_uiImageH;
  unsigned int    m_uiImageTextureId;

  unsigned int    m_uiScaleTextureId;
  tPositionScale *m_ScaleTextureScales;
  unsigned int    m_uiScaleTextureW;
  unsigned int    m_uiScaleTextureH;

  unsigned int    m_uiPBOId;

  unsigned int   *m_puiDetections;
  float          *m_pfDetectionsRet;
  unsigned int   *m_puiDetectionsCount;

  t_sTimeStamps   m_sTimeStamps;

  void CudaRegisterPBO();
  void CudaUnRegisterPBO();
  void PrepareTexture(unsigned int _uiImageW, unsigned int _uiImageH, unsigned int _uiImageWPitch, unsigned char _uiComponents, unsigned char *_Data);
  void PyramidScaled(float _fScaleFactor = DEFAULTSCALE);
  void CudaLRD(t_eCudaClassifierType _iPredictorType);
  void GetData(vector<t_cDetection> *_vDetections);


  t_sImageData(){
    m_uiVerbose = 0;
    m_uiState = 0;
    m_iStream = 0;
    m_d_cuaImage = NULL;
    m_d_pui2Detections = NULL;
    m_d_pfDetectionsRet = NULL;
    m_d_puiDetectionsCount = NULL;

    m_puiWeakCount = NULL;
    m_d_puiWeakCount = NULL;

    //texture<uchar1, 2, cudaReadModeNormalizedFloat>
    /*
    m_tImage.normalized = false;
    m_tImage.filterMode = cudaFilterModeLinear;
    m_tImage.addressMode[0] = cudaAddressModeClamp;
    m_tImage.addressMode[1] = cudaAddressModeClamp;
    m_tImage.addressMode[2] = cudaAddressModeClamp;

    m_tImage.channelDesc.x = 8;
    m_tImage.channelDesc.y = 8;
    m_tImage.channelDesc.z = 0;
    m_tImage.channelDesc.w = 0;
    m_tImage.channelDesc.f = cudaChannelFormatKindFloat;
    */
    m_uiImageW = 0;
    m_uiImageH = 0;
    m_uiImageTextureId = 0;

    m_uiScaleTextureId = 0;
    m_ScaleTextureScales = NULL;
    m_uiScaleTextureW = 0;
    m_uiScaleTextureH = 0;

    m_uiPBOId = 0;

    m_puiDetections = NULL;
    m_pfDetectionsRet = NULL;
    m_puiDetectionsCount =  NULL;
  }
};



class t_sContextData{
 private:
  unsigned int     m_uiVerbose;

 public:
  int              m_iUseDevice;
  int              m_iInitializedDevice;

  bool             m_bUseGL;
  bool             m_bInitializedGL;

  bool             m_bConstLoaded;
  bool             m_bClassifiersLoaded;


  vector<float>     m_vfTreshold;
  vector<tFloat17>  m_vfLRDPredictors;
  vector<tFloat100> m_vfLRPPredictors;
  vector<tFloat256> m_vfLBPPredictors;
  vector<tInt4>     m_vuiWeakParam;

  t_sImageData     m_sImageData[MAX_STREAMS];


 private:
  void InitGL();
  void InitCUDA();
  void LoadConstants();

 public:
  void SetVerbosity(unsigned int _uiVerbosity);
  
  void ImageStart(unsigned int _uiStreamId, unsigned int _uiImageW, unsigned int _uiImageH, unsigned int _uiImageWPitch, unsigned char _uiComponents, unsigned char *_Data);
  void ImageFinish(unsigned int _uiStreamId, vector<t_cDetection> *_vDetections);

  double (*GetTimeFunc)();

  t_sContextData();
  ~t_sContextData();

};






#endif

