
//#define LRD_DEBUG
//#define LRD_WEAK_COUNT

#include "comdlinemacros.h"

#define STREAMS 2
#define THREADS 128
#define MAX_BLOCK_DETECTIONS_COUNT1 1330//1070
#define MAX_BLOCK_DETECTIONS_COUNT2 600//260
#define MAX_DETECTIONS_COUNT 1000

#define DEFAULT_SCANW  240
#define DEFAULT_CUT1   6 //8
#define DEFAULT_CUT2   30
#define DEFAULT_CUT3   110


__device__ __constant__ unsigned int c_uiW;
__device__ __constant__ unsigned int c_uiH;
typedef texture<uchar1, 2, cudaReadModeNormalizedFloat> t_tImage;
t_tImage tex;

__device__ __constant__ unsigned int c_uiWeakClasifiersCount;
__device__ __constant__ float        c_uiTresholds[1100];
__device__ __constant__ uint4        c_uiWeakParam[1100];
texture<uint4, 1, cudaReadModeElementType> tWeakParam;
texture<float, 2, cudaReadModeElementType> tLRDPredictionValues;
texture<float, 2, cudaReadModeElementType> tLRPPredictionValues;
texture<float, 2, cudaReadModeElementType> tLBPPredictionValues;


//__device__ __shared__ uchar1 s_ucTmp0[MAX_BLOCK_DETECTIONS_COUNT1*6 + MAX_BLOCK_DETECTIONS_COUNT2*6];
__device__ __shared__ uchar1 s_ucTmp0[MAX_BLOCK_DETECTIONS_COUNT1*6];
__device__            uchar1 s_ucTmp1[MAX_BLOCK_DETECTIONS_COUNT2*6];

__device__ __shared__  uchar2      *s_uc2BlockDetections1;
__device__ __shared__  float       *s_fBlockDetectionsRet1;
__device__ __shared__  unsigned int s_uiBlockDetectionsCount1;
__device__             unsigned int g_uiBlockDetectionsCount1[10000];
#define s_uiBlockDetectionsCount1 g_uiBlockDetectionsCount1[gridDim.x*blockIdx.y+blockIdx.x]


__device__ __shared__  uchar2      *s_uc2BlockDetections2;
__device__ __shared__  float       *s_fBlockDetectionsRet2;
__device__ __shared__  unsigned int s_uiBlockDetectionsCount2;
__device__             unsigned int g_uiBlockDetectionsCount2[10000];
#define s_uiBlockDetectionsCount2 g_uiBlockDetectionsCount2[gridDim.x*blockIdx.y+blockIdx.x]






// ***********************************************
// auto definition
// ***********************************************

#ifdef LRD_WEAK_COUNT
  __device__ __constant__ unsigned short *c_puiWeakCount[STREAMS];
#endif

#ifdef LRD_DEBUG
  __device__ __constant__ unsigned int *c_puiCutDetectionCount;
  __device__ __constant__ unsigned int *c_puiDetectionOverflows;
#endif

#ifndef SCANW
  #define SCANW DEFAULT_SCANW
#endif

#ifndef CUT1
  #define CUT1 DEFAULT_CUT1
#endif
#ifndef CUT2
  #define CUT2 DEFAULT_CUT2
#endif
#ifndef CUT3
  #define CUT3 DEFAULT_CUT3
#endif



// ***********************************************
// implementation
// ***********************************************

__device__ float GetData(float _fX, float _fY)
{
  return tex2D( tex, _fX, _fY).x;
}

//**********************************************************************************************
//**********************************************************************************************
//******************************************** LRD *********************************************
//**********************************************************************************************
//**********************************************************************************************


#define _positionX (_WeakClasf.x)
#define _positionY (_WeakClasf.y)
#define _uiBlockSizeId (_WeakClasf.z)
#define _blockABId (_WeakClasf.w)

__device__ int LRDGetDifferentialRank(uint4 _WeakClasf, uchar2 _pos)
{
  unsigned int uiBlockWidth  = (_uiBlockSizeId >> 3);
  float         fBlockWidth  = float(uiBlockWidth) /2.0f;
  unsigned int uiBlockHeight = (_uiBlockSizeId & 7);
  float         fBlockHeight = float(uiBlockHeight)/2.0f;
  unsigned int uiBlockA = _blockABId >> 4;
  unsigned int uiBlockB = _blockABId & 15;
  unsigned int px = blockIdx.x*SCANW   + _positionX + _pos.x;
  unsigned int py = blockIdx.y*THREADS + _positionY + _pos.y;

  float fBlockASum = GetData( px + uiBlockWidth*(uiBlockA%3) +fBlockWidth, py + uiBlockHeight*(uiBlockA/3) +fBlockHeight);
  float fBlockBSum = GetData( px + uiBlockWidth*(uiBlockB%3) +fBlockWidth, py + uiBlockHeight*(uiBlockB/3) +fBlockHeight);

  float tmp;
  int iRankA=0;
  int iRankB=0;

  tmp = GetData( px+fBlockWidth, py+fBlockHeight);    iRankA += (tmp < fBlockASum);    iRankB -= (tmp < fBlockBSum);  px+=uiBlockWidth;
  tmp = GetData( px+fBlockWidth, py+fBlockHeight);    iRankA += (tmp < fBlockASum);    iRankB -= (tmp < fBlockBSum);  px+=uiBlockWidth;
  tmp = GetData( px+fBlockWidth, py+fBlockHeight);    iRankA += (tmp < fBlockASum);    iRankB -= (tmp < fBlockBSum);
  px -= 2*uiBlockWidth;    py+=uiBlockHeight;
  tmp = GetData( px+fBlockWidth, py+fBlockHeight);    iRankA += (tmp < fBlockASum);    iRankB -= (tmp < fBlockBSum);  px+=uiBlockWidth;
  tmp = GetData( px+fBlockWidth, py+fBlockHeight);    iRankA += (tmp < fBlockASum);    iRankB -= (tmp < fBlockBSum);  px+=uiBlockWidth;
  tmp = GetData( px+fBlockWidth, py+fBlockHeight);    iRankA += (tmp < fBlockASum);    iRankB -= (tmp < fBlockBSum);
  px -= 2*uiBlockWidth;    py+=uiBlockHeight;
  tmp = GetData( px+fBlockWidth, py+fBlockHeight);    iRankA += (tmp < fBlockASum);    iRankB -= (tmp < fBlockBSum);  px+=uiBlockWidth;
  tmp = GetData( px+fBlockWidth, py+fBlockHeight);    iRankA += (tmp < fBlockASum);    iRankB -= (tmp < fBlockBSum);  px+=uiBlockWidth;
  tmp = GetData( px+fBlockWidth, py+fBlockHeight);    iRankA += (tmp < fBlockASum);    iRankB -= (tmp < fBlockBSum);

  return iRankA + iRankB + 8;
}

#undef _positionX
#undef _positionY
#undef _uiBlockSizeId
#undef _blockABId


__device__ void LRDStart(uchar2 pos, cudaStream_t _StreamId)
{
  float ret = 0.0f;

  #pragma unroll
    for(unsigned int cid=0; cid<CUT1; cid++){  //cid - clasifier id
      ret += tex2D(tLRDPredictionValues, LRDGetDifferentialRank(c_uiWeakParam[cid], pos), cid); //tex1D(tWeakParam, cid)
      if(ret<c_uiTresholds[cid]){
        #ifdef LRD_WEAK_COUNT
          if(((blockIdx.y*THREADS + pos.y)<c_uiH) && ((blockIdx.x*SCANW + pos.x)<c_uiW))
            c_puiWeakCount[_StreamId][(blockIdx.y*THREADS + pos.y)*c_uiW + (blockIdx.x*SCANW + pos.x)] = cid;
        #endif
        return;
      }
    }

  unsigned int id = atomicInc( &s_uiBlockDetectionsCount1, MAX_BLOCK_DETECTIONS_COUNT1 );
  s_uc2BlockDetections1[id] = pos;
  s_fBlockDetectionsRet1[id] = ret;


#ifdef LRD_DEBUG
  atomicInc( c_puiCutDetectionCount+0, UINT_MAX );
  if(id==MAX_BLOCK_DETECTIONS_COUNT1) atomicAdd( c_puiDetectionOverflows, 1000);
#endif
}





__device__ void LRD12(unsigned int _id, cudaStream_t _StreamId)
{
  uchar2 pos = s_uc2BlockDetections1[_id];
  float  ret = s_fBlockDetectionsRet1[_id];

  #pragma unroll
    for(unsigned int cid=CUT1; cid<CUT2; cid++){  //cid - clasifier id
      ret += tex2D(tLRDPredictionValues, LRDGetDifferentialRank(c_uiWeakParam[cid], pos), cid);
      if(ret<c_uiTresholds[cid]){
        #ifdef LRD_WEAK_COUNT
          if(((blockIdx.y*THREADS + pos.y)<c_uiH) && ((blockIdx.x*SCANW + pos.x)<c_uiW))
            c_puiWeakCount[_StreamId][(blockIdx.y*THREADS + pos.y)*c_uiW + (blockIdx.x*SCANW + pos.x)] = cid;
        #endif
        return;
      }
    }

  unsigned int id = atomicInc( &s_uiBlockDetectionsCount2, MAX_BLOCK_DETECTIONS_COUNT2 );
  s_uc2BlockDetections2[id] = pos;
  s_fBlockDetectionsRet2[id] = ret;


#ifdef LRD_DEBUG
  atomicInc( c_puiCutDetectionCount+1, UINT_MAX );
  if(id==MAX_BLOCK_DETECTIONS_COUNT2) atomicAdd( c_puiDetectionOverflows, 100);
#endif
}


__device__ void LRD21(unsigned int _id, cudaStream_t _StreamId)
{
  uchar2 pos = s_uc2BlockDetections2[_id];
  float  ret = s_fBlockDetectionsRet2[_id];

  #pragma unroll
    for(unsigned int cid=CUT2; cid<CUT3; cid++){  //cid - clasifier id
      ret += tex2D(tLRDPredictionValues, LRDGetDifferentialRank(c_uiWeakParam[cid], pos), cid);
      if(ret<c_uiTresholds[cid]){
        #ifdef LRD_WEAK_COUNT
          if(((blockIdx.y*THREADS + pos.y)<c_uiH) && ((blockIdx.x*SCANW + pos.x)<c_uiW))
            c_puiWeakCount[_StreamId][(blockIdx.y*THREADS + pos.y)*c_uiW + (blockIdx.x*SCANW + pos.x)] = cid;
        #endif
        return;
      }
    }

  unsigned int id = atomicInc( &s_uiBlockDetectionsCount1, MAX_BLOCK_DETECTIONS_COUNT1 );
  s_uc2BlockDetections1[id] = pos;
  s_fBlockDetectionsRet1[id] = ret;


#ifdef LRD_DEBUG
  atomicInc( c_puiCutDetectionCount+2, UINT_MAX );
  if(id==MAX_BLOCK_DETECTIONS_COUNT1) atomicAdd( c_puiDetectionOverflows, 10);
#endif
}





__device__ void LRDfinish(unsigned int _id, uint2 *_odata, float *_pfDetectionsRet, unsigned int *_uiDetectionsCount, cudaStream_t _StreamId)
{
  uchar2 pos = s_uc2BlockDetections1[_id];
  float  ret = s_fBlockDetectionsRet1[_id];

  #pragma unroll 10
    for(unsigned int cid=CUT3; cid<c_uiWeakClasifiersCount; cid++){  //cid - clasifier id
      ret += tex2D(tLRDPredictionValues, LRDGetDifferentialRank(c_uiWeakParam[cid], pos), cid);
      if(ret<c_uiTresholds[cid]){
        #ifdef LRD_WEAK_COUNT
          if(((blockIdx.y*THREADS + pos.y)<c_uiH) && ((blockIdx.x*SCANW + pos.x)<c_uiW))
            c_puiWeakCount[_StreamId][(blockIdx.y*THREADS + pos.y)*c_uiW + (blockIdx.x*SCANW + pos.x)] = cid;
        #endif
        return;
      }
    }

  if(ret>=0.0f)
  {
    uint2 p;
    p.x = blockIdx.x*SCANW + pos.x;
    p.y = blockIdx.y*THREADS + pos.y;

    unsigned int id = atomicInc( _uiDetectionsCount, MAX_DETECTIONS_COUNT );
    _odata[id] = p;
    _pfDetectionsRet[id] = ret;

    #ifdef LRD_WEAK_COUNT
      if(((blockIdx.y*THREADS + pos.y)<c_uiH) && ((blockIdx.x*SCANW + pos.x)<c_uiW))
        c_puiWeakCount[_StreamId][(blockIdx.y*THREADS + pos.y)*c_uiW + (blockIdx.x*SCANW + pos.x)] = c_uiWeakClasifiersCount;
    #endif


    #ifdef LRD_DEBUG
      if(id==MAX_DETECTIONS_COUNT) atomicInc( c_puiDetectionOverflows, UINT_MAX);
    #endif
  }
}




__global__ void CudaKernelLRD(cudaStream_t _StreamId, uint2 *g_odata, float *_pfDetectionsRet, unsigned int *g_puiDetectionsCount) 
{
  
  if(threadIdx.y==0){
    s_uiBlockDetectionsCount1=0;
    
    s_uc2BlockDetections1  =  (uchar2 *)(s_ucTmp0 + 0);
    s_fBlockDetectionsRet1 =   (float *)(s_ucTmp0 + MAX_BLOCK_DETECTIONS_COUNT1*2);
    //s_uc2BlockDetections2  =  (uchar2 *)(s_ucTmp0 + MAX_BLOCK_DETECTIONS_COUNT1*6);
    //s_fBlockDetectionsRet2 =   (float *)(s_ucTmp0 + MAX_BLOCK_DETECTIONS_COUNT1*6 + MAX_BLOCK_DETECTIONS_COUNT2*2);
    s_uc2BlockDetections2  =  (uchar2 *)(s_ucTmp1 + 0);
    s_fBlockDetectionsRet2 =   (float *)(s_ucTmp1 + MAX_BLOCK_DETECTIONS_COUNT2*2);
  }

  __syncthreads();

  uchar2 pos;
  pos.y=threadIdx.y;
  #pragma unroll
    for(pos.x=0; pos.x<SCANW; pos.x++)
     LRDStart(pos, _StreamId);

  __syncthreads();

  if(threadIdx.y==0) s_uiBlockDetectionsCount2=0;
  
  __syncthreads();
  
  for(unsigned int f=threadIdx.y; f<s_uiBlockDetectionsCount1; f+=blockDim.y)
    LRD12(f, _StreamId);

  __syncthreads();

  if(threadIdx.y==0) s_uiBlockDetectionsCount1=0;
  
  __syncthreads();
  
  for(unsigned int f=threadIdx.y; f<s_uiBlockDetectionsCount2; f+=blockDim.y)
    LRD21(f, _StreamId);

  __syncthreads();
  
  for(unsigned int f=threadIdx.y; f<s_uiBlockDetectionsCount1; f+=blockDim.y)
    LRDfinish(f, g_odata, _pfDetectionsRet, g_puiDetectionsCount, _StreamId);

}

//**********************************************************************************************
//**********************************************************************************************
//******************************************** LRP *********************************************
//**********************************************************************************************
//**********************************************************************************************


#define _positionX (_WeakClasf.x)
#define _positionY (_WeakClasf.y)
#define _uiBlockSizeId (_WeakClasf.z)
#define _blockABId (_WeakClasf.w)

__device__ int LRPGetDifferentialRank(uint4 _WeakClasf, uchar2 _pos)
{
  unsigned int uiBlockWidth  = (_uiBlockSizeId >> 3);
  float         fBlockWidth  = float(uiBlockWidth)/2.0f;
  unsigned int uiBlockHeight = (_uiBlockSizeId & 7);
  float         fBlockHeight = float(uiBlockHeight)/2.0f;
  unsigned int uiBlockA = _blockABId >> 4;
  unsigned int uiBlockB = _blockABId & 15;
  unsigned int px = blockIdx.x*SCANW   + _positionX + _pos.x;
  unsigned int py = blockIdx.y*THREADS + _positionY + _pos.y;

  float fBlockASum = GetData( px + uiBlockWidth*(uiBlockA%3) +fBlockWidth, py + uiBlockHeight*(uiBlockA/3) +fBlockHeight);
  float fBlockBSum = GetData( px + uiBlockWidth*(uiBlockB%3) +fBlockWidth, py + uiBlockHeight*(uiBlockB/3) +fBlockHeight);

  float tmp;
  int iRankA=0;
  int iRankB=0;

  tmp = GetData( px+fBlockWidth, py+fBlockHeight);    iRankA += (tmp < fBlockASum);    iRankB += (tmp < fBlockBSum);  px+=uiBlockWidth;
  tmp = GetData( px+fBlockWidth, py+fBlockHeight);    iRankA += (tmp < fBlockASum);    iRankB += (tmp < fBlockBSum);  px+=uiBlockWidth;
  tmp = GetData( px+fBlockWidth, py+fBlockHeight);    iRankA += (tmp < fBlockASum);    iRankB += (tmp < fBlockBSum);
  px -= 2*uiBlockWidth;    py+=uiBlockHeight;
  tmp = GetData( px+fBlockWidth, py+fBlockHeight);    iRankA += (tmp < fBlockASum);    iRankB += (tmp < fBlockBSum);  px+=uiBlockWidth;
  tmp = GetData( px+fBlockWidth, py+fBlockHeight);    iRankA += (tmp < fBlockASum);    iRankB += (tmp < fBlockBSum);  px+=uiBlockWidth;
  tmp = GetData( px+fBlockWidth, py+fBlockHeight);    iRankA += (tmp < fBlockASum);    iRankB += (tmp < fBlockBSum);
  px -= 2*uiBlockWidth;    py+=uiBlockHeight;
  tmp = GetData( px+fBlockWidth, py+fBlockHeight);    iRankA += (tmp < fBlockASum);    iRankB += (tmp < fBlockBSum);  px+=uiBlockWidth;
  tmp = GetData( px+fBlockWidth, py+fBlockHeight);    iRankA += (tmp < fBlockASum);    iRankB += (tmp < fBlockBSum);  px+=uiBlockWidth;
  tmp = GetData( px+fBlockWidth, py+fBlockHeight);    iRankA += (tmp < fBlockASum);    iRankB += (tmp < fBlockBSum);

  return iRankA + 10*iRankB;
}

#undef _positionX
#undef _positionY
#undef _uiBlockSizeId
#undef _blockABId


__device__ void LRPStart(uchar2 pos, cudaStream_t _StreamId)
{
  float ret = 0.0f;

  #pragma unroll
    for(unsigned int cid=0; cid<CUT1; cid++){  //cid - clasifier id
      ret += tex2D(tLRPPredictionValues, LRPGetDifferentialRank(c_uiWeakParam[cid], pos), cid); //tex1D(tWeakParam, cid)
      if(ret<c_uiTresholds[cid]){
        #ifdef LRD_WEAK_COUNT
          if(((blockIdx.y*THREADS + pos.y)<c_uiH) && ((blockIdx.x*SCANW + pos.x)<c_uiW))
            c_puiWeakCount[_StreamId][(blockIdx.y*THREADS + pos.y)*c_uiW + (blockIdx.x*SCANW + pos.x)] = cid;
        #endif
        return;
      }
    }

  unsigned int id = atomicInc( &s_uiBlockDetectionsCount1, MAX_BLOCK_DETECTIONS_COUNT1 );
  s_uc2BlockDetections1[id] = pos;
  s_fBlockDetectionsRet1[id] = ret;


#ifdef LRD_DEBUG
  atomicInc( c_puiCutDetectionCount+0, UINT_MAX );
  if(id==MAX_BLOCK_DETECTIONS_COUNT1) atomicAdd( c_puiDetectionOverflows, 1000);
#endif
}





__device__ void LRP12(unsigned int _id, cudaStream_t _StreamId)
{
  uchar2 pos = s_uc2BlockDetections1[_id];
  float  ret = s_fBlockDetectionsRet1[_id];

  #pragma unroll
    for(unsigned int cid=CUT1; cid<CUT2; cid++){  //cid - clasifier id
      ret += tex2D(tLRPPredictionValues, LRPGetDifferentialRank(c_uiWeakParam[cid], pos), cid);
      if(ret<c_uiTresholds[cid]){
        #ifdef LRD_WEAK_COUNT
          if(((blockIdx.y*THREADS + pos.y)<c_uiH) && ((blockIdx.x*SCANW + pos.x)<c_uiW))
            c_puiWeakCount[_StreamId][(blockIdx.y*THREADS + pos.y)*c_uiW + (blockIdx.x*SCANW + pos.x)] = cid;
        #endif
        return;
      }
    }

  unsigned int id = atomicInc( &s_uiBlockDetectionsCount2, MAX_BLOCK_DETECTIONS_COUNT2 );
  s_uc2BlockDetections2[id] = pos;
  s_fBlockDetectionsRet2[id] = ret;


#ifdef LRD_DEBUG
  atomicInc( c_puiCutDetectionCount+1, UINT_MAX );
  if(id==MAX_BLOCK_DETECTIONS_COUNT2) atomicAdd( c_puiDetectionOverflows, 100);
#endif
}


__device__ void LRP21(unsigned int _id, cudaStream_t _StreamId)
{
  uchar2 pos = s_uc2BlockDetections2[_id];
  float  ret = s_fBlockDetectionsRet2[_id];

  #pragma unroll
    for(unsigned int cid=CUT2; cid<CUT3; cid++){  //cid - clasifier id
      ret += tex2D(tLRPPredictionValues, LRPGetDifferentialRank(c_uiWeakParam[cid], pos), cid);
      if(ret<c_uiTresholds[cid]){
        #ifdef LRD_WEAK_COUNT
          if(((blockIdx.y*THREADS + pos.y)<c_uiH) && ((blockIdx.x*SCANW + pos.x)<c_uiW))
            c_puiWeakCount[_StreamId][(blockIdx.y*THREADS + pos.y)*c_uiW + (blockIdx.x*SCANW + pos.x)] = cid;
        #endif
        return;
      }
    }

  unsigned int id = atomicInc( &s_uiBlockDetectionsCount1, MAX_BLOCK_DETECTIONS_COUNT1 );
  s_uc2BlockDetections1[id] = pos;
  s_fBlockDetectionsRet1[id] = ret;


#ifdef LRD_DEBUG
  atomicInc( c_puiCutDetectionCount+2, UINT_MAX );
  if(id==MAX_BLOCK_DETECTIONS_COUNT1) atomicAdd( c_puiDetectionOverflows, 10);
#endif
}





__device__ void LRPfinish(unsigned int _id, uint2 *_odata, float *_pfDetectionsRet, unsigned int *_uiDetectionsCount, cudaStream_t _StreamId)
{
  uchar2 pos = s_uc2BlockDetections1[_id];
  float  ret = s_fBlockDetectionsRet1[_id];

  #pragma unroll 10
    for(unsigned int cid=CUT3; cid<c_uiWeakClasifiersCount; cid++){  //cid - clasifier id
      ret += tex2D(tLRPPredictionValues, LRPGetDifferentialRank(c_uiWeakParam[cid], pos), cid);
      if(ret<c_uiTresholds[cid]){
        #ifdef LRD_WEAK_COUNT
          if(((blockIdx.y*THREADS + pos.y)<c_uiH) && ((blockIdx.x*SCANW + pos.x)<c_uiW))
            c_puiWeakCount[_StreamId][(blockIdx.y*THREADS + pos.y)*c_uiW + (blockIdx.x*SCANW + pos.x)] = cid;
        #endif
        return;
      }
    }

  if(ret>=0.0f)
  {
    uint2 p;
    p.x = blockIdx.x*SCANW + pos.x;
    p.y = blockIdx.y*THREADS + pos.y;

    unsigned int id = atomicInc( _uiDetectionsCount, MAX_DETECTIONS_COUNT );
    _odata[id] = p;
    _pfDetectionsRet[id] = ret;

    #ifdef LRD_WEAK_COUNT
      if(((blockIdx.y*THREADS + pos.y)<c_uiH) && ((blockIdx.x*SCANW + pos.x)<c_uiW))
        c_puiWeakCount[_StreamId][(blockIdx.y*THREADS + pos.y)*c_uiW + (blockIdx.x*SCANW + pos.x)] = c_uiWeakClasifiersCount;
    #endif


    #ifdef LRD_DEBUG
      if(id==MAX_DETECTIONS_COUNT) atomicInc( c_puiDetectionOverflows, UINT_MAX);
    #endif
  }
}


__global__ void CudaKernelLRP(cudaStream_t _StreamId, uint2 *g_odata, float *_pfDetectionsRet, unsigned int *g_puiDetectionsCount) 
{
  
  if(threadIdx.y==0){
    s_uiBlockDetectionsCount1=0;
    
    s_uc2BlockDetections1  =  (uchar2 *)(s_ucTmp0 + 0);
    s_fBlockDetectionsRet1 =   (float *)(s_ucTmp0 + MAX_BLOCK_DETECTIONS_COUNT1*2);
    //s_uc2BlockDetections2  =  (uchar2 *)(s_ucTmp0 + MAX_BLOCK_DETECTIONS_COUNT1*6);
    //s_fBlockDetectionsRet2 =   (float *)(s_ucTmp0 + MAX_BLOCK_DETECTIONS_COUNT1*6 + MAX_BLOCK_DETECTIONS_COUNT2*2);
    s_uc2BlockDetections2  =  (uchar2 *)(s_ucTmp1 + 0);
    s_fBlockDetectionsRet2 =   (float *)(s_ucTmp1 + MAX_BLOCK_DETECTIONS_COUNT2*2);
  }

  __syncthreads();

  uchar2 pos;
  pos.y=threadIdx.y;
  #pragma unroll
    for(pos.x=0; pos.x<SCANW; pos.x++)
     LRPStart(pos, _StreamId);

  __syncthreads();

  if(threadIdx.y==0) s_uiBlockDetectionsCount2=0;
  
  __syncthreads();
  
  for(unsigned int f=threadIdx.y; f<s_uiBlockDetectionsCount1; f+=blockDim.y)
    LRP12(f, _StreamId);

  __syncthreads();

  if(threadIdx.y==0) s_uiBlockDetectionsCount1=0;
  
  __syncthreads();
  
  for(unsigned int f=threadIdx.y; f<s_uiBlockDetectionsCount2; f+=blockDim.y)
    LRP21(f, _StreamId);

  __syncthreads();
  
  for(unsigned int f=threadIdx.y; f<s_uiBlockDetectionsCount1; f+=blockDim.y)
    LRPfinish(f, g_odata, _pfDetectionsRet, g_puiDetectionsCount, _StreamId);

}














//**********************************************************************************************
//**********************************************************************************************
//******************************************** LBP *********************************************
//**********************************************************************************************
//**********************************************************************************************


#define _positionX (_WeakClasf.x)
#define _positionY (_WeakClasf.y)
#define _uiBlockSizeId (_WeakClasf.z)
#define _blockABId (_WeakClasf.w)

__device__ int LBPGetDifferentialRank(uint4 _WeakClasf, uchar2 _pos)
{
  unsigned int uiBlockWidth  = (_uiBlockSizeId >> 3);
  float         fBlockWidth  = float(uiBlockWidth)/2.0f;
  unsigned int uiBlockHeight = (_uiBlockSizeId & 7);
  float         fBlockHeight = float(uiBlockHeight)/2.0f;
  unsigned int uiBlockA = _blockABId >> 4;
  unsigned int uiBlockB = _blockABId & 15;
  unsigned int px = blockIdx.x*SCANW   + _positionX + _pos.x;
  unsigned int py = blockIdx.y*THREADS + _positionY + _pos.y;

  float fBlockASum = GetData( px + uiBlockWidth +fBlockWidth, py + uiBlockHeight +fBlockHeight);

  float tmp;
  int iRankA=0;

  tmp = GetData( px+fBlockWidth, py+fBlockHeight);    iRankA += (tmp > fBlockASum);  px+=uiBlockWidth;
  tmp = GetData( px+fBlockWidth, py+fBlockHeight);    iRankA += 2*(tmp > fBlockASum);  px+=uiBlockWidth;
  tmp = GetData( px+fBlockWidth, py+fBlockHeight);    iRankA += 4*(tmp > fBlockASum);
  px -= 2*uiBlockWidth;    py+=uiBlockHeight;
  tmp = GetData( px+fBlockWidth, py+fBlockHeight);    iRankA += 128*(tmp > fBlockASum);  px+=2*uiBlockWidth;
  tmp = GetData( px+fBlockWidth, py+fBlockHeight);    iRankA += 8*(tmp > fBlockASum);
  px -= 2*uiBlockWidth;    py+=uiBlockHeight;
  tmp = GetData( px+fBlockWidth, py+fBlockHeight);    iRankA += 64*(tmp > fBlockASum);  px+=uiBlockWidth;
  tmp = GetData( px+fBlockWidth, py+fBlockHeight);    iRankA += 32*(tmp > fBlockASum);  px+=uiBlockWidth;
  tmp = GetData( px+fBlockWidth, py+fBlockHeight);    iRankA += 16*(tmp > fBlockASum);

  return iRankA;
}

#undef _positionX
#undef _positionY
#undef _uiBlockSizeId
#undef _blockABId


__device__ void LBPStart(uchar2 pos, cudaStream_t _StreamId)
{
  float ret = 0.0f;

  #pragma unroll
    for(unsigned int cid=0; cid<CUT1; cid++){  //cid - clasifier id
      ret += tex2D(tLBPPredictionValues, LBPGetDifferentialRank(c_uiWeakParam[cid], pos), cid); //tex1D(tWeakParam, cid)
      if(ret<c_uiTresholds[cid]){
        #ifdef LRD_WEAK_COUNT
          if(((blockIdx.y*THREADS + pos.y)<c_uiH) && ((blockIdx.x*SCANW + pos.x)<c_uiW))
            c_puiWeakCount[_StreamId][(blockIdx.y*THREADS + pos.y)*c_uiW + (blockIdx.x*SCANW + pos.x)] = cid;
        #endif
        return;
      }
    }

  unsigned int id = atomicInc( &s_uiBlockDetectionsCount1, MAX_BLOCK_DETECTIONS_COUNT1 );
  s_uc2BlockDetections1[id] = pos;
  s_fBlockDetectionsRet1[id] = ret;


#ifdef LRD_DEBUG
  atomicInc( c_puiCutDetectionCount+0, UINT_MAX );
  if(id==MAX_BLOCK_DETECTIONS_COUNT1) atomicAdd( c_puiDetectionOverflows, 1000);
#endif
}





__device__ void LBP12(unsigned int _id, cudaStream_t _StreamId)
{
  uchar2 pos = s_uc2BlockDetections1[_id];
  float  ret = s_fBlockDetectionsRet1[_id];

  #pragma unroll
    for(unsigned int cid=CUT1; cid<CUT2; cid++){  //cid - clasifier id
      ret += tex2D(tLBPPredictionValues, LBPGetDifferentialRank(c_uiWeakParam[cid], pos), cid);
      if(ret<c_uiTresholds[cid]){
        #ifdef LRD_WEAK_COUNT
          if(((blockIdx.y*THREADS + pos.y)<c_uiH) && ((blockIdx.x*SCANW + pos.x)<c_uiW))
            c_puiWeakCount[_StreamId][(blockIdx.y*THREADS + pos.y)*c_uiW + (blockIdx.x*SCANW + pos.x)] = cid;
        #endif
        return;
      }
    }

  unsigned int id = atomicInc( &s_uiBlockDetectionsCount2, MAX_BLOCK_DETECTIONS_COUNT2 );
  s_uc2BlockDetections2[id] = pos;
  s_fBlockDetectionsRet2[id] = ret;


#ifdef LRD_DEBUG
  atomicInc( c_puiCutDetectionCount+1, UINT_MAX );
  if(id==MAX_BLOCK_DETECTIONS_COUNT2) atomicAdd( c_puiDetectionOverflows, 100);
#endif
}


__device__ void LBP21(unsigned int _id, cudaStream_t _StreamId)
{
  uchar2 pos = s_uc2BlockDetections2[_id];
  float  ret = s_fBlockDetectionsRet2[_id];

  #pragma unroll
    for(unsigned int cid=CUT2; cid<CUT3; cid++){  //cid - clasifier id
      ret += tex2D(tLBPPredictionValues, LBPGetDifferentialRank(c_uiWeakParam[cid], pos), cid);
      if(ret<c_uiTresholds[cid]){
        #ifdef LRD_WEAK_COUNT
          if(((blockIdx.y*THREADS + pos.y)<c_uiH) && ((blockIdx.x*SCANW + pos.x)<c_uiW))
            c_puiWeakCount[_StreamId][(blockIdx.y*THREADS + pos.y)*c_uiW + (blockIdx.x*SCANW + pos.x)] = cid;
        #endif
        return;
      }
    }

  unsigned int id = atomicInc( &s_uiBlockDetectionsCount1, MAX_BLOCK_DETECTIONS_COUNT1 );
  s_uc2BlockDetections1[id] = pos;
  s_fBlockDetectionsRet1[id] = ret;


#ifdef LRD_DEBUG
  atomicInc( c_puiCutDetectionCount+2, UINT_MAX );
  if(id==MAX_BLOCK_DETECTIONS_COUNT1) atomicAdd( c_puiDetectionOverflows, 10);
#endif
}





__device__ void LBPfinish(unsigned int _id, uint2 *_odata, float *_pfDetectionsRet, unsigned int *_uiDetectionsCount, cudaStream_t _StreamId)
{
  uchar2 pos = s_uc2BlockDetections1[_id];
  float  ret = s_fBlockDetectionsRet1[_id];

  #pragma unroll 10
    for(unsigned int cid=CUT3; cid<c_uiWeakClasifiersCount; cid++){  //cid - clasifier id
      ret += tex2D(tLBPPredictionValues, LBPGetDifferentialRank(c_uiWeakParam[cid], pos), cid);
      if(ret<c_uiTresholds[cid]){
        #ifdef LRD_WEAK_COUNT
          if(((blockIdx.y*THREADS + pos.y)<c_uiH) && ((blockIdx.x*SCANW + pos.x)<c_uiW))
            c_puiWeakCount[_StreamId][(blockIdx.y*THREADS + pos.y)*c_uiW + (blockIdx.x*SCANW + pos.x)] = cid;
        #endif
        return;
      }
    }

  if(ret>=0.0f)
  {
    uint2 p;
    p.x = blockIdx.x*SCANW + pos.x;
    p.y = blockIdx.y*THREADS + pos.y;

    unsigned int id = atomicInc( _uiDetectionsCount, MAX_DETECTIONS_COUNT );
    _odata[id] = p;
    _pfDetectionsRet[id] = ret;

    #ifdef LRD_WEAK_COUNT
      if(((blockIdx.y*THREADS + pos.y)<c_uiH) && ((blockIdx.x*SCANW + pos.x)<c_uiW))
        c_puiWeakCount[_StreamId][(blockIdx.y*THREADS + pos.y)*c_uiW + (blockIdx.x*SCANW + pos.x)] = c_uiWeakClasifiersCount;
    #endif


    #ifdef LRD_DEBUG
      if(id==MAX_DETECTIONS_COUNT) atomicInc( c_puiDetectionOverflows, UINT_MAX);
    #endif
  }
}


__global__ void CudaKernelLBP(cudaStream_t _StreamId, uint2 *g_odata, float *_pfDetectionsRet, unsigned int *g_puiDetectionsCount) 
{
  
  if(threadIdx.y==0){
    s_uiBlockDetectionsCount1=0;
    
    s_uc2BlockDetections1  =  (uchar2 *)(s_ucTmp0 + 0);
    s_fBlockDetectionsRet1 =   (float *)(s_ucTmp0 + MAX_BLOCK_DETECTIONS_COUNT1*2);
    //s_uc2BlockDetections2  =  (uchar2 *)(s_ucTmp0 + MAX_BLOCK_DETECTIONS_COUNT1*6);
    //s_fBlockDetectionsRet2 =   (float *)(s_ucTmp0 + MAX_BLOCK_DETECTIONS_COUNT1*6 + MAX_BLOCK_DETECTIONS_COUNT2*2);
    s_uc2BlockDetections2  =  (uchar2 *)(s_ucTmp1 + 0);
    s_fBlockDetectionsRet2 =   (float *)(s_ucTmp1 + MAX_BLOCK_DETECTIONS_COUNT2*2);
  }

  __syncthreads();

  uchar2 pos;
  pos.y=threadIdx.y;
  #pragma unroll
    for(pos.x=0; pos.x<SCANW; pos.x++)
     LBPStart(pos, _StreamId);

  __syncthreads();

  if(threadIdx.y==0) s_uiBlockDetectionsCount2=0;
  
  __syncthreads();
  
  for(unsigned int f=threadIdx.y; f<s_uiBlockDetectionsCount1; f+=blockDim.y)
    LBP12(f, _StreamId);

  __syncthreads();

  if(threadIdx.y==0) s_uiBlockDetectionsCount1=0;
  
  __syncthreads();
  
  for(unsigned int f=threadIdx.y; f<s_uiBlockDetectionsCount2; f+=blockDim.y)
    LBP21(f, _StreamId);

  __syncthreads();
  
  for(unsigned int f=threadIdx.y; f<s_uiBlockDetectionsCount1; f+=blockDim.y)
    LBPfinish(f, g_odata, _pfDetectionsRet, g_puiDetectionsCount, _StreamId);

}