#include <aboninterface.h>
#include "lrd.h"


Timer* g_pfTimer;

struct t_cLRDDetector
{
	//bud prvni ve strukture, nebo pouzit dedicnost
	abonDetector header;
	//priklad vlastnich promennych

  t_sContextData *m_pcCUDALRD;
  
  abonErrorCode CUDADetectorInit();

};


double CudaLRDLibGetTime()
{
  return timerTime(g_pfTimer);
}


bool StrCmpN(char *_pcS0, char *_pcS1, unsigned int _uiN)
{
  for(unsigned int f=0; f<_uiN; f++)
    if(_pcS0[f]!=_pcS1[f]) return false;
  return true;
}

bool GetLRDLibParam(unsigned int _uiArgc, const char *_pcArgv[], char *_pcParam, float *_fValue)
{
  for(unsigned int f=1; f<_uiArgc; f++)
    if(StrCmpN((char*)(_pcArgv[f]), _pcParam, strlen(_pcParam))){
      float ret = float( atof(_pcArgv[f]+strlen(_pcParam)) );
      
      if(ret!=0.0f){
        *_fValue = ret;
        return true;
      }else{
        return false;
      }
    }

  return false;
}





extern "C" { // plugin interface ---------------------------------------------------

abonErrorCode CUDASetClassifier(abonDetector *detector, TClassifier * classifier)
{
  struct t_cLRDDetector *cLRDDetector = (struct t_cLRDDetector *)detector;
  abonErrorCode cErr = cLRDDetector->CUDADetectorInit();
  if( cErr != aboneOk) return cErr;

  if(classifier->tp==LRD){
    cout << "LRD Classifier" << endl;
    cout << "\tStage Count: " << classifier->evaluatedStageCount << endl;
    cout << "\tAlpha Count: " << classifier->alphaCount << endl;

    for(unsigned int f=0; f<classifier->evaluatedStageCount; f++){
      TStage &stage = ((TStage*)(classifier->stage))[f];

      if(cLRDDetector==NULL) cout << "cLRDDetector null" << endl;
      if(cLRDDetector->m_pcCUDALRD==NULL) cout << "m_pcCUDALRD null" << endl;

        
      cLRDDetector->m_pcCUDALRD->m_vuiWeakParam.push_back(tInt4( stage.x, stage.y,
								                                                 (stage.w<<3)+stage.h,
								                                                 (int(stage.A)<<4)+int(stage.B)));
      cLRDDetector->m_pcCUDALRD->m_vfLRDPredictors.push_back(tFloat17(stage.alpha-8));
      cLRDDetector->m_pcCUDALRD->m_vfTreshold.push_back(stage.theta_b);
    }

    if(cLRDDetector->m_pcCUDALRD->m_vfLRDPredictors.size() != cLRDDetector->m_pcCUDALRD->m_vuiWeakParam.size())
      printf("ERROR: size deffers in m_vfPredictors and m_vuiWeakParam\n");
    if(cLRDDetector->m_pcCUDALRD->m_vfTreshold.size() != cLRDDetector->m_pcCUDALRD->m_vuiWeakParam.size())
      printf("ERROR: size deffers in m_vfTreshold and m_vuiWeakParam\n"); 

    cout << "Classifier done" << endl;
  }else if(classifier->tp==LRP){
    cout << "LRP Classifier" << endl;
    cout << "\tStage Count: " << classifier->evaluatedStageCount << endl;
    cout << "\tAlpha Count: " << classifier->alphaCount << endl;

    for(unsigned int f=0; f<classifier->evaluatedStageCount; f++){
      TStage &stage = ((TStage*)(classifier->stage))[f];

      if(cLRDDetector==NULL) cout << "cLRDDetector null" << endl;
      if(cLRDDetector->m_pcCUDALRD==NULL) cout << "m_pcCUDALRD null" << endl;

      cLRDDetector->m_pcCUDALRD->m_vuiWeakParam.push_back(tInt4( stage.x, stage.y,
                                                                 (stage.w<<3)+stage.h,
                                                                 (int(stage.A)<<4)+int(stage.B)));
      cLRDDetector->m_pcCUDALRD->m_vfLRPPredictors.push_back(tFloat100(stage.alpha));
      cLRDDetector->m_pcCUDALRD->m_vfTreshold.push_back(stage.theta_b);
    }

    if(cLRDDetector->m_pcCUDALRD->m_vfLRPPredictors.size() != cLRDDetector->m_pcCUDALRD->m_vuiWeakParam.size())
      printf("ERROR: size deffers in m_vfPredictors and m_vuiWeakParam\n");
    if(cLRDDetector->m_pcCUDALRD->m_vfTreshold.size() != cLRDDetector->m_pcCUDALRD->m_vuiWeakParam.size())
      printf("ERROR: size deffers in m_vfTreshold and m_vuiWeakParam\n"); 

    cout << "Classifier done" << endl;
  }else if(classifier->tp==LBP){
    cout << "LBP Classifier" << endl;
    cout << "\tStage Count: " << classifier->evaluatedStageCount << endl;
    cout << "\tAlpha Count: " << classifier->alphaCount << endl;

    for(unsigned int f=0; f<classifier->evaluatedStageCount; f++){
      TStage &stage = ((TStage*)(classifier->stage))[f];

      if(cLRDDetector==NULL) cout << "cLRDDetector null" << endl;
      if(cLRDDetector->m_pcCUDALRD==NULL) cout << "m_pcCUDALRD null" << endl;

      cLRDDetector->m_pcCUDALRD->m_vuiWeakParam.push_back(tInt4( stage.x, stage.y,
                                                                 (stage.w<<3)+stage.h,
                                                                 (int(stage.A)<<4)+int(stage.B)));
      cLRDDetector->m_pcCUDALRD->m_vfLBPPredictors.push_back(tFloat256(stage.alpha));
      cLRDDetector->m_pcCUDALRD->m_vfTreshold.push_back(stage.theta_b);
    }

    if(cLRDDetector->m_pcCUDALRD->m_vfLBPPredictors.size() != cLRDDetector->m_pcCUDALRD->m_vuiWeakParam.size())
      printf("ERROR: size deffers in m_vfPredictors and m_vuiWeakParam\n");
    if(cLRDDetector->m_pcCUDALRD->m_vfTreshold.size() != cLRDDetector->m_pcCUDALRD->m_vuiWeakParam.size())
      printf("ERROR: size deffers in m_vfTreshold and m_vuiWeakParam\n"); 

    cout << "Classifier done" << endl;
  }else{
    cout << "Unknown Classifier Type :" << classifier->tp << endl;
  }
	return aboneOk;
}

abonErrorCode CUDASetArguments(abonDetector *detector, unsigned argc, const char *argv[])
{
  struct t_cLRDDetector *cLRDDetector = (struct t_cLRDDetector *)detector;
  abonErrorCode cErr = cLRDDetector->CUDADetectorInit();
  if( cErr != aboneOk) return cErr;
  
  cout << "LRDSetArguments" << endl;
/*
  for(unsigned int f=0; f<argc; f++)
    printf("%s\n",argv[f]);
*/
  float fVal;

  if(GetLRDLibParam(argc, argv, "v_", &fVal)){
    cLRDDetector->m_pcCUDALRD->SetVerbosity((unsigned int)fVal);
    printf("m_uiVerbose: %i\n", (unsigned int)fVal);
  }


  cout << "LRDSetArguments done" << endl;
  return aboneOk;
}

abonErrorCode CUDAProcess(abonDetector *detector, abonImageData *image)
{
  struct t_cLRDDetector *cLRDDetector = (struct t_cLRDDetector *)detector;
  abonErrorCode cErr = cLRDDetector->CUDADetectorInit();
  if( cErr != aboneOk) return cErr;

  //cout << "LRDProcess.\n";

  unsigned int uiStreamId = 0;
  vector<t_cDetection> vDetections;
  cLRDDetector->m_pcCUDALRD->ImageStart(uiStreamId, image->width, image->height, image->pitch, 1, (unsigned char*)image->data);
  cLRDDetector->m_pcCUDALRD->ImageFinish(uiStreamId, &vDetections);
  
  image->timestamps[abontsPyramid].start   = cLRDDetector->m_pcCUDALRD->m_sImageData[uiStreamId].m_sTimeStamps.g_pdTimeStamp[LRD_S];
  image->timestamps[abontsPyramid].end     = cLRDDetector->m_pcCUDALRD->m_sImageData[uiStreamId].m_sTimeStamps.g_pdTimeStamp[LRD_PYRAMID_E];
  image->timestamps[abontsDetection].start = cLRDDetector->m_pcCUDALRD->m_sImageData[uiStreamId].m_sTimeStamps.g_pdTimeStamp[LRD_CUDA_S];
  image->timestamps[abontsDetection].end   = cLRDDetector->m_pcCUDALRD->m_sImageData[uiStreamId].m_sTimeStamps.g_pdTimeStamp[LRD_GET_E];
  image->timestamps[abontsAux1].start      = cLRDDetector->m_pcCUDALRD->m_sImageData[uiStreamId].m_sTimeStamps.g_pdTimeStamp[LRD_CUDA_S];
  image->timestamps[abontsAux1].end        = cLRDDetector->m_pcCUDALRD->m_sImageData[uiStreamId].m_sTimeStamps.g_pdTimeStamp[LRD_CUDA_E];
  image->timestamps[abontsAux2].start      = cLRDDetector->m_pcCUDALRD->m_sImageData[uiStreamId].m_sTimeStamps.g_pdTimeStamp[LRD_GET_S];
  image->timestamps[abontsAux2].end        = cLRDDetector->m_pcCUDALRD->m_sImageData[uiStreamId].m_sTimeStamps.g_pdTimeStamp[LRD_GET_E];

  if(image->detectionCount>vDetections.size()) image->detectionCount = vDetections.size();
  if(image->detectionCount>0) memcpy(image->detections, &vDetections[0], image->detectionCount*sizeof(abonDetection));
  image->nmsDone = false;

  //cout << "LRDProcess done.\n";
	return aboneOk;
}

abonErrorCode CUDAStart(abonDetector *detector, abonCbGetImage *get, abonCbPutImage *put, void *param)
{
  struct t_cLRDDetector *cLRDDetector = (struct t_cLRDDetector *)detector;
  abonErrorCode cErr = cLRDDetector->CUDADetectorInit();
  if( cErr != aboneOk) return cErr;


  cout << "LRDStart.\n";

  //smycka pro zpracovani snimku

	unsigned int uiStreamId;
  vector<t_cDetection> vDetections;

  abonImageData *image[MAX_STREAMS];
	while(1)
	{
    uiStreamId = 0;
    if((image[uiStreamId] = get(param)) == NULL) break;
    cLRDDetector->m_pcCUDALRD->ImageStart(uiStreamId, image[uiStreamId]->width, image[uiStreamId]->height, image[uiStreamId]->pitch, 1, (unsigned char*)image[uiStreamId]->data);

    uiStreamId = 1;
    if((image[uiStreamId] = get(param)) == NULL) break;
    cLRDDetector->m_pcCUDALRD->ImageStart(uiStreamId, image[uiStreamId]->width, image[uiStreamId]->height, image[uiStreamId]->pitch, 1, (unsigned char*)image[uiStreamId]->data);

    uiStreamId = 0;
    cLRDDetector->m_pcCUDALRD->ImageFinish(uiStreamId, &vDetections);
    image[uiStreamId]->timestamps[abontsPyramid].start   = cLRDDetector->m_pcCUDALRD->m_sImageData[uiStreamId].m_sTimeStamps.g_pdTimeStamp[LRD_S];
    image[uiStreamId]->timestamps[abontsPyramid].end     = cLRDDetector->m_pcCUDALRD->m_sImageData[uiStreamId].m_sTimeStamps.g_pdTimeStamp[LRD_PYRAMID_E];
    image[uiStreamId]->timestamps[abontsDetection].start = cLRDDetector->m_pcCUDALRD->m_sImageData[uiStreamId].m_sTimeStamps.g_pdTimeStamp[LRD_CUDA_S];
    image[uiStreamId]->timestamps[abontsDetection].end   = cLRDDetector->m_pcCUDALRD->m_sImageData[uiStreamId].m_sTimeStamps.g_pdTimeStamp[LRD_E];
    image[uiStreamId]->timestamps[abontsAux1].start      = cLRDDetector->m_pcCUDALRD->m_sImageData[uiStreamId].m_sTimeStamps.g_pdTimeStamp[LRD_CUDA_S];
    image[uiStreamId]->timestamps[abontsAux1].end        = cLRDDetector->m_pcCUDALRD->m_sImageData[uiStreamId].m_sTimeStamps.g_pdTimeStamp[LRD_CUDA_E];
    image[uiStreamId]->timestamps[abontsAux2].start      = cLRDDetector->m_pcCUDALRD->m_sImageData[uiStreamId].m_sTimeStamps.g_pdTimeStamp[LRD_GET_S];
    image[uiStreamId]->timestamps[abontsAux2].end        = cLRDDetector->m_pcCUDALRD->m_sImageData[uiStreamId].m_sTimeStamps.g_pdTimeStamp[LRD_GET_E];
    
    image[uiStreamId]->detectionCount = (int(image[uiStreamId]->detectionsEnd)-int(image[uiStreamId]->detections))/sizeof(abonDetection);
    if(image[uiStreamId]->detectionCount>vDetections.size()) image[uiStreamId]->detectionCount = vDetections.size();
    if(image[uiStreamId]->detectionCount>0) memcpy(image[uiStreamId]->detections, &vDetections[0], image[uiStreamId]->detectionCount*sizeof(abonDetection));
    image[uiStreamId]->nmsDone = false;
    put(param, image[uiStreamId]);


    uiStreamId = 1;
    cLRDDetector->m_pcCUDALRD->ImageFinish(uiStreamId, &vDetections);
    image[uiStreamId]->timestamps[abontsPyramid].start   = cLRDDetector->m_pcCUDALRD->m_sImageData[uiStreamId].m_sTimeStamps.g_pdTimeStamp[LRD_S];
    image[uiStreamId]->timestamps[abontsPyramid].end     = cLRDDetector->m_pcCUDALRD->m_sImageData[uiStreamId].m_sTimeStamps.g_pdTimeStamp[LRD_PYRAMID_E];
    image[uiStreamId]->timestamps[abontsDetection].start = cLRDDetector->m_pcCUDALRD->m_sImageData[uiStreamId].m_sTimeStamps.g_pdTimeStamp[LRD_CUDA_S];
    image[uiStreamId]->timestamps[abontsDetection].end   = cLRDDetector->m_pcCUDALRD->m_sImageData[uiStreamId].m_sTimeStamps.g_pdTimeStamp[LRD_E];
    image[uiStreamId]->timestamps[abontsAux1].start      = cLRDDetector->m_pcCUDALRD->m_sImageData[uiStreamId].m_sTimeStamps.g_pdTimeStamp[LRD_CUDA_S];
    image[uiStreamId]->timestamps[abontsAux1].end        = cLRDDetector->m_pcCUDALRD->m_sImageData[uiStreamId].m_sTimeStamps.g_pdTimeStamp[LRD_CUDA_E];
    image[uiStreamId]->timestamps[abontsAux2].start      = cLRDDetector->m_pcCUDALRD->m_sImageData[uiStreamId].m_sTimeStamps.g_pdTimeStamp[LRD_GET_S];
    image[uiStreamId]->timestamps[abontsAux2].end        = cLRDDetector->m_pcCUDALRD->m_sImageData[uiStreamId].m_sTimeStamps.g_pdTimeStamp[LRD_GET_E];
    
    image[uiStreamId]->detectionCount = (int(image[uiStreamId]->detectionsEnd)-int(image[uiStreamId]->detections))/sizeof(abonDetection);
    if(image[uiStreamId]->detectionCount>vDetections.size()) image[uiStreamId]->detectionCount = vDetections.size();
    if(image[uiStreamId]->detectionCount>0) memcpy(image[uiStreamId]->detections, &vDetections[0], image[uiStreamId]->detectionCount*sizeof(abonDetection));
    image[uiStreamId]->nmsDone = false;
		put(param, image[uiStreamId]);


    //fprintf(stdout, ":");
    //fflush(stdout);
	}

  cout << "LRDStart done.\n";
	return aboneOk;
}

abonErrorCode CUDARelease(abonDetector *detector)
{
  cout << "LRDRelease.\n";
  struct t_cLRDDetector *cLRDDetector = (struct t_cLRDDetector *)detector;

	delete cLRDDetector;

  cout << "LRDRelease done.\n";
	return aboneOk;
}





abonDetector *abonCreate()
{
  cout << "abonCreate.\n";
  t_cLRDDetector *detector = new t_cLRDDetector;
	if(detector != NULL)
	{
    detector->header = abonDetectorInitializer;
		detector->header.setClassifier = CUDASetClassifier;
    detector->header.setArguments = CUDASetArguments;
		//detector->header.process = LRDProcess;
    detector->header.start = CUDAStart;
		detector->header.release = CUDARelease;
    detector->m_pcCUDALRD = NULL;

    
	}
  cout << "abonCreate done.\n";
	return (abonDetector*)detector;
}

}// plugin interface ---------------------------------------------------






abonErrorCode t_cLRDDetector::CUDADetectorInit()
{
  if(m_pcCUDALRD) return aboneOk;

  g_pfTimer = header.timer;
  
  cout << "LRDInit.\n";
  
  m_pcCUDALRD = new t_sContextData();
  m_pcCUDALRD->GetTimeFunc = (&CudaLRDLibGetTime);

  for(int f=0; f<MAX_STREAMS; f++)
    m_pcCUDALRD->m_sImageData[f].m_sTimeStamps.GetTimeFunc = m_pcCUDALRD->GetTimeFunc;
  

  cout << "LRDInit done.\n";
  return aboneOk;
}
