/*
								+-----------------------------------+
								|                                   |
								|     ***  Ascii histogram  ***     |
								|                                   |
								|   Copyright   -tHE SWINe- 2008   |
								|                                   |
								|           AsciiHist.cpp           |
								|                                   |
								+-----------------------------------+
*/

#include "UberLame_src/CallStack.h"
#include <string.h>
#include <limits.h>
#include <stdio.h>
#include "UberLame_src/MinMax.h"
#include "UberLame_src/Integer.h"
#include "AsciiHist.h"

#if defined(_MSC_VER) && !defined(__MWERKS__) && !defined(for)
#define for if(0) {} else for
#endif

/*
 *								=== CHistogram ===
 */

/**
 *	CHistogram::CHistogram(int n_bin_num, int n_min, int n_max)
 *		- default constructor; initializes histogram for
 *		  n_bin_num bins (up to max_Bin_Num) with the first bin
 *		  associated with n_min value and the last bin associated
 *		  with n_max value
 *		- all bins are cleared to zero
 */
CHistogram::CHistogram(int n_bin_num, int n_min, int n_max)
	:m_n_bin_num(min(int(max_Bin_Num), n_bin_num)),
	m_n_min(n_min), m_n_max(max(n_min + 1, n_max))
{
	memset(m_p_bin, 0, m_n_bin_num * sizeof(int));
}

/**
 *	void CHistogram::Add(int n_x, int n_count)
 *		- adds n_count to bin, associated with value n_x
 *		- note bins are saturated to INT_MAX on overflow
 */
void CHistogram::Add(int n_x, int n_count)
{
	int n_bin = (n_x - m_n_min) * m_n_bin_num / (m_n_max - m_n_min);
	n_bin = min(m_n_bin_num - 1, max(0, n_bin));
	if(m_p_bin[n_bin] < INT_MAX - n_count)
		m_p_bin[n_bin] += n_count;
	else
		m_p_bin[n_bin] = INT_MAX;
}

/**
 *	void CHistogram::Reset()
 *		- clears all bins to zero
 */
void CHistogram::Reset()
{
	memset(m_p_bin, 0, m_n_bin_num * sizeof(int));
}

/**
 *	void CHistogram::Print(const char *p_s_pre, const char *p_s_post) const
 *		- prints the histogram to stdount
 *		- p_s_pre is prepended to each line (may contain indent, etc)
 *		- p_s_post is added past each line
 */
void CHistogram::Print(const char *p_s_pre, const char *p_s_post) const
{
	int n_max_hist = 1; // avoid div by 0
	for(int i = 0; i < m_n_bin_num; ++ i)
		n_max_hist = max(n_max_hist, m_p_bin[i]);
	for(int i = 0; i < m_n_bin_num; ++ i) {
		printf("\t%s %-2d - %2d %s: %s\n", p_s_pre,
			m_n_min + i * (m_n_max - m_n_min) / m_n_bin_num + 1,
			m_n_min + (i + 1) * (m_n_max - m_n_min) / m_n_bin_num,
			p_s_post, "******************************" + 30 - m_p_bin[i] * 30 / n_max_hist);
	}
	printf("\n");
}

/*
 *								=== ~CHistogram ===
 */
