/*
								+-----------------------------------+
								|                                   |
								|     ***  Ascii histogram  ***     |
								|                                   |
								|   Copyright   -tHE SWINe- 2008   |
								|                                   |
								|            AsciiHist.h            |
								|                                   |
								+-----------------------------------+
*/

#ifndef __ASCII_HISTOGRAM_INCLUDED
#define __ASCII_HISTOGRAM_INCLUDED

/**
 *	class CHistogram
 *		- very simple histogram counter, able
 *		  to print the result as ascii to stdout
 */
class CHistogram {
public:
	enum {
		max_Bin_Num = 32
	};

protected:
	int m_n_bin_num;
	int m_p_bin[max_Bin_Num];
	int m_n_min, m_n_max;

public:
	/**
	 *	CHistogram::CHistogram(int n_bin_num, int n_min, int n_max)
	 *		- default constructor; initializes histogram for
	 *		  n_bin_num bins (up to max_Bin_Num) with the first bin
	 *		  associated with n_min value and the last bin associated
	 *		  with n_max value
	 *		- all bins are cleared to zero
	 */
	CHistogram(int n_bin_num, int n_min, int n_max);

	/**
	 *	void CHistogram::Add(int n_x, int n_count)
	 *		- adds n_count to bin, associated with value n_x
	 *		- note bins are saturated to INT_MAX on overflow
	 */
	void Add(int n_x, int n_count);

	/**
	 *	void CHistogram::Reset()
	 *		- clears all bins to zero
	 */
	void Reset();

	/**
	 *	void CHistogram::Print(const char *p_s_pre, const char *p_s_post) const
	 *		- prints the histogram to stdount
	 *		- p_s_pre is prepended to each line (may contain indent, etc)
	 *		- p_s_post is added past each line
	 */
	void Print(const char *p_s_pre, const char *p_s_post) const;
};

#endif //__ASCII_HISTOGRAM_INCLUDED
