/*
								+----------------------------------+
								|                                  |
								|  ***  berLame GLUT window  ***  |
								|                                  |
								|   Copyright  -tHE SWINe- 2009   |
								|                                  |
								|          DetectGPU.cpp           |
								|                                  |
								+----------------------------------+
*/

#include "./UberLame_src/CallStack.h"
#include "./UberLame_src/OpenGL20.h"
#include <GL/glut.h>
#include <math.h>
#include <time.h>
#include <stdio.h>
#include <vector>
#include <algorithm>
#include <functional>
#include "./UberLame_src/OpenGLState.h"
#include "./UberLame_src/Texture.h"
#include "./UberLame_src/Vector.h"
#include "./UberLame_src/RenderBuffer2.h"
#include "./UberLame_src/VertexBufferObject.h"
#include "./UberLame_src/ShaderSys.h"
#include "./UberLame_src/GPUImgProc.h"

#include "Load_XML.h"
#include "Convolve.h"
#include "WaldBoost.h"
#include "InitOpenGL.h"
#include "DetectGPU.h"

#ifdef ALL_IN_ONE_PASS
typedef CWaldBoost_AllInOne_ShaderGenerator CShaderGeneratorClass;
#else //ALL_IN_ONE_PASS
typedef CWaldBoostShaderGenerator CShaderGeneratorClass;
#endif //ALL_IN_ONE_PASS

// Detector parameters
struct GPUDetector {
	abonDetector header;
	// header

	bool b_initialized;
	int n_image_width, n_image_height;
	bool b_classifer_specified;
	// detector status

	CShaderGeneratorClass *p_sh_gen;
	CImageDownscaler *p_ids;
	std::vector<CImageConvolver*> *p_convolver_list;
	std::vector<CTileRenderer*> *p_renderer_list;
	CAlphaTextureAtlas *p_alpha_tex_atlas;
	// GPU computing

	TWaldBoostClassifier *p_classifier;
	// copy of the classifier

	CGLTexture_Rect *p_src_image;
	// texture with source image

	TDownscaleImageSet t_img_set;
	// some information about detector config
};

static int n_instance_num = 0;
extern CGLState *p_state;
static CGLImageProc *p_image_proc = 0;
// OpenGL state guard & image processing object

static int const n_downscale = 2;
static int const n_tile_size = 256;
static const float f_pyramid = .75f;
// tile renderer config

abonErrorCode DetectVisualObjects(abonDetector *detector, abonImageData *image)
{
	__FuncGuard("::DetectVisualObjects");

	if(detector->timer) {
		glFlush();
		glFinish();
		//image->timestamps[abontsTotal].start = // they say do not write to total
			image->timestamps[abontsAux1].start = timerTime(detector->timer);
	}

	GPUDetector *gpudet = (GPUDetector*)detector;
	// type cast detector

	if(image->width != image->pitch)
		fprintf(stderr, "["LIBNAME"]: " "error: image scanlines are padded, OpenGL won't like that\n");
	p_state->ActiveTextureUnit(0);
	gpudet->p_src_image->Bind_Enable(p_state);
	glTexSubImage2D(gpudet->p_src_image->n_Target(), 0, 0, 0, image->width,
		image->height, GL_LUMINANCE, GL_UNSIGNED_BYTE, image->data);
	// upload image

#if 0 // test what image opencv gives us
	static TBmp t_bmp;
	static int n_frame_no = -1;
	if(n_frame_no < 0) {
		t_bmp.n_width = image->width;
		t_bmp.n_height = image->height;
		t_bmp.b_grayscale = true;
		t_bmp.b_alpha = false;
		t_bmp.n_former_bpp = 8;
		t_bmp.p_buffer = new uint32_t[image->width * image->height];
		++ n_frame_no;
	}
	// alloc bitmap (debug)

	for(int i = 0, n = image->width * image->height; i < n; ++ i) {
		uint8_t n_grey = unsigned(image->data[i]);
		t_bmp.p_buffer[i] = 0xff000000 | (n_grey << 16) | (n_grey << 8) | n_grey;
	}
	// convert grey to RGB data (lame native)

	std::string s_filename;
	stl_ut::Format(s_filename, "debug_gpu_detector_inframe_%05d.tga", n_frame_no);
	CTgaCodec::Save_TGA(s_filename.c_str(), t_bmp);
	// save targa file
#endif

	if(detector->timer) {
		glFlush();
		glFinish();
		image->timestamps[abontsAux1].end =
			image->timestamps[abontsPyramid].start = timerTime(detector->timer);
	}

	if(!gpudet->p_ids->BuildDownscaledImages(p_state, gpudet->p_src_image)) {
		fprintf(stderr, "["LIBNAME"]: " "error: failed to build downscaled images\n");
		return aboneError;
	}
	// build downscaled images

	if(detector->timer) {
		glFlush();
		glFinish();
		image->timestamps[abontsPyramid].end =
			image->timestamps[abontsConvolution].start = timerTime(detector->timer);
	}

	for(int i = 0, n = gpudet->p_ids->n_Page_Num(); i < n; ++ i) {
		CGLTexture_Rect *p_tex = gpudet->p_ids->p_Page_Tex(i);
		CImageConvolver *p_convolver = (*gpudet->p_convolver_list)[i];
		if(!p_convolver->ConvolveImage(p_state, p_tex)) {
			fprintf(stderr, "["LIBNAME"]: " "error: failed to convolve images\n");
			return aboneError;
		}
	}
	// build convolutions

	if(detector->timer) {
		glFlush();
		glFinish();
		image->timestamps[abontsConvolution].end =
			image->timestamps[abontsDetection].start = timerTime(detector->timer);
	}

	{
#ifdef ALL_IN_ONE_PASS
		gpudet->p_sh_gen->Set_FinalThreshold(gpudet->p_classifier->f_final_thresh);
#endif //ALL_IN_ONE_PASS
		for(int i = 0; i < gpudet->p_ids->n_Page_Num(); ++ i) {
			if(!gpudet->p_sh_gen->Render(p_state, gpudet->p_ids->p_Page_Tex(i), // !!
			   (*gpudet->p_convolver_list)[i], (*gpudet->p_renderer_list)[i])) {
				fprintf(stderr, "["LIBNAME"]: " "error: failed to run adaboost shaders\n");
				return aboneError;
			}
		}
	}
	// waldboost

	if(detector->timer) {
		glFlush();
		glFinish();
		image->timestamps[abontsDetection].end =
			image->timestamps[abontsAux2].start = timerTime(detector->timer);
	}

	//glColor3f(1, 1, 1); // not required here
	CGLImageProc::DisableProgrammableShading(p_state);
	// want to display bare images

	std::vector<CTileRenderer::TDetection> detection_list;
	for(size_t i = 0; i < gpudet->p_renderer_list->size(); ++ i) {
#ifdef ALL_IN_ONE_PASS
		if(!(*gpudet->p_renderer_list)[i]->Download_Detections(p_state, detection_list, .5f)) // threshold is elsewhere
#else //ALL_IN_ONE_PASS
		if(!(*gpudet->p_renderer_list)[i]->Download_Detections(p_state, detection_list, gpudet->p_classifier->f_final_thresh))
#endif //ALL_IN_ONE_PASS
			fprintf(stderr, "["LIBNAME"]: " "error: failed to download detections\n");
	}
	// download 

	abonDetection * cursor = image->detections;
	abonDetection * end = image->detectionsEnd;
	image->detectionCount = min(size_t(end - cursor) /*image->detectionCount*/, detection_list.size()); // changed 2009-10-05
	for(int i = 0, n = image->detectionCount; i < n; ++ i) {
		CTileRenderer::TDetection t_det = detection_list[i];
		// get detection

		float f_scale = float(image->width) / t_det.n_scale;
		// calculate detection scale

		cursor[i].angle = 0;
		cursor[i].x = t_det.x * f_scale;
		cursor[i].y = t_det.y * f_scale;
		cursor[i].width = int(gpudet->p_classifier->n_window_width * f_scale);
		cursor[i].height = int(gpudet->p_classifier->n_window_height * f_scale);
		cursor[i].response = 1.0f; // no info right now
		// save detection in their format
	}
    //printf("d:%d\n", detection_list.size()); // debug
	// convert format

	if(detector->timer) {
		glFlush();
		glFinish();
		//image->timestamps[abontsTotal].end = // they say do not write to total
			image->timestamps[abontsAux2].end = timerTime(detector->timer);
	}

	return aboneOk;
}

void Cleanup(GPUDetector *p_detector)
{
	__FuncGuard("::Cleanup");

	if(p_detector) {
		if(p_detector->p_src_image)
			delete p_detector->p_src_image;
		if(p_detector->p_ids)
			delete p_detector->p_ids;
		if(p_detector->p_convolver_list) {
			for(int i = 0, n = p_detector->p_convolver_list->size(); i < n; ++ i)
				delete (*p_detector->p_convolver_list)[i];
			delete p_detector->p_convolver_list;
		}
		if(p_detector->p_renderer_list) {
			for(int i = 0, n = p_detector->p_renderer_list->size(); i < n; ++ i)
				delete (*p_detector->p_renderer_list)[i];
			delete p_detector->p_renderer_list;
		}
		if(p_detector->p_sh_gen)
			delete p_detector->p_sh_gen;
		if(p_detector->p_alpha_tex_atlas)
			delete p_detector->p_alpha_tex_atlas;
		if(p_detector->p_classifier)
			delete p_detector->p_classifier;
		// cleanup

		p_detector->b_initialized = false;
		p_detector->n_image_width = -1;
		p_detector->n_image_height = -1;
		p_detector->b_classifer_specified = false;
		p_detector->p_sh_gen = 0;
		p_detector->p_ids = 0;
		p_detector->p_convolver_list = 0;
		p_detector->p_renderer_list = 0;
		p_detector->p_alpha_tex_atlas = 0;
		p_detector->p_classifier = 0;
		p_detector->p_src_image = 0;
		// fill-in defaults
	}

	if(-- n_instance_num == 0) {
		if(p_image_proc)
			delete p_image_proc;
		Shutdown_OpenGL();
	}
	// shutdown OpenGL as well with the last one
}

bool CheckOpenGL_Extensions()
{
	__FuncGuard("::CheckOpenGL_Extensions");

	if(CGLExtensionHandler::n_GetGL12FuncPointers() ||
	   !CGLExtensionHandler::b_Support_OpenGL(1, 2)) {
		fprintf(stderr, "["LIBNAME"]: " "error: OpenGL 1.2 not supported\n");
		return false; // need 3D texturing
	}
	if(CGLExtensionHandler::n_GetGL13FuncPointers() ||
	   !CGLExtensionHandler::b_Support_OpenGL(1, 3))
		fprintf(stderr, "["LIBNAME"]: " "warning: OpenGL 1.3 not supported\n");
	if(CGLExtensionHandler::n_GetGL14FuncPointers() ||
	   !CGLExtensionHandler::b_Support_OpenGL(1, 4))
		fprintf(stderr, "["LIBNAME"]: " "warning: OpenGL 1.4 not supported\n");
	if(CGLExtensionHandler::n_GetGL15FuncPointers() ||
	   !CGLExtensionHandler::b_Support_OpenGL(1, 5))
		fprintf(stderr, "["LIBNAME"]: " "warning: OpenGL 1.5 not supported\n");
	bool b_have_gl20 = false;
	if(!CGLExtensionHandler::n_GetGL20FuncPointers() &&
	   CGLExtensionHandler::b_Support_OpenGL(2, 0))
		b_have_gl20 = CGL_Core_shader::b_Supported();
	else
		fprintf(stderr, "["LIBNAME"]: " "warning: OpenGL 2.0 not supported\n");
	if(CGLExtensionHandler::n_GetGL21FuncPointers() ||
	   !CGLExtensionHandler::b_Support_OpenGL(2, 1))
		fprintf(stderr, "["LIBNAME"]: " "warning: OpenGL 2.1 not supported\n");
	if(!CGLExtensionHandler::b_Support_OpenGL(3, 0))
		fprintf(stderr, "["LIBNAME"]: " "warning: OpenGL 3.0 not supported (no biggie)\n");
	/*printf("OpenGL %d.%d\n", CGLExtensionHandler::n_OpenGL_Version() / 10,
		CGLExtensionHandler::n_OpenGL_Version() % 10);*/
	if(CGL_ARB_shader::b_Supported())
		;//printf("GL_ARB_fragment_shader\n");
	else if(!b_have_gl20) {
		fprintf(stderr, "["LIBNAME"]: " "error: GL_ARB_fragment_shader not supported\n");
		return false;
	} else
		fprintf(stderr, "["LIBNAME"]: " "warning: GL_ARB_fragment_shader not supported\n");
	if(CGLFrameBuffer_FBO::b_Supported())
		;//printf("GL_EXT_framebuffer_object\n");
	else {
		fprintf(stderr, "["LIBNAME"]: " "error: GL_EXT_framebuffer_object not supported\n");
		return false;
	}
	if(CGLExtensionHandler::b_SupportedExtension("GL_ARB_texture_float"))
		;//printf("GL_ARB_texture_float\n");
	else {
		fprintf(stderr, "["LIBNAME"]: " "error: GL_ARB_texture_float not supported\n");
		return false;
	}
	if(CGLExtensionHandler::b_SupportedExtension("GL_ARB_texture_rg"))
		;//printf("GL_ARB_texture_float\n");
	else
		fprintf(stderr, "["LIBNAME"]: " "warning: GL_ARB_texture_rg not supported; convolver sub-optimal (no biggie)\n");
	if(CGLExtensionHandler::b_SupportedExtension("GL_ARB_texture_rectangle"))
		;//printf("GL_ARB_texture_rectangle\n");
	else {
		fprintf(stderr, "["LIBNAME"]: " "error: GL_ARB_texture_rectangle not supported\n");
		return false;
	}
	if(CGLExtensionHandler::b_SupportedExtension("GL_ARB_occlusion_query") &&
	   !CGLExtensionHandler::n_GetOcclusionQueryARBFuncPointers())
		;//printf("GL_ARB_occlusion_query\n");
	else {
		fprintf(stderr, "["LIBNAME"]: " "error: GL_ARB_occlusion_query not supported\n");
		return false;
	}
	if(CGLExtensionHandler::b_SupportedExtension("GL_NV_float_buffer"))
		;//printf("GL_NV_float_buffer\n");
	else {
		fprintf(stderr, "["LIBNAME"]: " "error: GL_NV_float_buffer not supported; waldboost sub-optimal\n");
		// don't have GL_FLOAT_R16_NV, waldboost code must use GL_RGB16F_ARB instead
	}
	// check necessary OpenGL extensions

	return true;
}

bool Init(GPUDetector *p_detector)
{
	__FuncGuard("::Init");

	if(++ n_instance_num == 1) {
		if(!Init_OpenGL()) {
			fprintf(stderr, "["LIBNAME"]: " "error: failed to initialize OpenGL\n");
			Cleanup(p_detector);
			return false;
		}
		// init OpenGL

		if(!(p_image_proc = new CGLImageProc())) {
			fprintf(stderr, "["LIBNAME"]: " "error: not enough memory\n");
			Cleanup(p_detector);
			return false;
		}
		// create GPU image processing util
	}
	// count instances

	if(p_detector->b_initialized)
		return true;
	// already initialized

	if(!CheckOpenGL_Extensions()) {
		Cleanup(p_detector);
		return false;
	}
	// check for necessary OpenGL extensions

	if(!p_detector->b_classifer_specified || !p_detector->p_classifier) {
		fprintf(stderr, "["LIBNAME"]: " "error: classifier not specified\n");
		Cleanup(p_detector);
		return false;
	}
	TConvolutionList t_conv_list;
	if(!t_conv_list.Create(*p_detector->p_classifier)) {
		fprintf(stderr, "["LIBNAME"]: " "error: not enough memory\n");
		Cleanup(p_detector);
		return false;
	}
	// create convolution list

#ifdef ALL_IN_ONE_PASS
	if(!(p_detector->p_alpha_tex_atlas = new CAlphaTextureAtlas(p_state,
	   *p_detector->p_classifier, 1024, false)) || !p_detector->p_alpha_tex_atlas->b_Status()) {
#else //ALL_IN_ONE_PASS
	if(!(p_detector->p_alpha_tex_atlas = new CAlphaTextureAtlas(p_state,
	   *p_detector->p_classifier, 512, true)) || !p_detector->p_alpha_tex_atlas->b_Status()) {
#endif //ALL_IN_ONE_PASS
		fprintf(stderr, "["LIBNAME"]: " "error: failed to create alpha texture atlas\n");
		Cleanup(p_detector);
		return false;
	}
	// create alpha texture atlas

	if(p_detector->n_image_width == -1 || p_detector->n_image_height == -1) {
		fprintf(stderr, "["LIBNAME"]: " "error: input image size not specified\n");
		Cleanup(p_detector);
		return false;
	}

	TDownscaleImageSet &t_img_set = p_detector->t_img_set;
	const int n_max_page_size = 2048;
	if(!t_img_set.Create(p_detector->n_image_width,
	   p_detector->n_image_height, f_pyramid, 24, 24, 64, 512, n_max_page_size)) {
		if(max(p_detector->n_image_width, p_detector->n_image_height) > n_max_page_size)
			fprintf(stderr, "["LIBNAME"]: " "error: failed to create image downscaler (input image too big)\n");
		else
			fprintf(stderr, "["LIBNAME"]: " "error: not enough memory\n");
		Cleanup(p_detector);
		return false;
	}
	// create downscale image list

	if(!(p_detector->p_src_image = new CGLTexture_Rect(p_state,
	   p_detector->n_image_width, p_detector->n_image_height, GL_LUMINANCE8)) ||
	   !p_detector->p_src_image->b_Status()) {
		fprintf(stderr, "["LIBNAME"]: " "error: not enough memory\n");
		Cleanup(p_detector);
		return false;
	}
	// error check

	if(!(p_detector->p_ids = new CImageDownscaler(p_state, t_img_set)) || !p_detector->p_ids->b_Status()) {
		fprintf(stderr, "["LIBNAME"]: " "error: failed to create image downscaler\n");
		Cleanup(p_detector);
		return false;
	}
	if(!p_detector->p_ids->BuildDownscaledImages(p_state, p_detector->p_src_image)) {
		fprintf(stderr, "["LIBNAME"]: " "error: failed to build downscaled images\n");
		Cleanup(p_detector);
		return false;
	}
	// create downscaler, build images

	if(!(p_detector->p_convolver_list = new std::vector<CImageConvolver*>)) {
		fprintf(stderr, "["LIBNAME"]: " "error: not enough memory\n");
		Cleanup(p_detector);
		return false;
	}
	if(!stl_ut::Reserve_N(*p_detector->p_convolver_list, p_detector->p_ids->n_Page_Num())) {
		fprintf(stderr, "["LIBNAME"]: " "error: not enough memory\n");
		Cleanup(p_detector);
		return false;
	}
	for(int i = 0, n = p_detector->p_ids->n_Page_Num(); i < n; ++ i) {
		CGLTexture_Rect *p_tex = p_detector->p_ids->p_Page_Tex(i);
		CImageConvolver *p_convolver;
		if(!(p_convolver = new CImageConvolver(p_state,
		   p_tex->n_Width(), p_tex->n_Height(), t_conv_list))) {
			fprintf(stderr, "["LIBNAME"]: " "error: not enough memory\n");
			Cleanup(p_detector);
			return false;
		}
		p_detector->p_convolver_list->push_back(p_convolver);
		if(!p_convolver->b_Status()) {
			fprintf(stderr, "["LIBNAME"]: " "error: failed to create convolver\n");
			Cleanup(p_detector);
			return false;
		}
		if(p_convolver->b_SourceImageNeedsBilinearFilter()) {
			CGLTextureParams &r_t_params = p_tex->r_Parameters(p_state);
			r_t_params.Set_Texture_Min_Filter(GL_LINEAR);
			r_t_params.Set_Texture_Mag_Filter(GL_LINEAR);
			_ASSERTE(!r_t_params.b_Generate_Mipmap()); // no mipmaps, right?
		}
		// set bilinear filter to source image
	}
	// create list with image sizes for convolver initialization

#ifdef ALL_IN_ONE_PASS
	if(!(p_detector->p_sh_gen = new CWaldBoost_AllInOne_ShaderGenerator(p_state,
	   *p_detector->p_convolver_list->begin(), p_detector->p_alpha_tex_atlas, *p_detector->p_classifier))) {
#else //ALL_IN_ONE_PASS
	const int n_pass_size_limit = 20;
	if(!(p_detector->p_sh_gen = new CWaldBoostShaderGenerator(p_state,
	   *p_detector->p_convolver_list->begin(), p_detector->p_alpha_tex_atlas, *p_detector->p_classifier,
	   n_downscale, n_pass_size_limit))) {
#endif //ALL_IN_ONE_PASS
		fprintf(stderr, "["LIBNAME"]: " "error: not enough memory\n");
		Cleanup(p_detector);
		return false;
	}
	if(!p_detector->p_sh_gen->b_Status()) {
		fprintf(stderr, "["LIBNAME"]: " "error: failed to create wald-boost shader generator\n");
		Cleanup(p_detector);
		return false;
	}
	// generate waldboost shaders

	if(!(p_detector->p_renderer_list = new std::vector<CTileRenderer*>())) {
		fprintf(stderr, "["LIBNAME"]: " "error: not enough memory\n");
		Cleanup(p_detector);
		return false;
	}
	if(!stl_ut::Reserve_N(*p_detector->p_renderer_list, p_detector->p_ids->n_Page_Num())) {
		fprintf(stderr, "["LIBNAME"]: " "error: not enough memory\n");
		Cleanup(p_detector);
		return false;
	}
	int n_pass_num = p_detector->p_sh_gen->n_Pass_Num();
	for(int i = 0, n = p_detector->p_ids->n_Page_Num(); i < n; ++ i) {
		CTileRenderer *p_renderer;
		if(!(p_renderer = new CTileRenderer(p_state, t_img_set.page_list[i],
		   n_pass_num, n_tile_size, n_downscale))) {
			fprintf(stderr, "["LIBNAME"]: " "error: not enough memory\n");
			Cleanup(p_detector);
			return false;
		}
		p_detector->p_renderer_list->push_back(p_renderer);
		if(!p_renderer->b_Status()) {
			fprintf(stderr, "["LIBNAME"]: " "error: failed to create wald-boost renderer\n");
			Cleanup(p_detector);
			return false;
		}
		// create tile renderer
	}

	//glColor3f(1, 1, 1); // not required here
	CGLImageProc::DisableProgrammableShading(p_state);
	// want to display bare images

	if(glGetError() != GL_NO_ERROR)
		fprintf(stderr, "["LIBNAME"]: " "error: OpenGL error(s) occured during initialization\n");

	p_detector->b_initialized = true;

	return true;
}

abonErrorCode GPU_SetVerbosity(abonDetector *detector, unsigned verbosity)
{
	__FuncGuard("::GPU_SetVerbosity");

	if(!detector)
		return aboneError;

	detector->verbosity = verbosity;
	return aboneOk;
}

abonErrorCode GPU_SetTimer(abonDetector *detector, Timer *timer)
{
	__FuncGuard("::GPU_SetTimer");

	if(!detector)
		return aboneError;

	detector->timer = timer;
	return aboneOk;
}

abonErrorCode GPU_SetClassifier(abonDetector *detector, TClassifier *classifier)
{
	__FuncGuard("::GPU_SetClassifier");

	if(!detector)
		return aboneError;

	GPUDetector *gpudet = (GPUDetector*)detector;
	// type cast detector

	if(gpudet->b_initialized)
		return aboneError;
	if(!gpudet->p_classifier && !(gpudet->p_classifier = new TWaldBoostClassifier))
		return aboneError;
	if(!(gpudet->p_classifier->CopyFrom(classifier)))
		return aboneError;
	gpudet->b_classifer_specified = true;
	return aboneOk;
}

abonErrorCode GPU_SetSize(abonDetector *detector, unsigned width, unsigned height, unsigned pitch)
{
	__FuncGuard("::GPU_SetSize");

	if(!detector)
		return aboneError;

	GPUDetector *gpudet = (GPUDetector*)detector;
	// type cast detector

	if(gpudet->b_initialized)
		return aboneError;
	if(pitch != width)
		return aboneParam;
	gpudet->n_image_width = width;
	gpudet->n_image_height = height;
	return aboneOk;
}

abonErrorCode GPU_Process(abonDetector *detector, abonImageData *image)
{
	__FuncGuard("::GPU_Process");

	if(!detector)
		return aboneError;

	if(!Init((GPUDetector*)detector)) {
		fprintf(stderr, "["LIBNAME"]: " "error: failed to initialize detector\n");
		return aboneError;
	}
	// lazy initialization here

	return DetectVisualObjects(detector, image);
}

abonErrorCode GPU_Start(abonDetector *detector, abonCbGetImage *get, abonCbPutImage *put, void *param)
{
	__FuncGuard("::GPU_Start");

	if(!detector)
		return aboneError;

	abonImageData * image = 0;
    while((image = get(param)) != NULL)
	{
		if(GPU_Process(detector, image))
			return aboneError;
		// process the image

        // Write the image to the output stream
        put(param, image);
	}
	return aboneOk;
}

abonErrorCode GPU_Release(abonDetector *detector)
{
	__FuncGuard("::GPU_Release");

	if(!detector)
		return aboneError;

	Cleanup((GPUDetector*)detector);

	return aboneOk;
}

abonErrorCode GPU_TimestampDescription(abonDetector *detector, abonTimeStamps *timestamp, const char **description)
{
	__FuncGuard("::GPU_TimestampDescription");

	return aboneError;
}

#if defined(WIN32) || defined(_WIN64)
extern "C" __declspec(dllexport) abonDetector * abonCreate()
#else
extern "C" abonDetector * abonCreate()
#endif
{
	__FuncGuard("::abonCreate");

	GPUDetector *detector = (GPUDetector*)malloc(sizeof(GPUDetector));
	if(detector != NULL) {
		memset(detector, 0, sizeof(*detector));
        detector->header.setVerbosity = GPU_SetVerbosity;
        detector->header.setTimer = GPU_SetTimer;
		detector->header.setClassifier = GPU_SetClassifier;
		detector->header.setSize = GPU_SetSize;
		detector->header.process = GPU_Process;
		detector->header.start = GPU_Start;
		detector->header.release = GPU_Release;
        
        // Internal parameters
        detector->b_initialized = false;
		detector->n_image_width = -1;
		detector->n_image_height = -1;
		detector->b_classifer_specified = false;
		detector->p_sh_gen = 0;
		detector->p_ids = 0;
		detector->p_convolver_list = 0;
		detector->p_renderer_list = 0;
		detector->p_alpha_tex_atlas = 0;
		detector->p_classifier = 0;
		detector->p_src_image = 0;
		new(&detector->t_img_set) TDownscaleImageSet(); // init TDownscaleImageSet

        printf("["LIBNAME"]: "LIBVERSION"\n");
	}
	return (abonDetector*)detector;
}
