#include "UberLame_src/CallStack.h"
#include "UberLame_src/OpenGL20.h"
#include "UberLame_src/OpenGLState.h"
#include "UberLame_src/Texture.h"
#include "UberLame_src/Shader2.h"
#include "UberLame_src/RenderBuffer2.h"
#include "UberLame_src/VertexBufferObject.h"
#include "OpenGL20Logo.h"
#include "InitOpenGL.h"

static bool b_pixel_buffer_supported = false;
static bool b_fence_supported = false;
static bool b_fbo_supported = false;
static bool b_imaging_supported = false;
static bool b_gl_images_supported = false;
static bool b_shaders_supported = false;
// opengl capabilities

CGLState *p_state = 0;
// opengl state cache

/*
 *								--- win32 OpenGL ---
 */

#ifdef WIN32

#include "./UberLame_src/OpenGLDrv.h"

static HWND h_wnd;
static CGLDriver *p_gl_driver = 0;

static const char *p_s_OpenGL_window_name = "berLame OpenGL";
static const char *p_s_OpenGL_window_classname = "UberLameGLWnd";
// window name and wndclass

/**
 *	bool Create_Window(const char *p_s_name, int n_width, int n_height)
 *		- creates window with given name and size
 *		- returns true on success, false on failure
 */
bool Create_Window(const char *p_s_name, int n_width, int n_height)
{
	WNDCLASSEX t_wnd_class;
	t_wnd_class.cbSize = sizeof(WNDCLASSEX);
	t_wnd_class.style = CS_HREDRAW | CS_VREDRAW;
	t_wnd_class.lpfnWndProc = DefWindowProc;
	t_wnd_class.cbClsExtra = 0;
	t_wnd_class.cbWndExtra = 0;
	t_wnd_class.hInstance = GetModuleHandle(NULL);
	t_wnd_class.hIcon = LoadIcon(GetModuleHandle(NULL), IDI_APPLICATION);
	t_wnd_class.hCursor = LoadCursor(GetModuleHandle(NULL), IDC_ARROW);
	t_wnd_class.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
	t_wnd_class.lpszMenuName = 0;
	t_wnd_class.lpszClassName = p_s_OpenGL_window_classname;
	t_wnd_class.hIconSm = LoadIcon(GetModuleHandle(NULL), IDI_APPLICATION);
	if(!RegisterClassEx(&t_wnd_class))
		return false;
	// create window class

	RECT t_rect;
	t_rect.left = t_rect.top = 0;
	t_rect.right = n_width;
	t_rect.bottom = n_height;
	if(!AdjustWindowRectEx(&t_rect, WS_BORDER | WS_CAPTION, false, WS_EX_TOOLWINDOW))
		return false;
	// calculate window size to contain client area with desired resolution

	if(!(h_wnd = CreateWindowEx(WS_EX_TOOLWINDOW, p_s_OpenGL_window_classname, p_s_name,
	   WS_BORDER | WS_CAPTION, CW_USEDEFAULT, CW_USEDEFAULT,
	   t_rect.right - t_rect.left, t_rect.bottom - t_rect.top,
	   NULL, NULL, GetModuleHandle(NULL), NULL)))
		return false;
	// create window

#ifdef GPU_LIB_WIN32_SHOW_WINDOW
	ShowWindow(h_wnd, SW_SHOW);
#else //GPU_LIB_WIN32_SHOW_WINDOW
	ShowWindow(h_wnd, SW_HIDE);
#endif //GPU_LIB_WIN32_SHOW_WINDOW
	UpdateWindow(h_wnd);
	// show & update window

	return true;
}

#endif //WIN32

/*
 *								--- ~win32 OpenGL ---
 */

/*
 *								--- OpenGL init code ---
 */

/**
 *	bool Init_OpenGL()
 *		- initializes OpenGL. under linux uses GLUT, under windows uses custom driver
 *		- in case running under windows, there's option to display OpenGL window,
 *		  containing "OpenGL 2.0" logo (enabled using GPU_LIB_WIN32_SHOW_WINDOW #define)
 *		- returns true on success (or multiple initialization), false on failure
 */
bool Init_OpenGL()
{
	if(p_state)
		return true;

	/*if(!Register_Shaders())
		return false;*/
	// register shaders to CShaderFactory

#ifdef WIN32
	if(!Create_Window(p_s_OpenGL_window_name, 150, 57))
		return false;
	// create window

	if(!(p_gl_driver = new CGLDriver()))
		return false;
	if(!p_gl_driver->Init(h_wnd, 150, 57, 24, 0)) {
		delete p_gl_driver;
		p_gl_driver = 0;
		return false;
	}
	// init open-gl

#ifdef GL_STATE_SINGLE_CONTEXT_ONLY
	if(!(p_state = new CGLState())) {
#else //GL_STATE_SINGLE_CONTEXT_ONLY
	if(!(p_state = new CGLState(wglGetCurrentDC(), wglGetCurrentContext()))) {
#endif //GL_STATE_SINGLE_CONTEXT_ONLY
		delete p_gl_driver;
		p_gl_driver = 0;
		return false;
	}
	// create state guard
#else //WIN32
	// t_odo -  linux opengl (init glut)

	int n_arg_num = 1;
	char *p_s_arg = "no-arg";
	glutInit(&n_arg_num, &p_s_arg);
    glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE);
    glutInitWindowSize(150, 57);
    glutCreateWindow(p_s_OpenGL_window_name);

#ifdef GL_STATE_SINGLE_CONTEXT_ONLY
	if(!(p_state = new CGLState()))
#else //GL_STATE_SINGLE_CONTEXT_ONLY
	if(!(p_state = new CGLState(NULL, NULL)))
#endif //GL_STATE_SINGLE_CONTEXT_ONLY
		return false;
	// create state guard
#endif //WIN32

	b_fbo_supported = CGLFrameBuffer_FBO::b_Supported(); // gets function pointers as well
	CGLExtensionHandler::n_GetGL12FuncPointers();
	CGLExtensionHandler::n_GetGL13FuncPointers();
	CGLExtensionHandler::n_GetMultitextureFuncPointers(); // opengl1.3 as well
	CGLExtensionHandler::n_GetGL14FuncPointers();
	CGLExtensionHandler::n_GetGL15FuncPointers();
	CGLExtensionHandler::n_GetGL20FuncPointers();
	b_pixel_buffer_supported = CGLPixelBufferObject::b_Supported(); // gets function pointers as well
	b_gl_images_supported = CGLPixelBufferObject::b_Supported();
	b_fence_supported = CGLExtensionHandler::b_SupportedExtension("GL_NV_fence") &&
		!CGLExtensionHandler::n_GetFenceNVFuncPointers();
	b_imaging_supported = CGLExtensionHandler::b_SupportedExtension("GL_ARB_imaging");
	b_shaders_supported = CGL_ARB_shader::b_Supported() || CGL_Core_shader::b_Supported();
	// get function pointers

#ifdef GPU_LIB_WIN32_SHOW_WINDOW
	unsigned __int32 *p_gl_logo;
	if((p_gl_logo = COpenGL20Logo::p_DecompressImage())) {
		/*glDrawPixels(n_opengl_logo_2_0_sm_jpg_width, n_opengl_logo_2_0_sm_jpg_height,
			GL_BGRA, GL_UNSIGNED_BYTE, p_gl_logo);*/ // upside-down
		CGLTexture_2D opengl_tex(p_state, COpenGL20Logo::n_Width(),
			COpenGL20Logo::n_Height(), GL_RGBA8, false, 0,
			GL_BGRA, GL_UNSIGNED_BYTE, p_gl_logo);
		delete[] p_gl_logo;

		p_state->DisableDepthTest();
		p_state->DisableCullFace();
		p_state->Color3f(1, 1, 1);

		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();

		opengl_tex.Bind_Enable(p_state);
		const float p_vertices[] = {
			0, 1, -1, -1, 0,
			0, 0, -1, +1, 0,
			1, 0, +1, +1, 0,
			1, 1, +1, -1, 0
		};
		glInterleavedArrays(GL_T2F_V3F, 0, p_vertices);
		glDrawArrays(GL_QUADS, 0, 4);
		glDisableClientState(GL_VERTEX_ARRAY);
		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
		// use VA, rather than glBegin

		//p_state->BindTexture2D(0); // destroying texture with logo

#ifdef WIN32
		p_gl_driver->Blit();
#else //WIN32
		glutSwapBuffers();
#endif //WIN32
	}
	// load OpenGL logo
#endif //GPU_LIB_WIN32_SHOW_WINDOW

	return true;
}

/**
 *	bool Shutdown_OpenGL();
 *		- shuts down OpenGL, previously initialized by Init_OpenGL(), calls Free_OpenGL_Objects()
 *		  to delete all kinds of temporary objects library uses
 *		- returns true on success, false on failure
 *		- note for linux - there's no function to shut down GLUT, it's expected
 *		  to call this just before exitting as it has really no effect on OpenGL, it frees
 *		  OpenGL objects only
 */
bool Shutdown_OpenGL()
{
	if(!p_state)
		return true;

	bool b_result = true;//Free_OpenGL_Objects();

	delete p_state;
	p_state = 0;

#ifdef WIN32
	b_result &= p_gl_driver->Shutdown();
	delete p_gl_driver;
	// shutdown open-gl

	UnregisterClass(p_s_OpenGL_window_classname, GetModuleHandle(NULL));
	DestroyWindow(h_wnd);
	// destroy window
#else
	// t_odo - linux opengl (shutdown glut)
	// no function to shutdown glut :(
#endif

	return b_result;
}

/*
 *								--- ~OpenGL init code ---
 */
