/*
								+--------------------------------+
								|                                |
								|   ***  Call stack guard  ***   |
								|                                |
								|  Copyright  -tHE SWINe- 2005  |
								|                                |
								|          CallStack.h           |
								|                                |
								+--------------------------------+
*/

/*
 *	passed code revision at 2006-05-16
 *
 *	added the p_s_Format() function, fixed some logging errors
 *	added couple of _ASSERTE-s to CCallStackGuard::file_printf
 *
 *	passed code revision at 2007-02-05
 *
 *	fixed pointer / iterator type mismatch in CCallStackGuard::~CCallStackGuard()
 *	removed handcrafted loops over std::vector, using std::for_each instead
 *	fixed 'the' typo in code history
 *
 *	2007-03-06
 *
 *	removed __declspec(dllexport)
 *	fixed error arround separator concatenating in call stack dump
 *
 *	2007-10-29
 *
 *	enhanced linux compatibility
 *
 *	2007-11-12
 *
 *	reformat (added line breaks where lines were longer than 100 characters)
 *
 *	2008-02-06
 *
 *	slightly improved assertion for console apps (needs no memory allocation)
 *	changed message formatting a bit
 *
 *	2008-04-20
 *
 *	added a couple of #ifdefs which disable compilation of CCallStackGuard
 *	in cases it's not needed
 *
 *	2008-08-08
 *
 *	added #ifdef for windows 64
 *
 */

#ifndef __CALL_STACK_GUARD_INCLUDED
#define __CALL_STACK_GUARD_INCLUDED

/*
 *	char *p_s_Format(const char *p_s_fmt, ...)
 *		- equivalent of sprintf, but it returns newly allocated string, instead of writing into
 *		  the one that is passed as first sprintf param
 *		- useful when it's necessary to sprintf into the string of unknown length
 *		- might cause waste of memory under msvc
 */
char *p_s_Format(const char *p_s_fmt, ...);

/*
 *	char *p_s_Format(char *p_s_str, int &r_n_size, const char *p_s_fmt, ...)
 *		- equivalent of sprintf, but it reallocates the string in case it's too short
 *		- useful when it's necessary to sprintf into the string of unknown length
 *		- might cause waste of memory under msvc
 */
char *p_s_Format(char *p_s_str, int &r_n_size, const char *p_s_fmt, ...);

/*#define _CALL_STACK_TRACKING_FILE "call_stack_debug.txt"
#define _TRACK_CALL_STACK
#define _TRACK_CALL_STACK_EXITTING*/
// you can permanently write call stack on any function entered / exit
// (note that this would propably be a significant performance hit)

//#define _FORCE_ASSERTIONS
// you can force assertions to appear in release versions as well

#if defined(_DEBUG) || defined(_FORCE_ASSERTIONS) || !defined(_DONT_USE_STACK_GUARD)

#include <vector>

class /*__declspec(dllexport)*/ CCallStackGuard {
protected:
	static std::vector<const char*> m_call_stack;

public:
	CCallStackGuard(const char *p_s_parent_function);
	~CCallStackGuard();

	static const std::vector<const char*> *p_CallStack();
	static char *p_s_CallStack(const char *p_s_separator, const char *p_s_pro = 0,
		const char *p_s_epi = 0);

#if defined(WIN32) || defined (_WIN64)
	static void Assert_Win(const char *p_s_expr, int n_line, const char *p_s_file);
#endif // WIN32, _WIN64
	static void Assert_Dos(const char *p_s_expr, int n_line, const char *p_s_file);

protected:
#ifdef _TRACK_CALL_STACK
	void file_printf(int b_over);
#endif
};

#endif //defined(_DEBUG) || defined(_FORCE_ASSERTIONS) || !defined(_DONT_USE_STACK_GUARD)

#ifdef _DONT_USE_STACK_GUARD
#define __FuncGuard(func_name)
#else
#define __FuncGuard(func_name) CCallStackGuard guard(func_name)
#endif

// write __FuncGuard("function_name"); into beginning of every your function

#ifdef _ASSERTE
#undef _ASSERTE
#endif

#if !defined(_DEBUG) && !defined(_FORCE_ASSERTIONS)
#define _ASSERTE(exp)
#else //_DEBUG
#if defined(WIN32) || defined(_WIN64)
#define _ASSERTE(exp)	do { if(!(exp)) CCallStackGuard::Assert_Win(#exp, __LINE__, __FILE__); } while(0)
#else //_WINDOWS
#define _ASSERTE(exp)	do { if(!(exp)) CCallStackGuard::Assert_Dos(#exp, __LINE__, __FILE__); } while(0)
#endif //_WINDOWS
#endif //_DEBUG

#endif //__CALL_STACK_GUARD_INCLUDED
