/*
								+---------------------------------+
								|                                 |
								|  ***  Standard hash funcs  ***  |
								|                                 |
								|  Copyright   -tHE SWINe- 2006  |
								|                                 |
								|            Hash.cpp             |
								|                                 |
								+---------------------------------+
*/

/*
 *
 *	2007-12-24
 *
 *	improved linux compatibility by adding posix integer types
 *
 *	2008-03-04
 *
 *	now using Integer.h header
 *
 *	2008-07-04
 *
 *	replaced all occurences of unsigned char with uint8_t
 *	changed CStreamHash::Process_Data(const unit8_t*, int) to
 *	CStreamHash::Process_Data(const void*, int) so there's no need
 *	to type cast data anymore (convenience improvement)
 *
 *	2009-05-04
 *
 *	fixed mixed windows / linux line endings
 *
 */

#include <string.h>
#include <stdio.h>
#include "Hash.h"

#if defined(_MSC_VER) && !defined(__MWERKS__) && !defined(for)
#define for if(0) {} else for
#endif

/*
 *								=== TMD5 ===
 */

TMD5::TMD5()
{
	m_n_hash[0] = 0x67452301;
	m_n_hash[1] = 0xEFCDAB89;
	m_n_hash[2] = 0x98BADCFE;
	m_n_hash[3] = 0x10325476;
}

TMD5 TMD5::operator =(const TMD5 &r_t_md5)
{
	memcpy(m_n_hash, r_t_md5.m_n_hash, 4 * sizeof(uint32_t));
	return *this;
}

bool TMD5::operator ==(const TMD5 &r_t_md5) const
{
	return m_n_hash[0] == r_t_md5.m_n_hash[0] && m_n_hash[1] == r_t_md5.m_n_hash[1] &&
		   m_n_hash[2] == r_t_md5.m_n_hash[2] && m_n_hash[3] == r_t_md5.m_n_hash[3];
}

bool TMD5::operator !=(const TMD5 &r_t_md5) const
{
	return m_n_hash[0] != r_t_md5.m_n_hash[0] || m_n_hash[1] != r_t_md5.m_n_hash[1] ||
		   m_n_hash[2] != r_t_md5.m_n_hash[2] || m_n_hash[3] != r_t_md5.m_n_hash[3];
}

bool TMD5::operator >(const TMD5 &r_t_md5) const
{
	return m_n_hash[0] > r_t_md5.m_n_hash[0] || (m_n_hash[0] == r_t_md5.m_n_hash[0] &&
		   m_n_hash[1] > r_t_md5.m_n_hash[1] || (m_n_hash[1] == r_t_md5.m_n_hash[1] &&
		   m_n_hash[2] > r_t_md5.m_n_hash[2] || (m_n_hash[2] == r_t_md5.m_n_hash[2] &&
		   m_n_hash[3] > r_t_md5.m_n_hash[3])));
}

bool TMD5::operator <(const TMD5 &r_t_md5) const
{
	return m_n_hash[0] < r_t_md5.m_n_hash[0] || (m_n_hash[0] == r_t_md5.m_n_hash[0] &&
		   m_n_hash[1] < r_t_md5.m_n_hash[1] || (m_n_hash[1] == r_t_md5.m_n_hash[1] &&
		   m_n_hash[2] < r_t_md5.m_n_hash[2] || (m_n_hash[2] == r_t_md5.m_n_hash[2] &&
		   m_n_hash[3] < r_t_md5.m_n_hash[3])));
}

bool TMD5::operator >=(const TMD5 &r_t_md5) const
{
	return m_n_hash[0] >= r_t_md5.m_n_hash[0] || (m_n_hash[0] == r_t_md5.m_n_hash[0] &&
		   m_n_hash[1] >= r_t_md5.m_n_hash[1] || (m_n_hash[1] == r_t_md5.m_n_hash[1] &&
		   m_n_hash[2] >= r_t_md5.m_n_hash[2] || (m_n_hash[2] == r_t_md5.m_n_hash[2] &&
		   m_n_hash[3] >= r_t_md5.m_n_hash[3])));
}

bool TMD5::operator <=(const TMD5 &r_t_md5) const
{
	return m_n_hash[0] <= r_t_md5.m_n_hash[0] || (m_n_hash[0] == r_t_md5.m_n_hash[0] &&
		   m_n_hash[1] <= r_t_md5.m_n_hash[1] || (m_n_hash[1] == r_t_md5.m_n_hash[1] &&
		   m_n_hash[2] <= r_t_md5.m_n_hash[2] || (m_n_hash[2] == r_t_md5.m_n_hash[2] &&
		   m_n_hash[3] <= r_t_md5.m_n_hash[3])));
}

/*inline uint32_t &TMD5::operator [](int n_index)
{
	return m_n_hash[n_index];
}

inline uint32_t TMD5::operator [](int n_index) const
{
	return m_n_hash[n_index];
}*/

void TMD5::Process_Data(const uint32_t *p_data)
{
	const int r[64] = {
		7, 12, 17, 22, 7, 12, 17, 22, 7, 12, 17, 22, 7, 12, 17, 22,
		5,  9, 14, 20, 5,  9, 14, 20, 5,  9, 14, 20, 5,  9, 14, 20,
		4, 11, 16, 23, 4, 11, 16, 23, 4, 11, 16, 23, 4, 11, 16, 23,
		6, 10, 15, 21, 6, 10, 15, 21, 6, 10, 15, 21, 6, 10, 15, 21
	};
	const uint32_t k[64] = {
		0xd76aa478, 0xe8c7b756, 0x242070db, 0xc1bdceee, 0xf57c0faf, 0x4787c62a, 0xa8304613, 0xfd469501,
		0x698098d8, 0x8b44f7af, 0xffff5bb1, 0x895cd7be, 0x6b901122, 0xfd987193, 0xa679438e, 0x49b40821,
		0xf61e2562, 0xc040b340, 0x265e5a51, 0xe9b6c7aa, 0xd62f105d, 0x02441453, 0xd8a1e681, 0xe7d3fbc8,
		0x21e1cde6, 0xc33707d6, 0xf4d50d87, 0x455a14ed, 0xa9e3e905, 0xfcefa3f8, 0x676f02d9, 0x8d2a4c8a,
		0xfffa3942, 0x8771f681, 0x6d9d6122, 0xfde5380c, 0xa4beea44, 0x4bdecfa9, 0xf6bb4b60, 0xbebfbc70,
		0x289b7ec6, 0xeaa127fa, 0xd4ef3085, 0x04881d05, 0xd9d4d039, 0xe6db99e5, 0x1fa27cf8, 0xc4ac5665,
		0xf4292244, 0x432aff97, 0xab9423a7, 0xfc93a039, 0x655b59c3, 0x8f0ccc92, 0xffeff47d, 0x85845dd1,
		0x6fa87e4f, 0xfe2ce6e0, 0xa3014314, 0x4e0811a1, 0xf7537e82, 0xbd3af235, 0x2ad7d2bb, 0xeb86d391
	};

	uint32_t a = m_n_hash[0], b = m_n_hash[1], c = m_n_hash[2], d = m_n_hash[3];

	for(int i = 0; i < 16; i ++) {
		uint32_t f = (d ^ (b & (c ^ d)));
		uint32_t n_temp = d;
		d = c;
		c = b;
		b += n_leftrot(a + f + k[i] + p_data[i], r[i]);
		a = n_temp;
	}
	for(int i = 16; i < 32; i ++) {
		uint32_t f = (c ^ (d & (b ^ c)));
		uint32_t n_temp = d;
		d = c;
		c = b;
		b += n_leftrot(a + f + k[i] + p_data[(5 * i + 1) & 0x0f], r[i]);
		a = n_temp;
	}
	for(int i = 32; i < 48; i ++) {
		uint32_t f = b ^ c ^ d;
		uint32_t n_temp = d;
		d = c;
		c = b;
		b += n_leftrot(a + f + k[i] + p_data[(3 * i + 5) & 0x0f], r[i]);
		a = n_temp;
	}
	for(int i = 48; i < 64; i ++) {
		uint32_t f = c ^ (b | ~d);
		uint32_t n_temp = d;
		d = c;
		c = b;
		b += n_leftrot(a + f + k[i] + p_data[(7 * i) & 0x0f], r[i]);
		a = n_temp;
	}
	// main processing loop

	m_n_hash[0] += a;
	m_n_hash[1] += b;
	m_n_hash[2] += c;
	m_n_hash[3] += d;
}

inline uint32_t TMD5::n_leftrot(uint32_t x, int y)
{
	return (x << y) | (x >> (32 - y));
}

/*
 *								=== ~TMD5 ===
 */

/*
 *								=== TSHA1 ===
 */

TSHA1::TSHA1()
{
	m_n_hash[0] = 0x67452301;
	m_n_hash[1] = 0xEFCDAB89;
	m_n_hash[2] = 0x98BADCFE;
	m_n_hash[3] = 0x10325476;
	m_n_hash[4] = 0xC3D2E1F0;
}

TSHA1 TSHA1::operator =(const TSHA1 &r_t_sha1)
{
	memcpy(m_n_hash, r_t_sha1.m_n_hash, 5 * sizeof(uint32_t));
	return *this;
}

bool TSHA1::operator ==(const TSHA1 &r_t_sha1) const
{
	return m_n_hash[0] == r_t_sha1.m_n_hash[0] && m_n_hash[1] == r_t_sha1.m_n_hash[1] &&
		   m_n_hash[2] == r_t_sha1.m_n_hash[2] && m_n_hash[3] == r_t_sha1.m_n_hash[3] &&
		   m_n_hash[4] == r_t_sha1.m_n_hash[4];
}

bool TSHA1::operator !=(const TSHA1 &r_t_sha1) const
{
	return m_n_hash[0] != r_t_sha1.m_n_hash[0] || m_n_hash[1] != r_t_sha1.m_n_hash[1] ||
		   m_n_hash[2] != r_t_sha1.m_n_hash[2] || m_n_hash[3] != r_t_sha1.m_n_hash[3] ||
		   m_n_hash[4] != r_t_sha1.m_n_hash[4];
}

bool TSHA1::operator >(const TSHA1 &r_t_sha1) const
{
	return m_n_hash[0] > r_t_sha1.m_n_hash[0] || (m_n_hash[0] == r_t_sha1.m_n_hash[0] &&
		   m_n_hash[1] > r_t_sha1.m_n_hash[1] || (m_n_hash[1] == r_t_sha1.m_n_hash[1] &&
		   m_n_hash[2] > r_t_sha1.m_n_hash[2] || (m_n_hash[2] == r_t_sha1.m_n_hash[2] &&
		   m_n_hash[3] > r_t_sha1.m_n_hash[3] || (m_n_hash[3] == r_t_sha1.m_n_hash[3] &&
		   m_n_hash[4] > r_t_sha1.m_n_hash[4]))));
}

bool TSHA1::operator <(const TSHA1 &r_t_sha1) const
{
	return m_n_hash[0] < r_t_sha1.m_n_hash[0] || (m_n_hash[0] == r_t_sha1.m_n_hash[0] &&
		   m_n_hash[1] < r_t_sha1.m_n_hash[1] || (m_n_hash[1] == r_t_sha1.m_n_hash[1] &&
		   m_n_hash[2] < r_t_sha1.m_n_hash[2] || (m_n_hash[2] == r_t_sha1.m_n_hash[2] &&
		   m_n_hash[3] < r_t_sha1.m_n_hash[3] || (m_n_hash[3] == r_t_sha1.m_n_hash[3] &&
		   m_n_hash[4] < r_t_sha1.m_n_hash[4]))));
}

bool TSHA1::operator >=(const TSHA1 &r_t_sha1) const
{
	return m_n_hash[0] >= r_t_sha1.m_n_hash[0] || (m_n_hash[0] == r_t_sha1.m_n_hash[0] &&
		   m_n_hash[1] >= r_t_sha1.m_n_hash[1] || (m_n_hash[1] == r_t_sha1.m_n_hash[1] &&
		   m_n_hash[2] >= r_t_sha1.m_n_hash[2] || (m_n_hash[2] == r_t_sha1.m_n_hash[2] &&
		   m_n_hash[3] >= r_t_sha1.m_n_hash[3] || (m_n_hash[3] == r_t_sha1.m_n_hash[3] &&
		   m_n_hash[4] >= r_t_sha1.m_n_hash[4]))));
}

bool TSHA1::operator <=(const TSHA1 &r_t_sha1) const
{
	return m_n_hash[0] <= r_t_sha1.m_n_hash[0] || (m_n_hash[0] == r_t_sha1.m_n_hash[0] &&
		   m_n_hash[1] <= r_t_sha1.m_n_hash[1] || (m_n_hash[1] == r_t_sha1.m_n_hash[1] &&
		   m_n_hash[2] <= r_t_sha1.m_n_hash[2] || (m_n_hash[2] == r_t_sha1.m_n_hash[2] &&
		   m_n_hash[3] <= r_t_sha1.m_n_hash[3] || (m_n_hash[3] == r_t_sha1.m_n_hash[3] &&
		   m_n_hash[4] <= r_t_sha1.m_n_hash[4]))));
}

/*inline uint32_t &TSHA1::operator [](int n_index)
{
	return m_n_hash[n_index];
}

inline uint32_t TSHA1::operator [](int n_index) const
{
	return m_n_hash[n_index];
}*/

void TSHA1::Process_Data(const uint32_t *p_data)
{
	uint32_t p_tmp_data[80];

	for(int i = 0; i < 16; i ++)
		p_tmp_data[i] = n_EndianConversion(p_data[i]);
	for(int i = 16; i < 80; i ++) {
		p_tmp_data[i] = n_leftrot(p_tmp_data[i - 3] ^ p_tmp_data[i - 8] ^
			p_tmp_data[i - 14] ^ p_tmp_data[i - 16], 1);
	}
	// expand data, convert endian

	uint32_t a = m_n_hash[0], b = m_n_hash[1],
		c = m_n_hash[2], d = m_n_hash[3], e = m_n_hash[4];

	for(int i = 0; i < 20; i ++) {
		uint32_t f = d ^ (b & (c ^ d));
		uint32_t n_temp = n_leftrot(a, 5) + f + e + 0x5A827999 + p_tmp_data[i];
		e = d;
		d = c;
		c = n_leftrot(b, 30);
		b = a;
		a = n_temp;
	}
	for(int i = 20; i < 40; i ++) {
		uint32_t f = b ^ c ^ d;
		uint32_t n_temp = n_leftrot(a, 5) + f + e + 0x6ED9EBA1 + p_tmp_data[i];
		e = d;
		d = c;
		c = n_leftrot(b, 30);
		b = a;
		a = n_temp;
	}
	for(int i = 40; i < 60; i ++) {
		uint32_t f = (b & c) | (d & (b | c));
		uint32_t n_temp = n_leftrot(a, 5) + f + e + 0x8F1BBCDC + p_tmp_data[i];
		e = d;
		d = c;
		c = n_leftrot(b, 30);
		b = a;
		a = n_temp;
	}
	for(int i = 60; i < 80; i ++) {
		uint32_t f = b ^ c ^ d;
		uint32_t n_temp = n_leftrot(a, 5) + f + e + 0xCA62C1D6 + p_tmp_data[i];
		e = d;
		d = c;
		c = n_leftrot(b, 30);
		b = a;
		a = n_temp;
	}
	// main processing loop

	m_n_hash[0] += a;
	m_n_hash[1] += b;
	m_n_hash[2] += c;
	m_n_hash[3] += d;
	m_n_hash[4] += e;
}

inline uint32_t TSHA1::n_leftrot(uint32_t x, int y)
{
	return (x << y) | (x >> (32 - y));
}

inline uint32_t TSHA1::n_EndianConversion(uint32_t x)
{
	return (x >> 24) | ((x >> 8) & 0xff00) | ((x & 0xff00) << 8) | (x << 24);
}

/*
 *								=== ~TSHA1 ===
 */

/*
 *								=== TTinyMD5 ===
 */

TTinyMD5::TTinyMD5()
{
	m_n_hash[0] = 0x67;
	m_n_hash[1] = 0xEF;
	m_n_hash[2] = 0x98;
	m_n_hash[3] = 0x10;
}

TTinyMD5 TTinyMD5::operator =(const TTinyMD5 &r_t_md5)
{
	memcpy(m_n_hash, r_t_md5.m_n_hash, 4 * sizeof(uint8_t));
	return *this;
}

bool TTinyMD5::operator ==(const TTinyMD5 &r_t_md5) const
{
	return m_n_hash[0] == r_t_md5.m_n_hash[0] && m_n_hash[1] == r_t_md5.m_n_hash[1] &&
		   m_n_hash[2] == r_t_md5.m_n_hash[2] && m_n_hash[3] == r_t_md5.m_n_hash[3];
}

bool TTinyMD5::operator !=(const TTinyMD5 &r_t_md5) const
{
	return m_n_hash[0] != r_t_md5.m_n_hash[0] || m_n_hash[1] != r_t_md5.m_n_hash[1] ||
		   m_n_hash[2] != r_t_md5.m_n_hash[2] || m_n_hash[3] != r_t_md5.m_n_hash[3];
}

bool TTinyMD5::operator >(const TTinyMD5 &r_t_md5) const
{
	return m_n_hash[0] > r_t_md5.m_n_hash[0] || (m_n_hash[0] == r_t_md5.m_n_hash[0] &&
		   m_n_hash[1] > r_t_md5.m_n_hash[1] || (m_n_hash[1] == r_t_md5.m_n_hash[1] &&
		   m_n_hash[2] > r_t_md5.m_n_hash[2] || (m_n_hash[2] == r_t_md5.m_n_hash[2] &&
		   m_n_hash[3] > r_t_md5.m_n_hash[3])));
}

bool TTinyMD5::operator <(const TTinyMD5 &r_t_md5) const
{
	return m_n_hash[0] < r_t_md5.m_n_hash[0] || (m_n_hash[0] == r_t_md5.m_n_hash[0] &&
		   m_n_hash[1] < r_t_md5.m_n_hash[1] || (m_n_hash[1] == r_t_md5.m_n_hash[1] &&
		   m_n_hash[2] < r_t_md5.m_n_hash[2] || (m_n_hash[2] == r_t_md5.m_n_hash[2] &&
		   m_n_hash[3] < r_t_md5.m_n_hash[3])));
}

bool TTinyMD5::operator >=(const TTinyMD5 &r_t_md5) const
{
	return m_n_hash[0] >= r_t_md5.m_n_hash[0] || (m_n_hash[0] == r_t_md5.m_n_hash[0] &&
		   m_n_hash[1] >= r_t_md5.m_n_hash[1] || (m_n_hash[1] == r_t_md5.m_n_hash[1] &&
		   m_n_hash[2] >= r_t_md5.m_n_hash[2] || (m_n_hash[2] == r_t_md5.m_n_hash[2] &&
		   m_n_hash[3] >= r_t_md5.m_n_hash[3])));
}

bool TTinyMD5::operator <=(const TTinyMD5 &r_t_md5) const
{
	return m_n_hash[0] <= r_t_md5.m_n_hash[0] || (m_n_hash[0] == r_t_md5.m_n_hash[0] &&
		   m_n_hash[1] <= r_t_md5.m_n_hash[1] || (m_n_hash[1] == r_t_md5.m_n_hash[1] &&
		   m_n_hash[2] <= r_t_md5.m_n_hash[2] || (m_n_hash[2] == r_t_md5.m_n_hash[2] &&
		   m_n_hash[3] <= r_t_md5.m_n_hash[3])));
}

void TTinyMD5::Process_Data(const uint32_t *p_data)
{
	const int r[16] = {2, 3, 5, 7, 1, 1, 2, 3, 3, 1, 4, 1, 1, 8, 6, 2};
	const uint8_t k[16] = {
		0xd7, 0xe8, 0x24, 0xc1,
		0xf5, 0x47, 0xa8, 0xfd,
		0x69, 0x8b, 0xff, 0x89,
		0x6b, 0xfd, 0xa6, 0x49
	};

	for(int n = 0; n < 2; n ++, p_data ++) {
		uint8_t a = m_n_hash[0], b = m_n_hash[1], c = m_n_hash[2], d = m_n_hash[3];

		for(int i = 0; i < 4; i ++) {
			uint8_t f = (d ^ (b & (c ^ d)));
			uint8_t n_temp = d;
			d = c;
			c = b;
			b += n_leftrot(a + f + k[i] + ((const uint8_t*)p_data)[i], r[i]);
			a = n_temp;
		}
		for(int i = 4; i < 8; i ++) {
			uint8_t f = (c ^ (d & (b ^ c)));
			uint8_t n_temp = d;
			d = c;
			c = b;
			b += n_leftrot(a + f + k[i] + ((const uint8_t*)p_data)[(5 * i + 1) & 0x03], r[i]);
			a = n_temp;
		}
		for(int i = 8; i < 12; i ++) {
			uint8_t f = b ^ c ^ d;
			uint8_t n_temp = d;
			d = c;
			c = b;
			b += n_leftrot(a + f + k[i] + ((const uint8_t*)p_data)[(3 * i + 5) & 0x03], r[i]);
			a = n_temp;
		}
		for(int i = 12; i < 16; i ++) {
			uint8_t f = c ^ (b | ~d);
			uint8_t n_temp = d;
			d = c;
			c = b;
			b += n_leftrot(a + f + k[i] + ((const uint8_t*)p_data)[(7 * i) & 0x03], r[i]);
			a = n_temp;
		}
		// main processing loop

		m_n_hash[0] += a;
		m_n_hash[1] += b;
		m_n_hash[2] += c;
		m_n_hash[3] += d;
	}
}

inline uint8_t TTinyMD5::n_leftrot(uint8_t x, int y)
{
	return (x << y) | (x >> (8 - y));
}

/*
 *								=== ~TTinyMD5 ===
 */

/*
 *		-end-of-file-
 */
