/*
								+--------------------------------+
								|                                |
								| *** Portable integer types *** |
								|                                |
								|  Copyright  -tHE SWINe- 2008  |
								|                                |
								|            Integer.h           |
								|                                |
								+--------------------------------+
*/

/*
 *	2008-08-02
 *
 *	added some template-based code for determining signedness and maximal value of integer types
 *
 *	2008-08-08
 *
 *	added #ifdef for windows 64
 *
 */

#ifndef __INTEGER_TYPES_INCLUDED
#define __INTEGER_TYPES_INCLUDED

#if defined(__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
	#include <inttypes.h>
#elif defined(__sun__)
	#include <inttypes.h>
	#if defined(__STDC__)
		typedef char int8_t;
		typedef unsigned char uint8_t;
		typedef short int16_t;
		typedef unsigned short uint16_t;
		typedef int int32_t;
		typedef unsigned int uint32_t;
		#if defined(__arch64__)
			typedef long int64_t;
			typedef unsigned long uint64_t;
		#else
			typedef long long int int64_t;
			typedef unsigned long long int uint64_t;
		#endif /* __arch64__ */
	#endif /* __STDC__ */
#elif defined(__VMS)
	#include <inttypes.h>
#elif defined(__SCO__) || defined(__USLC__)
	#include <stdint.h>
#elif defined(__UNIXOS2__) || defined(__SOL64__)
	typedef char int8_t;
	typedef unsigned char uint8_t;
	typedef short int16_t;
	typedef unsigned short uint16_t;
	typedef long int int32_t;
	typedef unsigned long int uint32_t;
	typedef long long int int64_t;
	typedef unsigned long long int uint64_t;
#elif defined(_WIN32) && defined(__GNUC__) // MinGW
	#include <stdint.h>
#elif defined(_WIN32) || defined (_WIN64) // MSVC
	typedef __int8 int8_t;
	typedef __int16 int16_t;
	typedef __int32 int32_t;
	typedef __int64 int64_t;
	typedef unsigned __int8 uint8_t;
	typedef unsigned __int16 uint16_t;
	typedef unsigned __int32 uint32_t;
	typedef unsigned __int64 uint64_t;
#else // fallback
	#include <inttypes.h>
#endif
// define int types

#ifndef UINT8_MAX
#define UINT8_MAX  (uint8_t(-1))
#endif
#ifndef UINT16_MAX
#define UINT16_MAX (uint16_t(-1))
#endif
#ifndef UINT32_MAX
#define UINT32_MAX (uint32_t(-1))
#endif
#ifndef UINT64_MAX
#define UINT64_MAX (uint64_t(-1))
#endif
#ifndef INT8_MAX
#define INT8_MAX  (int8_t(UINT8_MAX / 2))
#endif
#ifndef INT16_MAX
#define INT16_MAX (int16_t(UINT16_MAX / 2))
#endif
#ifndef INT32_MAX
#define INT32_MAX (int32_t(UINT32_MAX / 2))
#endif
#ifndef INT64_MAX
#define INT64_MAX (int64_t(UINT64_MAX / 2))
#endif
#ifndef INT8_MIN
#define INT8_MIN  (int8_t(-INT8_MAX - 1))
#endif
#ifndef INT16_MIN
#define INT16_MIN (int16_t(-INT16_MAX - 1))
#endif
#ifndef INT32_MIN
#define INT32_MIN (int32_t(-INT32_MAX - 1))
#endif
#ifndef INT64_MIN
#define INT64_MIN (int64_t(-INT64_MAX - 1))
#endif
// calc limits

/*
 *								=== templates for maximal value of integer types ===
 */

/*
 *	template <class _Ty>
 *	struct CIsSigned
 *		- value of CIsSigned<_Ty>::result reflect signedness of type _Ty
 */
template <class _Ty>
struct CIsSigned {
	enum { result = _Ty(-1) < 1 };
	// should be '_Ty(-1) < 0', using 1 to avoid g++ warning 'unsigned comparison < 0 is always false'
};

/*
 *	template <class _Ty>
 *	struct CIsInteger
 *		- CIsInteger<_Ty>::result is set to true uf _Ty is integer type, otherwise it's false
 */
template <class _Ty>
struct CIsInteger {
	enum { result = _Ty(.5) == 0 };
};

/*
 *	template <class _Ty>
 *	struct CMaxIntValue
 *		- template determining maximal value of _Ty, use CMaxIntValue<_Ty>::result()
 *		- note _Ty must be integer type, it won't work with float or double
 */
template <class _Ty>
struct CMaxIntValue {
	enum {
		n_bit_num = 8 * sizeof(_Ty),
		b_unsigned = (CIsSigned<_Ty>::result)? 0 : 1
	};

#define _Ty_max (((uint64_t(1) << (n_bit_num - 1)) - 1) + \
	b_unsigned * (uint64_t(1) << (n_bit_num - 1)))

#if defined(_MSC_VER) && !defined(__MWERKS__) // MSVC
	enum {
		n_result_hi = uint32_t(_Ty_max >> 32),
		n_result_lo = uint32_t(_Ty_max & 0xffffffff)
	};
	// msvc doesn't allow static const initialization so we keep it as hi-lo
	// enum values (note this relies on enum values being 32-bit integers)

	static inline uint64_t result()
	{
		_ASSERTE(CIsInteger<_Ty>::result); // only works for integers
		return (uint64_t(n_result_hi) << 32) | uint32_t(n_result_lo);
	}
#else //MSVC
	static const uint64_t n_result = _Ty_max; // g++ allows this (preferable)

	static inline uint64_t result()
	{
		_ASSERTE(CIsInteger<_Ty>::result); // only works for integers
		return n_result;
	}
#endif //MSVC
};

/*
 *	template <class _Ty>
 *	inline uint64_t n_MaxIntValue(_Ty x)
 *		- returns maximal value of type _Ty
 */
template <class _Ty>
inline uint64_t n_MaxIntValue(_Ty x)
{
	return CMaxIntValue<_Ty>::result();
}

/*
 *								=== ~templates for maximal value of integer types ===
 */

#endif //__INTEGER_TYPES_INCLUDED
