/*
								+---------------------------------+
								|                                 |
								|      ***   OpenGL2.0   ***      |
								|                                 |
								|  Copyright   -tHE SWINe- 2005  |
								|                                 |
								|            OpenGl20.h           |
								|                                 |
								+---------------------------------+
*/

/*
 *	passed code revision at 2006-05-16
 *
 *	integers, contanining just true or false were changed to bool
 *
 *	passed code revision at 2006-07-15
 *
 *	some more integers, contanining just true or false were changed to bool
 *
 *	2007-02-05
 *
 *	added support for GL_NV_fence extension
 *
 *	2007-02-20
 *
 *	added support for following new (geForce8800 (or DX10)) extensions:
 *		- GL_ARB_occlusion_query
 *		- OpenGL 2.1
 *		- GL_EXT_timer_query
 *		- GL_EXT_draw_buffers2
 *		- GL_EXT_texture_integer
 *		- GL_EXT_gpu_shader4
 *		- GL_EXT_gpu_program_parameters
 *		- GL_EXT_bindable_uniform
 *		- GL_EXT_geometry_shader4
 *		- GL_EXT_draw_instanced
 *		- GL_EXT_texture_array
 *		- GL_EXT_texture_object
 *	removed outdated GL_EXT_texture_object extension (from 1995/10/03, promoted to core
 *	extension in OpenGL 1.1)
 *	removed dllexport attribute
 *
 *	2007-10-29
 *
 *	added some documentation comments, clarified OpenGL 1.3 multitexture / ARB multitexture
 *	distinction and improved linux compatibility
 *
 *	2007-11-10
 *
 *	improved linux compatibility
 *
 *	2008-03-04
 *
 *	now using Integer.h header, hidden __stdcall into GLApi instead
 *	of defining __stdcall blank in linux
 *
 *	2008-08-08
 *
 *	added #ifdef for windows 64
 *
 *	2008-08-19
 *
 *	gl.h is now included in OpenGL20.h (convenience)
 *
 *	2009-05-03
 *
 *	added GL_FRAMEBUFFER_INCOMPLETE_DUPLICATE_ATTACHMENT_EXT enumerant
 *	(not defined in GL_EXT_packed_depth_stencil, nor in GL_EXT_framebuffer_object)
 *
 *	added CGLExtensionHandler::n_GetAllSupportedExtensionsFuncPointers()
 *
 *	2009-05-04
 *
 *	fixed mixed windows / linux line endings
 *
 */

/*
** License Applicability. Except to the extent portions of this file are
** made subject to an alternative license as permitted in the SGI Free
** Software License B, Version 1.1 (the "License"), the contents of this
** file are subject only to the provisions of the License. You may not use
** this file except in compliance with the License. You may obtain a copy
** of the License at Silicon Graphics, Inc., attn: Legal Services, 1600
** Amphitheatre Parkway, Mountain View, CA 94043-1351, or at:
** 
** http://oss.sgi.com/projects/FreeB
** 
** Note that, as provided in the License, the Software is distributed on an
** "AS IS" basis, with ALL EXPRESS AND IMPLIED WARRANTIES AND CONDITIONS
** DISCLAIMED, INCLUDING, WITHOUT LIMITATION, ANY IMPLIED WARRANTIES AND
** CONDITIONS OF MERCHANTABILITY, SATISFACTORY QUALITY, FITNESS FOR A
** PARTICULAR PURPOSE, AND NON-INFRINGEMENT.
** 
** Original Code. The Original Code is: OpenGL Sample Implementation,
** Version 1.2.1, released January 26, 2000, developed by Silicon Graphics,
** Inc. The Original Code is Copyright (c) 1991-2004 Silicon Graphics, Inc.
** Copyright in any portions created by third parties is as indicated
** elsewhere herein. All Rights Reserved.
** 
** Additional Notice Provisions: This software was created using the
** OpenGL(R) version 1.2.1 Sample Implementation published by SGI, but has
** not been independently verified as being compliant with the OpenGL(R)
** version 1.2.1 Specification.
*/

#ifndef __OPENGL20_TOOL_INCLUDED
#define __OPENGL20_TOOL_INCLUDED

/*
 *	makes life a little bit easier when including gl.h by mistake
 */
#if defined(__gl_h_) || defined(__GL_H__) || defined(GL_VERSION_1_1)
#pragma message("gl.h doesn't have to be included in files using OpenGL20.h")
#endif

/*
 *	makes life a little bit easier when including glext.h by mistake
 */
#if defined(__glext_h_) || defined(GL_GLEXT_VERSION)
#error("glext.h must NOT be included in files using OpenGL20.h")
#endif

/*
 *	NULL_FUNCTION_POINTERS
 *		- in case NULL_FUNCTION_POINTERS is defined (default), all extension function pointers
 *		  are initialized to 0, but user never needs to compare individual function values
 *		  so it's posible to save some space in executable file by skipping the initialization
 */
#define NULL_FUNCTION_POINTERS

/*
 *	class CGLExtensionHandler
 *		- class for handling OpenGL extensions
 *		- functions with name n_Get*FuncPointers() returns number of function pointers that were
 *		  not found (zero is success)
 */
class CGLExtensionHandler {
public:
	/*
	 *	static bool CGLExtensionHandler::b_Support_OpenGL(int n_min_major = 2, int n_min_minor = 0)
	 *		- returns true in case OpenGL version  is greater or equal to n_min_major.n_min_minor
	 *		- might return false on malformed OpenGL version string
	 */
	static bool b_Support_OpenGL(int n_min_major = 2, int n_min_minor = 0);

	/*
	 *	static int CGLExtensionHandler::n_OpenGL_Version()
	 *		- returns OpenGL version as major * 10 + minor (ie. for OpenGL 2.1 returns 21)
	 */
	static int n_OpenGL_Version();

	/*
	 *	static bool CGLExtensionHandler::b_SupportedExtension(const char *p_s_extension_name)
	 *		- returns true in case p_s_extension_name is supported, otherwise false
	 *		- requires space in extension string before and after the extension name
	 *		  (except on beginning / end of the string)
	 */
	static bool b_SupportedExtension(const char *p_s_extension_name);

#if defined(WIN32) || defined (_WIN64)
	/*
	 *	static bool CGLExtensionHandler::b_SupportedWGLExtension(const char *p_s_extension_name)
	 *		- returns true in case WGL extension p_s_extension_name is supported, otherwise false
	 *		- requires space in extension string before and after the extension name
	 *		  (except on beginning / end of the string)
	 */
	static bool b_SupportedWGLExtension(const char *p_s_extension_name);
#endif

	/*
	 *	static int CGLExtensionHandler::n_GetAllSupportedExtensionsFuncPointers()
	 *		- gets function pointers for all supported extensions and OpenGL versions
	 *		- returns number of missing functions
	 *		- note this is aimed at simple and test implementations without
	 *		  much need for error checking. it is always better to call individual
	 *		  functions, because in case this fails, there's no telling
	 *		  which extension exactly failed to initialize
	 */
	static int n_GetAllSupportedExtensionsFuncPointers();

	/*
	 *	static int CGLExtensionHandler::n_GetGL12FuncPointers()
	 *		- gets OpenGL 1.2 core function pointers
	 *		- returns number of missing functions
	 */
	static int n_GetGL12FuncPointers();

	/*
	 *	static int CGLExtensionHandler::n_GetGL13FuncPointers()
	 *		- gets OpenGL 1.3 core function pointers (including multitexture function pointers)
	 *		- returns number of missing functions
	 */
	static int n_GetGL13FuncPointers();

	/*
	 *	static int CGLExtensionHandler::n_GetMultitextureFuncPointers()
	 *		- gets OpenGL 1.3 multitexture function pointers
	 *		- returns number of missing functions
	 */
	static int n_GetMultitextureFuncPointers();

	/*
	 *	static int CGLExtensionHandler::n_GetGL14FuncPointers()
	 *		- gets OpenGL 1.4 core function pointers
	 *		- returns number of missing functions
	 */
	static int n_GetGL14FuncPointers();

	/*
	 *	static int CGLExtensionHandler::n_GetGL15FuncPointers()
	 *		- gets OpenGL 1.5 core function pointers
	 *		- returns number of missing functions
	 */
	static int n_GetGL15FuncPointers();

	/*
	 *	static int CGLExtensionHandler::n_GetGL20FuncPointers()
	 *		- gets OpenGL 2.0 core function pointers
	 *		- returns number of missing functions
	 */
	static int n_GetGL20FuncPointers();

	/*
	 *	static int CGLExtensionHandler::n_GetGL21FuncPointers()
	 *		- gets OpenGL 2.1 core function pointers
	 *		- returns number of missing functions
	 */
	static int n_GetGL21FuncPointers();

	/*
	 *	static int CGLExtensionHandler::n_GetMultitextureARBFuncPointers()
	 *		- gets GL_ARB_multitexture function pointers
	 *		- note function names are ones of OpenGL 1.3 multitexture, no ARB suffix here
	 *		- returns number of missing functions
	 */
	static int n_GetMultitextureARBFuncPointers();

	/*
	 *	static int CGLExtensionHandler::n_GetCompressedTextureARBFuncPointers()
	 *		- gets GL_ARB_texture_compression function pointers
	 *		- returns number of missing functions
	 */
	static int n_GetCompressedTextureARBFuncPointers();

	/*
	 *	static int CGLExtensionHandler::n_GetVertexBlendARBFuncPointers()
	 *		- gets GL_ARB_vertex_blend function pointers
	 *		- returns number of missing functions
	 */
	static int n_GetVertexBlendARBFuncPointers();

	/*
	 *	static int CGLExtensionHandler::n_GetMatrixPaletteARBFuncPointers()
	 *		- gets GL_ARB_matrix_palette function pointers
	 *		- returns number of missing functions
	 */
	static int n_GetMatrixPaletteARBFuncPointers();

	/*
	 *	static int CGLExtensionHandler::n_GetFramebufferObjectEXTFuncPointers()
	 *		- gets GL_EXT_framebuffer_object function pointers
	 *		- returns number of missing functions
	 */
	static int n_GetFramebufferObjectEXTFuncPointers();

	/*
	 *	static int CGLExtensionHandler::n_GetCompiledArrayEXTFuncPointers()
	 *		- gets GL_EXT_compiled_vertex_array function pointers
	 *		- returns number of missing functions
	 */
	static int n_GetCompiledArrayEXTFuncPointers();

	/*
	 *	static int CGLExtensionHandler::n_GetTextureObjectEXTFuncPointers()
	 *		- gets GL_EXT_texture_object function pointers
	 *		- outdated 20th EXT from 1995/10/03; deprecated
	 *		- returns number of missing functions
	 */
	//static int n_GetTextureObjectEXTFuncPointers();

	/*
	 *	static int CGLExtensionHandler::n_GetPolygonOffsetEXTFuncPointers()
	 *		- gets GL_EXT_polygon_offset function pointers
	 *		- outdated, present in OpenGL 1.1 which is required to cimpile this
	 *		- returns number of missing functions
	 */
	//static int n_GetPolygonOffsetEXTFuncPointers();

	/*
	 *	static int CGLExtensionHandler::n_GetColorBufferFloatARBFuncPointers()
	 *		- gets GL_ARB_color_buffer_float function pointers
	 *		- returns number of missing functions
	 */
	static int n_GetColorBufferFloatARBFuncPointers();

	/*
	 *	static int CGLExtensionHandler::n_GetVertexArrayEXTFuncPointers()
	 *		- gets GL_EXT_vertex_array function pointers
	 *		- outdated, present in OpenGL 1.1 which is required to cimpile this
	 *		- returns number of missing functions
	 */
	//static int n_GetVertexArrayEXTFuncPointers();

	/*
	 *	static int CGLExtensionHandler::n_GetVertexProgramARBFuncPointers()
	 *		- gets GL_ARB_vertex_program function pointers
	 *		- returns number of missing functions
	 */
	static int n_GetVertexProgramARBFuncPointers();

	/*
	 *	static int CGLExtensionHandler::n_GetFragmentProgramARBFuncPointers()
	 *		- gets GL_ARB_fragment_program function pointers
	 *		- returns number of missing functions
	 */
	static int n_GetFragmentProgramARBFuncPointers();

	/*
	 *	static int CGLExtensionHandler::n_GetVertexShaderARBFuncPointers()
	 *		- gets GL_ARB_vertex_shader function pointers
	 *		- returns number of missing functions
	 */
	static int n_GetVertexShaderARBFuncPointers();

	/*
	 *	static int CGLExtensionHandler::n_GetFragmentShaderARBFuncPointers()
	 *		- gets GL_ARB_fragment_shader function pointers
	 *		- returns number of missing functions
	 */
	static int n_GetFragmentShaderARBFuncPointers();

	/*
	 *	static int CGLExtensionHandler::n_GetShaderObjectARBFuncPointers()
	 *		- gets GL_ARB_shader_object function pointers
	 *		- returns number of missing functions
	 */
	static int n_GetShaderObjectARBFuncPointers();

	/*
	 *	static int CGLExtensionHandler::n_GetVertexBufferObjectARBFuncPointers()
	 *		- gets GL_ARB_vertex_buffer_object function pointers
	 *		- returns number of missing functions
	 */
	static int n_GetVertexBufferObjectARBFuncPointers();

	/*
	 *	static int CGLExtensionHandler::n_GetFenceNVFuncPointers()
	 *		- gets GL_NV_fence function pointers
	 *		- returns number of missing functions
	 */
	static int n_GetFenceNVFuncPointers();

	/*
	 *	static int CGLExtensionHandler::n_GetOcclusionQueryARBFuncPointers()
	 *		- gets GL_ARB_occlusion_query function pointers
	 *		- returns number of missing functions
	 */
	static int n_GetOcclusionQueryARBFuncPointers();

	/*
	 *	static int CGLExtensionHandler::n_GetTimerQueryEXTFuncPointers()
	 *		- gets GL_EXT_timer_query function pointers
	 *		- returns number of missing functions
	 */
	static int n_GetTimerQueryEXTFuncPointers();

	/*
	 *	static int CGLExtensionHandler::n_GetDrawBuffers2EXTFuncPointers()
	 *		- gets GL_EXT_draw_buffers2 function pointers
	 *		- returns number of missing functions
	 */
	static int n_GetDrawBuffers2EXTFuncPointers();

	/*
	 *	static int CGLExtensionHandler::n_GetTextureIntegerEXTFuncPointers()
	 *		- gets GL_EXT_texture_integer function pointers
	 *		- returns number of missing functions
	 */
	static int n_GetTextureIntegerEXTFuncPointers();

	/*
	 *	static int CGLExtensionHandler::n_GetGPUShader4EXTFuncPointers()
	 *		- gets GL_EXT_gpu_shader4 function pointers
	 *		- returns number of missing functions
	 */
	static int n_GetGPUShader4EXTFuncPointers();

	/*
	 *	static int CGLExtensionHandler::n_GetGPUProgramParametersEXTFuncPointers()
	 *		- gets GL_EXT_gpu_program_parameters function pointers
	 *		- returns number of missing functions
	 */
	static int n_GetGPUProgramParametersEXTFuncPointers();

	/*
	 *	static int CGLExtensionHandler::n_GetBindableUniformEXTFuncPointers()
	 *		- gets GL_EXT_bindable_uniform function pointers
	 *		- returns number of missing functions
	 */
	static int n_GetBindableUniformEXTFuncPointers();

	/*
	 *	static int CGLExtensionHandler::n_GetGeometryShader4EXTFuncPointers()
	 *		- gets GL_EXT_geometry_shader4 function pointers
	 *		- returns number of missing functions
	 */
	static int n_GetGeometryShader4EXTFuncPointers();

	/*
	 *	static int CGLExtensionHandler::n_GetDrawInstancedEXTFuncPointers()
	 *		- gets GL_EXT_draw_instanced function pointers
	 *		- returns number of missing functions
	 */
	static int n_GetDrawInstancedEXTFuncPointers();

	/*
	 *	static int CGLExtensionHandler::n_GetTextureArrayEXTFuncPointers()
	 *		- gets GL_EXT_texture_array function pointers
	 *		- note this extension overlaps with GL_EXT_geometry_shader4
	 *		- returns number of missing functions
	 */
	static int n_GetTextureArrayEXTFuncPointers(); 

	/*
	 *	static int CGLExtensionHandler::n_GetTextureObjectEXTFuncPointers()
	 *		- gets GL_EXT_texture_object function pointers
	 *		- returns number of missing functions
	 */
	static int n_GetTextureObjectEXTFuncPointers();

	/*
	 *	static int CGLExtensionHandler::n_GetBlendMinMaxEXTFuncPointers()
	 *		- gets GL_EXT_blend_minmax function pointers
	 *		- returns number of missing functions
	 */
	static int n_GetBlendMinMaxEXTFuncPointers();
};

enum {
	num_functions_gl12 = 38,
	num_functions_multitexture = 34,
	num_functions_gl13 = 12 + num_functions_multitexture,
	num_functions_arb_multitexture = num_functions_multitexture,
	num_functions_gl14 = 45,
	num_functions_gl15 = 19,
	num_functions_gl20 = 93,
	num_functions_compressed_texture_arb = 7,
	num_functions_vertex_blend_arb = 10,
	num_functions_matrix_palette_arb = 5,
	num_functions_framebuffer_object_ext = 17,
	num_functions_compiled_vertex_array_ext = 2,
	//num_functions_texture_object_ext = 6, (20. EXT from 1995/10/03; outdated)
	num_functions_polygon_offset_ext = 1,
	num_functions_color_buffer_float_arb = 1,
	num_functions_vertex_array_ext = 9,
	num_functions_vertex_program_arb = 62,
	num_functions_fragment_program_arb = 19,
	num_functions_vertex_shader_arb = 46,
	num_functions_fragment_shader_arb = 0,
	num_functions_shader_object_arb = 39,
	num_functions_vertex_buffer_object_arb = 11,
	num_functions_fence_nv = 7,
	num_functions_occlusion_query_ext = 8,
	num_functions_gl21 = 6,
	num_functions_timer_query_ext = 2,
	num_functions_draw_buffers2_ext = 6,
	num_functions_texture_integer_ext = 6,
	num_functions_gpu_shader4_ext = 34,
	num_functions_gpu_program_parameters_ext = 2,
	num_functions_bindable_uniform_ext = 3,
	num_functions_geometry_shader4_ext = 4,
	num_functions_draw_instanced_ext = 2,
	num_functions_texture_array_ext = 1,
	num_functions_texture_object_ext = 1,
	num_functions_blend_minmax_ext = 1
};

#if defined(WIN32) || defined (_WIN64)
#define GLApi __stdcall
#else
#define GLApi 
#endif // WIN32, _WIN64

#if defined(WIN32) || defined (_WIN64)
#include <windows.h> // gl.h needs some typedefs
#endif
#define GL_GLEXT_LEGACY
#include <GL/gl.h>
// include gl.h

#include <stddef.h>
#ifndef GL_VERSION_2_0
/* GL type for program/shader text */
typedef char GLchar;			/* native character */
#endif

#ifndef GL_VERSION_1_5
/* GL types for handling large vertex buffer objects */
typedef ptrdiff_t GLintptr;
typedef ptrdiff_t GLsizeiptr;
#endif

#ifndef GL_ARB_vertex_buffer_object
/* GL types for handling large vertex buffer objects */
typedef ptrdiff_t GLintptrARB;
typedef ptrdiff_t GLsizeiptrARB;
#endif

#ifndef GL_ARB_shader_objects
/* GL types for handling shader object handles and program/shader text */
typedef char GLcharARB;		/* native character */
typedef unsigned int GLhandleARB;	/* shader object handle */
#endif

/* GL types for "half" precision (s10e5) float data in host memory */
#ifndef GL_ARB_half_float_pixel
typedef unsigned short GLhalfARB;
#endif

#ifndef GL_NV_half_float
typedef unsigned short GLhalfNV;
#endif

#ifndef GLEXT_64_TYPES_DEFINED
/* This code block is duplicated in glext.h, so must be protected */
#define GLEXT_64_TYPES_DEFINED
#include "Integer.h"
#endif

#ifndef GL_EXT_timer_query
typedef int64_t GLint64EXT;
typedef uint64_t GLuint64EXT;
#endif

//							    --- opengl 1.2 core functions ---

#ifndef GL_VERSION_1_2
#define GL_VERSION_1_2 1
#define __GENERATE_GL_12__

#define GL_UNSIGNED_BYTE_3_3_2            0x8032
#define GL_UNSIGNED_SHORT_4_4_4_4         0x8033
#define GL_UNSIGNED_SHORT_5_5_5_1         0x8034
#define GL_UNSIGNED_INT_8_8_8_8           0x8035
#define GL_UNSIGNED_INT_10_10_10_2        0x8036
#define GL_RESCALE_NORMAL                 0x803A
#define GL_TEXTURE_BINDING_3D             0x806A
#define GL_PACK_SKIP_IMAGES               0x806B
#define GL_PACK_IMAGE_HEIGHT              0x806C
#define GL_UNPACK_SKIP_IMAGES             0x806D
#define GL_UNPACK_IMAGE_HEIGHT            0x806E
#define GL_TEXTURE_3D                     0x806F
#define GL_PROXY_TEXTURE_3D               0x8070
#define GL_TEXTURE_DEPTH                  0x8071
#define GL_TEXTURE_WRAP_R                 0x8072
#define GL_MAX_3D_TEXTURE_SIZE            0x8073
#define GL_UNSIGNED_BYTE_2_3_3_REV        0x8362
#define GL_UNSIGNED_SHORT_5_6_5           0x8363
#define GL_UNSIGNED_SHORT_5_6_5_REV       0x8364
#define GL_UNSIGNED_SHORT_4_4_4_4_REV     0x8365
#define GL_UNSIGNED_SHORT_1_5_5_5_REV     0x8366
#define GL_UNSIGNED_INT_8_8_8_8_REV       0x8367
#define GL_UNSIGNED_INT_2_10_10_10_REV    0x8368
#define GL_BGR                            0x80E0
#define GL_BGRA                           0x80E1
#define GL_MAX_ELEMENTS_VERTICES          0x80E8
#define GL_MAX_ELEMENTS_INDICES           0x80E9
#define GL_CLAMP_TO_EDGE                  0x812F
#define GL_TEXTURE_MIN_LOD                0x813A
#define GL_TEXTURE_MAX_LOD                0x813B
#define GL_TEXTURE_BASE_LEVEL             0x813C
#define GL_TEXTURE_MAX_LEVEL              0x813D
#define GL_LIGHT_MODEL_COLOR_CONTROL      0x81F8
#define GL_SINGLE_COLOR                   0x81F9
#define GL_SEPARATE_SPECULAR_COLOR        0x81FA
#define GL_SMOOTH_POINT_SIZE_RANGE        0x0B12
#define GL_SMOOTH_POINT_SIZE_GRANULARITY  0x0B13
#define GL_SMOOTH_LINE_WIDTH_RANGE        0x0B22
#define GL_SMOOTH_LINE_WIDTH_GRANULARITY  0x0B23
#define GL_ALIASED_POINT_SIZE_RANGE       0x846D
#define GL_ALIASED_LINE_WIDTH_RANGE       0x846E

extern void (GLApi *glBlendColor)(GLclampf, GLclampf, GLclampf, GLclampf);
extern void (GLApi *glBlendEquation)(GLenum);
extern void (GLApi *glDrawRangeElements)(GLenum, GLuint, GLuint, GLsizei, GLenum, const GLvoid*);
extern void (GLApi *glColorTable)(GLenum, GLenum, GLsizei, GLenum, GLenum, const GLvoid*);
extern void (GLApi *glColorTableParameterfv)(GLenum, GLenum, const GLfloat*);
extern void (GLApi *glColorTableParameteriv)(GLenum, GLenum, const GLint*);
extern void (GLApi *glCopyColorTable)(GLenum, GLenum, GLint, GLint, GLsizei);
extern void (GLApi *glGetColorTable)(GLenum, GLenum, GLenum, GLvoid*);
extern void (GLApi *glGetColorTableParameterfv)(GLenum, GLenum, GLfloat*);
extern void (GLApi *glGetColorTableParameteriv)(GLenum, GLenum, GLint*);
extern void (GLApi *glColorSubTable)(GLenum, GLsizei, GLsizei, GLenum, GLenum, const GLvoid*);
extern void (GLApi *glCopyColorSubTable)(GLenum, GLsizei, GLint, GLint, GLsizei);
extern void (GLApi *glConvolutionFilter1D)(GLenum, GLenum, GLsizei, GLenum, GLenum, const GLvoid*);
extern void (GLApi *glConvolutionFilter2D)(GLenum, GLenum, GLsizei, GLsizei, GLenum, GLenum, const GLvoid*);
extern void (GLApi *glConvolutionParameterf)(GLenum, GLenum, GLfloat);
extern void (GLApi *glConvolutionParameterfv)(GLenum, GLenum, const GLfloat*);
extern void (GLApi *glConvolutionParameteri)(GLenum, GLenum, GLint);
extern void (GLApi *glConvolutionParameteriv)(GLenum, GLenum, const GLint*);
extern void (GLApi *glCopyConvolutionFilter1D)(GLenum, GLenum, GLint, GLint, GLsizei);
extern void (GLApi *glCopyConvolutionFilter2D)(GLenum, GLenum, GLint, GLint, GLsizei, GLsizei);
extern void (GLApi *glGetConvolutionFilter)(GLenum, GLenum, GLenum, GLvoid*);
extern void (GLApi *glGetConvolutionParameterfv)(GLenum, GLenum, GLfloat*);
extern void (GLApi *glGetConvolutionParameteriv)(GLenum, GLenum, GLint*);
extern void (GLApi *glGetSeparableFilter)(GLenum, GLenum, GLenum, GLvoid *, GLvoid *, GLvoid*);
extern void (GLApi *glSeparableFilter2D)(GLenum, GLenum, GLsizei, GLsizei, GLenum, GLenum, const GLvoid *, const GLvoid*);
extern void (GLApi *glGetHistogram)(GLenum, GLboolean, GLenum, GLenum, GLvoid*);
extern void (GLApi *glGetHistogramParameterfv)(GLenum, GLenum, GLfloat*);
extern void (GLApi *glGetHistogramParameteriv)(GLenum, GLenum, GLint*);
extern void (GLApi *glGetMinmax)(GLenum, GLboolean, GLenum, GLenum, GLvoid*);
extern void (GLApi *glGetMinmaxParameterfv)(GLenum, GLenum, GLfloat*);
extern void (GLApi *glGetMinmaxParameteriv)(GLenum, GLenum, GLint*);
extern void (GLApi *glHistogram)(GLenum, GLsizei, GLenum, GLboolean);
extern void (GLApi *glMinmax)(GLenum, GLenum, GLboolean);
extern void (GLApi *glResetHistogram)(GLenum);
extern void (GLApi *glResetMinmax)(GLenum);
extern void (GLApi *glTexImage3D)(GLenum, GLint, GLint, GLsizei, GLsizei, GLsizei, GLint, GLenum, GLenum, const GLvoid*);
extern void (GLApi *glTexSubImage3D)(GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLenum, const GLvoid*);
extern void (GLApi *glCopyTexSubImage3D)(GLenum, GLint, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei);

#endif //GL_VERSION_1_2

//							    --- opengl 1.3 core functions ---

#ifndef GL_VERSION_1_3
#define GL_VERSION_1_3 1
#define __GENERATE_GL_13__

#define GL_TEXTURE0                       0x84C0
#define GL_TEXTURE1                       0x84C1
#define GL_TEXTURE2                       0x84C2
#define GL_TEXTURE3                       0x84C3
#define GL_TEXTURE4                       0x84C4
#define GL_TEXTURE5                       0x84C5
#define GL_TEXTURE6                       0x84C6
#define GL_TEXTURE7                       0x84C7
#define GL_TEXTURE8                       0x84C8
#define GL_TEXTURE9                       0x84C9
#define GL_TEXTURE10                      0x84CA
#define GL_TEXTURE11                      0x84CB
#define GL_TEXTURE12                      0x84CC
#define GL_TEXTURE13                      0x84CD
#define GL_TEXTURE14                      0x84CE
#define GL_TEXTURE15                      0x84CF
#define GL_TEXTURE16                      0x84D0
#define GL_TEXTURE17                      0x84D1
#define GL_TEXTURE18                      0x84D2
#define GL_TEXTURE19                      0x84D3
#define GL_TEXTURE20                      0x84D4
#define GL_TEXTURE21                      0x84D5
#define GL_TEXTURE22                      0x84D6
#define GL_TEXTURE23                      0x84D7
#define GL_TEXTURE24                      0x84D8
#define GL_TEXTURE25                      0x84D9
#define GL_TEXTURE26                      0x84DA
#define GL_TEXTURE27                      0x84DB
#define GL_TEXTURE28                      0x84DC
#define GL_TEXTURE29                      0x84DD
#define GL_TEXTURE30                      0x84DE
#define GL_TEXTURE31                      0x84DF
#define GL_ACTIVE_TEXTURE                 0x84E0
#define GL_CLIENT_ACTIVE_TEXTURE          0x84E1
#define GL_MAX_TEXTURE_UNITS              0x84E2
#define GL_TRANSPOSE_MODELVIEW_MATRIX     0x84E3
#define GL_TRANSPOSE_PROJECTION_MATRIX    0x84E4
#define GL_TRANSPOSE_TEXTURE_MATRIX       0x84E5
#define GL_TRANSPOSE_COLOR_MATRIX         0x84E6
#define GL_MULTISAMPLE                    0x809D
#define GL_SAMPLE_ALPHA_TO_COVERAGE       0x809E
#define GL_SAMPLE_ALPHA_TO_ONE            0x809F
#define GL_SAMPLE_COVERAGE                0x80A0
#define GL_SAMPLE_BUFFERS                 0x80A8
#define GL_SAMPLES                        0x80A9
#define GL_SAMPLE_COVERAGE_VALUE          0x80AA
#define GL_SAMPLE_COVERAGE_INVERT         0x80AB
#define GL_MULTISAMPLE_BIT                0x20000000
#define GL_NORMAL_MAP                     0x8511
#define GL_REFLECTION_MAP                 0x8512
#define GL_TEXTURE_CUBE_MAP               0x8513
#define GL_TEXTURE_BINDING_CUBE_MAP       0x8514
#define GL_TEXTURE_CUBE_MAP_POSITIVE_X    0x8515
#define GL_TEXTURE_CUBE_MAP_NEGATIVE_X    0x8516
#define GL_TEXTURE_CUBE_MAP_POSITIVE_Y    0x8517
#define GL_TEXTURE_CUBE_MAP_NEGATIVE_Y    0x8518
#define GL_TEXTURE_CUBE_MAP_POSITIVE_Z    0x8519
#define GL_TEXTURE_CUBE_MAP_NEGATIVE_Z    0x851A
#define GL_PROXY_TEXTURE_CUBE_MAP         0x851B
#define GL_MAX_CUBE_MAP_TEXTURE_SIZE      0x851C
#define GL_COMPRESSED_ALPHA               0x84E9
#define GL_COMPRESSED_LUMINANCE           0x84EA
#define GL_COMPRESSED_LUMINANCE_ALPHA     0x84EB
#define GL_COMPRESSED_INTENSITY           0x84EC
#define GL_COMPRESSED_RGB                 0x84ED
#define GL_COMPRESSED_RGBA                0x84EE
#define GL_TEXTURE_COMPRESSION_HINT       0x84EF
#define GL_TEXTURE_COMPRESSED_IMAGE_SIZE  0x86A0
#define GL_TEXTURE_COMPRESSED             0x86A1
#define GL_NUM_COMPRESSED_TEXTURE_FORMATS 0x86A2
#define GL_COMPRESSED_TEXTURE_FORMATS     0x86A3
#define GL_CLAMP_TO_BORDER                0x812D
#define GL_COMBINE                        0x8570
#define GL_COMBINE_RGB                    0x8571
#define GL_COMBINE_ALPHA                  0x8572
#define GL_SOURCE0_RGB                    0x8580
#define GL_SOURCE1_RGB                    0x8581
#define GL_SOURCE2_RGB                    0x8582
#define GL_SOURCE0_ALPHA                  0x8588
#define GL_SOURCE1_ALPHA                  0x8589
#define GL_SOURCE2_ALPHA                  0x858A
#define GL_OPERAND0_RGB                   0x8590
#define GL_OPERAND1_RGB                   0x8591
#define GL_OPERAND2_RGB                   0x8592
#define GL_OPERAND0_ALPHA                 0x8598
#define GL_OPERAND1_ALPHA                 0x8599
#define GL_OPERAND2_ALPHA                 0x859A
#define GL_RGB_SCALE                      0x8573
#define GL_ADD_SIGNED                     0x8574
#define GL_INTERPOLATE                    0x8575
#define GL_SUBTRACT                       0x84E7
#define GL_CONSTANT                       0x8576
#define GL_PRIMARY_COLOR                  0x8577
#define GL_PREVIOUS                       0x8578
#define GL_DOT3_RGB                       0x86AE
#define GL_DOT3_RGBA                      0x86AF

extern void (GLApi *glLoadTransposeMatrixf)(const GLfloat*);
extern void (GLApi *glLoadTransposeMatrixd)(const GLdouble*);
extern void (GLApi *glMultTransposeMatrixf)(const GLfloat*);
extern void (GLApi *glMultTransposeMatrixd)(const GLdouble*);
extern void (GLApi *glSampleCoverage)(GLclampf, GLboolean);
extern void (GLApi *glCompressedTexImage3D)(GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLsizei, const GLvoid*);
extern void (GLApi *glCompressedTexImage2D)(GLenum, GLint, GLenum, GLsizei, GLsizei, GLint, GLsizei, const GLvoid*);
extern void (GLApi *glCompressedTexImage1D)(GLenum, GLint, GLenum, GLsizei, GLint, GLsizei, const GLvoid*);
extern void (GLApi *glCompressedTexSubImage3D)(GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLsizei, const GLvoid*);
extern void (GLApi *glCompressedTexSubImage2D)(GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLsizei, const GLvoid*);
extern void (GLApi *glCompressedTexSubImage1D)(GLenum, GLint, GLint, GLsizei, GLenum, GLsizei, const GLvoid*);
extern void (GLApi *glGetCompressedTexImage)(GLenum, GLint, GLvoid*);

//								  --- 1.3 multitexture functions ---

extern void (GLApi *glActiveTexture)(GLenum);
extern void (GLApi *glClientActiveTexture)(GLenum);
extern void (GLApi *glMultiTexCoord1d)(GLenum, GLdouble);
extern void (GLApi *glMultiTexCoord1dv)(GLenum, const GLdouble*);
extern void (GLApi *glMultiTexCoord1f)(GLenum, GLfloat);
extern void (GLApi *glMultiTexCoord1fv)(GLenum, const GLfloat*);
extern void (GLApi *glMultiTexCoord1i)(GLenum, GLint);
extern void (GLApi *glMultiTexCoord1iv)(GLenum, const GLint*);
extern void (GLApi *glMultiTexCoord1s)(GLenum, GLshort);
extern void (GLApi *glMultiTexCoord1sv)(GLenum, const GLshort*);
extern void (GLApi *glMultiTexCoord2d)(GLenum, GLdouble, GLdouble);
extern void (GLApi *glMultiTexCoord2dv)(GLenum, const GLdouble*);
extern void (GLApi *glMultiTexCoord2f)(GLenum, GLfloat, GLfloat);
extern void (GLApi *glMultiTexCoord2fv)(GLenum, const GLfloat*);
extern void (GLApi *glMultiTexCoord2i)(GLenum, GLint, GLint);
extern void (GLApi *glMultiTexCoord2iv)(GLenum, const GLint*);
extern void (GLApi *glMultiTexCoord2s)(GLenum, GLshort, GLshort);
extern void (GLApi *glMultiTexCoord2sv)(GLenum, const GLshort*);
extern void (GLApi *glMultiTexCoord3d)(GLenum, GLdouble, GLdouble, GLdouble);
extern void (GLApi *glMultiTexCoord3dv)(GLenum, const GLdouble*);
extern void (GLApi *glMultiTexCoord3f)(GLenum, GLfloat, GLfloat, GLfloat);
extern void (GLApi *glMultiTexCoord3fv)(GLenum, const GLfloat*);
extern void (GLApi *glMultiTexCoord3i)(GLenum, GLint, GLint, GLint);
extern void (GLApi *glMultiTexCoord3iv)(GLenum, const GLint*);
extern void (GLApi *glMultiTexCoord3s)(GLenum, GLshort, GLshort, GLshort);
extern void (GLApi *glMultiTexCoord3sv)(GLenum, const GLshort*);
extern void (GLApi *glMultiTexCoord4d)(GLenum, GLdouble, GLdouble, GLdouble, GLdouble);
extern void (GLApi *glMultiTexCoord4dv)(GLenum, const GLdouble*);
extern void (GLApi *glMultiTexCoord4f)(GLenum, GLfloat, GLfloat, GLfloat, GLfloat);
extern void (GLApi *glMultiTexCoord4fv)(GLenum, const GLfloat*);
extern void (GLApi *glMultiTexCoord4i)(GLenum, GLint, GLint, GLint, GLint);
extern void (GLApi *glMultiTexCoord4iv)(GLenum, const GLint*);
extern void (GLApi *glMultiTexCoord4s)(GLenum, GLshort, GLshort, GLshort, GLshort);
extern void (GLApi *glMultiTexCoord4sv)(GLenum, const GLshort*);

#endif //GL_VERSION_1_3

//							    --- opengl 1.4 core functions ---

#ifndef GL_VERSION_1_4
#define GL_VERSION_1_4 1
#define __GENERATE_GL_14__

#define GL_BLEND_DST_RGB                  0x80C8
#define GL_BLEND_SRC_RGB                  0x80C9
#define GL_BLEND_DST_ALPHA                0x80CA
#define GL_BLEND_SRC_ALPHA                0x80CB
#define GL_POINT_SIZE_MIN                 0x8126
#define GL_POINT_SIZE_MAX                 0x8127
#define GL_POINT_FADE_THRESHOLD_SIZE      0x8128
#define GL_POINT_DISTANCE_ATTENUATION     0x8129
#define GL_GENERATE_MIPMAP                0x8191
#define GL_GENERATE_MIPMAP_HINT           0x8192
#define GL_DEPTH_COMPONENT16              0x81A5
#define GL_DEPTH_COMPONENT24              0x81A6
#define GL_DEPTH_COMPONENT32              0x81A7
#define GL_MIRRORED_REPEAT                0x8370
#define GL_FOG_COORDINATE_SOURCE          0x8450
#define GL_FOG_COORDINATE                 0x8451
#define GL_FRAGMENT_DEPTH                 0x8452
#define GL_CURRENT_FOG_COORDINATE         0x8453
#define GL_FOG_COORDINATE_ARRAY_TYPE      0x8454
#define GL_FOG_COORDINATE_ARRAY_STRIDE    0x8455
#define GL_FOG_COORDINATE_ARRAY_POINTER   0x8456
#define GL_FOG_COORDINATE_ARRAY           0x8457
#define GL_COLOR_SUM                      0x8458
#define GL_CURRENT_SECONDARY_COLOR        0x8459
#define GL_SECONDARY_COLOR_ARRAY_SIZE     0x845A
#define GL_SECONDARY_COLOR_ARRAY_TYPE     0x845B
#define GL_SECONDARY_COLOR_ARRAY_STRIDE   0x845C
#define GL_SECONDARY_COLOR_ARRAY_POINTER  0x845D
#define GL_SECONDARY_COLOR_ARRAY          0x845E
#define GL_MAX_TEXTURE_LOD_BIAS           0x84FD
#define GL_TEXTURE_FILTER_CONTROL         0x8500
#define GL_TEXTURE_LOD_BIAS               0x8501
#define GL_INCR_WRAP                      0x8507
#define GL_DECR_WRAP                      0x8508
#define GL_TEXTURE_DEPTH_SIZE             0x884A
#define GL_DEPTH_TEXTURE_MODE             0x884B
#define GL_TEXTURE_COMPARE_MODE           0x884C
#define GL_TEXTURE_COMPARE_FUNC           0x884D
#define GL_COMPARE_R_TO_TEXTURE           0x884E

extern void (GLApi *glBlendFuncSeparate)(GLenum, GLenum, GLenum, GLenum);
extern void (GLApi *glFogCoordf)(GLfloat);
extern void (GLApi *glFogCoordfv)(const GLfloat*);
extern void (GLApi *glFogCoordd)(GLdouble);
extern void (GLApi *glFogCoorddv)(const GLdouble*);
extern void (GLApi *glFogCoordPointer)(GLenum, GLsizei, const GLvoid*);
extern void (GLApi *glMultiDrawArrays)(GLenum, GLint *, GLsizei *, GLsizei);
extern void (GLApi *glMultiDrawElements)(GLenum, const GLsizei *, GLenum, const GLvoid* *, GLsizei);
extern void (GLApi *glPointParameterf)(GLenum, GLfloat);
extern void (GLApi *glPointParameterfv)(GLenum, const GLfloat*);
extern void (GLApi *glPointParameteri)(GLenum, GLint);
extern void (GLApi *glPointParameteriv)(GLenum, const GLint*);
extern void (GLApi *glSecondaryColor3b)(GLbyte, GLbyte, GLbyte);
extern void (GLApi *glSecondaryColor3bv)(const GLbyte*);
extern void (GLApi *glSecondaryColor3d)(GLdouble, GLdouble, GLdouble);
extern void (GLApi *glSecondaryColor3dv)(const GLdouble*);
extern void (GLApi *glSecondaryColor3f)(GLfloat, GLfloat, GLfloat);
extern void (GLApi *glSecondaryColor3fv)(const GLfloat*);
extern void (GLApi *glSecondaryColor3i)(GLint, GLint, GLint);
extern void (GLApi *glSecondaryColor3iv)(const GLint*);
extern void (GLApi *glSecondaryColor3s)(GLshort, GLshort, GLshort);
extern void (GLApi *glSecondaryColor3sv)(const GLshort*);
extern void (GLApi *glSecondaryColor3ub)(GLubyte, GLubyte, GLubyte);
extern void (GLApi *glSecondaryColor3ubv)(const GLubyte*);
extern void (GLApi *glSecondaryColor3ui)(GLuint, GLuint, GLuint);
extern void (GLApi *glSecondaryColor3uiv)(const GLuint*);
extern void (GLApi *glSecondaryColor3us)(GLushort, GLushort, GLushort);
extern void (GLApi *glSecondaryColor3usv)(const GLushort*);
extern void (GLApi *glSecondaryColorPointer)(GLint, GLenum, GLsizei, const GLvoid*);
extern void (GLApi *glWindowPos2d)(GLdouble, GLdouble);
extern void (GLApi *glWindowPos2dv)(const GLdouble*);
extern void (GLApi *glWindowPos2f)(GLfloat, GLfloat);
extern void (GLApi *glWindowPos2fv)(const GLfloat*);
extern void (GLApi *glWindowPos2i)(GLint, GLint);
extern void (GLApi *glWindowPos2iv)(const GLint*);
extern void (GLApi *glWindowPos2s)(GLshort, GLshort);
extern void (GLApi *glWindowPos2sv)(const GLshort*);
extern void (GLApi *glWindowPos3d)(GLdouble, GLdouble, GLdouble);
extern void (GLApi *glWindowPos3dv)(const GLdouble*);
extern void (GLApi *glWindowPos3f)(GLfloat, GLfloat, GLfloat);
extern void (GLApi *glWindowPos3fv)(const GLfloat*);
extern void (GLApi *glWindowPos3i)(GLint, GLint, GLint);
extern void (GLApi *glWindowPos3iv)(const GLint*);
extern void (GLApi *glWindowPos3s)(GLshort, GLshort, GLshort);
extern void (GLApi *glWindowPos3sv)(const GLshort*);

#endif //GL_VERSION_1_4

//							    --- opengl 1.5 core functions ---

#ifndef GL_VERSION_1_5
#define GL_VERSION_1_5 1
#define __GENERATE_GL_15__

#define GL_BUFFER_SIZE                    0x8764
#define GL_BUFFER_USAGE                   0x8765
#define GL_QUERY_COUNTER_BITS             0x8864
#define GL_CURRENT_QUERY                  0x8865
#define GL_QUERY_RESULT                   0x8866
#define GL_QUERY_RESULT_AVAILABLE         0x8867
#define GL_ARRAY_BUFFER                   0x8892
#define GL_ELEMENT_ARRAY_BUFFER           0x8893
#define GL_ARRAY_BUFFER_BINDING           0x8894
#define GL_ELEMENT_ARRAY_BUFFER_BINDING   0x8895
#define GL_VERTEX_ARRAY_BUFFER_BINDING    0x8896
#define GL_NORMAL_ARRAY_BUFFER_BINDING    0x8897
#define GL_COLOR_ARRAY_BUFFER_BINDING     0x8898
#define GL_INDEX_ARRAY_BUFFER_BINDING     0x8899
#define GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING 0x889A
#define GL_EDGE_FLAG_ARRAY_BUFFER_BINDING 0x889B
#define GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING 0x889C
#define GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING 0x889D
#define GL_WEIGHT_ARRAY_BUFFER_BINDING    0x889E
#define GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING 0x889F
#define GL_READ_ONLY                      0x88B8
#define GL_WRITE_ONLY                     0x88B9
#define GL_READ_WRITE                     0x88BA
#define GL_BUFFER_ACCESS                  0x88BB
#define GL_BUFFER_MAPPED                  0x88BC
#define GL_BUFFER_MAP_POINTER             0x88BD
#define GL_STREAM_DRAW                    0x88E0
#define GL_STREAM_READ                    0x88E1
#define GL_STREAM_COPY                    0x88E2
#define GL_STATIC_DRAW                    0x88E4
#define GL_STATIC_READ                    0x88E5
#define GL_STATIC_COPY                    0x88E6
#define GL_DYNAMIC_DRAW                   0x88E8
#define GL_DYNAMIC_READ                   0x88E9
#define GL_DYNAMIC_COPY                   0x88EA
#define GL_SAMPLES_PASSED                 0x8914
#define GL_FOG_COORD_SRC                  GL_FOG_COORDINATE_SOURCE
#define GL_FOG_COORD                      GL_FOG_COORDINATE
#define GL_CURRENT_FOG_COORD              GL_CURRENT_FOG_COORDINATE
#define GL_FOG_COORD_ARRAY_TYPE           GL_FOG_COORDINATE_ARRAY_TYPE
#define GL_FOG_COORD_ARRAY_STRIDE         GL_FOG_COORDINATE_ARRAY_STRIDE
#define GL_FOG_COORD_ARRAY_POINTER        GL_FOG_COORDINATE_ARRAY_POINTER
#define GL_FOG_COORD_ARRAY                GL_FOG_COORDINATE_ARRAY
#define GL_FOG_COORD_ARRAY_BUFFER_BINDING GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING
#define GL_SRC0_RGB                       GL_SOURCE0_RGB
#define GL_SRC1_RGB                       GL_SOURCE1_RGB
#define GL_SRC2_RGB                       GL_SOURCE2_RGB
#define GL_SRC0_ALPHA                     GL_SOURCE0_ALPHA
#define GL_SRC1_ALPHA                     GL_SOURCE1_ALPHA
#define GL_SRC2_ALPHA                     GL_SOURCE2_ALPHA

extern void (GLApi *glGenQueries)(GLsizei, GLuint*);
extern void (GLApi *glDeleteQueries)(GLsizei, const GLuint*);
extern GLboolean (GLApi *glIsQuery)(GLuint);
extern void (GLApi *glBeginQuery)(GLenum, GLuint);
extern void (GLApi *glEndQuery)(GLenum);
extern void (GLApi *glGetQueryiv)(GLenum, GLenum, GLint*);
extern void (GLApi *glGetQueryObjectiv)(GLuint, GLenum, GLint*);
extern void (GLApi *glGetQueryObjectuiv)(GLuint, GLenum, GLuint*);
extern void (GLApi *glBindBuffer)(GLenum, GLuint);
extern void (GLApi *glDeleteBuffers)(GLsizei, const GLuint*);
extern void (GLApi *glGenBuffers)(GLsizei, GLuint*);
extern GLboolean (GLApi *glIsBuffer)(GLuint);
extern void (GLApi *glBufferData)(GLenum, GLsizeiptr, const GLvoid *, GLenum);
extern void (GLApi *glBufferSubData)(GLenum, GLintptr, GLsizeiptr, const GLvoid*);
extern void (GLApi *glGetBufferSubData)(GLenum, GLintptr, GLsizeiptr, GLvoid*);
extern GLvoid* (GLApi *glMapBuffer)(GLenum, GLenum);
extern GLboolean (GLApi *glUnmapBuffer)(GLenum);
extern void (GLApi *glGetBufferParameteriv)(GLenum, GLenum, GLint*);
extern void (GLApi *glGetBufferPointerv)(GLenum, GLenum, GLvoid**);

#endif //GL_VERSION_1_5

//							   --- GL_ARB_texture_compression ---

#ifndef GL_ARB_texture_compression
#define GL_ARB_texture_compression 1
#define __GENERATE_GL_ARB_texture_compression__

extern void (GLApi *glCompressedTexImage3DARB)(GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLsizei, const GLvoid*);
extern void (GLApi *glCompressedTexImage2DARB)(GLenum, GLint, GLenum, GLsizei, GLsizei, GLint, GLsizei, const GLvoid*);
extern void (GLApi *glCompressedTexImage1DARB)(GLenum, GLint, GLenum, GLsizei, GLint, GLsizei, const GLvoid*);
extern void (GLApi *glCompressedTexSubImage3DARB)(GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLsizei, const GLvoid*);
extern void (GLApi *glCompressedTexSubImage2DARB)(GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLsizei, const GLvoid*);
extern void (GLApi *glCompressedTexSubImage1DARB)(GLenum, GLint, GLint, GLsizei, GLenum, GLsizei, const GLvoid*);
extern void (GLApi *glGetCompressedTexImageARB)(GLenum, GLint, GLvoid*);

#endif //GL_ARB_texture_compression

//								  --- GL_ARB_vertex_blend ---

#ifndef GL_ARB_vertex_blend
#define GL_ARB_vertex_blend 1
#define __GENERATE_GL_ARB_vertex_blend__

#define GL_MAX_VERTEX_UNITS_ARB           0x86A4
#define GL_ACTIVE_VERTEX_UNITS_ARB        0x86A5
#define GL_WEIGHT_SUM_UNITY_ARB           0x86A6
#define GL_VERTEX_BLEND_ARB               0x86A7
#define GL_CURRENT_WEIGHT_ARB             0x86A8
#define GL_WEIGHT_ARRAY_TYPE_ARB          0x86A9
#define GL_WEIGHT_ARRAY_STRIDE_ARB        0x86AA
#define GL_WEIGHT_ARRAY_SIZE_ARB          0x86AB
#define GL_WEIGHT_ARRAY_POINTER_ARB       0x86AC
#define GL_WEIGHT_ARRAY_ARB               0x86AD
#define GL_MODELVIEW0_ARB                 0x1700
#define GL_MODELVIEW1_ARB                 0x850A
#define GL_MODELVIEW2_ARB                 0x8722
#define GL_MODELVIEW3_ARB                 0x8723
#define GL_MODELVIEW4_ARB                 0x8724
#define GL_MODELVIEW5_ARB                 0x8725
#define GL_MODELVIEW6_ARB                 0x8726
#define GL_MODELVIEW7_ARB                 0x8727
#define GL_MODELVIEW8_ARB                 0x8728
#define GL_MODELVIEW9_ARB                 0x8729
#define GL_MODELVIEW10_ARB                0x872A
#define GL_MODELVIEW11_ARB                0x872B
#define GL_MODELVIEW12_ARB                0x872C
#define GL_MODELVIEW13_ARB                0x872D
#define GL_MODELVIEW14_ARB                0x872E
#define GL_MODELVIEW15_ARB                0x872F
#define GL_MODELVIEW16_ARB                0x8730
#define GL_MODELVIEW17_ARB                0x8731
#define GL_MODELVIEW18_ARB                0x8732
#define GL_MODELVIEW19_ARB                0x8733
#define GL_MODELVIEW20_ARB                0x8734
#define GL_MODELVIEW21_ARB                0x8735
#define GL_MODELVIEW22_ARB                0x8736
#define GL_MODELVIEW23_ARB                0x8737
#define GL_MODELVIEW24_ARB                0x8738
#define GL_MODELVIEW25_ARB                0x8739
#define GL_MODELVIEW26_ARB                0x873A
#define GL_MODELVIEW27_ARB                0x873B
#define GL_MODELVIEW28_ARB                0x873C
#define GL_MODELVIEW29_ARB                0x873D
#define GL_MODELVIEW30_ARB                0x873E
#define GL_MODELVIEW31_ARB                0x873F

extern void (GLApi *glWeightbvARB)(GLint, const GLbyte*);
extern void (GLApi *glWeightsvARB)(GLint, const GLshort*);
extern void (GLApi *glWeightivARB)(GLint, const GLint*);
extern void (GLApi *glWeightfvARB)(GLint, const GLfloat*);
extern void (GLApi *glWeightdvARB)(GLint, const GLdouble*);
extern void (GLApi *glWeightubvARB)(GLint, const GLubyte*);
extern void (GLApi *glWeightusvARB)(GLint, const GLushort*);
extern void (GLApi *glWeightuivARB)(GLint, const GLuint*);
extern void (GLApi *glWeightPointerARB)(GLint, GLenum, GLsizei, const GLvoid*);
extern void (GLApi *glVertexBlendARB)(GLint);

#endif //GL_ARB_vertex_blend

//								  --- GL_ARB_matrix_palette ---

#ifndef GL_ARB_matrix_palette
#define GL_ARB_matrix_palette 1
#define __GENERATE_GL_ARB_matrix_palette__

#define GL_MATRIX_PALETTE_ARB             0x8840
#define GL_MAX_MATRIX_PALETTE_STACK_DEPTH_ARB 0x8841
#define GL_MAX_PALETTE_MATRICES_ARB       0x8842
#define GL_CURRENT_PALETTE_MATRIX_ARB     0x8843
#define GL_MATRIX_INDEX_ARRAY_ARB         0x8844
#define GL_CURRENT_MATRIX_INDEX_ARB       0x8845
#define GL_MATRIX_INDEX_ARRAY_SIZE_ARB    0x8846
#define GL_MATRIX_INDEX_ARRAY_TYPE_ARB    0x8847
#define GL_MATRIX_INDEX_ARRAY_STRIDE_ARB  0x8848
#define GL_MATRIX_INDEX_ARRAY_POINTER_ARB 0x8849

extern void (GLApi *glCurrentPaletteMatrixARB)(GLint);
extern void (GLApi *glMatrixIndexubvARB)(GLint, const GLubyte*);
extern void (GLApi *glMatrixIndexusvARB)(GLint, const GLushort*);
extern void (GLApi *glMatrixIndexuivARB)(GLint, const GLuint*);
extern void (GLApi *glMatrixIndexPointerARB)(GLint, GLenum, GLsizei, const GLvoid*);

#endif //GL_ARB_matrix_palette

//							  --- GL_EXT_framebuffer_object ---

#ifndef GL_EXT_framebuffer_object
#define GL_EXT_framebuffer_object 1
#define __GENERATE_GL_EXT_framebuffer_object__

#define GL_INVALID_FRAMEBUFFER_OPERATION_EXT 0x0506
#define GL_MAX_RENDERBUFFER_SIZE_EXT      0x84E8
#define GL_FRAMEBUFFER_BINDING_EXT        0x8CA6
#define GL_RENDERBUFFER_BINDING_EXT       0x8CA7
#define GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE_EXT 0x8CD0
#define GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME_EXT 0x8CD1
#define GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL_EXT 0x8CD2
#define GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE_EXT 0x8CD3
#define GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_3D_ZOFFSET_EXT 0x8CD4
#define GL_FRAMEBUFFER_COMPLETE_EXT       0x8CD5
#define GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT_EXT 0x8CD6
#define GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT_EXT 0x8CD7
#define GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS_EXT 0x8CD9
#define GL_FRAMEBUFFER_INCOMPLETE_FORMATS_EXT 0x8CDA
#define GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER_EXT 0x8CDB
#define GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER_EXT 0x8CDC
#define GL_FRAMEBUFFER_UNSUPPORTED_EXT    0x8CDD
#define GL_MAX_COLOR_ATTACHMENTS_EXT      0x8CDF
#define GL_COLOR_ATTACHMENT0_EXT          0x8CE0
#define GL_COLOR_ATTACHMENT1_EXT          0x8CE1
#define GL_COLOR_ATTACHMENT2_EXT          0x8CE2
#define GL_COLOR_ATTACHMENT3_EXT          0x8CE3
#define GL_COLOR_ATTACHMENT4_EXT          0x8CE4
#define GL_COLOR_ATTACHMENT5_EXT          0x8CE5
#define GL_COLOR_ATTACHMENT6_EXT          0x8CE6
#define GL_COLOR_ATTACHMENT7_EXT          0x8CE7
#define GL_COLOR_ATTACHMENT8_EXT          0x8CE8
#define GL_COLOR_ATTACHMENT9_EXT          0x8CE9
#define GL_COLOR_ATTACHMENT10_EXT         0x8CEA
#define GL_COLOR_ATTACHMENT11_EXT         0x8CEB
#define GL_COLOR_ATTACHMENT12_EXT         0x8CEC
#define GL_COLOR_ATTACHMENT13_EXT         0x8CED
#define GL_COLOR_ATTACHMENT14_EXT         0x8CEE
#define GL_COLOR_ATTACHMENT15_EXT         0x8CEF
#define GL_DEPTH_ATTACHMENT_EXT           0x8D00
#define GL_STENCIL_ATTACHMENT_EXT         0x8D20
#define GL_FRAMEBUFFER_EXT                0x8D40
#define GL_RENDERBUFFER_EXT               0x8D41
#define GL_RENDERBUFFER_WIDTH_EXT         0x8D42
#define GL_RENDERBUFFER_HEIGHT_EXT        0x8D43
#define GL_RENDERBUFFER_INTERNAL_FORMAT_EXT 0x8D44
#define GL_STENCIL_INDEX1_EXT             0x8D46
#define GL_STENCIL_INDEX4_EXT             0x8D47
#define GL_STENCIL_INDEX8_EXT             0x8D48
#define GL_STENCIL_INDEX16_EXT            0x8D49
#define GL_RENDERBUFFER_RED_SIZE_EXT      0x8D50
#define GL_RENDERBUFFER_GREEN_SIZE_EXT    0x8D51
#define GL_RENDERBUFFER_BLUE_SIZE_EXT     0x8D52
#define GL_RENDERBUFFER_ALPHA_SIZE_EXT    0x8D53
#define GL_RENDERBUFFER_DEPTH_SIZE_EXT    0x8D54
#define GL_RENDERBUFFER_STENCIL_SIZE_EXT  0x8D55

extern GLboolean (GLApi *glIsRenderbufferEXT)(GLuint renderbuffer);
extern void (GLApi *glBindRenderbufferEXT)(GLenum target, GLuint renderbuffer);
extern void (GLApi *glDeleteRenderbuffersEXT)(GLsizei n, const GLuint *renderbuffers);
extern void (GLApi *glGenRenderbuffersEXT)(GLsizei n, GLuint *renderbuffers);
extern void (GLApi *glRenderbufferStorageEXT)(GLenum target, GLenum internalformat, GLsizei width, GLsizei height);
extern void (GLApi *glGetRenderbufferParameterivEXT)(GLenum target, GLenum pname, GLint* params);
extern GLboolean (GLApi *glIsFramebufferEXT)(GLuint framebuffer);
extern void (GLApi *glBindFramebufferEXT)(GLenum target, GLuint framebuffer);
extern void (GLApi *glDeleteFramebuffersEXT)(GLsizei n, const GLuint *framebuffers);
extern void (GLApi *glGenFramebuffersEXT)(GLsizei n, GLuint *framebuffers);
extern GLenum (GLApi *glCheckFramebufferStatusEXT)(GLenum target);
extern void (GLApi *glFramebufferTexture1DEXT)(GLenum target, GLenum attachment, GLenum textarget, GLuint texture, GLint level);
extern void (GLApi *glFramebufferTexture2DEXT)(GLenum target, GLenum attachment, GLenum textarget, GLuint texture, GLint level);
extern void (GLApi *glFramebufferTexture3DEXT)(GLenum target, GLenum attachment, GLenum textarget, GLuint texture, GLint level, GLint zoffset);
extern void (GLApi *glFramebufferRenderbufferEXT)(GLenum target, GLenum attachment, GLenum renderbuffertarget, GLuint renderbuffer);
extern void (GLApi *glGetFramebufferAttachmentParameterivEXT)(GLenum target, GLenum attachment, GLenum pname, GLint *params);
extern void (GLApi *glGenerateMipmapEXT)(GLenum target);

#endif //GL_EXT_framebuffer_object

//							   --- GL_EXT_compiled_vertex_array ---

#ifndef GL_EXT_compiled_vertex_array
#define GL_EXT_compiled_vertex_array 1
#define __GENERATE_GL_EXT_compiled_vertex_array__

#define GL_ARRAY_ELEMENT_LOCK_FIRST_EXT   0x81A8
#define GL_ARRAY_ELEMENT_LOCK_COUNT_EXT   0x81A9

extern void (GLApi *glLockArraysEXT)(GLint, GLsizei);
extern void (GLApi *glUnlockArraysEXT)(void);

#endif //GL_EXT_compiled_vertex_array

//								  --- GL_EXT_texture_object ---

/*extern GLboolean (GLApi *glAreTexturesResidentEXT)(GLsizei, const GLuint *, GLboolean*);
extern void (GLApi *glBindTextureEXT)(GLenum, GLuint);
extern void (GLApi *glDeleteTexturesEXT)(GLsizei, const GLuint*);
extern void (GLApi *glGenTexturesEXT)(GLsizei, GLuint*);
extern GLboolean (GLApi *glIsTextureEXT)(GLuint);
extern void (GLApi *glPrioritizeTexturesEXT)(GLsizei, const GLuint *, const GLclampf*);*/

//								  --- GL_EXT_polygon_offset ---

//extern void (GLApi *glPolygonOffsetEXT)(GLfloat, GLfloat);

//								--- GL_ARB_color_buffer_float ---

#ifndef GL_ARB_color_buffer_float
#define GL_ARB_color_buffer_float 1
#define __GENERATE_GL_ARB_color_buffer_float__

#define GL_RGBA_FLOAT_MODE_ARB            0x8820
#define GL_CLAMP_VERTEX_COLOR_ARB         0x891A
#define GL_CLAMP_FRAGMENT_COLOR_ARB       0x891B
#define GL_CLAMP_READ_COLOR_ARB           0x891C
#define GL_FIXED_ONLY_ARB                 0x891D

extern void (GLApi *glClampColorARB)(GLenum, GLenum);

#endif //GL_ARB_color_buffer_float

//								  --- GL_EXT_vertex_array ---

/*extern void (GLApi *glArrayElementEXT)(GLint);
extern void (GLApi *glColorPointerEXT)(GLint, GLenum, GLsizei, GLsizei, const GLvoid*);
extern void (GLApi *glDrawArraysEXT)(GLenum, GLint, GLsizei);
extern void (GLApi *glEdgeFlagPointerEXT)(GLsizei, GLsizei, const GLboolean*);
extern void (GLApi *glGetPointervEXT)(GLenum, GLvoid**);
extern void (GLApi *glIndexPointerEXT)(GLenum, GLsizei, GLsizei, const GLvoid*);
extern void (GLApi *glNormalPointerEXT)(GLenum, GLsizei, GLsizei, const GLvoid*);
extern void (GLApi *glTexCoordPointerEXT)(GLint, GLenum, GLsizei, GLsizei, const GLvoid*);
extern void (GLApi *glVertexPointerEXT)(GLint, GLenum, GLsizei, GLsizei, const GLvoid*);*/

//							  --- opengl 2.0 functions and tokens ---

#ifndef GL_VERSION_2_0
#define GL_VERSION_2_0 1
#define __GENERATE_GL_20__

// shader tokens
#define GL_CURRENT_PROGRAM						0x8B8D
#define GL_SHADER_TYPE							0x8B4E
#define GL_DELETE_STATUS						0x8B80
#define GL_COMPILE_STATUS						0x8B81
#define GL_LINK_STATUS							0x8B82
#define GL_VALIDATE_STATUS						0x8B83
#define GL_INFO_LOG_LENGTH						0x8B84
#define GL_ATTACHED_SHADERS						0x8B85
#define GL_ACTIVE_UNIFORMS						0x8B86
#define GL_ACTIVE_UNIFORM_MAX_LENGTH			0x8B87
#define GL_SHADER_SOURCE_LENGTH					0x8B88
#define GL_VERTEX_SHADER						0x8B31
#define GL_ACTIVE_ATTRIBUTES					0x8B89
#define GL_ACTIVE_ATTRIBUTE_MAX_LENGTH			0x8B8A
#define GL_FRAGMENT_SHADER						0x8B30

// uniform types
#define GL_FLOAT_VEC2							0x8B50
#define GL_FLOAT_VEC3							0x8B51
#define GL_FLOAT_VEC4							0x8B52
#define GL_INT_VEC2								0x8B53
#define GL_INT_VEC3								0x8B54
#define GL_INT_VEC4								0x8B55
#define GL_BOOL									0x8B56
#define GL_BOOL_VEC2							0x8B57
#define GL_BOOL_VEC3							0x8B58
#define GL_BOOL_VEC4							0x8B59
#define GL_FLOAT_MAT2							0x8B5A
#define GL_FLOAT_MAT3							0x8B5B
#define GL_FLOAT_MAT4							0x8B5C
#define GL_SAMPLER_1D							0x8B5D
#define GL_SAMPLER_2D							0x8B5E
#define GL_SAMPLER_3D							0x8B5F
#define GL_SAMPLER_CUBE							0x8B60
#define GL_SAMPLER_1D_SHADOW					0x8B61
#define GL_SAMPLER_2D_SHADOW					0x8B62

// vertex attrib arrays
#define GL_VERTEX_ATTRIB_ARRAY_ENABLED			0x8622
#define GL_VERTEX_ATTRIB_ARRAY_SIZE				0x8623
#define GL_VERTEX_ATTRIB_ARRAY_STRIDE			0x8624
#define GL_VERTEX_ATTRIB_ARRAY_TYPE				0x8625
#define GL_VERTEX_ATTRIB_ARRAY_NORMALIZED		0x886A
#define GL_CURRENT_VERTEX_ATTRIB				0x8626
#define GL_VERTEX_ATTRIB_ARRAY_POINTER			0x8645
#define GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING	0x889F

// hints
#define GL_FRAGMENT_SHADER_DERIVATIVE_HINT		0x8B8B

// raster control
#define GL_VERTEX_PROGRAM_POINT_SIZE			0x8642
#define GL_VERTEX_PROGRAM_TWO_SIDE				0x8643

// implementation dependent strings and limits
#define GL_SHADING_LANGUAGE_VERSION				0x8B8C
#define GL_MAX_VERTEX_ATTRIBS					0x8869
#define GL_MAX_FRAGMENT_UNIFORM_COMPONENTS		0x8B49
#define GL_MAX_VERTEX_UNIFORM_COMPONENTS		0x8B4A
#define GL_MAX_VARYING_FLOATS					0x8B4B
#define GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS		0x8B4C
#define GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS		0x8B4D
#define GL_MAX_TEXTURE_COORDS					0x8871
#define GL_MAX_TEXTURE_IMAGE_UNITS				0x8872

// multiple render targets
#define GL_MAX_DRAW_BUFFERS						0x8824
#define GL_DRAW_BUFFER0							0x8825
#define GL_DRAW_BUFFER1							0x8826
#define GL_DRAW_BUFFER2							0x8827
#define GL_DRAW_BUFFER3							0x8828
#define GL_DRAW_BUFFER4							0x8829
#define GL_DRAW_BUFFER5							0x882A
#define GL_DRAW_BUFFER6							0x882B
#define GL_DRAW_BUFFER7							0x882C
#define GL_DRAW_BUFFER8							0x882D
#define GL_DRAW_BUFFER9							0x882E
#define GL_DRAW_BUFFER10						0x882F
#define GL_DRAW_BUFFER11						0x8830
#define GL_DRAW_BUFFER12						0x8831
#define GL_DRAW_BUFFER13						0x8832
#define GL_DRAW_BUFFER14						0x8833
#define GL_DRAW_BUFFER15						0x8834

// point sprite
#define GL_POINT_SPRITE 						0x8861
#define GL_COORD_REPLACE 						0x8862
#define GL_POINT_SPRITE_COORD_ORIGIN 			0x8CA0
#define GL_LOWER_LEFT 							0x8CA1
#define GL_UPPER_LEFT 							0x8CA2

// two-sided stencil
#define GL_STENCIL_BACK_FUNC					0x8800
#define GL_STENCIL_BACK_VALUE_MASK 				0x8CA4
#define GL_STENCIL_BACK_REF 					0x8CA3
#define GL_STENCIL_BACK_FAIL 					0x8801
#define GL_STENCIL_BACK_PASS_DEPTH_FAIL 		0x8802
#define GL_STENCIL_BACK_PASS_DEPTH_PASS 		0x8803
#define GL_STENCIL_BACK_WRITEMASK 				0x8CA5

// separate alpha blending
#define GL_BLEND_EQUATION_RGB					0x8009
#define GL_BLEND_EQUATION_ALPHA					0x883D

// shader objects api
extern void (GLApi *glDeleteShader)(GLuint shader);
extern void (GLApi *glDetachShader)(GLuint program, GLuint shader);
extern GLuint (GLApi *glCreateShader)(GLenum type);
extern void (GLApi *glShaderSource)(GLuint shader, GLsizei count, const GLchar **string, const GLint *length);
extern void (GLApi *glCompileShader)(GLuint shader);

// program objects
extern GLuint (GLApi *glCreateProgram)(void);
extern void (GLApi *glAttachShader)(GLuint program, GLuint shader);
extern void (GLApi *glLinkProgram)(GLuint program);
extern void (GLApi *glUseProgram)(GLuint program);
extern void (GLApi *glDeleteProgram)(GLuint program);
extern void (GLApi *glValidateProgram)(GLuint program);

// uniforms
extern void (GLApi *glUniform1f)(GLint location, GLfloat v0);
extern void (GLApi *glUniform2f)(GLint location, GLfloat v0, GLfloat v1);
extern void (GLApi *glUniform3f)(GLint location, GLfloat v0, GLfloat v1, GLfloat v2);
extern void (GLApi *glUniform4f)(GLint location, GLfloat v0, GLfloat v1, GLfloat v2, GLfloat v3);
extern void (GLApi *glUniform1i)(GLint location, GLint v0);
extern void (GLApi *glUniform2i)(GLint location, GLint v0, GLint v1);
extern void (GLApi *glUniform3i)(GLint location, GLint v0, GLint v1, GLint v2);
extern void (GLApi *glUniform4i)(GLint location, GLint v0, GLint v1, GLint v2, GLint v3);
extern void (GLApi *glUniform1fv)(GLint location, GLsizei count, const GLfloat *value);
extern void (GLApi *glUniform2fv)(GLint location, GLsizei count, const GLfloat *value);
extern void (GLApi *glUniform3fv)(GLint location, GLsizei count, const GLfloat *value);
extern void (GLApi *glUniform4fv)(GLint location, GLsizei count, const GLfloat *value);
extern void (GLApi *glUniform1iv)(GLint location, GLsizei count, const GLint *value);
extern void (GLApi *glUniform2iv)(GLint location, GLsizei count, const GLint *value);
extern void (GLApi *glUniform3iv)(GLint location, GLsizei count, const GLint *value);
extern void (GLApi *glUniform4iv)(GLint location, GLsizei count, const GLint *value);
extern void (GLApi *glUniformMatrix2fv)(GLint location, GLsizei count, GLboolean transpose, const GLfloat *value);
extern void (GLApi *glUniformMatrix3fv)(GLint location, GLsizei count, GLboolean transpose, const GLfloat *value);
extern void (GLApi *glUniformMatrix4fv)(GLint location, GLsizei count, GLboolean transpose, const GLfloat *value);

// attrib locations
extern void (GLApi *glBindAttribLocation)(GLuint program, GLuint index, const GLchar *name);
extern GLint (GLApi *glGetAttribLocation)(GLuint program, const GLchar *name);

// vertex attributes
extern void (GLApi *glVertexAttrib1d)(GLuint index, GLdouble x);
extern void (GLApi *glVertexAttrib1dv)(GLuint index, const GLdouble *v);
extern void (GLApi *glVertexAttrib1f)(GLuint index, GLfloat x);
extern void (GLApi *glVertexAttrib1fv)(GLuint index, const GLfloat *v);
extern void (GLApi *glVertexAttrib1s)(GLuint index, GLshort x);
extern void (GLApi *glVertexAttrib1sv)(GLuint index, const GLshort *v);
extern void (GLApi *glVertexAttrib2d)(GLuint index, GLdouble x, GLdouble y);
extern void (GLApi *glVertexAttrib2dv)(GLuint index, const GLdouble *v);
extern void (GLApi *glVertexAttrib2f)(GLuint index, GLfloat x, GLfloat y);
extern void (GLApi *glVertexAttrib2fv)(GLuint index, const GLfloat *v);
extern void (GLApi *glVertexAttrib2s)(GLuint index, GLshort x, GLshort y);
extern void (GLApi *glVertexAttrib2sv)(GLuint index, const GLshort *v);
extern void (GLApi *glVertexAttrib3d)(GLuint index, GLdouble x, GLdouble y, GLdouble z);
extern void (GLApi *glVertexAttrib3dv)(GLuint index, const GLdouble *v);
extern void (GLApi *glVertexAttrib3f)(GLuint index, GLfloat x, GLfloat y, GLfloat z);
extern void (GLApi *glVertexAttrib3fv)(GLuint index, const GLfloat *v);
extern void (GLApi *glVertexAttrib3s)(GLuint index, GLshort x, GLshort y, GLshort z);
extern void (GLApi *glVertexAttrib3sv)(GLuint index, const GLshort *v);
extern void (GLApi *glVertexAttrib4Nbv)(GLuint index, const GLbyte *v);
extern void (GLApi *glVertexAttrib4Niv)(GLuint index, const GLint *v);
extern void (GLApi *glVertexAttrib4Nsv)(GLuint index, const GLshort *v);
extern void (GLApi *glVertexAttrib4Nub)(GLuint index, GLubyte x, GLubyte y, GLubyte z, GLubyte w);
extern void (GLApi *glVertexAttrib4Nubv)(GLuint index, const GLubyte *v);
extern void (GLApi *glVertexAttrib4Nuiv)(GLuint index, const GLuint *v);
extern void (GLApi *glVertexAttrib4Nusv)(GLuint index, const GLushort *v);
extern void (GLApi *glVertexAttrib4bv)(GLuint index, const GLbyte *v);
extern void (GLApi *glVertexAttrib4d)(GLuint index, GLdouble x, GLdouble y, GLdouble z, GLdouble w);
extern void (GLApi *glVertexAttrib4dv)(GLuint index, const GLdouble *v);
extern void (GLApi *glVertexAttrib4f)(GLuint index, GLfloat x, GLfloat y, GLfloat z, GLfloat w);
extern void (GLApi *glVertexAttrib4fv)(GLuint index, const GLfloat *v);
extern void (GLApi *glVertexAttrib4iv)(GLuint index, const GLint *v);
extern void (GLApi *glVertexAttrib4s)(GLuint index, GLshort x, GLshort y, GLshort z, GLshort w);
extern void (GLApi *glVertexAttrib4sv)(GLuint index, const GLshort *v);
extern void (GLApi *glVertexAttrib4ubv)(GLuint index, const GLubyte *v);
extern void (GLApi *glVertexAttrib4uiv)(GLuint index, const GLuint *v);
extern void (GLApi *glVertexAttrib4usv)(GLuint index, const GLushort *v);
extern void (GLApi *glVertexAttribPointer)(GLuint index, GLint size, GLenum type, GLboolean normalized, GLsizei stride, const GLvoid *pointer);
extern void (GLApi *glEnableVertexAttribArray)(GLuint index);
extern void (GLApi *glDisableVertexAttribArray)(GLuint index);
extern void (GLApi *glGetVertexAttribdv)(GLuint index, GLenum pname, GLdouble *params);
extern void (GLApi *glGetVertexAttribfv)(GLuint index, GLenum pname, GLfloat *params);
extern void (GLApi *glGetVertexAttribiv)(GLuint index, GLenum pname, GLint *params);
extern void (GLApi *glGetVertexAttribPointerv)(GLuint index, GLenum pname, GLvoid* *pointer);

// queries
extern GLboolean (GLApi *glIsShader)(GLuint shader);
extern GLboolean (GLApi *glIsProgram)(GLuint program);
extern void (GLApi *glGetShaderiv)(GLuint program, GLenum pname, GLint *params);
extern void (GLApi *glGetProgramiv)(GLuint program, GLenum pname, GLint *params);
extern void (GLApi *glGetAttachedShaders)(GLuint program, GLsizei maxCount, GLsizei *count, GLuint *shaders);
extern void (GLApi *glGetShaderInfoLog)(GLuint shader, GLsizei bufSize, GLsizei *length, GLchar *infoLog);
extern void (GLApi *glGetProgramInfoLog)(GLuint program, GLsizei bufSize, GLsizei *length, GLchar *infoLog);
extern GLint (GLApi *glGetUniformLocation)(GLuint program, const GLchar *name);
extern void (GLApi *glGetActiveUniform)(GLuint program, GLuint index, GLsizei bufSize, GLsizei *length, GLsizei *size, GLenum *type, GLchar *name);
extern void (GLApi *glGetUniformfv)(GLuint program, GLint location, GLfloat *params);
extern void (GLApi *glGetUniformiv)(GLuint program, GLint location, GLint *params);
extern void (GLApi *glGetShaderSource)(GLuint shader, GLsizei bufSize, GLsizei *length, GLchar *source);
extern void (GLApi *glGetActiveAttrib)(GLuint program, GLuint index, GLsizei bufSize, GLsizei *length, GLsizei *size, GLenum *type, GLchar *name);

// multiple draw buffers
extern void (GLApi *glDrawBuffers)(GLsizei n, const GLenum *bufs);

// two-sided stencil
extern void (GLApi *glStencilFuncSeparate)(GLenum face, GLenum func, GLint ref, GLuint mask);
extern void (GLApi *glStencilOpSeparate)(GLenum face, GLenum fail, GLenum zfail, GLenum zpass);
extern void (GLApi *glStencilMaskSeparate)(GLenum face, GLuint mask);

// separate alpha blending
extern void (GLApi *glBlendEquationSeparate)(GLenum modeRGB, GLenum modeAlpha);

#endif //GL_VERSION_2_0

/*
 *								--- GL_ARB_vertex_program ---
 */

#ifndef GL_ARB_vertex_program
#define GL_ARB_vertex_program 1
#define __GENERATE_GL_ARB_vertex_program__

#define GL_VERTEX_PROGRAM_ARB                              0x8620

#define GL_VERTEX_PROGRAM_POINT_SIZE_ARB                   0x8642
#define GL_VERTEX_PROGRAM_TWO_SIDE_ARB                     0x8643
#define GL_COLOR_SUM_ARB                                   0x8458

#define GL_PROGRAM_FORMAT_ASCII_ARB                        0x8875

#define GL_VERTEX_ATTRIB_ARRAY_ENABLED_ARB                 0x8622
#define GL_VERTEX_ATTRIB_ARRAY_SIZE_ARB                    0x8623
#define GL_VERTEX_ATTRIB_ARRAY_STRIDE_ARB                  0x8624
#define GL_VERTEX_ATTRIB_ARRAY_TYPE_ARB                    0x8625
#define GL_VERTEX_ATTRIB_ARRAY_NORMALIZED_ARB              0x886A
#define GL_CURRENT_VERTEX_ATTRIB_ARB                       0x8626

#define GL_VERTEX_ATTRIB_ARRAY_POINTER_ARB                 0x8645

#define GL_PROGRAM_LENGTH_ARB                              0x8627
#define GL_PROGRAM_FORMAT_ARB                              0x8876
#define GL_PROGRAM_BINDING_ARB                             0x8677
#define GL_PROGRAM_INSTRUCTIONS_ARB                        0x88A0
#define GL_MAX_PROGRAM_INSTRUCTIONS_ARB                    0x88A1
#define GL_PROGRAM_NATIVE_INSTRUCTIONS_ARB                 0x88A2
#define GL_MAX_PROGRAM_NATIVE_INSTRUCTIONS_ARB             0x88A3
#define GL_PROGRAM_TEMPORARIES_ARB                         0x88A4
#define GL_MAX_PROGRAM_TEMPORARIES_ARB                     0x88A5
#define GL_PROGRAM_NATIVE_TEMPORARIES_ARB                  0x88A6
#define GL_MAX_PROGRAM_NATIVE_TEMPORARIES_ARB              0x88A7
#define GL_PROGRAM_PARAMETERS_ARB                          0x88A8
#define GL_MAX_PROGRAM_PARAMETERS_ARB                      0x88A9
#define GL_PROGRAM_NATIVE_PARAMETERS_ARB                   0x88AA
#define GL_MAX_PROGRAM_NATIVE_PARAMETERS_ARB               0x88AB
#define GL_PROGRAM_ATTRIBS_ARB                             0x88AC
#define GL_MAX_PROGRAM_ATTRIBS_ARB                         0x88AD
#define GL_PROGRAM_NATIVE_ATTRIBS_ARB                      0x88AE
#define GL_MAX_PROGRAM_NATIVE_ATTRIBS_ARB                  0x88AF
#define GL_PROGRAM_ADDRESS_REGISTERS_ARB                   0x88B0
#define GL_MAX_PROGRAM_ADDRESS_REGISTERS_ARB               0x88B1
#define GL_PROGRAM_NATIVE_ADDRESS_REGISTERS_ARB            0x88B2
#define GL_MAX_PROGRAM_NATIVE_ADDRESS_REGISTERS_ARB        0x88B3
#define GL_MAX_PROGRAM_LOCAL_PARAMETERS_ARB                0x88B4
#define GL_MAX_PROGRAM_ENV_PARAMETERS_ARB                  0x88B5
#define GL_PROGRAM_UNDER_NATIVE_LIMITS_ARB                 0x88B6

#define GL_PROGRAM_STRING_ARB                              0x8628

#define GL_PROGRAM_ERROR_POSITION_ARB                      0x864B
#define GL_CURRENT_MATRIX_ARB                              0x8641
#define GL_TRANSPOSE_CURRENT_MATRIX_ARB                    0x88B7
#define GL_CURRENT_MATRIX_STACK_DEPTH_ARB                  0x8640
#define GL_MAX_VERTEX_ATTRIBS_ARB                          0x8869
#define GL_MAX_PROGRAM_MATRICES_ARB                        0x862F
#define GL_MAX_PROGRAM_MATRIX_STACK_DEPTH_ARB              0x862E

#define GL_PROGRAM_ERROR_STRING_ARB                        0x8874

#define GL_MATRIX0_ARB                                     0x88C0
#define GL_MATRIX1_ARB                                     0x88C1
#define GL_MATRIX2_ARB                                     0x88C2
#define GL_MATRIX3_ARB                                     0x88C3
#define GL_MATRIX4_ARB                                     0x88C4
#define GL_MATRIX5_ARB                                     0x88C5
#define GL_MATRIX6_ARB                                     0x88C6
#define GL_MATRIX7_ARB                                     0x88C7
#define GL_MATRIX8_ARB                                     0x88C8
#define GL_MATRIX9_ARB                                     0x88C9
#define GL_MATRIX10_ARB                                    0x88CA
#define GL_MATRIX11_ARB                                    0x88CB
#define GL_MATRIX12_ARB                                    0x88CC
#define GL_MATRIX13_ARB                                    0x88CD
#define GL_MATRIX14_ARB                                    0x88CE
#define GL_MATRIX15_ARB                                    0x88CF
#define GL_MATRIX16_ARB                                    0x88D0
#define GL_MATRIX17_ARB                                    0x88D1
#define GL_MATRIX18_ARB                                    0x88D2
#define GL_MATRIX19_ARB                                    0x88D3
#define GL_MATRIX20_ARB                                    0x88D4
#define GL_MATRIX21_ARB                                    0x88D5
#define GL_MATRIX22_ARB                                    0x88D6
#define GL_MATRIX23_ARB                                    0x88D7
#define GL_MATRIX24_ARB                                    0x88D8
#define GL_MATRIX25_ARB                                    0x88D9
#define GL_MATRIX26_ARB                                    0x88DA
#define GL_MATRIX27_ARB                                    0x88DB
#define GL_MATRIX28_ARB                                    0x88DC
#define GL_MATRIX29_ARB                                    0x88DD
#define GL_MATRIX30_ARB                                    0x88DE
#define GL_MATRIX31_ARB                                    0x88DF

// vertex program
extern void (GLApi *glVertexAttrib1sARB)(GLuint index, short x);
extern void (GLApi *glVertexAttrib1fARB)(GLuint index, float x);
extern void (GLApi *glVertexAttrib1dARB)(GLuint index, double x);
extern void (GLApi *glVertexAttrib2sARB)(GLuint index, short x, short y);
extern void (GLApi *glVertexAttrib2fARB)(GLuint index, float x, float y);
extern void (GLApi *glVertexAttrib2dARB)(GLuint index, double x, double y);
extern void (GLApi *glVertexAttrib3sARB)(GLuint index, short x, short y, short z);
extern void (GLApi *glVertexAttrib3fARB)(GLuint index, float x, float y, float z);
extern void (GLApi *glVertexAttrib3dARB)(GLuint index, double x, double y, double z);
extern void (GLApi *glVertexAttrib4sARB)(GLuint index, short x, short y, short z, short w);
extern void (GLApi *glVertexAttrib4fARB)(GLuint index, float x, float y, float z, float w);
extern void (GLApi *glVertexAttrib4dARB)(GLuint index, double x, double y, double z, double w);
extern void (GLApi *glVertexAttrib4NubARB)(GLuint index, GLubyte x, GLubyte y, GLubyte z, GLubyte w);
extern void (GLApi *glVertexAttrib1svARB)(GLuint index, const short *v);
extern void (GLApi *glVertexAttrib1fvARB)(GLuint index, const float *v);
extern void (GLApi *glVertexAttrib1dvARB)(GLuint index, const double *v);
extern void (GLApi *glVertexAttrib2svARB)(GLuint index, const short *v);
extern void (GLApi *glVertexAttrib2fvARB)(GLuint index, const float *v);
extern void (GLApi *glVertexAttrib2dvARB)(GLuint index, const double *v);
extern void (GLApi *glVertexAttrib3svARB)(GLuint index, const short *v);
extern void (GLApi *glVertexAttrib3fvARB)(GLuint index, const float *v);
extern void (GLApi *glVertexAttrib3dvARB)(GLuint index, const double *v);
extern void (GLApi *glVertexAttrib4bvARB)(GLuint index, const GLbyte *v);
extern void (GLApi *glVertexAttrib4svARB)(GLuint index, const short *v);
extern void (GLApi *glVertexAttrib4ivARB)(GLuint index, const int *v);
extern void (GLApi *glVertexAttrib4ubvARB)(GLuint index, const GLubyte *v);
extern void (GLApi *glVertexAttrib4usvARB)(GLuint index, const GLushort *v);
extern void (GLApi *glVertexAttrib4uivARB)(GLuint index, const GLuint *v);
extern void (GLApi *glVertexAttrib4fvARB)(GLuint index, const float *v);
extern void (GLApi *glVertexAttrib4dvARB)(GLuint index, const double *v);
extern void (GLApi *glVertexAttrib4NbvARB)(GLuint index, const GLbyte *v);
extern void (GLApi *glVertexAttrib4NsvARB)(GLuint index, const short *v);
extern void (GLApi *glVertexAttrib4NivARB)(GLuint index, const int *v);
extern void (GLApi *glVertexAttrib4NubvARB)(GLuint index, const GLubyte *v);
extern void (GLApi *glVertexAttrib4NusvARB)(GLuint index, const GLushort *v);
extern void (GLApi *glVertexAttrib4NuivARB)(GLuint index, const GLuint *v);
extern void (GLApi *glVertexAttribPointerARB)(GLuint index, int size, GLenum type, GLboolean normalized, GLsizei stride, const void *pointer);
extern void (GLApi *glEnableVertexAttribArrayARB)(GLuint index);
extern void (GLApi *glDisableVertexAttribArrayARB)(GLuint index);

extern void (GLApi *glProgramStringARB)(GLenum target, GLenum format, GLsizei len, const void *string); 
extern void (GLApi *glBindProgramARB)(GLenum target, GLuint program);
extern void (GLApi *glDeleteProgramsARB)(GLsizei n, const GLuint *programs);
extern void (GLApi *glGenProgramsARB)(GLsizei n, GLuint *programs);
extern void (GLApi *glProgramEnvParameter4dARB)(GLenum target, GLuint index, double x, double y, double z, double w);
extern void (GLApi *glProgramEnvParameter4dvARB)(GLenum target, GLuint index, const double *params);
extern void (GLApi *glProgramEnvParameter4fARB)(GLenum target, GLuint index, float x, float y, float z, float w);
extern void (GLApi *glProgramEnvParameter4fvARB)(GLenum target, GLuint index, const float *params);
extern void (GLApi *glProgramLocalParameter4dARB)(GLenum target, GLuint index, double x, double y, double z, double w);
extern void (GLApi *glProgramLocalParameter4dvARB)(GLenum target, GLuint index, const double *params);
extern void (GLApi *glProgramLocalParameter4fARB)(GLenum target, GLuint index, float x, float y, float z, float w);
extern void (GLApi *glProgramLocalParameter4fvARB)(GLenum target, GLuint index, const float *params);
extern void (GLApi *glGetProgramEnvParameterdvARB)(GLenum target, GLuint index, double *params);
extern void (GLApi *glGetProgramEnvParameterfvARB)(GLenum target, GLuint index, float *params);
extern void (GLApi *glGetProgramLocalParameterdvARB)(GLenum target, GLuint index, double *params);
extern void (GLApi *glGetProgramLocalParameterfvARB)(GLenum target, GLuint index, float *params);
extern void (GLApi *glGetProgramivARB)(GLenum target, GLenum pname, int *params);
extern void (GLApi *glGetProgramStringARB)(GLenum target, GLenum pname, void *string);
extern void (GLApi *glGetVertexAttribdvARB)(GLuint index, GLenum pname, double *params);
extern void (GLApi *glGetVertexAttribfvARB)(GLuint index, GLenum pname, float *params);
extern void (GLApi *glGetVertexAttribivARB)(GLuint index, GLenum pname, int *params);
extern void (GLApi *glGetVertexAttribPointervARB)(GLuint index, GLenum pname, void **pointer);
extern GLboolean (GLApi *glIsProgramARB)(GLuint program);

#endif //GL_ARB_vertex_program

/*
 *								--- GL_ARB_fragment_program ---
 */

#ifndef GL_ARB_fragment_program
#define GL_ARB_fragment_program 1
#define __GENERATE_GL_ARB_fragment_program__

#define GL_FRAGMENT_PROGRAM_ARB                            0x8804

#define GL_PROGRAM_ALU_INSTRUCTIONS_ARB                    0x8805
#define GL_PROGRAM_TEX_INSTRUCTIONS_ARB                    0x8806
#define GL_PROGRAM_TEX_INDIRECTIONS_ARB                    0x8807
#define GL_PROGRAM_NATIVE_ALU_INSTRUCTIONS_ARB             0x8808
#define GL_PROGRAM_NATIVE_TEX_INSTRUCTIONS_ARB             0x8809
#define GL_PROGRAM_NATIVE_TEX_INDIRECTIONS_ARB             0x880A
#define GL_MAX_PROGRAM_ALU_INSTRUCTIONS_ARB                0x880B
#define GL_MAX_PROGRAM_TEX_INSTRUCTIONS_ARB                0x880C
#define GL_MAX_PROGRAM_TEX_INDIRECTIONS_ARB                0x880D
#define GL_MAX_PROGRAM_NATIVE_ALU_INSTRUCTIONS_ARB         0x880E
#define GL_MAX_PROGRAM_NATIVE_TEX_INSTRUCTIONS_ARB         0x880F
#define GL_MAX_PROGRAM_NATIVE_TEX_INDIRECTIONS_ARB         0x8810

#define GL_MAX_TEXTURE_COORDS_ARB                          0x8871
#define GL_MAX_TEXTURE_IMAGE_UNITS_ARB                     0x8872

#endif //GL_ARB_fragment_program

/*
 *								--- GL_ARB_vertex_shader ---
 */

#ifndef GL_ARB_vertex_shader
#define GL_ARB_vertex_shader 1
#define __GENERATE_GL_ARB_vertex_shader__

#define GL_VERTEX_SHADER_ARB						0x8B31

#define GL_MAX_VERTEX_UNIFORM_COMPONENTS_ARB		0x8B4A
#define GL_MAX_VARYING_FLOATS_ARB					0x8B4B
#define GL_MAX_VERTEX_ATTRIBS_ARB					0x8869
#define GL_MAX_TEXTURE_IMAGE_UNITS_ARB				0x8872
#define GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS_ARB		0x8B4C
#define GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS_ARB		0x8B4D
#define GL_MAX_TEXTURE_COORDS_ARB					0x8871

#define GL_VERTEX_PROGRAM_POINT_SIZE_ARB			0x8642
#define GL_VERTEX_PROGRAM_TWO_SIDE_ARB				0x8643

#define GL_OBJECT_ACTIVE_ATTRIBUTES_ARB				0x8B89
#define GL_OBJECT_ACTIVE_ATTRIBUTE_MAX_LENGTH_ARB	0x8B8A

#define GL_VERTEX_ATTRIB_ARRAY_ENABLED_ARB			0x8622
#define GL_VERTEX_ATTRIB_ARRAY_SIZE_ARB				0x8623
#define GL_VERTEX_ATTRIB_ARRAY_STRIDE_ARB			0x8624
#define GL_VERTEX_ATTRIB_ARRAY_TYPE_ARB				0x8625
#define GL_VERTEX_ATTRIB_ARRAY_NORMALIZED_ARB		0x886A
#define GL_CURRENT_VERTEX_ATTRIB_ARB				0x8626

#define GL_VERTEX_ATTRIB_ARRAY_POINTER_ARB			0x8645

#define GL_FLOAT									0x1406
#define GL_FLOAT_VEC2_ARB							0x8B50
#define GL_FLOAT_VEC3_ARB							0x8B51
#define GL_FLOAT_VEC4_ARB							0x8B52
#define GL_FLOAT_MAT2_ARB							0x8B5A
#define GL_FLOAT_MAT3_ARB							0x8B5B
#define GL_FLOAT_MAT4_ARB							0x8B5C
// new tokens

// vertex shader
extern void (GLApi *glVertexAttrib1fARB)(GLuint index, float v0);
extern void (GLApi *glVertexAttrib1sARB)(GLuint index, short v0);
extern void (GLApi *glVertexAttrib1dARB)(GLuint index, double v0);
extern void (GLApi *glVertexAttrib2fARB)(GLuint index, float v0, float v1);
extern void (GLApi *glVertexAttrib2sARB)(GLuint index, short v0, short v1);
extern void (GLApi *glVertexAttrib2dARB)(GLuint index, double v0, double v1);
extern void (GLApi *glVertexAttrib3fARB)(GLuint index, float v0, float v1, float v2);
extern void (GLApi *glVertexAttrib3sARB)(GLuint index, short v0, short v1, short v2);
extern void (GLApi *glVertexAttrib3dARB)(GLuint index, double v0, double v1, double v2);
extern void (GLApi *glVertexAttrib4fARB)(GLuint index, float v0, float v1, float v2, float v3);
extern void (GLApi *glVertexAttrib4sARB)(GLuint index, short v0, short v1, short v2, short v3);
extern void (GLApi *glVertexAttrib4dARB)(GLuint index, double v0, double v1, double v2, double v3);
extern void (GLApi *glVertexAttrib4NubARB)(GLuint index, GLubyte x, GLubyte y, GLubyte z, GLubyte w);
extern void (GLApi *glVertexAttrib1fvARB)(GLuint index, const float *v);
extern void (GLApi *glVertexAttrib1svARB)(GLuint index, const short *v);
extern void (GLApi *glVertexAttrib1dvARB)(GLuint index, const double *v);
extern void (GLApi *glVertexAttrib2fvARB)(GLuint index, const float *v);
extern void (GLApi *glVertexAttrib2svARB)(GLuint index, const short *v);
extern void (GLApi *glVertexAttrib2dvARB)(GLuint index, const double *v);
extern void (GLApi *glVertexAttrib3fvARB)(GLuint index, const float *v);
extern void (GLApi *glVertexAttrib3svARB)(GLuint index, const short *v);
extern void (GLApi *glVertexAttrib3dvARB)(GLuint index, const double *v);
extern void (GLApi *glVertexAttrib4fvARB)(GLuint index, const float *v);
extern void (GLApi *glVertexAttrib4svARB)(GLuint index, const short *v);
extern void (GLApi *glVertexAttrib4dvARB)(GLuint index, const double *v);
extern void (GLApi *glVertexAttrib4ivARB)(GLuint index, const int *v);
extern void (GLApi *glVertexAttrib4bvARB)(GLuint index, const GLbyte *v);
extern void (GLApi *glVertexAttrib4ubvARB)(GLuint index, const GLubyte *v);
extern void (GLApi *glVertexAttrib4usvARB)(GLuint index, const GLushort *v);
extern void (GLApi *glVertexAttrib4uivARB)(GLuint index, const GLuint *v);
extern void (GLApi *glVertexAttrib4NbvARB)(GLuint index, const GLbyte *v);
extern void (GLApi *glVertexAttrib4NsvARB)(GLuint index, const short *v);
extern void (GLApi *glVertexAttrib4NivARB)(GLuint index, const int *v);
extern void (GLApi *glVertexAttrib4NubvARB)(GLuint index, const GLubyte *v);
extern void (GLApi *glVertexAttrib4NusvARB)(GLuint index, const GLushort *v);
extern void (GLApi *glVertexAttrib4NuivARB)(GLuint index, const GLuint *v);
extern void (GLApi *glVertexAttribPointerARB)(GLuint index, int size, GLenum type, GLboolean normalized, GLsizei stride, const void *pointer);
extern void (GLApi *glEnableVertexAttribArrayARB)(GLuint index);
extern void (GLApi *glDisableVertexAttribArrayARB)(GLuint index);
extern void (GLApi *glBindAttribLocationARB)(GLhandleARB programObj, GLuint index, const GLcharARB *name);
extern void (GLApi *glGetActiveAttribARB)(GLhandleARB programObj, GLuint index, GLsizei maxLength, GLsizei *length, int *size, GLenum *type, GLcharARB *name);
extern int (GLApi *glGetAttribLocationARB)(GLhandleARB programObj, const GLcharARB *name);
extern void (GLApi *glGetVertexAttribdvARB)(GLuint index, GLenum pname, double *params);
extern void (GLApi *glGetVertexAttribfvARB)(GLuint index, GLenum pname, float *params);
extern void (GLApi *glGetVertexAttribivARB)(GLuint index, GLenum pname, int *params);
extern void (GLApi *glGetVertexAttribPointervARB)(GLuint index, GLenum pname, void **pointer);

#endif //GL_ARB_vertex_shader

/*
 *				--- GL_ARB_fragment_shader ---
 */

#ifndef GL_ARB_fragment_shader
#define GL_ARB_fragment_shader 1
#define __GENERATE_GL_ARB_fragment_shader__

#define GL_FRAGMENT_SHADER_ARB						0x8B30

#define GL_MAX_FRAGMENT_UNIFORM_COMPONENTS_ARB		0x8B49
#define GL_MAX_TEXTURE_COORDS_ARB					0x8871
#define GL_MAX_TEXTURE_IMAGE_UNITS_ARB				0x8872

#define GL_FRAGMENT_SHADER_DERIVATIVE_HINT_ARB		0x8B8B

// (no new functions)

#endif //GL_ARB_fragment_shader

/*
 *				--- GL_ARB_shader_object ---
 */

#ifndef GL_ARB_shader_object
#define GL_ARB_shader_object 1
#define __GENERATE_GL_ARB_shader_object__

#define GL_PROGRAM_OBJECT_ARB						0x8B40

#define GL_OBJECT_TYPE_ARB							0x8B4E
#define GL_OBJECT_SUBTYPE_ARB						0x8B4F
#define GL_OBJECT_DELETE_STATUS_ARB					0x8B80
#define GL_OBJECT_COMPILE_STATUS_ARB				0x8B81
#define GL_OBJECT_LINK_STATUS_ARB					0x8B82
#define GL_OBJECT_VALIDATE_STATUS_ARB				0x8B83
#define GL_OBJECT_INFO_LOG_LENGTH_ARB				0x8B84
#define GL_OBJECT_ATTACHED_OBJECTS_ARB				0x8B85
#define GL_OBJECT_ACTIVE_UNIFORMS_ARB				0x8B86
#define GL_OBJECT_ACTIVE_UNIFORM_MAX_LENGTH_ARB		0x8B87
#define GL_OBJECT_SHADER_SOURCE_LENGTH_ARB			0x8B88

#define GL_SHADER_OBJECT_ARB						0x8B48

#define GL_FLOAT									0x1406
#define GL_FLOAT_VEC2_ARB							0x8B50
#define GL_FLOAT_VEC3_ARB							0x8B51
#define GL_FLOAT_VEC4_ARB							0x8B52
#define GL_INT										0x1404
#define GL_INT_VEC2_ARB								0x8B53
#define GL_INT_VEC3_ARB								0x8B54
#define GL_INT_VEC4_ARB								0x8B55
#define GL_BOOL_ARB									0x8B56
#define GL_BOOL_VEC2_ARB							0x8B57
#define GL_BOOL_VEC3_ARB							0x8B58
#define GL_BOOL_VEC4_ARB							0x8B59
#define GL_FLOAT_MAT2_ARB							0x8B5A
#define GL_FLOAT_MAT3_ARB							0x8B5B
#define GL_FLOAT_MAT4_ARB							0x8B5C
#define GL_SAMPLER_1D_ARB							0x8B5D
#define GL_SAMPLER_2D_ARB							0x8B5E
#define GL_SAMPLER_3D_ARB							0x8B5F
#define GL_SAMPLER_CUBE_ARB							0x8B60
#define GL_SAMPLER_1D_SHADOW_ARB					0x8B61
#define GL_SAMPLER_2D_SHADOW_ARB					0x8B62
#define GL_SAMPLER_2D_RECT_ARB						0x8B63
#define GL_SAMPLER_2D_RECT_SHADOW_ARB				0x8B64

// shader objects
extern void (GLApi *glDeleteObjectARB)(GLhandleARB obj);
extern GLhandleARB (GLApi *glGetHandleARB)(GLenum pname);
extern void (GLApi *glDetachObjectARB)(GLhandleARB containerObj, GLhandleARB attachedObj);
extern GLhandleARB (GLApi *glCreateShaderObjectARB)(GLenum shaderType);
extern void (GLApi *glShaderSourceARB)(GLhandleARB shaderObj, GLsizei count, const GLcharARB **string, const int *length);
extern void (GLApi *glCompileShaderARB)(GLhandleARB shaderObj);
extern GLhandleARB (GLApi *glCreateProgramObjectARB)(void);
extern void (GLApi *glAttachObjectARB)(GLhandleARB containerObj, GLhandleARB obj);
extern void (GLApi *glLinkProgramARB)(GLhandleARB programObj);
extern void (GLApi *glUseProgramObjectARB)(GLhandleARB programObj);
extern void (GLApi *glValidateProgramARB)(GLhandleARB programObj);
extern void (GLApi *glUniform1fARB)(int location, float v0);
extern void (GLApi *glUniform2fARB)(int location, float v0, float v1);
extern void (GLApi *glUniform3fARB)(int location, float v0, float v1, float v2);
extern void (GLApi *glUniform4fARB)(int location, float v0, float v1, float v2, float v3);
extern void (GLApi *glUniform1iARB)(int location, int v0);
extern void (GLApi *glUniform2iARB)(int location, int v0, int v1);
extern void (GLApi *glUniform3iARB)(int location, int v0, int v1, int v2);
extern void (GLApi *glUniform4iARB)(int location, int v0, int v1, int v2, int v3);
extern void (GLApi *glUniform1fvARB)(int location, GLsizei count, const float *value);
extern void (GLApi *glUniform2fvARB)(int location, GLsizei count, const float *value);
extern void (GLApi *glUniform3fvARB)(int location, GLsizei count, const float *value);
extern void (GLApi *glUniform4fvARB)(int location, GLsizei count, const float *value);
extern void (GLApi *glUniform1ivARB)(int location, GLsizei count, const int *value);
extern void (GLApi *glUniform2ivARB)(int location, GLsizei count, const int *value);
extern void (GLApi *glUniform3ivARB)(int location, GLsizei count, const int *value);
extern void (GLApi *glUniform4ivARB)(int location, GLsizei count, const int *value);
extern void (GLApi *glUniformMatrix2fvARB)(int location, GLsizei count, GLboolean transpose, const float *value);
extern void (GLApi *glUniformMatrix3fvARB)(int location, GLsizei count, GLboolean transpose, const float *value);
extern void (GLApi *glUniformMatrix4fvARB)(int location, GLsizei count, GLboolean transpose, const float *value);
extern void (GLApi *glGetObjectParameterfvARB)(GLhandleARB obj, GLenum pname, float *params);
extern void (GLApi *glGetObjectParameterivARB)(GLhandleARB obj, GLenum pname, int *params);
extern void (GLApi *glGetInfoLogARB)(GLhandleARB obj, GLsizei maxLength, GLsizei *length, GLcharARB *infoLog);
extern void (GLApi *glGetAttachedObjectsARB)(GLhandleARB containerObj, GLsizei maxCount, GLsizei *count, GLhandleARB *obj);
extern int (GLApi *glGetUniformLocationARB)(GLhandleARB programObj, const GLcharARB *name);
extern void (GLApi *glGetActiveUniformARB)(GLhandleARB programObj, GLuint index, GLsizei maxLength, GLsizei *length, int *size, GLenum *type, GLcharARB *name);
extern void (GLApi *glGetUniformfvARB)(GLhandleARB programObj, int location, float *params);
extern void (GLApi *glGetUniformivARB)(GLhandleARB programObj, int location, int *params);
extern void (GLApi *glGetShaderSourceARB)(GLhandleARB obj, GLsizei maxLength, GLsizei *length, GLcharARB *source);

#endif //GL_ARB_shader_object

/*
 *								--- GL_ARB_vertex_buffer_object ---
 */

#ifndef GL_ARB_vertex_buffer_object
#define GL_ARB_vertex_buffer_object 1
#define __GENERATE_GL_ARB_vertex_buffer_object__

// Accepted by the <target> parameters of BindBufferARB, BufferDataARB,
// BufferSubDataARB, MapBufferARB, UnmapBufferARB, GetBufferSubDataARB,
// GetBufferParameterivARB, and GetBufferPointervARB:

#define GL_ARRAY_BUFFER_ARB 0x8892
#define GL_ELEMENT_ARRAY_BUFFER_ARB 0x8893

// Accepted by the <pname> parameter of GetBooleanv, GetIntegerv, GetFloatv, and GetDoublev:

#define GL_ARRAY_BUFFER_BINDING_ARB 0x8894
#define GL_ELEMENT_ARRAY_BUFFER_BINDING_ARB 0x8895
#define GL_VERTEX_ARRAY_BUFFER_BINDING_ARB 0x8896
#define GL_NORMAL_ARRAY_BUFFER_BINDING_ARB 0x8897
#define GL_COLOR_ARRAY_BUFFER_BINDING_ARB 0x8898
#define GL_INDEX_ARRAY_BUFFER_BINDING_ARB 0x8899
#define GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING_ARB 0x889A
#define GL_EDGE_FLAG_ARRAY_BUFFER_BINDING_ARB 0x889B
#define GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING_ARB 0x889C
#define GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING_ARB 0x889D
#define GL_WEIGHT_ARRAY_BUFFER_BINDING_ARB 0x889E

// Accepted by the <pname> parameter of GetVertexAttribivARB:

#define GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING_ARB 0x889F

// Accepted by the <usage> parameter of BufferDataARB:

#define GL_STREAM_DRAW_ARB 0x88E0
#define GL_STREAM_READ_ARB 0x88E1
#define GL_STREAM_COPY_ARB 0x88E2
#define GL_STATIC_DRAW_ARB 0x88E4
#define GL_STATIC_READ_ARB 0x88E5
#define GL_STATIC_COPY_ARB 0x88E6
#define GL_DYNAMIC_DRAW_ARB 0x88E8
#define GL_DYNAMIC_READ_ARB 0x88E9
#define GL_DYNAMIC_COPY_ARB 0x88EA

// Accepted by the <access> parameter of MapBufferARB:

#define GL_READ_ONLY_ARB 0x88B8
#define GL_WRITE_ONLY_ARB 0x88B9
#define GL_READ_WRITE_ARB 0x88BA

// Accepted by the <pname> parameter of GetBufferParameterivARB:

#define GL_BUFFER_SIZE_ARB 0x8764
#define GL_BUFFER_USAGE_ARB 0x8765
#define GL_BUFFER_ACCESS_ARB 0x88BB
#define GL_BUFFER_MAPPED_ARB 0x88BC

// Accepted by the <pname> parameter of GetBufferPointervARB:

#define GL_BUFFER_MAP_POINTER_ARB 0x88BD

// new procedures and functions (contained in GL 1.5core as well)
extern void (GLApi *glBindBufferARB)(GLenum target, GLuint buffer);
extern void (GLApi *glDeleteBuffersARB)(GLsizei n, const GLuint *buffers);
extern void (GLApi *glGenBuffersARB)(GLsizei n, GLuint *buffers);
extern GLboolean (GLApi *glIsBufferARB)(GLuint buffer);
extern void (GLApi *glBufferDataARB)(GLenum target, GLsizeiptrARB size, const void *data, GLenum usage);
extern void (GLApi *glBufferSubDataARB)(GLenum target, GLintptrARB offset, GLsizeiptrARB size, const void *data);
extern void (GLApi *glGetBufferSubDataARB)(GLenum target, GLintptrARB offset, GLsizeiptrARB size, void *data);
extern void *(GLApi *glMapBufferARB)(GLenum target, GLenum access);
extern GLboolean (GLApi *glUnmapBufferARB)(GLenum target);
extern void (GLApi *glGetBufferParameterivARB)(GLenum target, GLenum pname, int *params);
extern void (GLApi *glGetBufferPointervARB)(GLenum target, GLenum pname, void **params);

#endif //GL_ARB_vertex_buffer_object

/*
 *								--- GL_ARB_pixel_buffer_object ---
 */

#ifndef GL_ARB_pixel_buffer_object
#define GL_ARB_pixel_buffer_object 1

// Accepted by the <target> parameters of BindBuffer, BufferData,
// BufferSubData, MapBuffer, UnmapBuffer, GetBufferSubData,
// GetBufferParameteriv, and GetBufferPointerv:

#define GL_PIXEL_PACK_BUFFER_ARB                        0x88EB
#define GL_PIXEL_UNPACK_BUFFER_ARB                      0x88EC

// Accepted by the <pname> parameter of GetBooleanv, GetIntegerv,
// GetFloatv, and GetDoublev:

#define GL_PIXEL_PACK_BUFFER_BINDING_ARB                0x88ED
#define GL_PIXEL_UNPACK_BUFFER_BINDING_ARB              0x88EF

#endif //GL_ARB_pixel_buffer_object

/*
 *								--- GL_NV_fence ---
 */

#ifndef GL_NV_fence
#define GL_NV_fence 1
#define __GENERATE_GL_NV_fence__

// Accepted by the <condition> parameter of SetFenceNV:
#define GL_ALL_COMPLETED_NV                   0x84F2

// Accepted by the <pname> parameter of GetFenceivNV:
#define GL_FENCE_STATUS_NV                    0x84F3
#define GL_FENCE_CONDITION_NV                 0x84F4

// New Procedures and Functions
extern void (GLApi *glGenFencesNV)(GLsizei n, GLuint *fences);
extern void (GLApi *glDeleteFencesNV)(GLsizei n, const GLuint *fences);
extern void (GLApi *glSetFenceNV)(GLuint fence, GLenum condition);
extern GLboolean (GLApi *glTestFenceNV)(GLuint fence);
extern void (GLApi *glFinishFenceNV)(GLuint fence);
extern GLboolean (GLApi *glIsFenceNV)(GLuint fence);
extern void (GLApi *glGetFenceivNV)(GLuint fence, GLenum pname, GLint *params);

#endif //GL_NV_fence

/*
 *								--- GL_ARB_occlusion_query ---
 */

#ifndef GL_ARB_occlusion_query
#define GL_ARB_occlusion_query 1
#define __GENERATE_GL_ARB_occlusion_query__

// Accepted by the <target> parameter of BeginQueryARB, EndQueryARB,
// and GetQueryivARB:

#define GL_SAMPLES_PASSED_ARB                             0x8914

// Accepted by the <pname> parameter of GetQueryivARB:

#define GL_QUERY_COUNTER_BITS_ARB                         0x8864
#define GL_CURRENT_QUERY_ARB                              0x8865

// Accepted by the <pname> parameter of GetQueryObjectivARB and
// GetQueryObjectuivARB:

#define GL_QUERY_RESULT_ARB                               0x8866
#define GL_QUERY_RESULT_AVAILABLE_ARB                     0x8867

// New Procedures and Functions

extern void (GLApi *glGenQueriesARB)(GLsizei n, GLuint *ids);
extern void (GLApi *glDeleteQueriesARB)(GLsizei n, const GLuint *ids);
extern GLboolean (GLApi *glIsQueryARB)(GLuint id);
extern void (GLApi *glBeginQueryARB)(GLenum target, GLuint id);
extern void (GLApi *glEndQueryARB)(GLenum target);
extern void (GLApi *glGetQueryivARB)(GLenum target, GLenum pname, GLint *params);
extern void (GLApi *glGetQueryObjectivARB)(GLuint id, GLenum pname, GLint *params);
extern void (GLApi *glGetQueryObjectuivARB)(GLuint id, GLenum pname, GLuint *params);

#endif //GL_ARB_occlusion_query

/*
 *								--- OpenGL 2.1 ---
 */

#ifndef GL_VERSION_2_1
#define GL_VERSION_2_1 1
#define __GENERATE_GL_21__

#define GL_CURRENT_RASTER_SECONDARY_COLOR 0x845F
#define GL_PIXEL_PACK_BUFFER              0x88EB
#define GL_PIXEL_UNPACK_BUFFER            0x88EC
#define GL_PIXEL_PACK_BUFFER_BINDING      0x88ED
#define GL_PIXEL_UNPACK_BUFFER_BINDING    0x88EF
#define GL_FLOAT_MAT2x3                   0x8B65
#define GL_FLOAT_MAT2x4                   0x8B66
#define GL_FLOAT_MAT3x2                   0x8B67
#define GL_FLOAT_MAT3x4                   0x8B68
#define GL_FLOAT_MAT4x2                   0x8B69
#define GL_FLOAT_MAT4x3                   0x8B6A
#define GL_SRGB                           0x8C40
#define GL_SRGB8                          0x8C41
#define GL_SRGB_ALPHA                     0x8C42
#define GL_SRGB8_ALPHA8                   0x8C43
#define GL_SLUMINANCE_ALPHA               0x8C44
#define GL_SLUMINANCE8_ALPHA8             0x8C45
#define GL_SLUMINANCE                     0x8C46
#define GL_SLUMINANCE8                    0x8C47
#define GL_COMPRESSED_SRGB                0x8C48
#define GL_COMPRESSED_SRGB_ALPHA          0x8C49
#define GL_COMPRESSED_SLUMINANCE          0x8C4A
#define GL_COMPRESSED_SLUMINANCE_ALPHA    0x8C4B

extern void (GLApi *glUniformMatrix2x3fv)(GLint location, GLsizei count, GLboolean transpose, const GLfloat *value);
extern void (GLApi *glUniformMatrix3x2fv)(GLint location, GLsizei count, GLboolean transpose, const GLfloat *value);
extern void (GLApi *glUniformMatrix2x4fv)(GLint location, GLsizei count, GLboolean transpose, const GLfloat *value);
extern void (GLApi *glUniformMatrix4x2fv)(GLint location, GLsizei count, GLboolean transpose, const GLfloat *value);
extern void (GLApi *glUniformMatrix3x4fv)(GLint location, GLsizei count, GLboolean transpose, const GLfloat *value);
extern void (GLApi *glUniformMatrix4x3fv)(GLint location, GLsizei count, GLboolean transpose, const GLfloat *value);

#endif //GL_VERSION_2_1

/*
 *								--- GL_EXT_timer_query ---
 */

#ifndef GL_EXT_timer_query
#define GL_EXT_timer_query 1
#define __GENERATE_GL_EXT_timer_query__

// Accepted by the <target> parameter of BeginQuery, EndQuery, and GetQueryiv:

#define GL_TIME_ELAPSED_EXT                               0x88BF

// New Procedures and Functions

extern void (GLApi *glGetQueryObjecti64vEXT)(GLuint id, GLenum pname, GLint64EXT *params);
extern void (GLApi *glGetQueryObjectui64vEXT)(GLuint id, GLenum pname, GLuint64EXT *params);

#endif //GL_EXT_timer_query

/*
 *								--- GL_EXT_draw_buffers2 ---
 */

#ifndef GL_EXT_draw_buffers2
#define GL_EXT_draw_buffers2 1
#define __GENERATE_GL_EXT_draw_buffers2__

extern void (GLApi *glColorMaskIndexedEXT)(GLuint buf, GLboolean r, GLboolean g, GLboolean b, GLboolean a);
extern void (GLApi *glGetGLbooleanIndexedvEXT)(GLenum value, GLuint index, GLboolean *data);
extern void (GLApi *glGetIntegerIndexedvEXT)(GLenum value, GLuint index, GLint *data);
extern void (GLApi *glEnableIndexedEXT)(GLenum target, GLuint index);
extern void (GLApi *glDisableIndexedEXT)(GLenum target, GLuint index);
extern GLboolean (GLApi *glIsEnabledIndexedEXT)(GLenum target, GLuint index);

#endif //GL_EXT_draw_buffers2

/*
 *								--- GL_EXT_texture_integer ---
 */

#ifndef GL_EXT_texture_integer
#define GL_EXT_texture_integer 1
#define __GENERATE_GL_EXT_texture_integer__

// New Tokens

// Accepted by the <pname> parameters of GetBooleanv, GetIntegerv,
// GetFloatv, and GetDoublev:

#define GL_RGBA_INTEGER_MODE_EXT                           0x8D9E

// Accepted by the <internalFormat> parameter of TexImage1D,
// TexImage2D, and TexImage3D:

#define GL_RGBA32UI_EXT                                    0x8D70
#define GL_RGB32UI_EXT                                     0x8D71
#define GL_ALPHA32UI_EXT                                   0x8D72
#define GL_INTENSITY32UI_EXT                               0x8D73
#define GL_LUMINANCE32UI_EXT                               0x8D74
#define GL_LUMINANCE_ALPHA32UI_EXT                         0x8D75

#define GL_RGBA16UI_EXT                                    0x8D76
#define GL_RGB16UI_EXT                                     0x8D77
#define GL_ALPHA16UI_EXT                                   0x8D78
#define GL_INTENSITY16UI_EXT                               0x8D79
#define GL_LUMINANCE16UI_EXT                               0x8D7A
#define GL_LUMINANCE_ALPHA16UI_EXT                         0x8D7B

#define GL_RGBA8UI_EXT                                     0x8D7C
#define GL_RGB8UI_EXT                                      0x8D7D
#define GL_ALPHA8UI_EXT                                    0x8D7E
#define GL_INTENSITY8UI_EXT                                0x8D7F
#define GL_LUMINANCE8UI_EXT                                0x8D80
#define GL_LUMINANCE_ALPHA8UI_EXT                          0x8D81

#define GL_RGBA32I_EXT                                     0x8D82
#define GL_RGB32I_EXT                                      0x8D83
#define GL_ALPHA32I_EXT                                    0x8D84
#define GL_INTENSITY32I_EXT                                0x8D85
#define GL_LUMINANCE32I_EXT                                0x8D86
#define GL_LUMINANCE_ALPHA32I_EXT                          0x8D87

#define GL_RGBA16I_EXT                                     0x8D88
#define GL_RGB16I_EXT                                      0x8D89
#define GL_ALPHA16I_EXT                                    0x8D8A
#define GL_INTENSITY16I_EXT                                0x8D8B
#define GL_LUMINANCE16I_EXT                                0x8D8C
#define GL_LUMINANCE_ALPHA16I_EXT                          0x8D8D

#define GL_RGBA8I_EXT                                      0x8D8E
#define GL_RGB8I_EXT                                       0x8D8F
#define GL_ALPHA8I_EXT                                     0x8D90
#define GL_INTENSITY8I_EXT                                 0x8D91
#define GL_LUMINANCE8I_EXT                                 0x8D92
#define GL_LUMINANCE_ALPHA8I_EXT                           0x8D93

// Accepted by the <format> parameter of TexImage1D, TexImage2D,
// TexImage3D, TexSubImage1D, TexSubImage2D, TexSubImage3D,
// DrawPixels and ReadPixels:

#define GL_RED_INTEGER_EXT                                 0x8D94
#define GL_GREEN_INTEGER_EXT                               0x8D95
#define GL_BLUE_INTEGER_EXT                                0x8D96
#define GL_ALPHA_INTEGER_EXT                               0x8D97
#define GL_RGB_INTEGER_EXT                                 0x8D98
#define GL_RGBA_INTEGER_EXT                                0x8D99
#define GL_BGR_INTEGER_EXT                                 0x8D9A
#define GL_BGRA_INTEGER_EXT                                0x8D9B
#define GL_LUMINANCE_INTEGER_EXT                           0x8D9C
#define GL_LUMINANCE_ALPHA_INTEGER_EXT                     0x8D9D

// New Procedures and Functions

extern void (GLApi *glClearColorIiEXT)(GLint r, GLint g, GLint b, GLint a);
extern void (GLApi *glClearColorIuiEXT)(GLuint r, GLuint g, GLuint b, GLuint a);
extern void (GLApi *glTexParameterIivEXT)(GLenum target, GLenum pname, GLint *params);
extern void (GLApi *glTexParameterIuivEXT)(GLenum target, GLenum pname, GLuint *params);
extern void (GLApi *glGetTexParameterIivEXT)(GLenum target, GLenum pname, GLint *params);
extern void (GLApi *glGetTexParameterIiuvEXT)(GLenum target, GLenum pname, GLuint *params);

#endif //GL_EXT_texture_integer

/*
 *								--- GL_EXT_gpu_shader4 ---
 */

#ifndef GL_EXT_gpu_shader4
#define GL_EXT_gpu_shader4 1
#define __GENERATE_GL_EXT_gpu_shader4__

// New Tokens

// Accepted by the <pname> parameters of GetVertexAttribdv,
// GetVertexAttribfv, GetVertexAttribiv, GetVertexAttribIuivEXT and
// GetVertexAttribIivEXT:

#define GL_VERTEX_ATTRIB_ARRAY_INTEGER_EXT                    0x88FD

// Returned by the <type> parameter of GetActiveUniform:

#define GL_SAMPLER_1D_ARRAY_EXT                               0x8DC0
#define GL_SAMPLER_2D_ARRAY_EXT                               0x8DC1
#define GL_SAMPLER_BUFFER_EXT                                 0x8DC2
#define GL_SAMPLER_1D_ARRAY_SHADOW_EXT                        0x8DC3
#define GL_SAMPLER_2D_ARRAY_SHADOW_EXT                        0x8DC4
#define GL_SAMPLER_CUBE_SHADOW_EXT                            0x8DC5
#define GL_UNSIGNED_INT                                       0x1405
#define GL_UNSIGNED_INT_VEC2_EXT                              0x8DC6
#define GL_UNSIGNED_INT_VEC3_EXT                              0x8DC7
#define GL_UNSIGNED_INT_VEC4_EXT                              0x8DC8
#define GL_INT_SAMPLER_1D_EXT                                 0x8DC9
#define GL_INT_SAMPLER_2D_EXT                                 0x8DCA
#define GL_INT_SAMPLER_3D_EXT                                 0x8DCB
#define GL_INT_SAMPLER_CUBE_EXT                               0x8DCC
#define GL_INT_SAMPLER_2D_RECT_EXT                            0x8DCD
#define GL_INT_SAMPLER_1D_ARRAY_EXT                           0x8DCE
#define GL_INT_SAMPLER_2D_ARRAY_EXT                           0x8DCF
#define GL_INT_SAMPLER_BUFFER_EXT                             0x8DD0
#define GL_UNSIGNED_INT_SAMPLER_1D_EXT                        0x8DD1
#define GL_UNSIGNED_INT_SAMPLER_2D_EXT                        0x8DD2
#define GL_UNSIGNED_INT_SAMPLER_3D_EXT                        0x8DD3
#define GL_UNSIGNED_INT_SAMPLER_CUBE_EXT                      0x8DD4
#define GL_UNSIGNED_INT_SAMPLER_2D_RECT_EXT                   0x8DD5
#define GL_UNSIGNED_INT_SAMPLER_1D_ARRAY_EXT                  0x8DD6
#define GL_UNSIGNED_INT_SAMPLER_2D_ARRAY_EXT                  0x8DD7
#define GL_UNSIGNED_INT_SAMPLER_BUFFER_EXT                    0x8DD8

// Accepted by the <pname> parameter of GetBooleanv, GetIntegerv, GetFloatv,
// and GetDoublev:

#define GL_MIN_PROGRAM_TEXEL_OFFSET_EXT                       0x8904
#define GL_MAX_PROGRAM_TEXEL_OFFSET_EXT                       0x8905

// New Procedures and Functions

extern void (GLApi *glVertexAttribI1iEXT)(GLuint index, GLint x);
extern void (GLApi *glVertexAttribI2iEXT)(GLuint index, GLint x, GLint y);
extern void (GLApi *glVertexAttribI3iEXT)(GLuint index, GLint x, GLint y, GLint z);
extern void (GLApi *glVertexAttribI4iEXT)(GLuint index, GLint x, GLint y, GLint z, GLint w);
extern void (GLApi *glVertexAttribI1uiEXT)(GLuint index, GLuint x);
extern void (GLApi *glVertexAttribI2uiEXT)(GLuint index, GLuint x, GLuint y);
extern void (GLApi *glVertexAttribI3uiEXT)(GLuint index, GLuint x, GLuint y, GLuint z);
extern void (GLApi *glVertexAttribI4uiEXT)(GLuint index, GLuint x, GLuint y, GLuint z, GLuint w);
extern void (GLApi *glVertexAttribI1ivEXT)(GLuint index, const GLint *v);
extern void (GLApi *glVertexAttribI2ivEXT)(GLuint index, const GLint *v);
extern void (GLApi *glVertexAttribI3ivEXT)(GLuint index, const GLint *v);
extern void (GLApi *glVertexAttribI4ivEXT)(GLuint index, const GLint *v);
extern void (GLApi *glVertexAttribI1uivEXT)(GLuint index, const GLuint *v);
extern void (GLApi *glVertexAttribI2uivEXT)(GLuint index, const GLuint *v);
extern void (GLApi *glVertexAttribI3uivEXT)(GLuint index, const GLuint *v);
extern void (GLApi *glVertexAttribI4uivEXT)(GLuint index, const GLuint *v);
extern void (GLApi *glVertexAttribI4bvEXT)(GLuint index, const GLbyte *v);
extern void (GLApi *glVertexAttribI4svEXT)(GLuint index, const GLshort *v);
extern void (GLApi *glVertexAttribI4ubvEXT)(GLuint index, const GLubyte *v);
extern void (GLApi *glVertexAttribI4usvEXT)(GLuint index, const GLushort *v);
extern void (GLApi *glVertexAttribIPointerEXT)(GLuint index,
	GLint size, GLenum type, GLsizei stride, const void *pointer);
extern void (GLApi *glGetVertexAttribIivEXT)(GLuint index, GLenum pname, GLint *params);
extern void (GLApi *glGetVertexAttribIuivEXT)(GLuint index, GLenum pname, GLuint *params);
extern void (GLApi *glUniform1uiEXT)(GLint location, GLuint v0);
extern void (GLApi *glUniform2uiEXT)(GLint location, GLuint v0, GLuint v1);
extern void (GLApi *glUniform3uiEXT)(GLint location, GLuint v0, GLuint v1, GLuint v2);
extern void (GLApi *glUniform4uiEXT)(GLint location, GLuint v0, GLuint v1, GLuint v2, GLuint v3);
extern void (GLApi *glUniform1uivEXT)(GLint location, GLsizei count, const GLuint *value);
extern void (GLApi *glUniform2uivEXT)(GLint location, GLsizei count, const GLuint *value);
extern void (GLApi *glUniform3uivEXT)(GLint location, GLsizei count, const GLuint *value);
extern void (GLApi *glUniform4uivEXT)(GLint location, GLsizei count, const GLuint *value);
extern void (GLApi *glGetUniformuivEXT)(GLuint program, GLint location, GLuint *params);
extern void (GLApi *glBindFragDataLocationEXT)(GLuint program, GLuint colorNumber, const char *name);
extern int (GLApi *glGetFragDataLocationEXT)(GLuint program, const char *name);

#endif //GL_EXT_gpu_shader4

/*
 *								--- GL_EXT_gpu_program_parameters ---
 */

#ifndef GL_EXT_gpu_program_parameters
#define GL_EXT_gpu_program_parameters 1
#define __GENERATE_GL_EXT_gpu_program_parameters__

extern void (GLApi *glProgramEnvParameters4fvEXT)(GLenum target, GLuint index, GLsizei count, 
                               const GLfloat *params);
extern void (GLApi *glProgramLocalParameters4fvEXT)(GLenum target, GLuint index, GLsizei count, 
                                 const GLfloat *params);

#endif //GL_EXT_gpu_program_parameters

/*
 *								--- GL_EXT_bindable_uniform ---
 */

#ifndef GL_EXT_bindable_uniform
#define GL_EXT_bindable_uniform 1
#define __GENERATE_GL_EXT_bindable_uniform__

// New Tokens

// Accepted by the <pname> parameter of GetBooleanv, GetIntegerv, GetFloatv,
// and GetDoublev:

#define GL_MAX_VERTEX_BINDABLE_UNIFORMS_EXT                0x8DE2
#define GL_MAX_FRAGMENT_BINDABLE_UNIFORMS_EXT              0x8DE3
#define GL_MAX_GEOMETRY_BINDABLE_UNIFORMS_EXT              0x8DE4
#define GL_MAX_BINDABLE_UNIFORM_SIZE_EXT                   0x8DED
#define GL_UNIFORM_BUFFER_BINDING_EXT                      0x8DEF

// Accepted by the <target> parameters of BindBuffer, BufferData,
// BufferSubData, MapBuffer, UnmapBuffer, GetBufferSubData, and
// GetBufferPointerv:

#define GL_UNIFORM_BUFFER_EXT                              0x8DEE

// New Procedures and Functions

extern void (GLApi *glUniformBufferEXT)(GLuint program, GLint location, GLuint buffer);
extern int (GLApi *glGetUniformBufferSizeEXT)(GLuint program, GLint location);
extern GLintptr (GLApi *glGetUniformOffsetEXT)(GLuint program, GLint location);

#endif //GL_EXT_bindable_uniform

/*
 *								--- GL_EXT_geometry_shader4 ---
 */

#ifndef GL_EXT_geometry_shader4
#define GL_EXT_geometry_shader4 1
#define __GENERATE_GL_EXT_geometry_shader4__

// New Tokens

// Accepted by the <type> parameter of CreateShader and returned by the
// <params> parameter of GetShaderiv:

#define GL_GEOMETRY_SHADER_EXT                             0x8DD9

// Accepted by the <pname> parameter of ProgramParameteriEXT and
// GetProgramiv:

#define GL_GEOMETRY_VERTICES_OUT_EXT                       0x8DDA
#define GL_GEOMETRY_INPUT_TYPE_EXT                         0x8DDB
#define GL_GEOMETRY_OUTPUT_TYPE_EXT                        0x8DDC

// Accepted by the <pname> parameter of GetBooleanv, GetIntegerv,
// GetFloatv, and GetDoublev:

#define GL_MAX_GEOMETRY_TEXTURE_IMAGE_UNITS_EXT             0x8C29
#define GL_MAX_GEOMETRY_VARYING_COMPONENTS_EXT              0x8DDD
#define GL_MAX_VERTEX_VARYING_COMPONENTS_EXT                0x8DDE
#define GL_MAX_VARYING_COMPONENTS_EXT                       0x8B4B
#define GL_MAX_GEOMETRY_UNIFORM_COMPONENTS_EXT              0x8DDF
#define GL_MAX_GEOMETRY_OUTPUT_VERTICES_EXT                 0x8DE0
#define GL_MAX_GEOMETRY_TOTAL_OUTPUT_COMPONENTS_EXT         0x8DE1

// Accepted by the <mode> parameter of Begin, DrawArrays,
// MultiDrawArrays, DrawElements, MultiDrawElements, and
// DrawRangeElements:

#ifndef GL_NV_geometry_program4 // redefinition from glext.h
#define GL_NV_geometry_program4
#define GL_LINES_ADJACENCY_EXT                              0xA
#define GL_LINE_STRIP_ADJACENCY_EXT                         0xB
#define GL_TRIANGLES_ADJACENCY_EXT                          0xC
#define GL_TRIANGLE_STRIP_ADJACENCY_EXT                     0xD
#endif

// Returned by CheckFramebufferStatusEXT:

#define GL_FRAMEBUFFER_INCOMPLETE_LAYER_TARGETS_EXT         0x8DA8
#define GL_FRAMEBUFFER_INCOMPLETE_LAYER_COUNT_EXT           0x8DA9

// Accepted by the <pname> parameter of GetFramebufferAttachment-
// ParameterivEXT:

#define GL_FRAMEBUFFER_ATTACHMENT_LAYERED_EXT               0x8DA7
#define GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LAYER_EXT         0x8CD4

// Accepted by the <cap> parameter of Enable, Disable, and IsEnabled,
// and by the <pname> parameter of GetIntegerv, GetFloatv, GetDoublev,
// and GetBooleanv:

#define GL_PROGRAM_POINT_SIZE_EXT                           0x8642

// (Note: FRAMEBUFFER_ATTACHMENT_TEXTURE_LAYER_EXT is simply an alias for the
// FRAMEBUFFER_ATTACHMENT_TEXTURE_3D_ZOFFSET_EXT token provided in
// EXT_framebuffer_object.  This extension generalizes the notion of
// "<zoffset>" to include layers of an array texture.)

// (Note:  PROGRAM_POINT_SIZE_EXT is simply an alias for the
// VERTEX_PROGRAM_POINT_SIZE token provided in OpenGL 2.0, which is itself an
// alias for VERTEX_PROGRAM_POINT_SIZE_ARB provided by
// ARB_vertex_program. Program-computed point sizes can be enabled if
// geometry shaders are enabled.)

// New Procedures and Functions

extern void (GLApi *glProgramParameteriEXT)(GLuint program,
	GLenum pname, GLint value);
extern void (GLApi *glFramebufferTextureEXT)(GLenum target,
	GLenum attachment, GLuint texture, GLint level);
extern void (GLApi *glFramebufferTextureLayerEXT)(GLenum target,
	GLenum attachment, GLuint texture, GLint level, GLint layer);
extern void (GLApi *glFramebufferTextureFaceEXT)(GLenum target,
	GLenum attachment, GLuint texture, GLint level, GLenum face);

#endif //GL_EXT_geometry_shader4

/*
 *								--- GL_EXT_draw_instanced ---
 */

#ifndef GL_EXT_draw_instanced
#define GL_EXT_draw_instanced 1
#define __GENERATE_GL_EXT_draw_instanced__

extern void (GLApi *glDrawArraysInstancedEXT)(GLenum mode,
	GLint first, GLsizei count, GLsizei primcount);
extern void (GLApi *glDrawElementsInstancedEXT)(GLenum mode,
	GLsizei count, GLenum type, const void *indices, GLsizei primcount);

#endif //GL_EXT_draw_instanced

/*
 *								--- GL_EXT_texture_array ---
 */

#ifndef GL_EXT_texture_array
#define GL_EXT_texture_array 1
#define __GENERATE_GL_EXT_texture_array__

// New Tokens

// Accepted by the <target> parameter of TexParameteri, TexParameteriv,
// TexParameterf, TexParameterfv, and BindTexture:

#define GL_TEXTURE_1D_ARRAY_EXT                            0x8C18
#define GL_TEXTURE_2D_ARRAY_EXT                            0x8C1A

// Accepted by the <target> parameter of TexImage3D, TexSubImage3D,
// CopyTexSubImage3D, CompressedTexImage3D, and CompressedTexSubImage3D:

#ifndef GL_EXT_texture_array
#define GL_TEXTURE_2D_ARRAY_EXT								0x8C1A
#endif
#define GL_PROXY_TEXTURE_2D_ARRAY_EXT                      0x8C1B

// Accepted by the <target> parameter of TexImage2D, TexSubImage2D,
// CopyTexImage2D, CopyTexSubImage2D, CompressedTexImage2D, and
// CompressedTexSubImage2D:

#ifndef GL_EXT_texture_array
#define GL_TEXTURE_1D_ARRAY_EXT								0x8C18
#endif
#define GL_PROXY_TEXTURE_1D_ARRAY_EXT                      0x8C19

// Accepted by the <pname> parameter of GetBooleanv, GetDoublev, GetIntegerv
// and GetFloatv:

#define GL_TEXTURE_BINDING_1D_ARRAY_EXT                    0x8C1C
#define GL_TEXTURE_BINDING_2D_ARRAY_EXT                    0x8C1D
#define GL_MAX_ARRAY_TEXTURE_LAYERS_EXT                    0x88FF

// Accepted by the <param> parameter of TexParameterf, TexParameteri,
// TexParameterfv, and TexParameteriv when the <pname> parameter is
// TEXTURE_COMPARE_MODE_ARB:

#define GL_COMPARE_REF_DEPTH_TO_TEXTURE_EXT                0x884E

// (Note:  COMPARE_REF_DEPTH_TO_TEXTURE_EXT is simply an alias for the
// existing COMPARE_R_TO_TEXTURE token in OpenGL 2.0; the alternate name
// reflects the fact that the R coordinate is not always used.)

// Accepted by the <internalformat> parameter of TexImage3D and
// CompressedTexImage3D, and by the <format> parameter of
// CompressedTexSubImage3D:

#ifndef GL_EXT_texture_compression_s3tc // macro redefinition from glext.h
#define GL_COMPRESSED_RGB_S3TC_DXT1_EXT  0x83F0
#define GL_COMPRESSED_RGBA_S3TC_DXT1_EXT  0x83F1
#define GL_COMPRESSED_RGBA_S3TC_DXT3_EXT  0x83F2
#define GL_COMPRESSED_RGBA_S3TC_DXT5_EXT  0x83F3
#endif

// Accepted by the <pname> parameter of
// GetFramebufferAttachmentParameterivEXT:

#define GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LAYER_EXT          0x8CD4

// (Note:  FRAMEBUFFER_ATTACHMENT_TEXTURE_LAYER is simply an alias for the
// FRAMEBUFFER_ATTACHMENT_TEXTURE_3D_ZOFFSET_EXT token provided in
// EXT_framebuffer_object.  This extension generalizes the notion of
// "<zoffset>" to include layers of an array texture.)

// Returned by the <type> parameter of GetActiveUniform:

#define GL_SAMPLER_1D_ARRAY_EXT                             0x8DC0
#define GL_SAMPLER_2D_ARRAY_EXT                             0x8DC1
#define GL_SAMPLER_1D_ARRAY_SHADOW_EXT                      0x8DC3
#define GL_SAMPLER_2D_ARRAY_SHADOW_EXT                      0x8DC4

// New Procedures and Functions

extern void (GLApi *glFramebufferTextureLayerEXT)(GLenum target, GLenum attachment,
	GLuint texture, GLint level, GLint layer);

#endif //GL_EXT_texture_array

/*
 *								--- GL_EXT_texture_buffer_object ---
 */

#ifndef GL_EXT_texture_buffer_object
#define GL_EXT_texture_buffer_object 1
#define __GENERATE_GL_EXT_texture_buffer_object__

// New Tokens

// Accepted by the <target> parameter of BindBuffer, BufferData,
// BufferSubData, MapBuffer, BindTexture, UnmapBuffer, GetBufferSubData,
// GetBufferParameteriv, GetBufferPointerv, and TexBufferEXT, and
// the <pname> parameter of GetBooleanv, GetDoublev, GetFloatv, and
// GetIntegerv:

#define GL_TEXTURE_BUFFER_EXT                              0x8C2A

// Accepted by the <pname> parameters of GetBooleanv, GetDoublev,
// GetFloatv, and GetIntegerv:

#define GL_MAX_TEXTURE_BUFFER_SIZE_EXT                     0x8C2B
#define GL_TEXTURE_BINDING_BUFFER_EXT                      0x8C2C
#define GL_TEXTURE_BUFFER_DATA_STORE_BINDING_EXT           0x8C2D
#define GL_TEXTURE_BUFFER_FORMAT_EXT                       0x8C2E

// New Procedures and Functions

extern void (GLApi *glTexBufferEXT)(GLenum target, GLenum internalformat, GLuint buffer);

#endif //GL_EXT_texture_buffer_object

/*
 *								--- GL_EXT_blend_minmax ---
 */

#ifndef GL_EXT_blend_minmax
#define GL_EXT_blend_minmax 1
#define __GENERATE_GL_EXT_blend_minmax__

// New Tokens
// Accepted by the <mode> parameter of BlendEquationEXT:

#define GL_FUNC_ADD_EXT                     0x8006
#define GL_MIN_EXT                          0x8007
#define GL_MAX_EXT                          0x8008

// Accepted by the <pname> parameter of GetBooleanv, GetIntegerv,
// GetFloatv, and GetDoublev:

#define GL_BLEND_EQUATION_EXT               0x8009

// New Procedures and Functions
extern void (GLApi *glBlendEquationEXT)(GLenum mode);

#endif //GL_EXT_blend_minmax

/*
 *								--- GL_EXT_blend_subtract ---
 */

#ifndef GL_EXT_blend_subtract
#define GL_EXT_blend_subtract 1

// New Tokens
// Accepted by the <mode> parameter of BlendEquationEXT:

#define GL_FUNC_SUBTRACT_EXT                0x800A
#define GL_FUNC_REVERSE_SUBTRACT_EXT        0x800B

#endif //GL_EXT_blend_subtract

/*
 *								--- GL_EXT_packed_depth_stencil ---
 */

#ifndef GL_EXT_packed_depth_stencil
#define GL_EXT_packed_depth_stencil 1

#define GL_DEPTH_STENCIL_EXT              0x84F9
#define GL_UNSIGNED_INT_24_8_EXT          0x84FA
#define GL_DEPTH24_STENCIL8_EXT           0x88F0
#define GL_TEXTURE_STENCIL_SIZE_EXT       0x88F1

#ifndef GL_FRAMEBUFFER_INCOMPLETE_DUPLICATE_ATTACHMENT_EXT
#define GL_FRAMEBUFFER_INCOMPLETE_DUPLICATE_ATTACHMENT_EXT   0x8CD8
#endif //GL_FRAMEBUFFER_INCOMPLETE_DUPLICATE_ATTACHMENT_EXT
// not defined in original extension, nor in GL_EXT_framebuffer_object

#endif //GL_EXT_packed_depth_stencil

#include "glext.h"

#endif // __OPENGL20_TOOL_INCLUDED
