/*
								+---------------------------------+
								|                                 |
								|      ***   OpenGlDrv   ***      |
								|                                 |
								|  Copyright   -tHE SWINe- 2002  |
								|                                 |
								|           OpenGlDrv.h           |
								|                                 |
								+---------------------------------+
*/

/*
 *	passed code revision at 2006-05-16
 *
 *	renamed arguments passed to CGLDriver::Init
 *	all integers storing only true or false values were replaced by bool
 *	erased all open-gl init code from CGLDriver::Init, but glDepthFunc(GL_LEQUAL); (default GL_LESS)
 *
 *	passed code revision at 2006-10-31
 *
 *	added bool CGLDriver::MakeCurrent()
 *	renamed bool CGLDriver::Terminate() to bool CGLDriver::Shutdown()
 *	propagated comments from .cpp to .h
 *
 *	passed code revision at 2006-11-01
 *
 *	added int CGLDriver::n_Width(), int CGLDriver::n_Height() and
 *	void CGLDriver::ResetViewport() functions
 *	parameters int n_depth_bpp and int n_stencil_bpp added to CGLDriver::Init function
 *
 *	passed code revision at 2006-11-13
 *
 *	fixed bug in n_depth_bpp and int n_stencil_bpp ussage in CGLDriver::Init function
 *	added CGLDriver::n_PixelFormat() and CGLDriver::t_Pixelormat() functions
 *	fixed function const-corectness
 *
 *	2007-03-06
 *
 *	removed __declspec(dllexport)
 *
 *	2008-03-04
 *
 *	__inline changed to inline
 *	added default value 32 of parameter n_bpp in CGLDriver::Init
 *
 */

#ifndef __GL_DRIVER_INCLUDED
#define __GL_DRIVER_INCLUDED

class CGLDriver {
    HDC m_h_dc;
    HGLRC m_h_glrc;
    HWND m_h_wnd;
    bool m_b_fullscreen;

    int m_n_width;
    int m_n_height;

    bool m_b_status;

	PIXELFORMATDESCRIPTOR m_t_pixel_format;
    GLuint m_n_pixel_format_id;

public:
	/*
	 *	CGLDriver::CGLDriver()
	 *		- default constructor
	 */
    CGLDriver();

	/*
	 *	CGLDriver::~CGLDriver()
	 *		- default destructor
	 */
    ~CGLDriver();

	/*
	 *	CGLDriver::Init(HWND h_wnd, int n_width, int n_height, int n_bpp = 32,
	 *		int n_depth_bpp = 24, int n_stencil_bpp = 0, bool b_fullscreen = false)
	 *		- create OpenGL context for window h_wnd
	 *		- display area should have size n_width per n_height pixels, with bit depth n_bpp
	 *		- neither window size nor position nor type is changed
	 *		- if b_fullscreen is true, it attempts to set display mode n_width per n_height
	 *		  pixels and bit depth n_bpp and to hide cursor
	 *		- return true on success otherwise false
	 *		- on failure it pops ut message box with error description
	 */
    bool Init(HWND h_wnd, int n_width, int n_height, int n_bpp = 32,
		int n_depth_bpp = 24, int n_stencil_bpp = 0, bool b_fullscreen = false);

	/*
	 *	bool CGLDriver::Shutdown()
	 *		- terminates previously initalized context
	 *		- return true if no context was initialized or on succes, otherwise false
	 *		- if fulscreen mode was set, sets default fullscreen mode and shows cursor
	 *		- on failure it pops ut message box with error description
	 */
    bool Shutdown();

	/*
	 *	bool CGLDriver::b_Status() const
	 *		- returns status of open-gl (false = not initialized, true = running)
	 */
	bool b_Status() const;

	/*
	 *	bool CGLDriver::MakeCurrent()
	 *		- makes this opengl context current
	 *		- returns true on success, false on failure
	 */
	bool MakeCurrent();

	/*
	 *	inline int CGLDriver::n_Width() const
	 *		- returns window width
	 */
	inline int n_Width() const { return m_n_width; }

	/*
	 *	inline int CGLDriver::n_Height() const
	 *		- returns window height
	 */
	inline int n_Height() const { return m_n_height; }

	/*
	 *	inline const PIXELFORMATDESCRIPTOR &t_PixelFormat() const
	 *		- returns pixel format
	 */
	inline const PIXELFORMATDESCRIPTOR &t_PixelFormat() const { return m_t_pixel_format; }

	/*
	 *	inline const GLuint n_PixelFormat() const
	 *		- returns OpenGL pixel format id
	 */
	inline GLuint n_PixelFormat() const { return m_n_pixel_format_id; }

	/*
	 *	void CGLDriver::ResetViewport() const
	 *		- calls glViewport() with window dimensions
	 */
	void ResetViewport() const;

	/*
	 *	void CGLDriver::Blit() const
	 *		- commits page flip (call glSwapBuffers())
	 *		- doesn't call neither glFlush() nor glFinish()
	 */
    void Blit() const;
};

#endif //__GL_DRIVER_INCLUDED
