/*
								+-----------------------------------+
								|                                   |
								|  ***  LRD wald-boost engine  ***  |
								|                                   |
								|   Copyright   -tHE SWINe- 2008   |
								|                                   |
								|           WaldBoost.cpp           |
								|                                   |
								+-----------------------------------+
*/

#include "./UberLame_src/CallStack.h"
#include "./UberLame_src/OpenGL20.h"
#include <GL/glut.h>
#include <math.h>
#include <time.h>
#include <stdio.h>
#include <vector>
#include <algorithm>
#include <functional>
#include "./UberLame_src/OpenGLState.h"
#include "./UberLame_src/Texture.h"
#include "./UberLame_src/Vector.h"
#include "./UberLame_src/RenderBuffer2.h"
#include "./UberLame_src/VertexBufferObject.h"
#include "./UberLame_src/ShaderSys.h"
#include "./UberLame_src/GPUImgProc.h"
#include "Load_XML.h"
#include "Convolve.h"
#include "WaldBoost.h"

//int n_fukwitdaddy_value = 1;

/*
 *								=== CAlphaTextureAtlas ===
 */

/**
 *	CAlphaTextureAtlas::CAlphaTextureAtlas(CGLState *p_state,
 *		const TWaldBoostClassifier &r_t_classifier,
 *		int n_max_texture_size = 512, bool b_allow_more_columns = false)
 *		- default constructor; creates texture atlas for classifier r_t_classifier
 *		  (ie. allocates row of pixels to store alpha table for each classifier
 *		  and creates those textures (uses GL_FLOAT_R16_NV or GL_RGB16F_ARB format))
 *		- n_max_texture_size is maximal texture size
 *		- setting b_allow_more_columns enables storing alphas in more than one column
 *		  (makes it possible to create wider textures, containing more alphas,
 *		  but requires x-coordinate offset for alpha texture fetch)
 *		- it's recommended to call b_Status() afterwards to check constructor success
 */
CAlphaTextureAtlas::CAlphaTextureAtlas(CGLState *p_state,
	const TWaldBoostClassifier &r_t_classifier,
	int n_max_texture_size, bool b_allow_more_columns)
{
	n_alpha_num = r_t_classifier.p_weak_classifier[0].n_alpha_num;
	// should be the same for all classifiers (checking that later on)

	if(n_alpha_num > n_max_texture_size) {
		n_alpha_num = -1; // to mark error
		return;
	}
	// texture must be wide enough to have alphas
	// for a single classifier on a single scanline

	int n_column_num = (b_allow_more_columns)? n_max_texture_size / n_alpha_num : 1;
	// calculate number of possible columns

	int n_alphas_per_page = n_max_texture_size * n_column_num;
	int n_page_num = (r_t_classifier.n_weak_classifier_num +
		n_alphas_per_page - 1) / n_alphas_per_page;
	// calculate number of texture pages

	texture_list.reserve(n_page_num);
	if(int(texture_list.capacity()) < n_page_num) {
		n_alpha_num = -1; // to mark error
		return;
	}
	// make sure there's space in the list

	GLenum n_tex_fmt = (CGLExtensionHandler::b_SupportedExtension(
		"GL_NV_float_buffer"))? GL_FLOAT_R16_NV : GL_RGB16F_ARB;
	// use single-channel textures, if supported

	for(int n_first = 0, n_left = r_t_classifier.n_weak_classifier_num; n_left;) {
		int n_alphas_on_page = min(n_left, n_alphas_per_page);
		// determine number of alphas on this page

		int n_columns_on_page = (n_alphas_on_page +
			n_max_texture_size - 1) / n_max_texture_size;
		// determine number of columns on page

		int n_avg_alphas_column = (n_alphas_on_page +
			n_columns_on_page - 1) / n_columns_on_page;
		// determine average number of alphas per column - texture height

		int n_alphas_last_column = n_alphas_on_page -
			n_avg_alphas_column * (n_columns_on_page - 1);
		_ASSERTE(!n_columns_on_page || n_alphas_last_column <= n_avg_alphas_column);
		_ASSERTE(n_columns_on_page || n_alphas_last_column == n_avg_alphas_column);
		// determine number of alphas in the last column

		int n_width = n_columns_on_page * n_alpha_num;
		int n_height = n_avg_alphas_column;
		_ASSERTE(n_width <= n_max_texture_size && n_height <= n_max_texture_size);
		// get texture size

		float *p_texture_data;
		if(!(p_texture_data = new float[n_width * n_height]))
			return;
		memset(p_texture_data, 0, n_width * n_height * sizeof(float));
		// allocate buffer for texture data

		const TWeakClassifier *p_classifier_ptr =
			r_t_classifier.p_weak_classifier + n_first;
		for(int j = 0; j < n_columns_on_page; ++ j) {
			for(int i = 0, n = (j + 1 < n_columns_on_page)?
			   n_avg_alphas_column : n_alphas_last_column; i < n; ++ i, ++ p_classifier_ptr) {
				memcpy(p_texture_data + (j * n_alpha_num) + i * n_width,
					p_classifier_ptr->p_alpha, n_alpha_num * sizeof(float));
			}
		}
		// copy alphas to the buffer

		TPage t_page;
		t_page.n_classifier_num = n_alphas_on_page;
		t_page.n_first_classifier = n_first;
		if(!(t_page.p_texture = new CGLTexture_Rect(p_state, n_width,
		   n_height, n_tex_fmt, GL_LUMINANCE, GL_FLOAT, p_texture_data)) ||
		   !t_page.p_texture->b_Status()) {
			delete[] p_texture_data;
			if(t_page.p_texture)
				delete t_page.p_texture;
			n_alpha_num = -1; // to mark error
			return;
		}
		delete[] p_texture_data;
		// alloc texture, fill page props

		CGLTextureParams &r_params = t_page.p_texture->r_Parameters(p_state);
		r_params.Set_Texture_Min_Filter(GL_NEAREST);
		r_params.Set_Texture_Mag_Filter(GL_NEAREST);
		// set nearest filtering

		/*const TWeakClassifier *p_classifier_ptr =
			r_t_classifier.p_weak_classifier + n_first;
		t_page.p_texture->Bind_Enable(p_state);
		for(int j = 0; j < n_columns_on_page; ++ j) {
			for(int i = 0, n = (j + 1 < n_columns_on_page)?
			   n_avg_alphas_column : n_alphas_last_column; i < n; ++ i, ++ p_classifier_ptr) {
				glTexSubImage2D(t_page.p_texture->n_Target(), 0, j * n_alpha_num, i,
					n_alpha_num, 1, GL_LUMINANCE, GL_FLOAT, p_classifier_ptr->p_alpha);
				// f_ixme - can use GL_LUMINANCE instead of GL_RED? - yes
			}
		}*/
		// fill the texture with alphas // note this yields bad results
		// when the texture is viewed in gdebugger (but works anyway)

		texture_list.push_back(t_page);
		// add to the list

		n_first += n_alphas_on_page;
		n_left -= n_alphas_on_page;
	}
}

/**
 *	CAlphaTextureAtlas::~CAlphaTextureAtlas()
 *		- destructor
 */
CAlphaTextureAtlas::~CAlphaTextureAtlas()
{
	std::for_each(texture_list.begin(), texture_list.end(), DeletePageTexture);
}

/**
 *	bool CAlphaTextureAtlas::b_Status() const
 *		- returns true if constructor succeeded and the atlas is ready to use,
 *		  otherwise returns false
 */
bool CAlphaTextureAtlas::b_Status() const
{
	return n_alpha_num >= 0;
}

/**
 *	int CAlphaTextureAtlas::n_Texture(int n_classifier_index, int &r_n_x, int &r_n_y) const
 *		- returns index of texture page (not OpenGL id) containing alphas of
 *		  weak classifier n_classifier_index
 *		- r_n_x and r_n_y are set to offset to the first pixel of alpha table
 *		  (table is oriented horizontally and next alphas are adressed by
 *		  increasing x-offset)
 *		- returns -1 in case n_classifier_index is greater or equal to number of
 *		  weak classifiers contained in classifier, passed to constructor
 */
int CAlphaTextureAtlas::n_Texture(int n_classifier_index, int &r_n_x, int &r_n_y) const
{
	for(int i = 0, n = texture_list.size(); i < n; ++ i) {
		const TPage &r_t_page = texture_list[i];
		if(n_classifier_index < r_t_page.n_first_classifier ||
		   n_classifier_index >= r_t_page.n_first_classifier + r_t_page.n_classifier_num)
			continue;
		// find page with the right classifier range

		n_classifier_index -= r_t_page.n_first_classifier;
		r_n_x = (n_classifier_index / r_t_page.p_texture->n_Height()) * n_alpha_num;
		r_n_y = n_classifier_index % r_t_page.p_texture->n_Height();
		// calculate first alpha position (next ones are along x+)

		return i;
	}
	return -1;
}

/**
 *	CGLTexture_Rect *CAlphaTextureAtlas::p_Texture(int n_index)
 *		- returns texture with index n_index (zero-based texture page index,
 *		  returned by n_Texture())
 */
CGLTexture_Rect *CAlphaTextureAtlas::p_Texture(int n_index)
{
	return texture_list[n_index].p_texture;
}

/**
 *	void CAlphaTextureAtlas::Dump() const
 *		- prints information about texture atlas to stdout (debug)
 */
void CAlphaTextureAtlas::Dump() const
{
	printf("have %d alpha texture pages:\n", texture_list.size());
	for(int i = 0, n = texture_list.size(); i < n; ++ i) {
		const TPage &r_t_page = texture_list[i];
		printf("\tpage %d: %d classifier alphas in %d x %d columns\n",
			i, r_t_page.n_classifier_num, r_t_page.p_texture->n_Height(),
			(r_t_page.n_classifier_num + r_t_page.p_texture->n_Height() - 1) /
			r_t_page.p_texture->n_Height());
	}
}

inline void CAlphaTextureAtlas::DeletePageTexture(TPage &r_t_page)
{
	delete r_t_page.p_texture;
}

/*
 *								=== ~CAlphaTextureAtlas ===
 */

/*
 *								=== CTileRenderer ===
 */

/**
 *	CTileRenderer::CTileRenderer(CGLState *p_state, const TImagePage &r_t_page,
 *		int n_pass_num, int n_tile_size = 32, int n_downscale = 1)
 *		- default constructor; creates tile renderer
 *		- p_state is OpenGL state guard, r_t_page is downscaled image pyramid page,
 *		  n_pass_num is number of rendering passes for drawing, n_tile_size is
 *		  size of tile in pixels and n_downscale is downscale factor of detection
 *		  raster (recommended using powers of two only)
 *		- it's recommended to check b_Status() to see constructor outcome
 */
CTileRenderer::CTileRenderer(CGLState *p_state, const TImagePage &r_t_page,
	int n_pass_num, int n_tile_size, int n_downscale)
	:m_n_tile_num(0),
#ifndef WBOOST_DISABLE_OCCLUSION_QUERIES
	m_p_occlusion_query(0),
#endif //WBOOST_DISABLE_OCCLUSION_QUERIES
	m_p_fbo(0), m_b_status(false), m_p_tmp_buffer(0), m_p_tile_data(0),
	m_n_downscale(n_downscale), m_b_histogram_triggered(/*true*/false),
	m_t_hist((n_pass_num + 1) / 2, 0, n_pass_num)
{
	int n_width = max(1, r_t_page.n_width / n_downscale);
	int n_height = max(1, r_t_page.n_height / n_downscale);
	// too lazy to do it the right way

	for(int i = 0; i < 2; ++ i) {
		if(!(m_p_texture[i] = p_AllocTexture(p_state, n_width, n_height)))
			return;
	}
	// alloc two textures for ping-pong rendering

	if(!(m_p_fbo = CGLImageProc::p_GetRenderBuffer(n_width, n_height,
	   1, true, 0, false, false, 0, false, false, 0, false)))
		return;
	// alloc framebuffer

	const int n_skip_border = 24;

	int n_tile_num = 0;
	for(int i = 0, n = r_t_page.image_list.size(); i < n; ++ i) {
		TImage *p_image = r_t_page.image_list[i];
		int n_tile_x_num = max(1, ((p_image->v_size.x - n_skip_border) / n_downscale) / n_tile_size);
		int n_tile_y_num = max(1, ((p_image->v_size.y - n_skip_border) / n_downscale) / n_tile_size);
		n_tile_num += n_tile_x_num * n_tile_y_num;
	}
	m_n_tile_num = n_tile_num;
	// calculate number of tiles

	if(!(m_p_tile_data = new TTileData[n_tile_num]))
		return;
	// allocate buffer for tile data (for faster detection picking)

#ifndef WBOOST_DISABLE_OCCLUSION_QUERIES
	if(!(m_p_occlusion_query = new GLuint[n_tile_num]))
		return;
	glGenQueriesARB(m_n_tile_num, m_p_occlusion_query);
	if(glGetError() != GL_NO_ERROR) {
		delete[] m_p_occlusion_query;
		m_p_occlusion_query = 0; // don't want to free invalid queries
		return;
	}
	// alloc occlusion queries
#endif //WBOOST_DISABLE_OCCLUSION_QUERIES

	if(!m_tile_flags.Alloc(n_tile_num))
		return;
	// alloc bit array with tile calculation running flags

	if(!(m_p_tmp_buffer = new float[n_tile_size * n_tile_size * 4]))
		return;
	// alloc temproral buffer for downloading tile-sized bitmaps

	float *p_vertex;
	if(!(p_vertex = new float[n_tile_num * 4 * 5]))
		return;
	TTileData *p_tile_ptr = m_p_tile_data;
	float *p_vertex_ptr = p_vertex;
	for(int i = 0, n = r_t_page.image_list.size(); i < n; ++ i) {
		TImage *p_image = r_t_page.image_list[i];
		int n_tile_x_num = max(1, ((p_image->v_size.x - n_skip_border) / n_downscale) / n_tile_size);
		int n_tile_y_num = max(1, ((p_image->v_size.y - n_skip_border) / n_downscale) / n_tile_size);
		int n_last_tile_w = max(0, p_image->v_size.x - n_skip_border) / n_downscale - (n_tile_x_num - 1) * n_tile_size;
		int n_last_tile_h = max(0, p_image->v_size.y - n_skip_border) / n_downscale - (n_tile_y_num - 1) * n_tile_size;
		//int n_last_tile_w2 = p_image->v_size.x - (n_tile_x_num - 1) * n_tile_size * n_downscale;
		//int n_last_tile_h2 = p_image->v_size.y - (n_tile_y_num - 1) * n_tile_size * n_downscale;
		// determine number of tiles and tile sizes

		for(int j = 0; j < n_tile_y_num; ++ j) {
			int y = p_image->v_position.y / n_downscale + j * n_tile_size;
			//int t = (p_image->v_position.y / n_downscale) * n_downscale + j * n_tile_size * n_downscale;
			int n_tile_h = (j + 1 < n_tile_y_num)? n_tile_size : n_last_tile_h;
			//int n_tile_h2 = (j + 1 < n_tile_y_num)? n_tile_size * n_downscale : n_last_tile_h2;
			for(int k = 0; k < n_tile_x_num; ++ k, ++ p_tile_ptr) {
				int x = p_image->v_position.x / n_downscale + k * n_tile_size;
				//int s = (p_image->v_position.x / n_downscale) * n_downscale + k * n_tile_size * n_downscale;
				int n_tile_w = (k + 1 < n_tile_x_num)? n_tile_size : n_last_tile_w;
				//int n_tile_w2 = (k + 1 < n_tile_x_num)? n_tile_size * n_downscale : n_last_tile_w2;
				// get tile coordinates

				p_tile_ptr->v_pos = Vector2i(x, y);
				p_tile_ptr->v_size = Vector2i(n_tile_w, n_tile_h);
				p_tile_ptr->p_image = p_image;
				// save tile info

				for(int l = 0; l < 4; ++ l, p_vertex_ptr += 5) {
					int u = "0011"[l] - '0';
#ifdef WBOOST_USE_TRISTRIPS
					int v = "0101"[l] - '0'; // GL_TRIANGLE_STRIP (drawn separately because of occlusion queries anyway)
#else //WBOOST_USE_TRISTRIPS
					int v = "0110"[l] - '0'; // GL_QUADS (deprecated by OpenGL 3.0)
#endif //WBOOST_USE_TRISTRIPS
					// s, t coordinates but in [0, 1] range

					//p_vertex_ptr[0] = s + u * n_tile_w2;
					//p_vertex_ptr[1] = t + v * n_tile_h2; // texture rect texcoords are not normalized
					p_vertex_ptr[2] = float(x + u * n_tile_w) / n_width * 2 - 1;
					p_vertex_ptr[3] = float(y + v * n_tile_h) / n_height * 2 - 1; // normalized position
					/*if(fabs(p_vertex_ptr[0] - (p_vertex_ptr[2] + 1) / 2 * r_t_page.n_width) > 1e-3f ||
					   fabs(p_vertex_ptr[1] - (p_vertex_ptr[3] + 1) / 2 * r_t_page.n_height) > 1e-3f)
						j = j;*/ // texture rect texcoords are not normalized
					p_vertex_ptr[0] = (p_vertex_ptr[2] + 1) / 2 * r_t_page.n_width;
					p_vertex_ptr[1] = (p_vertex_ptr[3] + 1) / 2 * r_t_page.n_height; // this gives the right results ... for some reason
					p_vertex_ptr[4] = 0; // z = 0
				}
				// generate quad vertices
			}
		}
	}
	_ASSERTE(p_vertex_ptr == p_vertex + n_tile_num * 4 * 5);
	// generate mesh vertices

	if(!m_vbo.Alloc(p_state, n_tile_num * 4 * 5 * sizeof(float), p_vertex)) {
		delete[] p_vertex;
		return;
	}
	// put vertex data to VBO

	delete[] p_vertex;
	// don't need this anymore

	m_n_iteration = -1;
	m_n_active_tile_num = 0;

	m_b_status = true;
}

/**
 *	CTileRenderer::~CTileRenderer()
 *		- destructor
 */
CTileRenderer::~CTileRenderer()
{
	if(m_p_tmp_buffer)
		delete[] m_p_tmp_buffer;
	if(m_p_tile_data)
		delete[] m_p_tile_data;
#ifndef WBOOST_DISABLE_OCCLUSION_QUERIES
	if(m_p_occlusion_query) {
		glDeleteQueriesARB(m_n_tile_num, m_p_occlusion_query);
		delete[] m_p_occlusion_query;
	}
#endif //WBOOST_DISABLE_OCCLUSION_QUERIES
	for(int i = 0; i < 2; ++ i) {
		if(m_p_texture[i])
			delete m_p_texture[i];
	}
}

/**
 *	bool CTileRenderer::b_Status() const
 *		- returns true if constructor succeeded, otherwise returns false
 */
bool CTileRenderer::b_Status() const
{
	return m_b_status;
}

/**
 *	void CTileRenderer::Trigger_Histogram()
 *		- debug function
 *		- triggers tile execution histogram (it will be printed after the next pass)
 */
void CTileRenderer::Trigger_Histogram()
{
	m_b_histogram_triggered = true;
}

/**
 *	bool CTileRenderer::Download_Detections(CGLState *p_state,
 *		std::vector<TDetection> &r_detection_list, float f_thresh)
 *		- downloads detections from FBO
 *		- p_state is OpenGL state guard, r_detection_list (output) is list of
 *		  detected objects and f_thresh is detection threshold
 *		- returns true on success, false on failure
 */
bool CTileRenderer::Download_Detections(CGLState *p_state,
	std::vector<TDetection> &r_detection_list, float f_thresh)
{
	if(!m_p_fbo->Bind() ||
	   !m_p_fbo->BindColorTexture_2D(*m_p_texture[0], GL_TEXTURE_RECTANGLE_ARB))
		return false;
	glViewport(0, 0, m_p_fbo->n_Width(), m_p_fbo->n_Height());
	// bind texture to fbo

	for(int i = 0; i < m_n_tile_num; ++ i) {
		if(m_tile_flags[i]) {
			const TTileData *p_tile = &m_p_tile_data[i];
			glReadPixels(p_tile->v_pos.x, p_tile->v_pos.y,
				p_tile->v_size.x, p_tile->v_size.y, GL_RED, GL_FLOAT, m_p_tmp_buffer);
			// get alive tile raster

			float *p_pixel = m_p_tmp_buffer;
			for(int y = 0, h = p_tile->v_size.y; y < h; ++ y) {
				for(int x = 0, w = p_tile->v_size.x; x < w; ++ x, ++ p_pixel) {
					if(*p_pixel > f_thresh) { // t_odo - add proper threshold
						const TImage *p_image = p_tile->p_image;

						TDetection d;
						d.x = (p_tile->v_pos.x + x) * m_n_downscale - p_image->v_position.x;
						d.y = (p_tile->v_pos.y + y) * m_n_downscale - p_image->v_position.y;
						d.n_scale = p_image->v_size.x;
						d.f_response = *p_pixel; // copy response
						// generate detection position

						if(r_detection_list.capacity() == r_detection_list.size()) {
							r_detection_list.reserve((r_detection_list.size() * 2) | 1);
							if(r_detection_list.capacity() == r_detection_list.size()) {
								m_p_fbo->BindColorTexture_2D(0, GL_TEXTURE_RECTANGLE_ARB);
								m_p_fbo->Release(); // release fbo, result texture
								return false;
							}
						}
						r_detection_list.push_back(d);
						// add to the list
					}
				}
			}
			// threshold, find detections, recalculate coordinates
		}
	}
	// download active tiles

	if(!m_p_fbo->BindColorTexture_2D(0, GL_TEXTURE_RECTANGLE_ARB) ||
	   !m_p_fbo->Release())
		return false;
	// release fbo, result texture

	return true;
}

/**
 *	int CTileRenderer::n_Tile_Num() const
 *		- returns number of rendering tiles
 */
int CTileRenderer::n_Tile_Num() const
{
	return m_n_tile_num;
}

/**
 *	int CTileRenderer::n_Active_Tile_Num() const
 *		- returns number of active rendering tiles (such tiles where occlusion
 *		  query returned nonzero in the last pass, or all tiles before first pass)
 */
int CTileRenderer::n_Active_Tile_Num() const
{
	return m_n_active_tile_num;
}

/**
 *	bool CTileRenderer::Render_Begin(CGLState *p_state)
 *		- begins rendering (sets up FBO, ...)
 *		- p_state is OpenGL state guard
 *		- returns true on success, false on failure
 */
bool CTileRenderer::Render_Begin(CGLState *p_state)
{
	if(m_n_iteration != -1)
		return false;
	// make sure rendering was properly ended

	m_tile_flags = 1;
	// reset tile flags

	CGLImageProc::DisableFixedFragmentProcessing(p_state);
	CGLImageProc::DisableFixedVertexProcessing(p_state);
	CGLImageProc::DisableProgrammableShading(p_state);
	CGLImageProc::IdentityMatrices();
	// make sure we're ready for gpgpu

#ifndef WBOOST_UNBIND_FBO_EVERY_WBOOST_PASS
	if(!m_p_fbo->Bind() ||
	   !m_p_fbo->BindColorTexture_2D(*m_p_texture[0], GL_TEXTURE_RECTANGLE_ARB))
		return false;
#endif
	glViewport(0, 0, m_p_fbo->n_Width(), m_p_fbo->n_Height());
	// bind renderbuffer and texture to render to

	m_vbo.Bind(p_state);
	glInterleavedArrays(GL_T2F_V3F, 0, m_vbo.p_OffsetPointer(0));
	// GL3 deprecated

	p_state->ActiveTextureUnit(0);
	p_state->DisableTexture2D();
	p_state->EnableTextureRect();
	// enable tex rect texture (pong texture)

	m_n_iteration = 0;
	m_n_active_tile_num = m_n_tile_num;
	// rendering is initialized

	if(m_b_histogram_triggered)
		m_t_hist.Reset();

	return true;
}

/**
 *	bool CTileRenderer::Render_Iteration(CGLState *p_state)
 *		- renders one iteration (draws active tiles with occlusion query
 *		  while reading results of occlusion query from the previous pass
 *		  (except in the first pass))
 *		- returns true on success, false on failure
 */
bool CTileRenderer::Render_Iteration(CGLState *p_state)
{
	if(m_n_iteration < 0)
		return false;
	// make sure rendering was properly initialized

	if(m_n_iteration) {
		CGLTexture_Rect *p_tmp = m_p_texture[0];
		m_p_texture[0] = m_p_texture[1];
		m_p_texture[1] = p_tmp;

#ifndef WBOOST_UNBIND_FBO_EVERY_WBOOST_PASS
		if(!m_p_fbo->BindColorTexture_2D(*m_p_texture[0], GL_TEXTURE_RECTANGLE_ARB))
			return false;
#endif
		// bind texture to render to
	}
	// swap textures

#ifdef WBOOST_UNBIND_FBO_EVERY_WBOOST_PASS
	if(!m_p_fbo->Bind() ||
	   !m_p_fbo->BindColorTexture_2D(*m_p_texture[0], GL_TEXTURE_RECTANGLE_ARB))
		return false;
#endif

	p_state->ActiveTextureUnit(0);
	m_p_texture[1]->Bind(p_state);
	// bind pong texture to read from

#ifndef WBOOST_DISABLE_OCCLUSION_QUERIES
	for(int i = 0, n = m_n_tile_num; i < n && m_n_active_tile_num; ++ i) {
		if(!m_tile_flags[i])
			continue;
		// skip dead tiles

		if(m_n_iteration > 2 && m_n_iteration <= WBOOST_LAST_OCCLUSION_QUERY_PASS) {
			GLuint n_value;
			glGetQueryObjectuivARB(m_p_occlusion_query[i], GL_QUERY_RESULT_ARB, &n_value);
			/*const int n_drawn_pixel_thresh = n_fukwitdaddy_value; // hack
			if(n_drawn_pixel_thresh > 1) {
				static bool _b_ = true;
				if(_b_) {
					_b_ = false;
					fprintf(stderr, "!!!!!!!!! warning: fukwitdaddy "
						"early optimisation enabled !!!!!!!!!\n");
				}
			}*/
			if(n_value < 1) {
				_ASSERTE(m_tile_flags[i]);
				m_tile_flags[i] = 0;
				-- m_n_active_tile_num;
				continue;
			}
		}
		// in case this is not the first iteration, get results of the occlusion query

		if(m_n_iteration && m_n_iteration < WBOOST_LAST_OCCLUSION_QUERY_PASS)
			glBeginQueryARB(GL_SAMPLES_PASSED_ARB, m_p_occlusion_query[i]);
#ifdef WBOOST_USE_TRISTRIPS
		glDrawArrays(GL_TRIANGLE_STRIP, i * 4, 4);
#else //WBOOST_USE_TRISTRIPS
		glDrawArrays(GL_QUADS, i * 4, 4);
#endif //WBOOST_USE_TRISTRIPS
		if(m_n_iteration && m_n_iteration < WBOOST_LAST_OCCLUSION_QUERY_PASS)
			glEndQueryARB(GL_SAMPLES_PASSED_ARB);
		// draw a single tile, surround by occlusion query
	}
#else //WBOOST_DISABLE_OCCLUSION_QUERIES
#ifdef WBOOST_USE_TRISTRIPS
	for(int i = 0, n = m_n_tile_num; i < n; ++ i)
		glDrawArrays(GL_TRIANGLE_STRIP, i * 4, 4);
#else //WBOOST_USE_TRISTRIPS
	glDrawArrays(GL_QUADS, 0, 4 * m_n_tile_num);
#endif //WBOOST_USE_TRISTRIPS
#endif //WBOOST_DISABLE_OCCLUSION_QUERIES

#ifdef WBOOST_UNBIND_FBO_EVERY_WBOOST_PASS
	if(!m_p_fbo->BindColorTexture_2D(0, GL_TEXTURE_RECTANGLE_ARB) ||
	   !m_p_fbo->Release())
		return false;
#endif
	// release texture

	if(m_b_histogram_triggered)
		m_t_hist.Add(m_n_iteration, m_n_active_tile_num);

	++ m_n_iteration;

	//printf("m_n_iteration: %d, m_n_active_tile_num: %d\n", m_n_iteration, m_n_active_tile_num);
	// debug

	//glFlush();
	// will this make it any faster?

	return true;
}

/**
 *	bool CTileRenderer::Render_Finish(CGLState *p_state)
 *		- finalizes rendering (release FBO, ...)
 *		- returns true on success, false on failure
 */
bool CTileRenderer::Render_Finish(CGLState *p_state)
{
	if(m_n_iteration == -1)
		return false;
	// make sure rendering was properly initialized

#ifndef WBOOST_UNBIND_FBO_EVERY_WBOOST_PASS
	if(!m_p_fbo->BindColorTexture_2D(0, GL_TEXTURE_RECTANGLE_ARB) ||
	   !m_p_fbo->Release())
		return false;
#endif
	// release texture, fbo

	p_state->BindVertexArrayBuffer(0);
	// release VBO

	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	glDisableClientState(GL_VERTEX_ARRAY);
	// GL3 deprecated

	//printf("%d active tiles\n", m_n_active_tile_num);
	m_n_iteration = -1;
	// rendering is over

	if(m_b_histogram_triggered) {
		m_b_histogram_triggered = false;
		printf("tile execution histogram:\n");
		m_t_hist.Print("pass", "");
	}

	return true;//glGetError() == GL_NO_ERROR;
}

/**
 *	CGLTexture_Rect *CTileRenderer::p_Result_Texture()
 *		- returns texture, containing rendering result
 */
CGLTexture_Rect *CTileRenderer::p_Result_Texture()
{
	//_ASSERTE(m_n_iteration == -1); // make sure rendering is over
	return m_p_texture[0];
}

CGLTexture_Rect *CTileRenderer::p_AllocTexture(CGLState *p_state, int n_width, int n_height)
{
#ifdef WBOOST_TILE_RENDERER_USE_RGB8
	GLenum n_tex_fmt = (CGLExtensionHandler::b_SupportedExtension(
		"GL_ARB_texture_rg"))? GL_R8 : GL_RGB8;;
#else //WBOOST_TILE_RENDERER_USE_RGB8
	GLenum n_tex_fmt = (CGLExtensionHandler::b_SupportedExtension(
		"GL_NV_float_buffer"))? GL_FLOAT_R16_NV : GL_RGB16F_ARB;
#endif //WBOOST_TILE_RENDERER_USE_RGB8
	// use single-channel textures, if supported

	CGLTexture_Rect *p_texture;
	if(!(p_texture = new CGLTexture_Rect(p_state, n_width,
	   n_height, n_tex_fmt)) || !p_texture->b_Status()) {
		if(p_texture)
			delete p_texture;
		return 0;
	}
	// alloc texture

	CGLTextureParams &r_params = p_texture->r_Parameters(p_state);
	r_params.Set_Texture_Min_Filter(GL_NEAREST);
	r_params.Set_Texture_Mag_Filter(GL_NEAREST);
	// set nearest filtering

	return p_texture;
}

/*
 *								=== CTileRenderer~ ===
 */

/*
 *								=== CWaldBoostShaderGenerator ===
 */

/**
 *	CWaldBoostShaderGenerator::CWaldBoostShaderGenerator(CGLState *p_state,
 *		CImageConvolver *p_convolver, CAlphaTextureAtlas *p_alpha_tex_atlas,
 *		const TWaldBoostClassifier &r_t_classifier, int n_downscale = 1,
 *		int n_max_shaders_pass = 20)
 *		- default constructor; creates data textures and assembles + compiles shader
 *		- p_state is OpenGL state guard, p_convolver is convolution manager,
 *		  p_alpha_tex_atlas is alpha texture atlas, r_t_classifier contains classifier
 *		  information, n_downscale is detection raster downscale factor (factors greater
 *		  than 1 yield less precise, but faster detections) and n_max_shaders_pass
 *		  is limit of weak classifiers in a single pass (limited by maximal shader length)
 *		- note this allocates memory and creates OpenGL objects. it's recommended to
 *		  call b_Status() afterwards
 */
CWaldBoostShaderGenerator::CWaldBoostShaderGenerator(CGLState *p_state,
	CImageConvolver *p_convolver, CAlphaTextureAtlas *p_alpha_tex_atlas,
	const TWaldBoostClassifier &r_t_classifier, int n_downscale, int n_max_shaders_pass)
	:m_b_status(false)
{
	glGetIntegerv(GL_MAX_TEXTURE_UNITS, &m_n_texture_unit_num);
	_ASSERTE(m_n_texture_unit_num <= max_TextureUnits);
	// get number of available texture units

	CHistogram pass_length_hist(5, 0, n_max_shaders_pass);
	// histogram of pass lengths

	int n_min_length = n_max_shaders_pass, n_max_length = 0;
	for(int i = 0, n_pass = 0, n = r_t_classifier.n_weak_classifier_num; i < n; ++ i, ++ n_pass) {
		if(m_pass_list.capacity() == m_pass_list.size()) {
			m_pass_list.reserve((m_pass_list.size() * 2) | 1);
			if(m_pass_list.capacity() == m_pass_list.size())
				return;
		}
		// make sure  there's enough space in the list

		const TWeakClassifier &r_t_cl0 = r_t_classifier.p_weak_classifier[i];
		// first weak classifier in this pass

		if(r_t_cl0.n_width != 3 || r_t_cl0.n_height != 3)
			return;
		// this implementation is limited to 3x3 features

		TPass t_pass = {0};

		int n_alpha0_x, n_alpha0_y;
		int n_alpha_tex;
		if((n_alpha_tex = p_alpha_tex_atlas->n_Texture(i, n_alpha0_x, n_alpha0_y)) < 0)
			return;
		t_pass.p_alpha_texture = p_alpha_tex_atlas->p_Texture(n_alpha_tex);
		// first bound texture is pong (not here), second is alpha texture

		t_pass.n_tex_num = 1;
		int n_conv_texture_slot0 = 0;
		if((t_pass.p_tex_binding[0] = n_GetTexture(r_t_cl0.n_block_width,
		   r_t_cl0.n_block_height, p_convolver)) == -2)
			return;
		// third is convolution texture

		std::string s_classifier_body;
		if(!GenerateShaderSection(s_classifier_body, r_t_cl0,
		   n_conv_texture_slot0, v_GetTextureOffset(r_t_cl0.n_block_width,
		   r_t_cl0.n_block_height, p_convolver), n_alpha0_x, n_alpha0_y))
			return;
		// generate execution section in classifier body

		int n_max_classifiers_in_this_pass = (n_pass < WBOOST_SUBDIVIDE_PASS_NUM)?
			min(n_max_shaders_pass, WBOOST_SUBDIVIDE_PASS_SIZE) : n_max_shaders_pass;
		// have few first passes with less classifiers than the latter passes

		int n_extra_classifier_num = 0;
		for(int j = i + 1, m = min(n, i + n_max_classifiers_in_this_pass);
		   j < m; ++ j, ++ n_extra_classifier_num) {
			const TWeakClassifier &r_t_cl = r_t_classifier.p_weak_classifier[j];
			// first weak classifier in this pass

			if(r_t_cl.n_width != 3 || r_t_cl.n_height != 3)
				return;
			// this implementation is limited to 3x3 features

			int n_alpha_x, n_alpha_y;
			if(p_alpha_tex_atlas->n_Texture(j, n_alpha_x, n_alpha_y) != n_alpha_tex)
				break;
			// can't append, this one uses different alpha texture (unlikely)

			int n_conv_texture;
			if((n_conv_texture = n_GetTexture(r_t_cl.n_block_width,
			   r_t_cl.n_block_height, p_convolver)) == -2)
				return;
			// get the convolution texture

			int n_conv_texture_slot = -1;
			for(int k = 0; k < t_pass.n_tex_num; ++ k) {
				if(t_pass.p_tex_binding[k] == n_conv_texture) {
					n_conv_texture_slot = k;
					break;
				}
			}
			if(n_conv_texture_slot == -1) {
				if(t_pass.n_tex_num == m_n_texture_unit_num - 2) {
					break;
					// can't append, uses different texture than anyone
				} else {
					n_conv_texture_slot = t_pass.n_tex_num;
					t_pass.p_tex_binding[t_pass.n_tex_num] = n_conv_texture;
					++ t_pass.n_tex_num;
					// add next texture
				}
			}
			// find convolution texture / try to add it

			// now it's ceratin we can append this classifier

			if(!GenerateShaderSection(s_classifier_body, r_t_cl,
			   n_conv_texture_slot, v_GetTextureOffset(r_t_cl.n_block_width,
			   r_t_cl.n_block_height, p_convolver), n_alpha_x, n_alpha_y))
				return;
			// generate execution section in classifier body
		}
		// go trough the next shaders, see how much of them is possible to be appended

		const char *p_s_shader_head =
			"#extension ARB_texture_rectangle : enable\n"
#ifdef DISABLE_TEXTURE_RECT_NV_WARNING
			"#define samplerRect sampler2DRect\n"
			"#define textureRect texture2DRect\n" // remove those lines on ati
#endif //DISABLE_TEXTURE_RECT_NV_WARNING
			"uniform samplerRect n_pong_tex, n_alpha_tex;\n";
		std::string s_shader;
		if(!Append(s_shader, p_s_shader_head))
			return;
		// start with shader head

		if(!Append(s_shader, "uniform samplerRect "))
			return;
		for(int j = 0; j < t_pass.n_tex_num; ++ j) {
			if((j && !Append(s_shader, ", ")) ||
			   !Append(s_shader, p_s_ConvSampler_Name(j)))
				return;
		}
		if(!Append(s_shader, ";\n"))
			return;
		// append convolution image sampler names

		const char *p_s_body_head = "void main()\n"
			"{\n"
			"    float f_accum = textureRect(n_pong_tex, gl_TexCoord[0].xy).x;\n" // read previous value
			"    if(f_accum < -5000.0) {\n"
			"        if(f_accum > -15000.0) {\n"
			"            gl_FragColor.x = -20000.0;\n" // must flip to the other buffer as well
			"            return;\n"
			"        } else\n"
			"            discard;\n"
			"    }\n"
			"\n";
		char p_s_head[512];
		if(n_downscale != 1) {
			const char *p_s_body_head_fmt = "void main()\n"
				"{\n"
				"    float f_accum = textureRect(n_pong_tex, gl_TexCoord[0].xy * %g).x;\n" // read previous value
				"    if(f_accum < -5000.0) {\n"
				"        if(f_accum < -15000.0)\n"
				"            discard;\n"
				"        else {\n"
				"            gl_FragColor.x = -20000.0;\n" // must flip to the other buffer as well
				"            return;\n"
				"        }\n"
				"    }\n"
				"\n";
			int n_char_num = sprintf(p_s_head, p_s_body_head_fmt, 1.0f / n_downscale);
			_ASSERTE(n_char_num + 1 < sizeof(p_s_head) / sizeof(char)); // make sure it fits fixed-size buffer
			p_s_body_head = p_s_head;
		}
		// body head; reads pong (previous) and discards immediately if below active pixel thresh

		const char *p_s_first_pass_body_head = "void main()\n"
			"{\n"
			"    float f_accum = 0.0;\n"
			"\n";
		// first pass body head; starts with fresh accumulator

		if(!Append(s_shader, (i)? p_s_body_head : p_s_first_pass_body_head))
			return;
		// append body head

		if(!Append(s_shader, s_classifier_body.c_str()))
			return;
		// append executive section

		const char *p_s_body_tail =
			"\n"
			"    gl_FragColor.x = f_accum;\n"
			"}\n";
		if(!Append(s_shader, p_s_body_tail))
			return;
		// append body tail; write accumulator

		std::string s_samplers;
		if(!Append(s_samplers, "n_pong_tex|n_alpha_tex"))
			return;
		for(int j = 0; j < t_pass.n_tex_num; ++ j) {
			if(!Append(s_samplers, "|") ||
			   !Append(s_samplers, p_s_ConvSampler_Name(j)))
				return;
		}
		// concatenate samplers string

		/*printf("-------------------------------------------------------------"
			"-------------------\nlrd shader (%d wc):\n%s\n------------------"
			"--------------------------------------------------------------\n",
			1 + n_extra_classifier_num, s_shader.c_str());*/
		// dump shader source so we know

		n_min_length = min(n_min_length, 1 + n_extra_classifier_num);
		n_max_length = max(n_max_length, 1 + n_extra_classifier_num);
		// keep track of min / max shader length

		TShaderInfo t_fs("waldboost_fs", TShaderInfo::proc_Fragment,
			s_shader.c_str(), s_samplers.c_str());
		if(!t_fs.p_high)
			return;
		// create shader info

		if(!(t_pass.p_shader = CGLImageProc::p_CompileLinkReport(
		   p_state, 0, 0, &t_fs, false)))
			return;
		// compile shader

		m_pass_list.push_back(t_pass);
		// add to the list

		i += n_extra_classifier_num;
		// skip classifiers we appended

		pass_length_hist.Add(n_extra_classifier_num + 1, 1);
	}

	printf("have %d passes (min length: %d, max length: %d, avg length: %.2f)\n",
		m_pass_list.size(), n_min_length, n_max_length,
		float(r_t_classifier.n_weak_classifier_num) / m_pass_list.size());
	// print simple info

	pass_length_hist.Print("", "shaders");
	// print histogram

	m_b_status = true;
}

/**
 *	bool CWaldBoostShaderGenerator::b_Status() const
 *		- returns true if constructor succeeded and shader generator
 *		  is ready for use, otherwise returns false
 */
bool CWaldBoostShaderGenerator::b_Status() const
{
	return m_b_status;
}

/**
 *	int CWaldBoostShaderGenerator::n_Pass_Num() const
 *		- returns number of passes (a lot)
 */
int CWaldBoostShaderGenerator::n_Pass_Num() const
{
	return m_pass_list.size();
}

/**
 *	bool CWaldBoostShaderGenerator::Render(CGLState *p_state,
 *		CGLTexture_Rect *p_base_texture, CImageConvolver *p_convolver,
 *		CTileRenderer *p_renderer)
 *		- renders a single frame (detects faces in a single frame)
 *		- p_state is OpenGL state guard, p_base_texture is base texture
 *		  (source image, or downscaled image pyramid), p_convolver is image
 *		  convolver (unused) and p_renderer is tile renderer
 *		- returns true on success, false on failure
 */
bool CWaldBoostShaderGenerator::Render(CGLState *p_state, CGLTexture_Rect *p_base_texture,
	CImageConvolver *p_convolver, CTileRenderer *p_renderer)
{
	if(!p_renderer->Render_Begin(p_state))
		return false;
	// begin rendering

	for(int i = 1; i < m_n_texture_unit_num; ++ i) {
		p_state->ActiveTextureUnit(i);
		p_state->EnableTextureRect();
	}
	// enable textures in higher texture units

	for(int i = 0, n = m_pass_list.size(); i < n; ++ i) {
		const TPass &r_t_pass = m_pass_list[i];

		if(!i || r_t_pass.p_alpha_texture !=
		   m_pass_list[i - 1].p_alpha_texture) {
			p_state->ActiveTextureUnit(1);
			r_t_pass.p_alpha_texture->Bind(p_state);
		}
		// bind alpha texture (probably just once)

		for(int j = 0; j < r_t_pass.n_tex_num; ++ j) {
			p_state->ActiveTextureUnit(j + 2);
			if(r_t_pass.p_tex_binding[j] == -1)
				p_base_texture->Bind(p_state);
			else
				p_convolver->p_Image(r_t_pass.p_tex_binding[j])->Bind(p_state);
		}
		// bind convolution textures

		if(!r_t_pass.p_shader->Bind(p_state))
			return false;
		// bind waldboost shader

		p_state->ActiveTextureUnit(0);
		if(!p_renderer->Render_Iteration(p_state))
			return false;
		if(!p_renderer->n_Active_Tile_Num())
			break;
	}

	for(int i = 1; i < m_n_texture_unit_num; ++ i) {
		p_state->ActiveTextureUnit(i);
		p_state->DisableTextureRect();
	}
	p_state->ActiveTextureUnit(0);
	// disable textures in higher texture units

	if(!p_renderer->Render_Finish(p_state))
		return false;
	// finish rendering

	return true;
}

bool CWaldBoostShaderGenerator::GenerateShaderSection(std::string &r_s_body,
	const TWeakClassifier &r_t_cl, int n_conv_texture_slot, Vector2f v_texture_off,
	int n_alpha_x, int n_alpha_y)
{
	const char *p_s_body_head =
		"    {\n"
		"        vec3 lrd0, lrd1, lrd2;\n";
	if(!Append(r_s_body, p_s_body_head))
		return false;
	// begin with body head

	const char *p_s_sampler_name = p_s_ConvSampler_Name(n_conv_texture_slot);
//#define __ENJOY_RGB_PACKED_CONVOLUTIONS

#ifdef __ENJOY_RGB_PACKED_CONVOLUTIONS
	for(int i = 0; i < 3; ++ i) {
		int y = i;
		char p_s_buffer[128];
		int n = sprintf(p_s_buffer, "        lrd%c = textureRect(%s, gl_TexCoord[0].xy"
			" + vec2(%.1f, %.1f)).xxx;\n", '0' + y, p_s_sampler_name,
			r_t_cl.n_position_x + 0 * r_t_cl.n_block_width + v_texture_off.x,
			r_t_cl.n_position_y + y * r_t_cl.n_block_height + v_texture_off.y); // sample order is ok
		_ASSERTE(n + 1 < sizeof(p_s_buffer) / sizeof(char)); // make sure it fits fixed-size buffer
		if(!Append(r_s_body, p_s_buffer))
			return false;
		// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! fake !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	}
#else //__ENJOY_RGB_PACKED_CONVOLUTIONS
	for(int i = 0; i < 9; ++ i) {
		int x = i % 3;
		int y = i / 3;
		char p_s_buffer[128];
		int n = sprintf(p_s_buffer, "        lrd%c.%c = textureRect(%s, gl_TexCoord[0].xy"
			" + vec2(%.1f, %.1f)).x;\n", '0' + x, 'x' + y, p_s_sampler_name,
			r_t_cl.n_position_x + x * r_t_cl.n_block_width + v_texture_off.x,
			r_t_cl.n_position_y + y * r_t_cl.n_block_height + v_texture_off.y); // sample order is ok
		_ASSERTE(n + 1 < sizeof(p_s_buffer) / sizeof(char)); // make sure it fits fixed-size buffer
		if(!Append(r_s_body, p_s_buffer))
			return false;
	}
#endif //__ENJOY_RGB_PACKED_CONVOLUTIONS
	// append code to read texture samples (this might need some simple optimization)

	{
		char p_s_buffer[64];
		int ax = r_t_cl.n_block_a % 3;
		int ay = r_t_cl.n_block_a / 3;
		int n = sprintf(p_s_buffer, "        vec3 a = vec3(lrd%c.%c);\n", '0' + ax, 'x' + ay); // sample order is ok
		_ASSERTE(n + 1 < sizeof(p_s_buffer) / sizeof(char)); // make sure it fits fixed-size buffer
		if(!Append(r_s_body, p_s_buffer))
			return false;
	}
	// append code to select block a

	const char *p_s_compare_a =
		"        vec3 lrd = vec3(greaterThan(a, lrd0));\n"
		"        lrd += vec3(greaterThan(a, lrd1));\n"
		"        lrd += vec3(greaterThan(a, lrd2));\n";
	if(!Append(r_s_body, p_s_compare_a))
		return false;
	// append code to calculate block a rank

	{
		char p_s_buffer[64];
		int bx = r_t_cl.n_block_b % 3;
		int by = r_t_cl.n_block_b / 3;
		int n = sprintf(p_s_buffer, "        vec3 b = vec3(lrd%c.%c);\n", '0' + bx, 'x' + by); // sample order is ok
		_ASSERTE(n + 1 < sizeof(p_s_buffer) / sizeof(char)); // make sure it fits fixed-size buffer
		if(!Append(r_s_body, p_s_buffer))
			return false;
	}
	// append code to select block b

	const char *p_s_compare_b =
		"        lrd -= vec3(greaterThan(b, lrd0));\n"
		"        lrd -= vec3(greaterThan(b, lrd1));\n"
		"        lrd -= vec3(greaterThan(b, lrd2));\n";
	if(!Append(r_s_body, p_s_compare_b))
		return false;
	// append code to subtract block b rank

	{
		char p_s_buffer[128];
		int n = sprintf(p_s_buffer, "        f_accum += textureRect(n_alpha_tex, "
			"vec2(%d.0 + dot(vec3(1.0), lrd), %d.0)).x;\n", // %d.5 = sampling point + half-texel right
			n_alpha_x + r_t_cl.n_alpha_num / 2, n_alpha_y);
		_ASSERTE(n + 1 < sizeof(p_s_buffer) / sizeof(char)); // make sure it fits fixed-size buffer
		if(!Append(r_s_body, p_s_buffer))
			return false;
	}
	//fprintf(stderr, "!!!!!!!!! warning: alpha fetch altered !!!!!!!!!\n");
	// append code to read alpha texture

#if 1
	if(r_t_cl.f_neg_thresh > -1e30f) {
		char p_s_buffer[256];
		int n = sprintf(p_s_buffer,
			"        if(f_accum < %f) {\n"
			"            gl_FragColor.x = -10000.0;\n"
			"            return;\n"
			"        }\n",
			r_t_cl.f_neg_thresh);
		_ASSERTE(n + 1 < sizeof(p_s_buffer) / sizeof(char)); // make sure it fits fixed-size buffer
		if(!Append(r_s_body, p_s_buffer))
			return false;
	}
#else
	static bool _b_ = true;
	if(_b_) {
		_b_ = false;
		fprintf(stderr, "!!!!!!!!! warning: early cutoff disabled !!!!!!!!!\n");
	}
	// debug - no thresh
#endif
	// append optional code to terminate calculation

	const char *p_s_body_tail =
		"    }\n";
	if(!Append(r_s_body, p_s_body_tail))
		return false;
	// end with body tail

	return true;
}

const char *CWaldBoostShaderGenerator::p_s_ConvSampler_Name(int n_conv_texture_slot)
{
	static char p_s_buffer[32];
	int n = sprintf(p_s_buffer, "n_conv%d_tex",n_conv_texture_slot);
	_ASSERTE(n + 1 < sizeof(p_s_buffer) / sizeof(char)); // make sure it fits fixed-size buffer
	return p_s_buffer;
}

inline bool CWaldBoostShaderGenerator::Append(std::string &r_s_str, const char *p_s_string)
{
	unsigned int n_str_length = strlen(p_s_string);
	if(r_s_str.capacity() < r_s_str.length() + n_str_length) {
		r_s_str.reserve(r_s_str.length() + n_str_length);
		if(r_s_str.capacity() < r_s_str.length() + n_str_length)
			return false;
	}
	// make sure there's space in the string

	r_s_str.append(p_s_string);
	// append

	return true;
}

Vector2f CWaldBoostShaderGenerator::v_GetTextureOffset(int n_block_width,
	int n_block_height, CImageConvolver *p_convolver)
{
	for(int i = 0, n = p_convolver->n_Kernel_Num(); i < n; ++ i) {
		if(p_convolver->v_Kernel(i) == Vector2i(n_block_width, n_block_height))
			return p_convolver->v_TextureOffset(i);
	}
	return Vector2f(0, 0);
}

int CWaldBoostShaderGenerator::n_GetTexture(int n_block_width,
	int n_block_height, CImageConvolver *p_convolver)
{
	for(int i = 0, n = p_convolver->n_Kernel_Num(); i < n; ++ i) {
		if(p_convolver->v_Kernel(i) == Vector2i(n_block_width, n_block_height)) {
			if(!p_convolver->p_Image(i))
				return -1; // using source texture
			return i;
		}
	}
	return -2;
}

/*
 *								=== ~CWaldBoostShaderGenerator ===
 */

/*
 *								=== CWaldBoost_AllInOne_ShaderGenerator ===
 */

/**
 *	CWaldBoost_AllInOne_ShaderGenerator::CWaldBoost_AllInOne_ShaderGenerator(CGLState *p_state,
 *		CImageConvolver *p_convolver, CAlphaTextureAtlas *p_alpha_tex_atlas,
 *		const TWaldBoostClassifier &r_t_classifier)
 *		- default constructor; creates data textures and assembles + compiles shader
 *		- p_state is OpenGL state guard, p_convolver is convolution unit (it isn't
 *		  really required here, it is only intended to have interface similar to
 *		  CWaldBoostShaderGenerator) and p_alpha_tex_atlas is alpha texture atlas,
 *		  r_t_classifier is classifier
 *		- note there are some restrictions, all convolutions must be calculated using
 *		  bilinear filtering (ie. 1x1, 1x2, 2x1 or 2x2 only) and all alphas must be
 *		  in a single texture page, in a single column (both restrictions could be removed
 *		  if necessary, the aim was to keep the code as simple and fast as possible)
 *		- note this allocates memory and creates OpenGL objects. it's recommended to
 *		  call b_Status() afterwards
 */
CWaldBoost_AllInOne_ShaderGenerator::CWaldBoost_AllInOne_ShaderGenerator(CGLState *p_state,
	CImageConvolver *p_convolver, CAlphaTextureAtlas *p_alpha_tex_atlas,
	const TWaldBoostClassifier &r_t_classifier)
	:m_b_status(false), p_alpha_texture(p_alpha_tex_atlas->p_Texture(0)), p_data_texture(0), p_shader(0)
{
	int n_texture_unit_num;
	glGetIntegerv(GL_MAX_TEXTURE_UNITS, &n_texture_unit_num);
	// get number of available texture units

	if(n_texture_unit_num < 4)
		return;
	// now needs three texture units + one is wasted on nothing

	for(int i = 0; i < p_convolver->n_Kernel_Num(); ++ i) {
		if(p_convolver->v_Kernel(i).x > 2 ||
		   p_convolver->v_Kernel(i).y > 2) {
			fprintf(stderr, "error: all in one can run with convolutions 1x1, 1x2, 2x1 or 2x2 only\n");
			return;
		}
		// only 1x1, 1x2, 2x1 or 2x2

		if(p_convolver->p_Image(i)) {
			fprintf(stderr, "error: all in one can't run without bilinear filter convolutions\n");
			return;
		}
		// make sure it's carried-out using bilinear filter
		// (there are no convolution images, everything is sourcing from base image)
	}
	// check convolver; can only afford a single convolution texture
	// (that's the source image texture + bilinear filtering)

	int n_max_loop_iters = 250;
	int n_loop_count = (r_t_classifier.n_weak_classifier_num + n_max_loop_iters - 1) / n_max_loop_iters;
	// get number of loops

	float *p_classifier_texture_data;
	if(!(p_classifier_texture_data = new float[n_max_loop_iters * n_loop_count * 2 * 4]))
		return;
	memset(p_classifier_texture_data, 0, n_max_loop_iters * n_loop_count * 2 * 4 * sizeof(float));
	// alloc texture for classifier data (number of classifiers per loop wide / loop count * 2 tall, RGBA)

	for(int i = 0; i < r_t_classifier.n_weak_classifier_num; ++ i) {
		const TWeakClassifier &r_t_cl = r_t_classifier.p_weak_classifier[i];
		// get weak classifier

		int n_alpha_x, n_alpha_y;
		int n_alpha_page = p_alpha_tex_atlas->n_Texture(i, n_alpha_x, n_alpha_y);
		if(n_alpha_page != 0 || n_alpha_x != 0 || n_alpha_y != i) {
			fprintf(stderr, "error: invalid alpha texture atlas for all in one\n");
			return;
		}
		// make sure alpha atlas is just a single texture with a single column

		Vector2i v_texcoord_step(r_t_cl.n_block_width, r_t_cl.n_block_height);
		// get texcoord step

		Vector2f v_texcoord_off(r_t_cl.n_position_x, r_t_cl.n_position_y); // get base texcoords
		v_texcoord_off += v_GetTextureOffset(v_texcoord_step, p_convolver); // account for bilinear filter offset (fake convo)
		// get texcoord off

		float f_thresh = r_t_cl.f_neg_thresh;
		// get threshold (always used, can't branch away in the loop)

		int a = r_t_cl.n_block_a;
		int b = r_t_cl.n_block_b;
		// get a, b

		int n_loop = i / n_max_loop_iters;
		int n_iter = i % n_max_loop_iters;
		// which iteration of which loop is the classifier in

		float *p_data_a = &p_classifier_texture_data[((n_loop * 2 + 0) * n_max_loop_iters + n_iter) * 4];
		p_data_a[0] = v_texcoord_off.x;
		p_data_a[1] = v_texcoord_off.y;
		p_data_a[2] = v_texcoord_step.x;
		p_data_a[3] = v_texcoord_step.y;
		float *p_data_b = &p_classifier_texture_data[((n_loop * 2 + 1) * n_max_loop_iters + n_iter) * 4];
		p_data_b[0] = a;
		p_data_b[1] = b;
		p_data_b[2] = f_thresh;
		p_data_b[3] = 0;
		// put classifier data into linear block so it can be stored inside texture
	}
	// fill the data

	if(!(p_data_texture = new CGLTexture_Rect(p_state, n_max_loop_iters, n_loop_count * 2,
	   GL_RGBA16F_ARB, GL_RGBA, GL_FLOAT, p_classifier_texture_data)) || !p_data_texture->b_Status()) {
		delete[] p_classifier_texture_data;
		return;
	}
	delete[] p_classifier_texture_data;
	{
		CGLTextureParams &r_params = p_data_texture->r_Parameters(p_state);
		r_params.Set_Texture_Mag_Filter(GL_NEAREST);
		r_params.Set_Texture_Min_Filter(GL_NEAREST);
	}
	// create texture, containing classifier data

	//test_AB_multiplexing();

	const char *p_s_shader_head =
		"#extension ARB_texture_rectangle : enable\n"
#ifdef DISABLE_TEXTURE_RECT_NV_WARNING
		"#define samplerRect sampler2DRect\n"
		"#define textureRect texture2DRect\n" // remove those lines on ati
#endif //DISABLE_TEXTURE_RECT_NV_WARNING
		"uniform samplerRect n_prev_pong, n_alphas, n_cl_data, n_src_image;\n" // note n_prev_pong is unused and is there only because it is bound by 
		"uniform float f_final_thresh;\n" // final threshold
		"\n"
		"void main()\n"
		"{\n"
		"	float f_accum = 0.0;\n"
		"	// result accumulator\n"
		"\n"
		"	int classifier_idx = 0;\n"
		"	// classifier index counter\n"
		"\n";

	const char *p_s_shader_loop =
		"	for(int i = 0; i < %d; ++ i, ++ classifier_idx) {\n"
		"		vec4 v_data_a = textureRect(n_cl_data, vec2(i, %.1f)); // off.x, off.y, step.x, step.y\n"
		"		vec3 v_data_b = textureRect(n_cl_data, vec2(i, %.1f)).xyz; // a, b, threshold\n"
		"		// get classifier data\n" // t_odo - replace y-coord by correct value, based on loop index (loop index, not loop iteration)
		"\n"
		"		vec3 lrd0, lrd1, lrd2;\n"
		"		{\n"
		"			vec4 v_tc01 = gl_TexCoord[0].xyxy + v_data_a.xyxy;\n"
		"			v_tc01.z += v_data_a.z;\n"
		"			vec2 v_tc2 = v_tc01.zw;\n"
		"			v_tc2.x += v_data_a.z;\n"
		"			// get texcoords for first three pixels of LRD grid\n"
		"\n"
		"			lrd0.x = textureRect(n_src_image, v_tc01.xy).x;\n"
		"			lrd1.x = textureRect(n_src_image, v_tc01.zw).x;\n"
		"			lrd2.x = textureRect(n_src_image, v_tc2).x;\n"
		"\n"
		"			v_tc01.yw += v_data_a.ww;\n"
		"			v_tc2.y += v_data_a.w;\n"
		"			// shift texcoords to the next pixels of LRD grid\n"
		"\n"
		"			lrd0.y = textureRect(n_src_image, v_tc01.xy).x;\n"
		"			lrd1.y = textureRect(n_src_image, v_tc01.zw).x;\n"
		"			lrd2.y = textureRect(n_src_image, v_tc2).x;\n"
		"\n"
		"			v_tc01.yw += v_data_a.ww;\n"
		"			v_tc2.y += v_data_a.w;\n"
		"			// shift texcoords to the next pixels of LRD grid\n"
		"\n"
		"			lrd0.z = textureRect(n_src_image, v_tc01.xy).x;\n"
		"			lrd1.z = textureRect(n_src_image, v_tc01.zw).x;\n"
		"			lrd2.z = textureRect(n_src_image, v_tc2).x;\n"
		"		}\n"
		"		// read LRD grid 3x3 pixels (convolutions hidden inside texture sampling units)\n"
		"\n"
		"		float a, b;\n"
		"		{\n"
		"			vec4 ax_ay_bx_by;\n"
		"			ax_ay_bx_by.xz = mod(v_data_b.xy, 3.0); // get x-coords of a, b\n"
		"			ax_ay_bx_by.yw = v_data_b.xy / 3.0; // get y-coords of a, b\n"
		"\n"
		"			vec4 v_first_col = vec4(lessThan(ax_ay_bx_by, vec4(1.0)));\n"
		"			vec4 v_third_col = vec4(greaterThanEqual(ax_ay_bx_by, vec4(2.0)));\n"
		"			vec4 v_second_col = vec4(1.0) - v_first_col - v_third_col;\n"
		"			// compare coords to < 1, >= 1 && < 2, >= 2\n"
		"			// (index to x-y conversion and multiplexing, using fp arithmetic ... yucky)\n"
		"\n"
		"			vec3 v_a_row = vec3(v_first_col.y, v_second_col.y, v_third_col.y);\n"
		"			vec3 v_b_row = vec3(v_first_col.w, v_second_col.w, v_third_col.w);\n"
		"			a = dot(lrd0 * v_first_col.x + lrd1 * v_second_col.x + lrd2 * v_third_col.x, v_a_row);\n"
		"			b = dot(lrd0 * v_first_col.z + lrd1 * v_second_col.z + lrd2 * v_third_col.z, v_b_row);\n"
		"		}\n"
		"		// demultiplex values of a and b blocks\n"
		"\n"
		"		vec3 lrd = vec3(greaterThan(vec3(a), lrd0));\n"
		"		lrd += vec3(greaterThan(vec3(a), lrd1));\n"
		"		lrd += vec3(greaterThan(vec3(a), lrd2));\n"
		"		lrd -= vec3(greaterThan(vec3(b), lrd0));\n"
		"		lrd -= vec3(greaterThan(vec3(b), lrd1));\n"
		"		lrd -= vec3(greaterThan(vec3(b), lrd2));\n"
		"		// calculate rank difference\n"
		"\n"
		"		f_accum += textureRect(n_alphas, vec2(8.0 + dot(vec3(1.0), lrd), classifier_idx)).x;\n"
		"		// fetch alpha\n" // t_odo - replace y-coord with zero-based classifier index, not just loop iteration
		"\n"
		"		if(f_accum < v_data_b.z)// { gl_FragColor.xyz = vec3(-10000.0);\n" // t_odo - remove, replace by glClear(GL_COLOR_BUFFER_BIT);\n"
		"			discard;//return; }\n" // t_odo - replace by discard as soon as glClear(GL_COLOR_BUFFER_BIT); is executed prior to calling this shader
		"		// early cutoff, as simple as that\n"
		"	}\n"
		"	// loop trough all classifiers\n"
		"\n";
	// there are following printf arguments:
	//		int loop length (250 and less)
	//		float y-coord of the first line with classifier data (0, 2, ...)
	//		float y-coord of second line with classifier data (1, 3, ...)

	const char *p_s_shader_tail =
		"	gl_FragColor.xyz = vec3(f_accum > f_final_thresh);\n"
		"	// perform threshold here\n"
		"}\n";
	// note this version of shader outputs thresholded classifier output
	// or discards on early cutoff (=> requires clearing output)

	std::string s_shader_source;
	if(!stl_ut::AssignCStr(s_shader_source, p_s_shader_head))
		return;
	for(int i = 0, n_loop = 0; i < r_t_classifier.n_weak_classifier_num; i += n_max_loop_iters, ++ n_loop) {
		int n_loop_length = min(n_max_loop_iters, r_t_classifier.n_weak_classifier_num - i);
		// calcuilate loop length

		std::string s_custom_loop;
		if(!stl_ut::Format(s_custom_loop, p_s_shader_loop,
		   n_loop_length, float(n_loop * 2), float(n_loop * 2 + 1)) ||
		   !stl_ut::Append(s_shader_source, s_custom_loop))
			return;
		// format & append
	}
	if(!stl_ut::AppendCStr(s_shader_source, p_s_shader_tail))
		return;
	// concatenate shader source (t_odo - use multiple loops)

	/*printf("####################################################"
		"############################\n%s\n#######################"
		"#########################################################\n",
		s_shader_source.c_str());*/
	// debug - dump shader source

	TShaderInfo t_fs("waldboost_fs", TShaderInfo::proc_Fragment,
		s_shader_source.c_str(), "n_prev_pong|n_alphas|n_cl_data|n_src_image");
	if(!t_fs.p_high)
		return;
	// create shader info

	if(!(p_shader = CGLImageProc::p_CompileLinkReport(p_state, 0, 0, &t_fs, false)))
		return;
	p_shader->t_Find_Parameter("f_final_thresh").SetValue(.0f);
	// compile shader

	m_b_status = true;
	// success, all done
}

/**
 *	CWaldBoost_AllInOne_ShaderGenerator::~CWaldBoost_AllInOne_ShaderGenerator()
 *		- destructor
 */
CWaldBoost_AllInOne_ShaderGenerator::~CWaldBoost_AllInOne_ShaderGenerator()
{
	if(p_data_texture)
		delete p_data_texture;
	if(p_shader)
		CGLImageProc::ReturnShaderObject(p_shader);
}

/**
 *	bool CWaldBoost_AllInOne_ShaderGenerator::b_Status() const
 *		- returns true if constructor succeeded and shader generator
 *		  is ready for use, otherwise returns false
 */
bool CWaldBoost_AllInOne_ShaderGenerator::b_Status() const
{
	return m_b_status;
}

/**
 *	int CWaldBoost_AllInOne_ShaderGenerator::n_Pass_Num() const
 *		- returns number of passes (1)
 */
int CWaldBoost_AllInOne_ShaderGenerator::n_Pass_Num() const
{
	return 1;
}

/**
 *	void CWaldBoost_AllInOne_ShaderGenerator::Set_FinalThreshold(float f_thresh)
 *		- sets final threshold to f_thresh (default 0.0)
 */
void CWaldBoost_AllInOne_ShaderGenerator::Set_FinalThreshold(float f_thresh)
{
	p_shader->t_Find_Parameter("f_final_thresh").SetValue(f_thresh);
}

/**
 *	bool CWaldBoost_AllInOne_ShaderGenerator::Render(CGLState *p_state,
 *		CGLTexture_Rect *p_base_texture, CImageConvolver *p_convolver,
 *		CTileRenderer *p_renderer)
 *		- renders a single frame (detects faces in a single frame)
 *		- p_state is OpenGL state guard, p_base_texture is base texture
 *		  (source image, or downscaled image pyramid), p_convolver is image
 *		  convolver (unused) and p_renderer is tile renderer
 *		- returns true on success, false on failure
 */
bool CWaldBoost_AllInOne_ShaderGenerator::Render(CGLState *p_state, CGLTexture_Rect *p_base_texture,
	CImageConvolver *p_convolver, CTileRenderer *p_renderer)
{
	if(!p_renderer->Render_Begin(p_state))
		return false;
	// begin rendering

	p_state->ClearColor4f(0, 0, 0, 0);
	glClear(GL_COLOR_BUFFER_BIT);
	// clear color buffer (required for our shader to work)

	p_state->ActiveTextureUnit(1); // "n_prev_pong|n_alphas|n_cl_data|n_src_image"
	p_alpha_texture->Bind_Enable(p_state);
	p_state->ActiveTextureUnit(2);
	p_data_texture->Bind_Enable(p_state);
	p_state->ActiveTextureUnit(3);
	p_base_texture->Bind_Enable(p_state);
	// bind textures

	if(!p_shader->Bind(p_state))
		return false;
	// bind waldboost shader

	p_state->ActiveTextureUnit(0);
	if(!p_renderer->Render_Iteration(p_state))
		return false;
	// render a single iteration (hey, this is all in one)

	for(int i = 1; i < 4; ++ i) {
		p_state->ActiveTextureUnit(i);
		p_state->DisableTextureRect();
	}
	p_state->ActiveTextureUnit(0);
	// disable textures in higher texture units

	if(!p_renderer->Render_Finish(p_state))
		return false;
	// finish rendering

	return true;
}

Vector2f CWaldBoost_AllInOne_ShaderGenerator::v_GetTextureOffset(Vector2i v_texcoord_step, CImageConvolver *p_convolver)
{
	for(int i = 0, n = p_convolver->n_Kernel_Num(); i < n; ++ i) {
		if(p_convolver->v_Kernel(i) == v_texcoord_step)
			return p_convolver->v_TextureOffset(i);
	}
	return Vector2f(0, 0);
}

int CWaldBoost_AllInOne_ShaderGenerator::n_Transpose3x3Index(int i)
{
	int x = i % 3;
	int y = i / 3;
	return x * 3 + y;
}

Vector4f CWaldBoost_AllInOne_ShaderGenerator::lessThan(Vector4f v, float f) // glsl lookalike
{
	return Vector4f(v.x < f, v.y < f, v.z < f, v.w < f);
}

Vector4f CWaldBoost_AllInOne_ShaderGenerator::greaterThanEqual(Vector4f v, float f) // glsl lookalike
{
	return Vector4f(v.x >= f, v.y >= f, v.z >= f, v.w >= f);
}

void CWaldBoost_AllInOne_ShaderGenerator::test_AB_multiplexing() // this is more of a programming check, not runtime check
{
	for(int a = 0; a < 9; ++ a) {
		for(int b = 0; b < 9; ++ b) {
			if(a == b)
				continue;

			Vector3f lrd0(0, 0, 0);
			Vector3f lrd1(0, 0, 0);
			Vector3f lrd2(0, 0, 0);

			{
				int ax = a % 3;
				int ay = a / 3;
				if(ax == 0)
					lrd0[ay] = 1;
				else if(ax == 1)
					lrd1[ay] = 1;
				else if(ax == 2)
					lrd2[ay] = 1;
			}
			{
				int bx = b % 3;
				int by = b / 3;
				if(bx == 0)
					lrd0[by] = 2;
				else if(bx == 1)
					lrd1[by] = 2;
				else if(bx == 2)
					lrd2[by] = 2;
			}
			// mark expected position of A and B by values 1 and 2

			Vector4f ax_ay_bx_by;
			ax_ay_bx_by.x = a % 3;
			ax_ay_bx_by.z = b % 3;
			ax_ay_bx_by.y = a / 3.0f;
			ax_ay_bx_by.w = b / 3.0f;

			Vector4f v_first_col = lessThan(ax_ay_bx_by, 1);
			Vector4f v_third_col = greaterThanEqual(ax_ay_bx_by, 2);
			Vector4f v_second_col = Vector4f(1, 1, 1, 1) - v_first_col - v_third_col;

			Vector3f v_a_row(v_first_col.y, v_second_col.y, v_third_col.y);
			Vector3f v_b_row(v_first_col.w, v_second_col.w, v_third_col.w);

			float fa = (lrd0 * v_first_col.x + lrd1 * v_second_col.x + lrd2 * v_third_col.x).f_Dot(v_a_row);
			float fb = (lrd0 * v_first_col.z + lrd1 * v_second_col.z + lrd2 * v_third_col.z).f_Dot(v_b_row);

			if(fa != 1.0f)
				fprintf(stderr, "error: AB multiplexing failed on A = %d !!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n", a);
			if(fb != 2.0f)
				fprintf(stderr, "error: AB multiplexing failed on B = %d !!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n", b);
		}
	}
}

/*
 *								=== ~CWaldBoost_AllInOne_ShaderGenerator ===
 */
