//-----------------------------------------------------------------------------
// libsse.h
// Pyramid of images and auxiliary structures for detection.
// Jiri Havel, DCGM FIT BUT, Brno
// $Id: libsse.h 86 2009-04-21 07:13:21Z ihavel $
//-----------------------------------------------------------------------------
#ifndef _LIBSSE_H_
#define _LIBSSE_H_

#include <aboninterface.h>

#include <vector>

#include "lrd_engine.h"
#include "imagetools.h"

typedef std::vector<abonDetection> Detections;

class SSEDetector : public abonDetector
{
protected :

    struct Level
    {
        float w, h;
        //image
        unsigned width, height, pitch;
        char *data;
        Convolutions *convolutions;
        int *xAddressTable;
        int *yAddressTable;
        Rank *ranks;

        Level();

        //alloc image, convolutions
        //alloc and fill address tables and ranks
        void Init(const unsigned width, const unsigned height, const unsigned pitch, const TClassifier * classifier);
        void Kill();

        //Calculate convolutions from image
        void Convolve();

        unsigned Scan(const TClassifier * classifier, const unsigned hstep, const unsigned vstep, Detections & detections);
    };

    typedef std::vector<Level> Levels;
    Levels levels;

    Detections detections;

    static unsigned getLevelCount(const unsigned width, const unsigned height, const TClassifier * classifier, const unsigned octaves = 0);

    void recalculateCorners(const unsigned level, Detections::iterator begin, Detections::iterator end);
public :

    SSEDetector();
    ~SSEDetector() throw();

    void SetClassifier(const TClassifier * cls);
    void SetSize(unsigned width, unsigned height);

    //allocate pyramid levels
    void Init();
    void Kill();

    //fill pyramid levels
    void Build(abonImageData *image);

    //calculate convolutions
    void Convolve();

    //run detection
    Detections & Scan();
};

#ifdef _OPENMP

class OMPSSEDetector : public SSEDetector
{
private :

    //temporary detection list for each thread
    //thread 0 uses original list
    typedef std::vector<Detections> ThreadTemps;
    ThreadTemps threadTemps;

public :

    OMPSSEDetector();
    ~OMPSSEDetector() throw();

    Detections & ConvolveScan();

};

#endif

#endif
