//-----------------------------------------------------------------------------
// lrd_engine.h
// Otimized detection with WaldBoost classifier with LRD, LRP or LBP features
// Roman Juranek, DCGM FIT BUT, Brno
// $Id: lrd_engine.h 171 2009-09-30 10:32:35Z ijuranek $
//-----------------------------------------------------------------------------

#ifndef _LRD_ENGINE_H_
#define _LRD_ENGINE_H_

#include <aboninterface.h>

#include <vector>
#include "imagetools.h"


// When MAKE_STATS is set to 1, then the engine will count
// number of evaluated windows and hypotheses.
// This WILL RAPIDLY DECREASE performance!!
#define MAKE_STATS 0

extern long long _windows;
extern long long _hypotheses;

void resetStats();

///
/// Convolution management
///

typedef std::vector<TConvolution*> Convolutions;

Convolutions * createConvolutions(const unsigned width, const unsigned height);
void releaseConvolutions(Convolutions ** conv);

///
/// Adressing table
///

int * createColAddressTable(const Convolutions * c);
int * createRowAddressTable(const Convolutions * c);

///
/// Rank table
///

typedef int Rank[16];

Rank * createClassifierRankTable(const TClassifier * classifier, const Convolutions * conv);

///
/// LRD Engine
///

/// Scan given image with classifier.
unsigned scanConvolvedImage(
        const Convolutions * const convolutions, // The convolved image
        const int * const xtable, const int * const ytable, // Tables for addressing the convolutions
        const TClassifier * const classifier, const Rank * const ranks, // The classifier
        const unsigned hstep, const unsigned vstep, // Scan step
        std::vector<abonDetection> & detections // Detection vector
);

#endif

