//-----------------------------------------------------------------------------
// timer.c
// Timer implementation
// Jiri Havel, DCGM FIT BUT, Brno
// $Id: timer.cpp 79 2009-04-16 10:34:08Z ijuranek $
//-----------------------------------------------------------------------------
#include <timer.h>

#include <assert.h>
#include <stdlib.h>
#include <time.h>
#include <stdio.h>

#ifdef _WIN32//TIMER_USE_PERFORMANCE_COUNTER

#include <windows.h>

typedef struct _TimerImplementation
{
	LARGE_INTEGER freq, start;
} TimerImplementation;

Timer *timerCreate()
{
	TimerImplementation *timer = (TimerImplementation*)malloc(sizeof(TimerImplementation));
	if(timer != NULL)
	{
		QueryPerformanceFrequency(&timer->freq);
		QueryPerformanceCounter(&timer->start);
	}
	return (Timer*)timer;
}

double timerTime(const Timer *t)
{
	const TimerImplementation *timer = (const TimerImplementation*)t;
	LARGE_INTEGER time;

	assert(timer != NULL);

	QueryPerformanceCounter(&time);
	time.QuadPart -= timer->start.QuadPart;
	return (double)time.QuadPart/(double)timer->freq.QuadPart;
}

void timerReset(Timer *t)
{
	TimerImplementation *timer = (TimerImplementation*)t;

	assert(t != NULL);

	QueryPerformanceCounter(&timer->start);
}

#else

typedef struct _TimerImplementation
{
    struct timespec start;
} TimerImplementation;

Timer *timerCreate()
{
	TimerImplementation *timer = (TimerImplementation*)malloc(sizeof(TimerImplementation));
	if(timer != NULL)
	{
		clock_gettime(CLOCK_PROCESS_CPUTIME_ID, &timer->start);
	}
	return (Timer*)timer;
}

double timerTime(const Timer *t)
{
	//const TimerImplementation *timer = (const TimerImplementation*)t;
	struct timespec time;

	assert(t != NULL);

	clock_gettime(CLOCK_PROCESS_CPUTIME_ID, &time);

	double s = (double)time.tv_sec + time.tv_nsec / 1000000000.0;

	return s;
}

void timerReset(Timer *t)
{
	TimerImplementation *timer = (TimerImplementation*)t;

	assert(t != NULL);

	clock_gettime(CLOCK_PROCESS_CPUTIME_ID, &timer->start);
}

#endif

void timerRelease(Timer *timer)
{
	free(timer);
}
