#include "window.h"

#ifdef _WIN32
  #include <windows.h>
#endif

#include <GL/freeglut.h>
#include <GL/glext.h>

#include <cassert>

#include <iostream>

// Callbacks

Window *Window::window = NULL;
unsigned width = 0, height = 0;

void Window::onDisplay()
{
    if(window->abonImage != NULL)
    {
        if(window->fullscreen)
        {
            float wa = (float)width/(float)window->abonImage->width;
            float ha = (float)height/(float)window->abonImage->height;
            glPixelZoom(wa, -ha);
        }
        else
        {
            glPixelZoom(1, -1);
        }
        glPixelStorei(GL_UNPACK_ROW_LENGTH, window->abonImage->pitch);
        glDrawPixels(
            window->abonImage->width, window->abonImage->height,
            GL_LUMINANCE, GL_UNSIGNED_BYTE, window->abonImage->data);
    }
    if(window->cvImage != NULL)
    {
        if(window->fullscreen)
        {
            float wa = (float)width/(float)window->cvImage->width;
            float ha = (float)height/(float)window->cvImage->height;
            glPixelZoom(wa, -ha);
        }
        else
        {
            glPixelZoom(1, -1);
        }
        //glPixelStorei(GL_UNPACK_ROW_LENGTH, window->cvImage->widthStep);
        glDrawPixels(
            window->cvImage->width, window->cvImage->height,
            GL_BGR, GL_UNSIGNED_BYTE, window->cvImage->imageData);
    }
    glutSwapBuffers();
}

void Window::onKeyboard(unsigned char key, int x, int y)
{
    if((key == 27) || (key == 'q'))
    {
        window->close = true;
    }
    if(key == 'f')
    {
        //window->setFullScreen();
        window->toggleFS = true;
    }
}

void Window::onReshape(int w, int h)
{
    width = w;
    height = h;
    glViewport(0, 0, w, h);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(0, w, h, 0, 0, 1);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glRasterPos2i(0, 0);
    glutPostRedisplay();
}

// Interface

void Window::setSize(unsigned width, unsigned height)
{
    if(!fullscreen)
    {
        glutReshapeWindow(width, height);
    }
}

void Window::setFullScreen()
{
    if(!fullscreen)
    {
        glutFullScreen();
        fullscreen = true;
    }
}

void Window::show(abonImageData *image)
{
    this->abonImage = image;
    this->cvImage = NULL;
    glutPostRedisplay();
}

void Window::show(IplImage *image)
{
    this->abonImage = NULL;
    this->cvImage = image;
    glutPostRedisplay();
}

void Window::update()
{
    glutMainLoopEvent();
    if (toggleFS)
    {
        setFullScreen();
        toggleFS = false;
    }
}

bool Window::isClosed()
{
    return close;
}

void Window::open()
{
    int x = 0;
    glutInit(&x,0);
    glutInitDisplayMode(GLUT_DOUBLE);
    glutCreateWindow("Output");

    glutDisplayFunc(onDisplay);
    glutKeyboardFunc(onKeyboard);
    glutReshapeFunc(onReshape);
}

Window::Window()
: abonImage(NULL), cvImage(NULL), fullscreen(false), close(false), toggleFS(false)
{
    assert(window == NULL);
    window = this;
}

Window::~Window() throw()
{
}
