/*
								+---------------------------------+
								|                                 |
								|      ***   OpenGL2.0   ***      |
								|                                 |
								|  Copyright   -tHE SWINe- 2005  |
								|                                 |
								|           OpenGl20.cpp          |
								|                                 |
								+---------------------------------+
*/

/*
 *	passed code revision at 2006-05-16
 *
 *	integers, contanining just true or false were changed to bool
 *
 *	passed code revision at 2006-07-15
 *
 *	some more integers, contanining just true or false were changed to bool
 *
 *	2007-02-05
 *
 *	added support for GL_NV_fence extension
 *
 *	2007-02-20
 *
 *	added support for following new (geForce8800 (or DX10)) extensions:
 *		- GL_ARB_occlusion_query
 *		- OpenGL 2.1
 *		- GL_EXT_timer_query
 *		- GL_EXT_draw_buffers2
 *		- GL_EXT_texture_integer
 *		- GL_EXT_gpu_shader4
 *		- GL_EXT_gpu_program_parameters
 *		- GL_EXT_bindable_uniform
 *		- GL_EXT_geometry_shader4
 *		- GL_EXT_draw_instanced
 *		- GL_EXT_texture_array
 *		- GL_EXT_texture_object
 *	removed outdated GL_EXT_texture_object extension (from 1995/10/03, promoted to core
 *	extension in OpenGL 1.1)
 *	removed dllexport attribute
 *
 *	2007-10-29
 *
 *	added some documentation comments, clarified OpenGL 1.3 multitexture / ARB multitexture
 *	distinction and improved linux compatibility
 *
 *	2007-11-10
 *
 *	improved linux compatibility
 *
 *	2008-03-04
 *
 *	now using Integer.h header, hidden __stdcall into GLApi instead
 *	of defining __stdcall blank in linux
 *
 *	2008-08-08
 *
 *	added #ifdef for windows 64
 *
 *	2008-08-19
 *
 *	gl.h is now included in OpenGL20.h (convenience)
 *
 *	2009-05-03
 *
 *	added GL_FRAMEBUFFER_INCOMPLETE_DUPLICATE_ATTACHMENT_EXT enumerant
 *	(not defined in GL_EXT_packed_depth_stencil, nor in GL_EXT_framebuffer_object)
 *
 *	added CGLExtensionHandler::n_GetAllSupportedExtensionsFuncPointers()
 *
 *	2009-05-04
 *
 *	fixed mixed windows / linux line endings
 *
 */

#include <vector>
#include "CallStack.h"
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <time.h>
#include "OpenGL20.h"

#if defined(WIN32) || defined (_WIN64)
#include <windows.h>
#endif // WIN32, _WIN64
/*#define GL_GLEXT_LEGACY
#include <GL/gl.h>*/
#if defined(WIN32) || defined (_WIN64)
#include <GL/wglext.h>
#else // WIN32, _WIN64
//#include "glxext.h"
// my glxext.h doesn't cope nicely with some unices; gotta get another version
#include <GL/glx.h>
#define glXGetProcAddress(x) (*glXGetProcAddressARB)((const GLubyte*)x)
#define wglGetProcAddress(x) (*glXGetProcAddress)((GLubyte*)(x))
#endif // WIN32, _WIN64

#ifdef NULL_FUNCTION_POINTERS
#define INIT_FUNC_POINTERS = 0
#else
#define INIT_FUNC_POINTERS 
#endif

/*
 *								=== CGLExtensionHandler ===
 */

/*
 *	static bool CGLExtensionHandler::b_Support_OpenGL(int n_min_major = 2, int n_min_minor = 0)
 *		- returns true in case OpenGL version  is greater or equal to n_min_major.n_min_minor
 *		- might return false on malformed OpenGL version string
 */
bool CGLExtensionHandler::b_Support_OpenGL(int n_min_major, int n_min_minor)
{
	__FuncGuard("CGLExtensionHandler::b_Support_OpenGL");

	const char *p_s_version;
	int n_major, n_minor;

	p_s_version = (const char*)glGetString(GL_VERSION);
	if(!p_s_version) {
#if defined(WIN32) || defined (_WIN64)
		MessageBox(0, "OpenGL extension string is 0!", "CGLExtensionHandler", MB_OK);
#endif
		return false;
	}

	if(sscanf(p_s_version, "%d.%d", &n_major, &n_minor) == 2) {
		if(n_major > n_min_major)
			return true;
		if(n_major == n_min_major && n_minor >= n_min_minor)
			return true;
	} else {
#if defined(WIN32) || defined (_WIN64)
		MessageBox(0, "OpenGL version string malformed!", "CGLExtensionHandler", MB_OK);
#endif
		_ASSERTE(0); // error - OpenGL version string malformed!
		return false;
	}

	return false;
}

/*
 *	static int CGLExtensionHandler::n_OpenGL_Version()
 *		- returns OpenGL version as major * 10 + minor (ie. for OpenGL 2.1 returns 21)
 */
int CGLExtensionHandler::n_OpenGL_Version()
{
	__FuncGuard("CGLExtensionHandler::n_OpenGL_Version");

	const char *p_s_version;
	int n_major, n_minor;

	p_s_version = (const char*)glGetString(GL_VERSION);
	if(!p_s_version) {
#if defined(WIN32) || defined (_WIN64)
		MessageBox(0, "OpenGL extension string is 0!", "CGLExtensionHandler", MB_OK);
#endif
		return -1;
	}

	if(sscanf(p_s_version, "%d.%d", &n_major, &n_minor) != 2) {
#if defined(WIN32) || defined (_WIN64)
		MessageBox(0, "OpenGL version string malformed!", "CGLExtensionHandler", MB_OK);
#endif
		_ASSERTE(0); // error - OpenGL version string malformed!
		return -1;
	}
	_ASSERTE(n_minor >= 0 && n_minor < 10);

	return n_major * 10 + n_minor;
}

/*
 *	static bool CGLExtensionHandler::b_SupportedExtension(const char *p_s_extension_name)
 *		- returns true in case p_s_extension_name is supported, otherwise false
 *		- requires space in extension string before and after the extension name
 *		  (except on beginning / end of the string)
 */
bool CGLExtensionHandler::b_SupportedExtension(const char *p_s_extension_name)
{
	__FuncGuard("CGLExtensionHandler::b_SupportedExtension");

	const char *p_s_extensions, *p_s_search_str;

	p_s_extensions = (const char*)glGetString(GL_EXTENSIONS);
	_ASSERTE(p_s_extensions);
	p_s_search_str = p_s_extensions;

	do {
		if((p_s_search_str = strstr(p_s_search_str, p_s_extension_name))) {
			if(p_s_search_str > p_s_extensions && !isspace(*(p_s_search_str - 1)))
				continue;
			if(strlen(p_s_search_str + strlen(p_s_extension_name)) &&
			   !isspace(*(p_s_search_str + strlen(p_s_extension_name))))
				continue;

			return true;
		}
	} while(p_s_search_str && strlen(++ p_s_search_str));

	return false;
}

#if defined(WIN32) || defined (_WIN64)

/*
 *	static bool CGLExtensionHandler::b_SupportedWGLExtension(const char *p_s_extension_name)
 *		- returns true in case WGL extension p_s_extension_name is supported, otherwise false
 *		- requires space in extension string before and after the extension name
 *		  (except on beginning / end of the string)
 */
bool CGLExtensionHandler::b_SupportedWGLExtension(const char *p_s_extension_name)
{
	__FuncGuard("CGLExtensionHandler::b_SupportedWGLExtension");

	const char *p_s_extensions, *p_s_search_str;

	PFNWGLGETEXTENSIONSSTRINGARBPROC wglGetExtensionsStringARB;
	if(!(wglGetExtensionsStringARB = (PFNWGLGETEXTENSIONSSTRINGARBPROC)wglGetProcAddress("wglGetExtensionsStringARB")))
		return false;

	p_s_extensions = (const char*)wglGetExtensionsStringARB(wglGetCurrentDC());
	p_s_search_str = p_s_extensions;

	do {
		if((p_s_search_str = strstr(p_s_search_str, p_s_extension_name))) {
			if(p_s_search_str > p_s_extensions && !isspace(*(p_s_search_str - 1)))
				continue;
			if(strlen(p_s_search_str + strlen(p_s_extension_name)) &&
			   !isspace(*(p_s_search_str + strlen(p_s_extension_name))))
				continue;

			return true;
		}
	} while(p_s_search_str && strlen(++ p_s_search_str));

	return false;
}

#endif

/*
 *	static int CGLExtensionHandler::n_GetAllSupportedExtensionsFuncPointers()
 *		- gets function pointers for all supported extensions and OpenGL versions
 *		- returns number of missing functions
 *		- note this is aimed at simple and test implementations without
 *		  much need for error checking. it is always better to call individual
 *		  functions, because in case this fails, there's no telling
 *		  which extension exactly failed to initialize
 */
int CGLExtensionHandler::n_GetAllSupportedExtensionsFuncPointers()
{
	return n_GetGL12FuncPointers() +
		n_GetGL13FuncPointers() +
		n_GetMultitextureFuncPointers() +
		n_GetGL14FuncPointers() +
		n_GetGL15FuncPointers() +
		n_GetGL20FuncPointers() +
		n_GetGL21FuncPointers() +
		n_GetMultitextureARBFuncPointers() +
		n_GetCompressedTextureARBFuncPointers() +
		n_GetVertexBlendARBFuncPointers() +
		n_GetMatrixPaletteARBFuncPointers() +
		n_GetFramebufferObjectEXTFuncPointers() +
		n_GetCompiledArrayEXTFuncPointers() +
		n_GetColorBufferFloatARBFuncPointers() +
		n_GetVertexProgramARBFuncPointers() +
		n_GetFragmentProgramARBFuncPointers() +
		n_GetVertexShaderARBFuncPointers() +
		n_GetFragmentShaderARBFuncPointers() +
		n_GetShaderObjectARBFuncPointers() +
		n_GetVertexBufferObjectARBFuncPointers() +
		n_GetFenceNVFuncPointers() +
		n_GetOcclusionQueryARBFuncPointers() +
		n_GetTimerQueryEXTFuncPointers() +
		n_GetDrawBuffers2EXTFuncPointers() +
		n_GetTextureIntegerEXTFuncPointers() +
		n_GetGPUShader4EXTFuncPointers() +
		n_GetGPUProgramParametersEXTFuncPointers() +
		n_GetBindableUniformEXTFuncPointers() +
		n_GetGeometryShader4EXTFuncPointers() +
		n_GetDrawInstancedEXTFuncPointers() +
		n_GetTextureArrayEXTFuncPointers() + 
		n_GetTextureObjectEXTFuncPointers() +
		n_GetBlendMinMaxEXTFuncPointers();
}

/*
 *		--- opengl 1.2 core func pointers ---
 */

#ifdef __GENERATE_GL_12__

void (GLApi *glBlendColor)(GLclampf, GLclampf, GLclampf, GLclampf) INIT_FUNC_POINTERS;
void (GLApi *glBlendEquation)(GLenum) INIT_FUNC_POINTERS;
void (GLApi *glDrawRangeElements)(GLenum, GLuint, GLuint, GLsizei, GLenum, const GLvoid*) INIT_FUNC_POINTERS;
void (GLApi *glColorTable)(GLenum, GLenum, GLsizei, GLenum, GLenum, const GLvoid*) INIT_FUNC_POINTERS;
void (GLApi *glColorTableParameterfv)(GLenum, GLenum, const GLfloat*) INIT_FUNC_POINTERS;
void (GLApi *glColorTableParameteriv)(GLenum, GLenum, const GLint*) INIT_FUNC_POINTERS;
void (GLApi *glCopyColorTable)(GLenum, GLenum, GLint, GLint, GLsizei) INIT_FUNC_POINTERS;
void (GLApi *glGetColorTable)(GLenum, GLenum, GLenum, GLvoid*) INIT_FUNC_POINTERS;
void (GLApi *glGetColorTableParameterfv)(GLenum, GLenum, GLfloat*) INIT_FUNC_POINTERS;
void (GLApi *glGetColorTableParameteriv)(GLenum, GLenum, GLint*) INIT_FUNC_POINTERS;
void (GLApi *glColorSubTable)(GLenum, GLsizei, GLsizei, GLenum, GLenum, const GLvoid*) INIT_FUNC_POINTERS;
void (GLApi *glCopyColorSubTable)(GLenum, GLsizei, GLint, GLint, GLsizei) INIT_FUNC_POINTERS;
void (GLApi *glConvolutionFilter1D)(GLenum, GLenum, GLsizei, GLenum, GLenum, const GLvoid*) INIT_FUNC_POINTERS;
void (GLApi *glConvolutionFilter2D)(GLenum, GLenum, GLsizei, GLsizei, GLenum, GLenum, const GLvoid*) INIT_FUNC_POINTERS;
void (GLApi *glConvolutionParameterf)(GLenum, GLenum, GLfloat) INIT_FUNC_POINTERS;
void (GLApi *glConvolutionParameterfv)(GLenum, GLenum, const GLfloat*) INIT_FUNC_POINTERS;
void (GLApi *glConvolutionParameteri)(GLenum, GLenum, GLint) INIT_FUNC_POINTERS;
void (GLApi *glConvolutionParameteriv)(GLenum, GLenum, const GLint*) INIT_FUNC_POINTERS;
void (GLApi *glCopyConvolutionFilter1D)(GLenum, GLenum, GLint, GLint, GLsizei) INIT_FUNC_POINTERS;
void (GLApi *glCopyConvolutionFilter2D)(GLenum, GLenum, GLint, GLint, GLsizei, GLsizei) INIT_FUNC_POINTERS;
void (GLApi *glGetConvolutionFilter)(GLenum, GLenum, GLenum, GLvoid*) INIT_FUNC_POINTERS;
void (GLApi *glGetConvolutionParameterfv)(GLenum, GLenum, GLfloat*) INIT_FUNC_POINTERS;
void (GLApi *glGetConvolutionParameteriv)(GLenum, GLenum, GLint*) INIT_FUNC_POINTERS;
void (GLApi *glGetSeparableFilter)(GLenum, GLenum, GLenum, GLvoid *, GLvoid *, GLvoid*) INIT_FUNC_POINTERS;
void (GLApi *glSeparableFilter2D)(GLenum, GLenum, GLsizei, GLsizei, GLenum, GLenum, const GLvoid *, const GLvoid*) INIT_FUNC_POINTERS;
void (GLApi *glGetHistogram)(GLenum, GLboolean, GLenum, GLenum, GLvoid*) INIT_FUNC_POINTERS;
void (GLApi *glGetHistogramParameterfv)(GLenum, GLenum, GLfloat*) INIT_FUNC_POINTERS;
void (GLApi *glGetHistogramParameteriv)(GLenum, GLenum, GLint*) INIT_FUNC_POINTERS;
void (GLApi *glGetMinmax)(GLenum, GLboolean, GLenum, GLenum, GLvoid*) INIT_FUNC_POINTERS;
void (GLApi *glGetMinmaxParameterfv)(GLenum, GLenum, GLfloat*) INIT_FUNC_POINTERS;
void (GLApi *glGetMinmaxParameteriv)(GLenum, GLenum, GLint*) INIT_FUNC_POINTERS;
void (GLApi *glHistogram)(GLenum, GLsizei, GLenum, GLboolean) INIT_FUNC_POINTERS;
void (GLApi *glMinmax)(GLenum, GLenum, GLboolean) INIT_FUNC_POINTERS;
void (GLApi *glResetHistogram)(GLenum) INIT_FUNC_POINTERS;
void (GLApi *glResetMinmax)(GLenum) INIT_FUNC_POINTERS;
void (GLApi *glTexImage3D)(GLenum, GLint, GLint, GLsizei, GLsizei, GLsizei, GLint, GLenum, GLenum, const GLvoid*) INIT_FUNC_POINTERS;
void (GLApi *glTexSubImage3D)(GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLenum, const GLvoid*) INIT_FUNC_POINTERS;
void (GLApi *glCopyTexSubImage3D)(GLenum, GLint, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei) INIT_FUNC_POINTERS;

#endif //__GENERATE_GL_12__

/*
 *	static int CGLExtensionHandler::n_GetGL12FuncPointers()
 *		- gets OpenGL 1.2 core function pointers
 *		- returns number of missing functions
 */
int CGLExtensionHandler::n_GetGL12FuncPointers()
{
	__FuncGuard("CGLExtensionHandler::n_GetGL12FuncPointers");

	int n_failed_functions = 0;

#ifdef __GENERATE_GL_12__

	if(!(glBlendColor = (void(GLApi*)(GLclampf, GLclampf, GLclampf, GLclampf))wglGetProcAddress("glBlendColor"))) ++ n_failed_functions;
	if(!(glBlendEquation = (void(GLApi*)(GLenum))wglGetProcAddress("glBlendEquation"))) ++ n_failed_functions;
	if(!(glDrawRangeElements = (void(GLApi*)(GLenum, GLuint, GLuint, GLsizei, GLenum, const GLvoid*))wglGetProcAddress("glDrawRangeElements"))) ++ n_failed_functions;
	if(!(glColorTable = (void(GLApi*)(GLenum, GLenum, GLsizei, GLenum, GLenum, const GLvoid*))wglGetProcAddress("glColorTable"))) ++ n_failed_functions;
	if(!(glColorTableParameterfv = (void(GLApi*)(GLenum, GLenum, const GLfloat*))wglGetProcAddress("glColorTableParameterfv"))) ++ n_failed_functions;
	if(!(glColorTableParameteriv = (void(GLApi*)(GLenum, GLenum, const GLint*))wglGetProcAddress("glColorTableParameteriv"))) ++ n_failed_functions;
	if(!(glCopyColorTable = (void(GLApi*)(GLenum, GLenum, GLint, GLint, GLsizei))wglGetProcAddress("glCopyColorTable"))) ++ n_failed_functions;
	if(!(glGetColorTable = (void(GLApi*)(GLenum, GLenum, GLenum, GLvoid*))wglGetProcAddress("glGetColorTable"))) ++ n_failed_functions;
	if(!(glGetColorTableParameterfv = (void(GLApi*)(GLenum, GLenum, GLfloat*))wglGetProcAddress("glGetColorTableParameterfv"))) ++ n_failed_functions;
	if(!(glGetColorTableParameteriv = (void(GLApi*)(GLenum, GLenum, GLint*))wglGetProcAddress("glGetColorTableParameteriv"))) ++ n_failed_functions;
	if(!(glColorSubTable = (void(GLApi*)(GLenum, GLsizei, GLsizei, GLenum, GLenum, const GLvoid*))wglGetProcAddress("glColorSubTable"))) ++ n_failed_functions;
	if(!(glCopyColorSubTable = (void(GLApi*)(GLenum, GLsizei, GLint, GLint, GLsizei))wglGetProcAddress("glCopyColorSubTable"))) ++ n_failed_functions;
	if(!(glConvolutionFilter1D = (void(GLApi*)(GLenum, GLenum, GLsizei, GLenum, GLenum, const GLvoid*))wglGetProcAddress("glConvolutionFilter1D"))) ++ n_failed_functions;
	if(!(glConvolutionFilter2D = (void(GLApi*)(GLenum, GLenum, GLsizei, GLsizei, GLenum, GLenum, const GLvoid*))wglGetProcAddress("glConvolutionFilter2D"))) ++ n_failed_functions;
	if(!(glConvolutionParameterf = (void(GLApi*)(GLenum, GLenum, GLfloat))wglGetProcAddress("glConvolutionParameterf"))) ++ n_failed_functions;
	if(!(glConvolutionParameterfv = (void(GLApi*)(GLenum, GLenum, const GLfloat*))wglGetProcAddress("glConvolutionParameterfv"))) ++ n_failed_functions;
	if(!(glConvolutionParameteri = (void(GLApi*)(GLenum, GLenum, GLint))wglGetProcAddress("glConvolutionParameteri"))) ++ n_failed_functions;
	if(!(glConvolutionParameteriv = (void(GLApi*)(GLenum, GLenum, const GLint*))wglGetProcAddress("glConvolutionParameteriv"))) ++ n_failed_functions;
	if(!(glCopyConvolutionFilter1D = (void(GLApi*)(GLenum, GLenum, GLint, GLint, GLsizei))wglGetProcAddress("glCopyConvolutionFilter1D"))) ++ n_failed_functions;
	if(!(glCopyConvolutionFilter2D = (void(GLApi*)(GLenum, GLenum, GLint, GLint, GLsizei, GLsizei))wglGetProcAddress("glCopyConvolutionFilter2D"))) ++ n_failed_functions;
	if(!(glGetConvolutionFilter = (void(GLApi*)(GLenum, GLenum, GLenum, GLvoid*))wglGetProcAddress("glGetConvolutionFilter"))) ++ n_failed_functions;
	if(!(glGetConvolutionParameterfv = (void(GLApi*)(GLenum, GLenum, GLfloat*))wglGetProcAddress("glGetConvolutionParameterfv"))) ++ n_failed_functions;
	if(!(glGetConvolutionParameteriv = (void(GLApi*)(GLenum, GLenum, GLint*))wglGetProcAddress("glGetConvolutionParameteriv"))) ++ n_failed_functions;
	if(!(glGetSeparableFilter = (void(GLApi*)(GLenum, GLenum, GLenum, GLvoid *, GLvoid *, GLvoid*))wglGetProcAddress("glGetSeparableFilter"))) ++ n_failed_functions;
	if(!(glSeparableFilter2D = (void(GLApi*)(GLenum, GLenum, GLsizei, GLsizei, GLenum, GLenum, const GLvoid *, const GLvoid*))wglGetProcAddress("glSeparableFilter2D"))) ++ n_failed_functions;
	if(!(glGetHistogram = (void(GLApi*)(GLenum, GLboolean, GLenum, GLenum, GLvoid*))wglGetProcAddress("glGetHistogram"))) ++ n_failed_functions;
	if(!(glGetHistogramParameterfv = (void(GLApi*)(GLenum, GLenum, GLfloat*))wglGetProcAddress("glGetHistogramParameterfv"))) ++ n_failed_functions;
	if(!(glGetHistogramParameteriv = (void(GLApi*)(GLenum, GLenum, GLint*))wglGetProcAddress("glGetHistogramParameteriv"))) ++ n_failed_functions;
	if(!(glGetMinmax = (void(GLApi*)(GLenum, GLboolean, GLenum, GLenum, GLvoid*))wglGetProcAddress("glGetMinmax"))) ++ n_failed_functions;
	if(!(glGetMinmaxParameterfv = (void(GLApi*)(GLenum, GLenum, GLfloat*))wglGetProcAddress("glGetMinmaxParameterfv"))) ++ n_failed_functions;
	if(!(glGetMinmaxParameteriv = (void(GLApi*)(GLenum, GLenum, GLint*))wglGetProcAddress("glGetMinmaxParameteriv"))) ++ n_failed_functions;
	if(!(glHistogram = (void(GLApi*)(GLenum, GLsizei, GLenum, GLboolean))wglGetProcAddress("glHistogram"))) ++ n_failed_functions;
	if(!(glMinmax = (void(GLApi*)(GLenum, GLenum, GLboolean))wglGetProcAddress("glMinmax"))) ++ n_failed_functions;
	if(!(glResetHistogram = (void(GLApi*)(GLenum))wglGetProcAddress("glResetHistogram"))) ++ n_failed_functions;
	if(!(glResetMinmax = (void(GLApi*)(GLenum))wglGetProcAddress("glResetMinmax"))) ++ n_failed_functions;
	if(!(glTexImage3D = (void(GLApi*)(GLenum, GLint, GLint, GLsizei, GLsizei, GLsizei, GLint, GLenum, GLenum, const GLvoid*))wglGetProcAddress("glTexImage3D"))) ++ n_failed_functions;
	if(!(glTexSubImage3D = (void(GLApi*)(GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLenum, const GLvoid*))wglGetProcAddress("glTexSubImage3D"))) ++ n_failed_functions;
	if(!(glCopyTexSubImage3D = (void(GLApi*)(GLenum, GLint, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei))wglGetProcAddress("glCopyTexSubImage3D"))) ++ n_failed_functions;

#endif //__GENERATE_GL_12__

	return n_failed_functions;
}

/*
 *		--- opengl 1.3 core func pointers ---
 */

#ifdef __GENERATE_GL_13__

void (GLApi *glLoadTransposeMatrixf)(const GLfloat*) INIT_FUNC_POINTERS;
void (GLApi *glLoadTransposeMatrixd)(const GLdouble*) INIT_FUNC_POINTERS;
void (GLApi *glMultTransposeMatrixf)(const GLfloat*) INIT_FUNC_POINTERS;
void (GLApi *glMultTransposeMatrixd)(const GLdouble*) INIT_FUNC_POINTERS;
void (GLApi *glSampleCoverage)(GLclampf, GLboolean) INIT_FUNC_POINTERS;
void (GLApi *glCompressedTexImage3D)(GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLsizei, const GLvoid*) INIT_FUNC_POINTERS;
void (GLApi *glCompressedTexImage2D)(GLenum, GLint, GLenum, GLsizei, GLsizei, GLint, GLsizei, const GLvoid*) INIT_FUNC_POINTERS;
void (GLApi *glCompressedTexImage1D)(GLenum, GLint, GLenum, GLsizei, GLint, GLsizei, const GLvoid*) INIT_FUNC_POINTERS;
void (GLApi *glCompressedTexSubImage3D)(GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLsizei, const GLvoid*) INIT_FUNC_POINTERS;
void (GLApi *glCompressedTexSubImage2D)(GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLsizei, const GLvoid*) INIT_FUNC_POINTERS;
void (GLApi *glCompressedTexSubImage1D)(GLenum, GLint, GLint, GLsizei, GLenum, GLsizei, const GLvoid*) INIT_FUNC_POINTERS;
void (GLApi *glGetCompressedTexImage)(GLenum, GLint, GLvoid*) INIT_FUNC_POINTERS;

#endif //__GENERATE_GL_13__

/*
 *	static int CGLExtensionHandler::n_GetGL13FuncPointers()
 *		- gets OpenGL 1.3 core function pointers (including multitexture function pointers)
 *		- returns number of missing functions
 */
int CGLExtensionHandler::n_GetGL13FuncPointers()
{
	__FuncGuard("CGLExtensionHandler::n_GetGL13FuncPointers");

	int n_failed_functions = 0;

#ifdef __GENERATE_GL_13__

	if(!(glLoadTransposeMatrixf = (void(GLApi*)(const GLfloat*))wglGetProcAddress("glLoadTransposeMatrixf"))) ++ n_failed_functions;
	if(!(glLoadTransposeMatrixd = (void(GLApi*)(const GLdouble*))wglGetProcAddress("glLoadTransposeMatrixd"))) ++ n_failed_functions;
	if(!(glMultTransposeMatrixf = (void(GLApi*)(const GLfloat*))wglGetProcAddress("glMultTransposeMatrixf"))) ++ n_failed_functions;
	if(!(glMultTransposeMatrixd = (void(GLApi*)(const GLdouble*))wglGetProcAddress("glMultTransposeMatrixd"))) ++ n_failed_functions;
	if(!(glSampleCoverage = (void(GLApi*)(GLclampf, GLboolean))wglGetProcAddress("glSampleCoverage"))) ++ n_failed_functions;
	if(!(glCompressedTexImage3D = (void(GLApi*)(GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLsizei, const GLvoid*))wglGetProcAddress("glCompressedTexImage3D"))) ++ n_failed_functions;
	if(!(glCompressedTexImage2D = (void(GLApi*)(GLenum, GLint, GLenum, GLsizei, GLsizei, GLint, GLsizei, const GLvoid*))wglGetProcAddress("glCompressedTexImage2D"))) ++ n_failed_functions;
	if(!(glCompressedTexImage1D = (void(GLApi*)(GLenum, GLint, GLenum, GLsizei, GLint, GLsizei, const GLvoid*))wglGetProcAddress("glCompressedTexImage1D"))) ++ n_failed_functions;
	if(!(glCompressedTexSubImage3D = (void(GLApi*)(GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLsizei, const GLvoid*))wglGetProcAddress("glCompressedTexSubImage3D"))) ++ n_failed_functions;
	if(!(glCompressedTexSubImage2D = (void(GLApi*)(GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLsizei, const GLvoid*))wglGetProcAddress("glCompressedTexSubImage2D"))) ++ n_failed_functions;
	if(!(glCompressedTexSubImage1D = (void(GLApi*)(GLenum, GLint, GLint, GLsizei, GLenum, GLsizei, const GLvoid*))wglGetProcAddress("glCompressedTexSubImage1D"))) ++ n_failed_functions;
	if(!(glGetCompressedTexImage = (void(GLApi*)(GLenum, GLint, GLvoid*))wglGetProcAddress("glGetCompressedTexImage"))) ++ n_failed_functions;

#endif //__GENERATE_GL_13__

	return n_failed_functions + n_GetMultitextureFuncPointers();
}

/*
 *		--- Multitexture func pointers ---
 */

#ifdef __GENERATE_GL_13__

void (GLApi *glActiveTexture)(GLenum) INIT_FUNC_POINTERS;
void (GLApi *glClientActiveTexture)(GLenum) INIT_FUNC_POINTERS;
void (GLApi *glMultiTexCoord1d)(GLenum, GLdouble) INIT_FUNC_POINTERS;
void (GLApi *glMultiTexCoord1dv)(GLenum, const GLdouble*) INIT_FUNC_POINTERS;
void (GLApi *glMultiTexCoord1f)(GLenum, GLfloat) INIT_FUNC_POINTERS;
void (GLApi *glMultiTexCoord1fv)(GLenum, const GLfloat*) INIT_FUNC_POINTERS;
void (GLApi *glMultiTexCoord1i)(GLenum, GLint) INIT_FUNC_POINTERS;
void (GLApi *glMultiTexCoord1iv)(GLenum, const GLint *) INIT_FUNC_POINTERS;
void (GLApi *glMultiTexCoord1s)(GLenum, GLshort) INIT_FUNC_POINTERS;
void (GLApi *glMultiTexCoord1sv)(GLenum, const GLshort*) INIT_FUNC_POINTERS;
void (GLApi *glMultiTexCoord2d)(GLenum, GLdouble, GLdouble) INIT_FUNC_POINTERS;
void (GLApi *glMultiTexCoord2dv)(GLenum, const GLdouble*) INIT_FUNC_POINTERS;
void (GLApi *glMultiTexCoord2f)(GLenum, GLfloat, GLfloat) INIT_FUNC_POINTERS;
void (GLApi *glMultiTexCoord2fv)(GLenum, const GLfloat*) INIT_FUNC_POINTERS;
void (GLApi *glMultiTexCoord2i)(GLenum, GLint, GLint) INIT_FUNC_POINTERS;
void (GLApi *glMultiTexCoord2iv)(GLenum, const GLint*) INIT_FUNC_POINTERS;
void (GLApi *glMultiTexCoord2s)(GLenum, GLshort, GLshort) INIT_FUNC_POINTERS;
void (GLApi *glMultiTexCoord2sv)(GLenum, const GLshort*) INIT_FUNC_POINTERS;
void (GLApi *glMultiTexCoord3d)(GLenum, GLdouble, GLdouble, GLdouble) INIT_FUNC_POINTERS;
void (GLApi *glMultiTexCoord3dv)(GLenum, const GLdouble*) INIT_FUNC_POINTERS;
void (GLApi *glMultiTexCoord3f)(GLenum, GLfloat, GLfloat, GLfloat) INIT_FUNC_POINTERS;
void (GLApi *glMultiTexCoord3fv)(GLenum, const GLfloat*) INIT_FUNC_POINTERS;
void (GLApi *glMultiTexCoord3i)(GLenum, GLint, GLint, GLint) INIT_FUNC_POINTERS;
void (GLApi *glMultiTexCoord3iv)(GLenum, const GLint*) INIT_FUNC_POINTERS;
void (GLApi *glMultiTexCoord3s)(GLenum, GLshort, GLshort, GLshort) INIT_FUNC_POINTERS;
void (GLApi *glMultiTexCoord3sv)(GLenum, const GLshort*) INIT_FUNC_POINTERS;
void (GLApi *glMultiTexCoord4d)(GLenum, GLdouble, GLdouble, GLdouble, GLdouble) INIT_FUNC_POINTERS;
void (GLApi *glMultiTexCoord4dv)(GLenum, const GLdouble*) INIT_FUNC_POINTERS;
void (GLApi *glMultiTexCoord4f)(GLenum, GLfloat, GLfloat, GLfloat, GLfloat) INIT_FUNC_POINTERS;
void (GLApi *glMultiTexCoord4fv)(GLenum, const GLfloat*) INIT_FUNC_POINTERS;
void (GLApi *glMultiTexCoord4i)(GLenum, GLint, GLint, GLint, GLint) INIT_FUNC_POINTERS;
void (GLApi *glMultiTexCoord4iv)(GLenum, const GLint*) INIT_FUNC_POINTERS;
void (GLApi *glMultiTexCoord4s)(GLenum, GLshort, GLshort, GLshort, GLshort) INIT_FUNC_POINTERS;
void (GLApi *glMultiTexCoord4sv)(GLenum, const GLshort*) INIT_FUNC_POINTERS;

#endif //__GENERATE_GL_13__

/*
 *	static int CGLExtensionHandler::n_GetMultitextureFuncPointers()
 *		- gets OpenGL 1.3 multitexture function pointers
 *		- returns number of missing functions
 */
int CGLExtensionHandler::n_GetMultitextureFuncPointers()
{
	__FuncGuard("CGLExtensionHandler::n_GetMultitextureFuncPointers");

	int n_failed_functions = 0;

#ifdef __GENERATE_GL_13__

	if(!(glActiveTexture = (void(GLApi*)(GLenum))wglGetProcAddress("glActiveTexture"))) ++ n_failed_functions;
	if(!(glClientActiveTexture = (void(GLApi*)(GLenum))wglGetProcAddress("glClientActiveTexture"))) ++ n_failed_functions;
	if(!(glMultiTexCoord1d = (void(GLApi*)(GLenum, GLdouble))wglGetProcAddress("glMultiTexCoord1d"))) ++ n_failed_functions;
	if(!(glMultiTexCoord1dv = (void(GLApi*)(GLenum, const GLdouble*))wglGetProcAddress("glMultiTexCoord1dv"))) ++ n_failed_functions;
	if(!(glMultiTexCoord1f = (void(GLApi*)(GLenum, GLfloat))wglGetProcAddress("glMultiTexCoord1f"))) ++ n_failed_functions;
	if(!(glMultiTexCoord1fv = (void(GLApi*)(GLenum, const GLfloat*))wglGetProcAddress("glMultiTexCoord1fv"))) ++ n_failed_functions;
	if(!(glMultiTexCoord1i = (void(GLApi*)(GLenum, GLint))wglGetProcAddress("glMultiTexCoord1i"))) ++ n_failed_functions;
	if(!(glMultiTexCoord1iv = (void(GLApi*)(GLenum, const GLint *))wglGetProcAddress("glMultiTexCoord1iv"))) ++ n_failed_functions;
	if(!(glMultiTexCoord1s = (void(GLApi*)(GLenum, GLshort))wglGetProcAddress("glMultiTexCoord1s"))) ++ n_failed_functions;
	if(!(glMultiTexCoord1sv = (void(GLApi*)(GLenum, const GLshort*))wglGetProcAddress("glMultiTexCoord1sv"))) ++ n_failed_functions;
	if(!(glMultiTexCoord2d = (void(GLApi*)(GLenum, GLdouble, GLdouble))wglGetProcAddress("glMultiTexCoord2d"))) ++ n_failed_functions;
	if(!(glMultiTexCoord2dv = (void(GLApi*)(GLenum, const GLdouble*))wglGetProcAddress("glMultiTexCoord2dv"))) ++ n_failed_functions;
	if(!(glMultiTexCoord2f = (void(GLApi*)(GLenum, GLfloat, GLfloat))wglGetProcAddress("glMultiTexCoord2f"))) ++ n_failed_functions;
	if(!(glMultiTexCoord2fv = (void(GLApi*)(GLenum, const GLfloat*))wglGetProcAddress("glMultiTexCoord2fv"))) ++ n_failed_functions;
	if(!(glMultiTexCoord2i = (void(GLApi*)(GLenum, GLint, GLint))wglGetProcAddress("glMultiTexCoord2i"))) ++ n_failed_functions;
	if(!(glMultiTexCoord2iv = (void(GLApi*)(GLenum, const GLint*))wglGetProcAddress("glMultiTexCoord2iv"))) ++ n_failed_functions;
	if(!(glMultiTexCoord2s = (void(GLApi*)(GLenum, GLshort, GLshort))wglGetProcAddress("glMultiTexCoord2s"))) ++ n_failed_functions;
	if(!(glMultiTexCoord2sv = (void(GLApi*)(GLenum, const GLshort*))wglGetProcAddress("glMultiTexCoord2sv"))) ++ n_failed_functions;
	if(!(glMultiTexCoord3d = (void(GLApi*)(GLenum, GLdouble, GLdouble, GLdouble))wglGetProcAddress("glMultiTexCoord3d"))) ++ n_failed_functions;
	if(!(glMultiTexCoord3dv = (void(GLApi*)(GLenum, const GLdouble*))wglGetProcAddress("glMultiTexCoord3dv"))) ++ n_failed_functions;
	if(!(glMultiTexCoord3f = (void(GLApi*)(GLenum, GLfloat, GLfloat, GLfloat))wglGetProcAddress("glMultiTexCoord3f"))) ++ n_failed_functions;
	if(!(glMultiTexCoord3fv = (void(GLApi*)(GLenum, const GLfloat*))wglGetProcAddress("glMultiTexCoord3fv"))) ++ n_failed_functions;
	if(!(glMultiTexCoord3i = (void(GLApi*)(GLenum, GLint, GLint, GLint))wglGetProcAddress("glMultiTexCoord3i"))) ++ n_failed_functions;
	if(!(glMultiTexCoord3iv = (void(GLApi*)(GLenum, const GLint*))wglGetProcAddress("glMultiTexCoord3iv"))) ++ n_failed_functions;
	if(!(glMultiTexCoord3s = (void(GLApi*)(GLenum, GLshort, GLshort, GLshort))wglGetProcAddress("glMultiTexCoord3s"))) ++ n_failed_functions;
	if(!(glMultiTexCoord3sv = (void(GLApi*)(GLenum, const GLshort*))wglGetProcAddress("glMultiTexCoord3sv"))) ++ n_failed_functions;
	if(!(glMultiTexCoord4d = (void(GLApi*)(GLenum, GLdouble, GLdouble, GLdouble, GLdouble))wglGetProcAddress("glMultiTexCoord4d"))) ++ n_failed_functions;
	if(!(glMultiTexCoord4dv = (void(GLApi*)(GLenum, const GLdouble*))wglGetProcAddress("glMultiTexCoord4dv"))) ++ n_failed_functions;
	if(!(glMultiTexCoord4f = (void(GLApi*)(GLenum, GLfloat, GLfloat, GLfloat, GLfloat))wglGetProcAddress("glMultiTexCoord4f"))) ++ n_failed_functions;
	if(!(glMultiTexCoord4fv = (void(GLApi*)(GLenum, const GLfloat*))wglGetProcAddress("glMultiTexCoord4fv"))) ++ n_failed_functions;
	if(!(glMultiTexCoord4i = (void(GLApi*)(GLenum, GLint, GLint, GLint, GLint))wglGetProcAddress("glMultiTexCoord4i"))) ++ n_failed_functions;
	if(!(glMultiTexCoord4iv = (void(GLApi*)(GLenum, const GLint*))wglGetProcAddress("glMultiTexCoord4iv"))) ++ n_failed_functions;
	if(!(glMultiTexCoord4s = (void(GLApi*)(GLenum, GLshort, GLshort, GLshort, GLshort))wglGetProcAddress("glMultiTexCoord4s"))) ++ n_failed_functions;
	if(!(glMultiTexCoord4sv = (void(GLApi*)(GLenum, const GLshort*))wglGetProcAddress("glMultiTexCoord4sv"))) ++ n_failed_functions;

#endif //__GENERATE_GL_13__

	return n_failed_functions;
}

/*
 *	static int CGLExtensionHandler::n_GetMultitextureARBFuncPointers()
 *		- gets GL_ARB_multitexture function pointers
 *		- note function names are ones of OpenGL 1.3 multitexture, no ARB suffix here
 *		- returns number of missing functions
 */
int CGLExtensionHandler::n_GetMultitextureARBFuncPointers()
{
	__FuncGuard("CGLExtensionHandler::n_GetMultitextureARBFuncPointers");

	int n_failed_functions = 0;

#ifdef __GENERATE_GL_13__

	if(!(glActiveTexture = (void(GLApi*)(GLenum))wglGetProcAddress("glActiveTextureARB"))) ++ n_failed_functions;
	if(!(glClientActiveTexture = (void(GLApi*)(GLenum))wglGetProcAddress("glClientActiveTextureARB"))) ++ n_failed_functions;
	if(!(glMultiTexCoord1d = (void(GLApi*)(GLenum, GLdouble))wglGetProcAddress("glMultiTexCoord1dARB"))) ++ n_failed_functions;
	if(!(glMultiTexCoord1dv = (void(GLApi*)(GLenum, const GLdouble*))wglGetProcAddress("glMultiTexCoord1dvARB"))) ++ n_failed_functions;
	if(!(glMultiTexCoord1f = (void(GLApi*)(GLenum, GLfloat))wglGetProcAddress("glMultiTexCoord1fARB"))) ++ n_failed_functions;
	if(!(glMultiTexCoord1fv = (void(GLApi*)(GLenum, const GLfloat*))wglGetProcAddress("glMultiTexCoord1fvARB"))) ++ n_failed_functions;
	if(!(glMultiTexCoord1i = (void(GLApi*)(GLenum, GLint))wglGetProcAddress("glMultiTexCoord1iARB"))) ++ n_failed_functions;
	if(!(glMultiTexCoord1iv = (void(GLApi*)(GLenum, const GLint *))wglGetProcAddress("glMultiTexCoord1ivARB"))) ++ n_failed_functions;
	if(!(glMultiTexCoord1s = (void(GLApi*)(GLenum, GLshort))wglGetProcAddress("glMultiTexCoord1sARB"))) ++ n_failed_functions;
	if(!(glMultiTexCoord1sv = (void(GLApi*)(GLenum, const GLshort*))wglGetProcAddress("glMultiTexCoord1svARB"))) ++ n_failed_functions;
	if(!(glMultiTexCoord2d = (void(GLApi*)(GLenum, GLdouble, GLdouble))wglGetProcAddress("glMultiTexCoord2dARB"))) ++ n_failed_functions;
	if(!(glMultiTexCoord2dv = (void(GLApi*)(GLenum, const GLdouble*))wglGetProcAddress("glMultiTexCoord2dvARB"))) ++ n_failed_functions;
	if(!(glMultiTexCoord2f = (void(GLApi*)(GLenum, GLfloat, GLfloat))wglGetProcAddress("glMultiTexCoord2fARB"))) ++ n_failed_functions;
	if(!(glMultiTexCoord2fv = (void(GLApi*)(GLenum, const GLfloat*))wglGetProcAddress("glMultiTexCoord2fvARB"))) ++ n_failed_functions;
	if(!(glMultiTexCoord2i = (void(GLApi*)(GLenum, GLint, GLint))wglGetProcAddress("glMultiTexCoord2iARB"))) ++ n_failed_functions;
	if(!(glMultiTexCoord2iv = (void(GLApi*)(GLenum, const GLint*))wglGetProcAddress("glMultiTexCoord2ivARB"))) ++ n_failed_functions;
	if(!(glMultiTexCoord2s = (void(GLApi*)(GLenum, GLshort, GLshort))wglGetProcAddress("glMultiTexCoord2sARB"))) ++ n_failed_functions;
	if(!(glMultiTexCoord2sv = (void(GLApi*)(GLenum, const GLshort*))wglGetProcAddress("glMultiTexCoord2svARB"))) ++ n_failed_functions;
	if(!(glMultiTexCoord3d = (void(GLApi*)(GLenum, GLdouble, GLdouble, GLdouble))wglGetProcAddress("glMultiTexCoord3dARB"))) ++ n_failed_functions;
	if(!(glMultiTexCoord3dv = (void(GLApi*)(GLenum, const GLdouble*))wglGetProcAddress("glMultiTexCoord3dvARB"))) ++ n_failed_functions;
	if(!(glMultiTexCoord3f = (void(GLApi*)(GLenum, GLfloat, GLfloat, GLfloat))wglGetProcAddress("glMultiTexCoord3fARB"))) ++ n_failed_functions;
	if(!(glMultiTexCoord3fv = (void(GLApi*)(GLenum, const GLfloat*))wglGetProcAddress("glMultiTexCoord3fvARB"))) ++ n_failed_functions;
	if(!(glMultiTexCoord3i = (void(GLApi*)(GLenum, GLint, GLint, GLint))wglGetProcAddress("glMultiTexCoord3iARB"))) ++ n_failed_functions;
	if(!(glMultiTexCoord3iv = (void(GLApi*)(GLenum, const GLint*))wglGetProcAddress("glMultiTexCoord3ivARB"))) ++ n_failed_functions;
	if(!(glMultiTexCoord3s = (void(GLApi*)(GLenum, GLshort, GLshort, GLshort))wglGetProcAddress("glMultiTexCoord3sARB"))) ++ n_failed_functions;
	if(!(glMultiTexCoord3sv = (void(GLApi*)(GLenum, const GLshort*))wglGetProcAddress("glMultiTexCoord3svARB"))) ++ n_failed_functions;
	if(!(glMultiTexCoord4d = (void(GLApi*)(GLenum, GLdouble, GLdouble, GLdouble, GLdouble))wglGetProcAddress("glMultiTexCoord4dARB"))) ++ n_failed_functions;
	if(!(glMultiTexCoord4dv = (void(GLApi*)(GLenum, const GLdouble*))wglGetProcAddress("glMultiTexCoord4dvARB"))) ++ n_failed_functions;
	if(!(glMultiTexCoord4f = (void(GLApi*)(GLenum, GLfloat, GLfloat, GLfloat, GLfloat))wglGetProcAddress("glMultiTexCoord4fARB"))) ++ n_failed_functions;
	if(!(glMultiTexCoord4fv = (void(GLApi*)(GLenum, const GLfloat*))wglGetProcAddress("glMultiTexCoord4fvARB"))) ++ n_failed_functions;
	if(!(glMultiTexCoord4i = (void(GLApi*)(GLenum, GLint, GLint, GLint, GLint))wglGetProcAddress("glMultiTexCoord4iARB"))) ++ n_failed_functions;
	if(!(glMultiTexCoord4iv = (void(GLApi*)(GLenum, const GLint*))wglGetProcAddress("glMultiTexCoord4ivARB"))) ++ n_failed_functions;
	if(!(glMultiTexCoord4s = (void(GLApi*)(GLenum, GLshort, GLshort, GLshort, GLshort))wglGetProcAddress("glMultiTexCoord4sARB"))) ++ n_failed_functions;
	if(!(glMultiTexCoord4sv = (void(GLApi*)(GLenum, const GLshort*))wglGetProcAddress("glMultiTexCoord4svARB"))) ++ n_failed_functions;

#endif //__GENERATE_GL_13__

	return n_failed_functions;
}

/*
 *		--- opengl 1.4 core func pointers ---
 */

#ifdef __GENERATE_GL_14__

void (GLApi *glBlendFuncSeparate)(GLenum, GLenum, GLenum, GLenum) INIT_FUNC_POINTERS;
void (GLApi *glFogCoordf)(GLfloat) INIT_FUNC_POINTERS;
void (GLApi *glFogCoordfv)(const GLfloat*) INIT_FUNC_POINTERS;
void (GLApi *glFogCoordd)(GLdouble) INIT_FUNC_POINTERS;
void (GLApi *glFogCoorddv)(const GLdouble*) INIT_FUNC_POINTERS;
void (GLApi *glFogCoordPointer)(GLenum, GLsizei, const GLvoid*) INIT_FUNC_POINTERS;
void (GLApi *glMultiDrawArrays)(GLenum, GLint *, GLsizei *, GLsizei) INIT_FUNC_POINTERS;
void (GLApi *glMultiDrawElements)(GLenum, const GLsizei *, GLenum, const GLvoid* *, GLsizei) INIT_FUNC_POINTERS;
void (GLApi *glPointParameterf)(GLenum, GLfloat) INIT_FUNC_POINTERS;
void (GLApi *glPointParameterfv)(GLenum, const GLfloat*) INIT_FUNC_POINTERS;
void (GLApi *glPointParameteri)(GLenum, GLint) INIT_FUNC_POINTERS;
void (GLApi *glPointParameteriv)(GLenum, const GLint*) INIT_FUNC_POINTERS;
void (GLApi *glSecondaryColor3b)(GLbyte, GLbyte, GLbyte) INIT_FUNC_POINTERS;
void (GLApi *glSecondaryColor3bv)(const GLbyte*) INIT_FUNC_POINTERS;
void (GLApi *glSecondaryColor3d)(GLdouble, GLdouble, GLdouble) INIT_FUNC_POINTERS;
void (GLApi *glSecondaryColor3dv)(const GLdouble*) INIT_FUNC_POINTERS;
void (GLApi *glSecondaryColor3f)(GLfloat, GLfloat, GLfloat) INIT_FUNC_POINTERS;
void (GLApi *glSecondaryColor3fv)(const GLfloat*) INIT_FUNC_POINTERS;
void (GLApi *glSecondaryColor3i)(GLint, GLint, GLint) INIT_FUNC_POINTERS;
void (GLApi *glSecondaryColor3iv)(const GLint*) INIT_FUNC_POINTERS;
void (GLApi *glSecondaryColor3s)(GLshort, GLshort, GLshort) INIT_FUNC_POINTERS;
void (GLApi *glSecondaryColor3sv)(const GLshort*) INIT_FUNC_POINTERS;
void (GLApi *glSecondaryColor3ub)(GLubyte, GLubyte, GLubyte) INIT_FUNC_POINTERS;
void (GLApi *glSecondaryColor3ubv)(const GLubyte*) INIT_FUNC_POINTERS;
void (GLApi *glSecondaryColor3ui)(GLuint, GLuint, GLuint) INIT_FUNC_POINTERS;
void (GLApi *glSecondaryColor3uiv)(const GLuint*) INIT_FUNC_POINTERS;
void (GLApi *glSecondaryColor3us)(GLushort, GLushort, GLushort) INIT_FUNC_POINTERS;
void (GLApi *glSecondaryColor3usv)(const GLushort*) INIT_FUNC_POINTERS;
void (GLApi *glSecondaryColorPointer)(GLint, GLenum, GLsizei, const GLvoid*) INIT_FUNC_POINTERS;
void (GLApi *glWindowPos2d)(GLdouble, GLdouble) INIT_FUNC_POINTERS;
void (GLApi *glWindowPos2dv)(const GLdouble*) INIT_FUNC_POINTERS;
void (GLApi *glWindowPos2f)(GLfloat, GLfloat) INIT_FUNC_POINTERS;
void (GLApi *glWindowPos2fv)(const GLfloat*) INIT_FUNC_POINTERS;
void (GLApi *glWindowPos2i)(GLint, GLint) INIT_FUNC_POINTERS;
void (GLApi *glWindowPos2iv)(const GLint*) INIT_FUNC_POINTERS;
void (GLApi *glWindowPos2s)(GLshort, GLshort) INIT_FUNC_POINTERS;
void (GLApi *glWindowPos2sv)(const GLshort*) INIT_FUNC_POINTERS;
void (GLApi *glWindowPos3d)(GLdouble, GLdouble, GLdouble) INIT_FUNC_POINTERS;
void (GLApi *glWindowPos3dv)(const GLdouble*) INIT_FUNC_POINTERS;
void (GLApi *glWindowPos3f)(GLfloat, GLfloat, GLfloat) INIT_FUNC_POINTERS;
void (GLApi *glWindowPos3fv)(const GLfloat*) INIT_FUNC_POINTERS;
void (GLApi *glWindowPos3i)(GLint, GLint, GLint) INIT_FUNC_POINTERS;
void (GLApi *glWindowPos3iv)(const GLint*) INIT_FUNC_POINTERS;
void (GLApi *glWindowPos3s)(GLshort, GLshort, GLshort) INIT_FUNC_POINTERS;
void (GLApi *glWindowPos3sv)(const GLshort*) INIT_FUNC_POINTERS;

#endif //__GENERATE_GL_14__

/*
 *	static int CGLExtensionHandler::n_GetGL14FuncPointers()
 *		- gets OpenGL 1.4 core function pointers
 *		- returns number of missing functions
 */
int CGLExtensionHandler::n_GetGL14FuncPointers()
{
	__FuncGuard("CGLExtensionHandler::n_GetGL14FuncPointers");

	int n_failed_functions = 0;

#ifdef __GENERATE_GL_14__

	if(!(glBlendFuncSeparate = (void(GLApi*)(GLenum, GLenum, GLenum, GLenum))wglGetProcAddress("glBlendFuncSeparate"))) ++ n_failed_functions;
	if(!(glFogCoordf = (void(GLApi*)(GLfloat))wglGetProcAddress("glFogCoordf"))) ++ n_failed_functions;
	if(!(glFogCoordfv = (void(GLApi*)(const GLfloat*))wglGetProcAddress("glFogCoordfv"))) ++ n_failed_functions;
	if(!(glFogCoordd = (void(GLApi*)(GLdouble))wglGetProcAddress("glFogCoordd"))) ++ n_failed_functions;
	if(!(glFogCoorddv = (void(GLApi*)(const GLdouble*))wglGetProcAddress("glFogCoorddv"))) ++ n_failed_functions;
	if(!(glFogCoordPointer = (void(GLApi*)(GLenum, GLsizei, const GLvoid*))wglGetProcAddress("glFogCoordPointer"))) ++ n_failed_functions;
	if(!(glMultiDrawArrays = (void(GLApi*)(GLenum, GLint *, GLsizei *, GLsizei))wglGetProcAddress("glMultiDrawArrays"))) ++ n_failed_functions;
	if(!(glMultiDrawElements = (void(GLApi*)(GLenum, const GLsizei *, GLenum, const GLvoid* *, GLsizei))wglGetProcAddress("glMultiDrawElements"))) ++ n_failed_functions;
	if(!(glPointParameterf = (void(GLApi*)(GLenum, GLfloat))wglGetProcAddress("glPointParameterf"))) ++ n_failed_functions;
	if(!(glPointParameterfv = (void(GLApi*)(GLenum, const GLfloat*))wglGetProcAddress("glPointParameterfv"))) ++ n_failed_functions;
	if(!(glPointParameteri = (void(GLApi*)(GLenum, GLint))wglGetProcAddress("glPointParameteri"))) ++ n_failed_functions;
	if(!(glPointParameteriv = (void(GLApi*)(GLenum, const GLint*))wglGetProcAddress("glPointParameteriv"))) ++ n_failed_functions;
	if(!(glSecondaryColor3b = (void(GLApi*)(GLbyte, GLbyte, GLbyte))wglGetProcAddress("glSecondaryColor3b"))) ++ n_failed_functions;
	if(!(glSecondaryColor3bv = (void(GLApi*)(const GLbyte*))wglGetProcAddress("glSecondaryColor3bv"))) ++ n_failed_functions;
	if(!(glSecondaryColor3d = (void(GLApi*)(GLdouble, GLdouble, GLdouble))wglGetProcAddress("glSecondaryColor3d"))) ++ n_failed_functions;
	if(!(glSecondaryColor3dv = (void(GLApi*)(const GLdouble*))wglGetProcAddress("glSecondaryColor3dv"))) ++ n_failed_functions;
	if(!(glSecondaryColor3f = (void(GLApi*)(GLfloat, GLfloat, GLfloat))wglGetProcAddress("glSecondaryColor3f"))) ++ n_failed_functions;
	if(!(glSecondaryColor3fv = (void(GLApi*)(const GLfloat*))wglGetProcAddress("glSecondaryColor3fv"))) ++ n_failed_functions;
	if(!(glSecondaryColor3i = (void(GLApi*)(GLint, GLint, GLint))wglGetProcAddress("glSecondaryColor3i"))) ++ n_failed_functions;
	if(!(glSecondaryColor3iv = (void(GLApi*)(const GLint*))wglGetProcAddress("glSecondaryColor3iv"))) ++ n_failed_functions;
	if(!(glSecondaryColor3s = (void(GLApi*)(GLshort, GLshort, GLshort))wglGetProcAddress("glSecondaryColor3s"))) ++ n_failed_functions;
	if(!(glSecondaryColor3sv = (void(GLApi*)(const GLshort*))wglGetProcAddress("glSecondaryColor3sv"))) ++ n_failed_functions;
	if(!(glSecondaryColor3ub = (void(GLApi*)(GLubyte, GLubyte, GLubyte))wglGetProcAddress("glSecondaryColor3ub"))) ++ n_failed_functions;
	if(!(glSecondaryColor3ubv = (void(GLApi*)(const GLubyte*))wglGetProcAddress("glSecondaryColor3ubv"))) ++ n_failed_functions;
	if(!(glSecondaryColor3ui = (void(GLApi*)(GLuint, GLuint, GLuint))wglGetProcAddress("glSecondaryColor3ui"))) ++ n_failed_functions;
	if(!(glSecondaryColor3uiv = (void(GLApi*)(const GLuint*))wglGetProcAddress("glSecondaryColor3uiv"))) ++ n_failed_functions;
	if(!(glSecondaryColor3us = (void(GLApi*)(GLushort, GLushort, GLushort))wglGetProcAddress("glSecondaryColor3us"))) ++ n_failed_functions;
	if(!(glSecondaryColor3usv = (void(GLApi*)(const GLushort*))wglGetProcAddress("glSecondaryColor3usv"))) ++ n_failed_functions;
	if(!(glSecondaryColorPointer = (void(GLApi*)(GLint, GLenum, GLsizei, const GLvoid*))wglGetProcAddress("glSecondaryColorPointer"))) ++ n_failed_functions;
	if(!(glWindowPos2d = (void(GLApi*)(GLdouble, GLdouble))wglGetProcAddress("glWindowPos2d"))) ++ n_failed_functions;
	if(!(glWindowPos2dv = (void(GLApi*)(const GLdouble*))wglGetProcAddress("glWindowPos2dv"))) ++ n_failed_functions;
	if(!(glWindowPos2f = (void(GLApi*)(GLfloat, GLfloat))wglGetProcAddress("glWindowPos2f"))) ++ n_failed_functions;
	if(!(glWindowPos2fv = (void(GLApi*)(const GLfloat*))wglGetProcAddress("glWindowPos2fv"))) ++ n_failed_functions;
	if(!(glWindowPos2i = (void(GLApi*)(GLint, GLint))wglGetProcAddress("glWindowPos2i"))) ++ n_failed_functions;
	if(!(glWindowPos2iv = (void(GLApi*)(const GLint*))wglGetProcAddress("glWindowPos2iv"))) ++ n_failed_functions;
	if(!(glWindowPos2s = (void(GLApi*)(GLshort, GLshort))wglGetProcAddress("glWindowPos2s"))) ++ n_failed_functions;
	if(!(glWindowPos2sv = (void(GLApi*)(const GLshort*))wglGetProcAddress("glWindowPos2sv"))) ++ n_failed_functions;
	if(!(glWindowPos3d = (void(GLApi*)(GLdouble, GLdouble, GLdouble))wglGetProcAddress("glWindowPos3d"))) ++ n_failed_functions;
	if(!(glWindowPos3dv = (void(GLApi*)(const GLdouble*))wglGetProcAddress("glWindowPos3dv"))) ++ n_failed_functions;
	if(!(glWindowPos3f = (void(GLApi*)(GLfloat, GLfloat, GLfloat))wglGetProcAddress("glWindowPos3f"))) ++ n_failed_functions;
	if(!(glWindowPos3fv = (void(GLApi*)(const GLfloat*))wglGetProcAddress("glWindowPos3fv"))) ++ n_failed_functions;
	if(!(glWindowPos3i = (void(GLApi*)(GLint, GLint, GLint))wglGetProcAddress("glWindowPos3i"))) ++ n_failed_functions;
	if(!(glWindowPos3iv = (void(GLApi*)(const GLint*))wglGetProcAddress("glWindowPos3iv"))) ++ n_failed_functions;
	if(!(glWindowPos3s = (void(GLApi*)(GLshort, GLshort, GLshort))wglGetProcAddress("glWindowPos3s"))) ++ n_failed_functions;
	if(!(glWindowPos3sv = (void(GLApi*)(const GLshort*))wglGetProcAddress("glWindowPos3sv"))) ++ n_failed_functions;

#endif //__GENERATE_GL_14__

	return n_failed_functions;
}

/*
 *		--- opengl 1.5 core func pointers ---
 */

#ifdef __GENERATE_GL_15__

void (GLApi *glGenQueries)(GLsizei, GLuint*) INIT_FUNC_POINTERS;
void (GLApi *glDeleteQueries)(GLsizei, const GLuint*) INIT_FUNC_POINTERS;
GLboolean (GLApi *glIsQuery)(GLuint) INIT_FUNC_POINTERS;
void (GLApi *glBeginQuery)(GLenum, GLuint) INIT_FUNC_POINTERS;
void (GLApi *glEndQuery)(GLenum) INIT_FUNC_POINTERS;
void (GLApi *glGetQueryiv)(GLenum, GLenum, GLint*) INIT_FUNC_POINTERS;
void (GLApi *glGetQueryObjectiv)(GLuint, GLenum, GLint*) INIT_FUNC_POINTERS;
void (GLApi *glGetQueryObjectuiv)(GLuint, GLenum, GLuint*) INIT_FUNC_POINTERS;
void (GLApi *glBindBuffer)(GLenum, GLuint) INIT_FUNC_POINTERS;
void (GLApi *glDeleteBuffers)(GLsizei, const GLuint*) INIT_FUNC_POINTERS;
void (GLApi *glGenBuffers)(GLsizei, GLuint*) INIT_FUNC_POINTERS;
GLboolean (GLApi *glIsBuffer)(GLuint) INIT_FUNC_POINTERS;
void (GLApi *glBufferData)(GLenum, GLsizeiptr, const GLvoid *, GLenum) INIT_FUNC_POINTERS;
void (GLApi *glBufferSubData)(GLenum, GLintptr, GLsizeiptr, const GLvoid*) INIT_FUNC_POINTERS;
void (GLApi *glGetBufferSubData)(GLenum, GLintptr, GLsizeiptr, GLvoid*) INIT_FUNC_POINTERS;
GLvoid* (GLApi *glMapBuffer)(GLenum, GLenum) INIT_FUNC_POINTERS;
GLboolean (GLApi *glUnmapBuffer)(GLenum) INIT_FUNC_POINTERS;
void (GLApi *glGetBufferParameteriv)(GLenum, GLenum, GLint*) INIT_FUNC_POINTERS;
void (GLApi *glGetBufferPointerv)(GLenum, GLenum, GLvoid**) INIT_FUNC_POINTERS;

#endif //__GENERATE_GL_15__

/*
 *	static int CGLExtensionHandler::n_GetGL15FuncPointers()
 *		- gets OpenGL 1.5 core function pointers
 *		- returns number of missing functions
 */
int CGLExtensionHandler::n_GetGL15FuncPointers()
{
	__FuncGuard("CGLExtensionHandler::n_GetGL15FuncPointers");

	int n_failed_functions = 0;

#ifdef __GENERATE_GL_15__

	if(!(glGenQueries = (void(GLApi*)(GLsizei, GLuint*))wglGetProcAddress("glGenQueries"))) ++ n_failed_functions;
	if(!(glDeleteQueries = (void(GLApi*)(GLsizei, const GLuint*))wglGetProcAddress("glDeleteQueries"))) ++ n_failed_functions;
	if(!(glIsQuery = (GLboolean(GLApi*)(GLuint))wglGetProcAddress("glIsQuery"))) ++ n_failed_functions;
	if(!(glBeginQuery = (void(GLApi*)(GLenum, GLuint))wglGetProcAddress("glBeginQuery"))) ++ n_failed_functions;
	if(!(glEndQuery = (void(GLApi*)(GLenum))wglGetProcAddress("glEndQuery"))) ++ n_failed_functions;
	if(!(glGetQueryiv = (void(GLApi*)(GLenum, GLenum, GLint*))wglGetProcAddress("glGetQueryiv"))) ++ n_failed_functions;
	if(!(glGetQueryObjectiv = (void(GLApi*)(GLuint, GLenum, GLint*))wglGetProcAddress("glGetQueryObjectiv"))) ++ n_failed_functions;
	if(!(glGetQueryObjectuiv = (void(GLApi*)(GLuint, GLenum, GLuint*))wglGetProcAddress("glGetQueryObjectuiv"))) ++ n_failed_functions;
	if(!(glBindBuffer = (void(GLApi*)(GLenum, GLuint))wglGetProcAddress("glBindBuffer"))) ++ n_failed_functions;
	if(!(glDeleteBuffers = (void(GLApi*)(GLsizei, const GLuint*))wglGetProcAddress("glDeleteBuffers"))) ++ n_failed_functions;
	if(!(glGenBuffers = (void(GLApi*)(GLsizei, GLuint*))wglGetProcAddress("glGenBuffers"))) ++ n_failed_functions;
	if(!(glIsBuffer = (GLboolean(GLApi*)(GLuint))wglGetProcAddress("glIsBuffer"))) ++ n_failed_functions;
	if(!(glBufferData = (void(GLApi*)(GLenum, GLsizeiptr, const GLvoid *, GLenum))wglGetProcAddress("glBufferData"))) ++ n_failed_functions;
	if(!(glBufferSubData = (void(GLApi*)(GLenum, GLintptr, GLsizeiptr, const GLvoid*))wglGetProcAddress("glBufferSubData"))) ++ n_failed_functions;
	if(!(glGetBufferSubData = (void(GLApi*)(GLenum, GLintptr, GLsizeiptr, GLvoid*))wglGetProcAddress("glGetBufferSubData"))) ++ n_failed_functions;
	if(!(glMapBuffer = (GLvoid*(GLApi*)(GLenum, GLenum))wglGetProcAddress("glMapBuffer"))) ++ n_failed_functions;
	if(!(glUnmapBuffer = (GLboolean(GLApi*)(GLenum))wglGetProcAddress("glUnmapBuffer"))) ++ n_failed_functions;
	if(!(glGetBufferParameteriv = (void(GLApi*)(GLenum, GLenum, GLint*))wglGetProcAddress("glGetBufferParameteriv"))) ++ n_failed_functions;
	if(!(glGetBufferPointerv = (void(GLApi*)(GLenum, GLenum, GLvoid**))wglGetProcAddress("glGetBufferPointerv"))) ++ n_failed_functions;

#endif //__GENERATE_GL_15__

	return n_failed_functions;
}

/*
 *		--- compressed texture arb func pointers ---
 */

#ifdef __GENERATE_GL_ARB_texture_compression__

void (GLApi *glCompressedTexImage3DARB)(GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLsizei, const GLvoid*) INIT_FUNC_POINTERS;
void (GLApi *glCompressedTexImage2DARB)(GLenum, GLint, GLenum, GLsizei, GLsizei, GLint, GLsizei, const GLvoid*) INIT_FUNC_POINTERS;
void (GLApi *glCompressedTexImage1DARB)(GLenum, GLint, GLenum, GLsizei, GLint, GLsizei, const GLvoid*) INIT_FUNC_POINTERS;
void (GLApi *glCompressedTexSubImage3DARB)(GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLsizei, const GLvoid*) INIT_FUNC_POINTERS;
void (GLApi *glCompressedTexSubImage2DARB)(GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLsizei, const GLvoid*) INIT_FUNC_POINTERS;
void (GLApi *glCompressedTexSubImage1DARB)(GLenum, GLint, GLint, GLsizei, GLenum, GLsizei, const GLvoid*) INIT_FUNC_POINTERS;
void (GLApi *glGetCompressedTexImageARB)(GLenum, GLint, GLvoid*) INIT_FUNC_POINTERS;

#endif //__GENERATE_GL_ARB_texture_compression__

/*
 *	static int CGLExtensionHandler::n_GetCompressedTextureARBFuncPointers()
 *		- gets GL_ARB_texture_compression function pointers
 *		- returns number of missing functions
 */
int CGLExtensionHandler::n_GetCompressedTextureARBFuncPointers()
{
	__FuncGuard("CGLExtensionHandler::n_GetCompressedTextureARBFuncPointers");

	int n_failed_functions = 0;

#ifdef __GENERATE_GL_ARB_texture_compression__

	if(!(glCompressedTexImage3DARB = (void(GLApi*)(GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLsizei, const GLvoid*))wglGetProcAddress("glCompressedTexImage3DARB"))) ++ n_failed_functions;
	if(!(glCompressedTexImage2DARB = (void(GLApi*)(GLenum, GLint, GLenum, GLsizei, GLsizei, GLint, GLsizei, const GLvoid*))wglGetProcAddress("glCompressedTexImage2DARB"))) ++ n_failed_functions;
	if(!(glCompressedTexImage1DARB = (void(GLApi*)(GLenum, GLint, GLenum, GLsizei, GLint, GLsizei, const GLvoid*))wglGetProcAddress("glCompressedTexImage1DARB"))) ++ n_failed_functions;
	if(!(glCompressedTexSubImage3DARB = (void(GLApi*)(GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLsizei, const GLvoid*))wglGetProcAddress("glCompressedTexSubImage3DARB"))) ++ n_failed_functions;
	if(!(glCompressedTexSubImage2DARB = (void(GLApi*)(GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLsizei, const GLvoid*))wglGetProcAddress("glCompressedTexSubImage2DARB"))) ++ n_failed_functions;
	if(!(glCompressedTexSubImage1DARB = (void(GLApi*)(GLenum, GLint, GLint, GLsizei, GLenum, GLsizei, const GLvoid*))wglGetProcAddress("glCompressedTexSubImage1DARB"))) ++ n_failed_functions;
	if(!(glGetCompressedTexImageARB = (void(GLApi*)(GLenum, GLint, GLvoid*))wglGetProcAddress("glGetCompressedTexImageARB"))) ++ n_failed_functions;

#endif //__GENERATE_GL_ARB_texture_compression__

	return n_failed_functions;
}

/*
 *		--- vertex blend arb func pointers ---
 */

#ifdef __GENERATE_GL_ARB_vertex_blend__

void (GLApi *glWeightbvARB)(GLint, const GLbyte*) INIT_FUNC_POINTERS;
void (GLApi *glWeightsvARB)(GLint, const GLshort*) INIT_FUNC_POINTERS;
void (GLApi *glWeightivARB)(GLint, const GLint*) INIT_FUNC_POINTERS;
void (GLApi *glWeightfvARB)(GLint, const GLfloat*) INIT_FUNC_POINTERS;
void (GLApi *glWeightdvARB)(GLint, const GLdouble*) INIT_FUNC_POINTERS;
void (GLApi *glWeightubvARB)(GLint, const GLubyte*) INIT_FUNC_POINTERS;
void (GLApi *glWeightusvARB)(GLint, const GLushort*) INIT_FUNC_POINTERS;
void (GLApi *glWeightuivARB)(GLint, const GLuint*) INIT_FUNC_POINTERS;
void (GLApi *glWeightPointerARB)(GLint, GLenum, GLsizei, const GLvoid*) INIT_FUNC_POINTERS;
void (GLApi *glVertexBlendARB)(GLint) INIT_FUNC_POINTERS;

#endif //__GENERATE_GL_ARB_vertex_blend__

/*
 *	static int CGLExtensionHandler::n_GetVertexBlendARBFuncPointers()
 *		- gets GL_ARB_vertex_blend function pointers
 *		- returns number of missing functions
 */
int CGLExtensionHandler::n_GetVertexBlendARBFuncPointers()
{
	__FuncGuard("CGLExtensionHandler::n_GetVertexBlendARBFuncPointers");

	int n_failed_functions = 0;

#ifdef __GENERATE_GL_ARB_vertex_blend__

	if(!(glWeightbvARB = (void(GLApi*)(GLint, const GLbyte*))wglGetProcAddress("glWeightbvARB"))) ++ n_failed_functions;
	if(!(glWeightsvARB = (void(GLApi*)(GLint, const GLshort*))wglGetProcAddress("glWeightsvARB"))) ++ n_failed_functions;
	if(!(glWeightivARB = (void(GLApi*)(GLint, const GLint*))wglGetProcAddress("glWeightivARB"))) ++ n_failed_functions;
	if(!(glWeightfvARB = (void(GLApi*)(GLint, const GLfloat*))wglGetProcAddress("glWeightfvARB"))) ++ n_failed_functions;
	if(!(glWeightdvARB = (void(GLApi*)(GLint, const GLdouble*))wglGetProcAddress("glWeightdvARB"))) ++ n_failed_functions;
	if(!(glWeightubvARB = (void(GLApi*)(GLint, const GLubyte*))wglGetProcAddress("glWeightubvARB"))) ++ n_failed_functions;
	if(!(glWeightusvARB = (void(GLApi*)(GLint, const GLushort*))wglGetProcAddress("glWeightusvARB"))) ++ n_failed_functions;
	if(!(glWeightuivARB = (void(GLApi*)(GLint, const GLuint*))wglGetProcAddress("glWeightuivARB"))) ++ n_failed_functions;
	if(!(glWeightPointerARB = (void(GLApi*)(GLint, GLenum, GLsizei, const GLvoid*))wglGetProcAddress("glWeightPointerARB"))) ++ n_failed_functions;
	if(!(glVertexBlendARB = (void(GLApi*)(GLint))wglGetProcAddress("glVertexBlendARB"))) ++ n_failed_functions;

#endif //__GENERATE_GL_ARB_vertex_blend__

	return n_failed_functions;
}

/*
 *		--- matrix palette arb func pointers ---
 */

#ifdef __GENERATE_GL_ARB_matrix_palette__

void (GLApi *glCurrentPaletteMatrixARB)(GLint) INIT_FUNC_POINTERS;
void (GLApi *glMatrixIndexubvARB)(GLint, const GLubyte*) INIT_FUNC_POINTERS;
void (GLApi *glMatrixIndexusvARB)(GLint, const GLushort*) INIT_FUNC_POINTERS;
void (GLApi *glMatrixIndexuivARB)(GLint, const GLuint*) INIT_FUNC_POINTERS;
void (GLApi *glMatrixIndexPointerARB)(GLint, GLenum, GLsizei, const GLvoid*) INIT_FUNC_POINTERS;

#endif //__GENERATE_GL_ARB_matrix_palette__

/*
 *	static int CGLExtensionHandler::n_GetMatrixPaletteARBFuncPointers()
 *		- gets GL_ARB_matrix_palette function pointers
 *		- returns number of missing functions
 */
int CGLExtensionHandler::n_GetMatrixPaletteARBFuncPointers()
{
	__FuncGuard("CGLExtensionHandler::n_GetMatrixPaletteARBFuncPointers");

	int n_failed_functions = 0;

#ifdef __GENERATE_GL_ARB_matrix_palette__

	if(!(glCurrentPaletteMatrixARB = (void(GLApi*)(GLint))wglGetProcAddress("glCurrentPaletteMatrixARB"))) ++ n_failed_functions;
	if(!(glMatrixIndexubvARB = (void(GLApi*)(GLint, const GLubyte*))wglGetProcAddress("glMatrixIndexubvARB"))) ++ n_failed_functions;
	if(!(glMatrixIndexusvARB = (void(GLApi*)(GLint, const GLushort*))wglGetProcAddress("glMatrixIndexusvARB"))) ++ n_failed_functions;
	if(!(glMatrixIndexuivARB = (void(GLApi*)(GLint, const GLuint*))wglGetProcAddress("glMatrixIndexuivARB"))) ++ n_failed_functions;
	if(!(glMatrixIndexPointerARB = (void(GLApi*)(GLint, GLenum, GLsizei, const GLvoid*))wglGetProcAddress("glMatrixIndexPointerARB"))) ++ n_failed_functions;

#endif //__GENERATE_GL_ARB_matrix_palette__

	return n_failed_functions;
}

/*
 *		--- ext framebuffer object func pointers ---
 */

#ifdef __GENERATE_GL_EXT_framebuffer_object__

GLboolean (GLApi *glIsRenderbufferEXT)(GLuint renderbuffer) INIT_FUNC_POINTERS;
void (GLApi *glBindRenderbufferEXT)(GLenum target, GLuint renderbuffer) INIT_FUNC_POINTERS;
void (GLApi *glDeleteRenderbuffersEXT)(GLsizei n, const GLuint *renderbuffers) INIT_FUNC_POINTERS;
void (GLApi *glGenRenderbuffersEXT)(GLsizei n, GLuint *renderbuffers) INIT_FUNC_POINTERS;
void (GLApi *glRenderbufferStorageEXT)(GLenum target, GLenum internalformat, GLsizei width, GLsizei height) INIT_FUNC_POINTERS;
void (GLApi *glGetRenderbufferParameterivEXT)(GLenum target, GLenum pname, GLint* params) INIT_FUNC_POINTERS;
GLboolean (GLApi *glIsFramebufferEXT)(GLuint framebuffer) INIT_FUNC_POINTERS;
void (GLApi *glBindFramebufferEXT)(GLenum target, GLuint framebuffer) INIT_FUNC_POINTERS;
void (GLApi *glDeleteFramebuffersEXT)(GLsizei n, const GLuint *framebuffers) INIT_FUNC_POINTERS;
void (GLApi *glGenFramebuffersEXT)(GLsizei n, GLuint *framebuffers) INIT_FUNC_POINTERS;
GLenum (GLApi *glCheckFramebufferStatusEXT)(GLenum target) INIT_FUNC_POINTERS;
void (GLApi *glFramebufferTexture1DEXT)(GLenum target, GLenum attachment, GLenum textarget, GLuint texture, GLint level) INIT_FUNC_POINTERS;
void (GLApi *glFramebufferTexture2DEXT)(GLenum target, GLenum attachment, GLenum textarget, GLuint texture, GLint level) INIT_FUNC_POINTERS;
void (GLApi *glFramebufferTexture3DEXT)(GLenum target, GLenum attachment, GLenum textarget, GLuint texture, GLint level, GLint zoffset) INIT_FUNC_POINTERS;
void (GLApi *glFramebufferRenderbufferEXT)(GLenum target, GLenum attachment, GLenum renderbuffertarget, GLuint renderbuffer) INIT_FUNC_POINTERS;
void (GLApi *glGetFramebufferAttachmentParameterivEXT)(GLenum target, GLenum attachment, GLenum pname, GLint *params) INIT_FUNC_POINTERS;
void (GLApi *glGenerateMipmapEXT)(GLenum target) INIT_FUNC_POINTERS;

#endif //__GENERATE_GL_EXT_framebuffer_object__

/*
 *	static int CGLExtensionHandler::n_GetFramebufferObjectEXTFuncPointers()
 *		- gets GL_EXT_framebuffer_object function pointers
 *		- returns number of missing functions
 */
int CGLExtensionHandler::n_GetFramebufferObjectEXTFuncPointers()
{
	__FuncGuard("CGLExtensionHandler::n_GetFramebufferObjectEXTFuncPointers");

	int n_failed_functions = 0;

#ifdef __GENERATE_GL_EXT_framebuffer_object__

	if(!(glIsRenderbufferEXT = (GLboolean(GLApi*)(GLuint renderbuffer))wglGetProcAddress("glIsRenderbufferEXT"))) ++ n_failed_functions;
	if(!(glBindRenderbufferEXT = (void(GLApi*)(GLenum target, GLuint renderbuffer))wglGetProcAddress("glBindRenderbufferEXT"))) ++ n_failed_functions;
	if(!(glDeleteRenderbuffersEXT = (void(GLApi*)(GLsizei n, const GLuint *renderbuffers))wglGetProcAddress("glDeleteRenderbuffersEXT"))) ++ n_failed_functions;
	if(!(glGenRenderbuffersEXT = (void(GLApi*)(GLsizei n, GLuint *renderbuffers))wglGetProcAddress("glGenRenderbuffersEXT"))) ++ n_failed_functions;
	if(!(glRenderbufferStorageEXT = (void(GLApi*)(GLenum target, GLenum internalformat, GLsizei width, GLsizei height))wglGetProcAddress("glRenderbufferStorageEXT"))) ++ n_failed_functions;
	if(!(glGetRenderbufferParameterivEXT = (void(GLApi*)(GLenum target, GLenum pname, GLint* params))wglGetProcAddress("glGetRenderbufferParameterivEXT"))) ++ n_failed_functions;
	if(!(glIsFramebufferEXT = (GLboolean(GLApi*)(GLuint framebuffer))wglGetProcAddress("glIsFramebufferEXT"))) ++ n_failed_functions;
	if(!(glBindFramebufferEXT = (void(GLApi*)(GLenum target, GLuint framebuffer))wglGetProcAddress("glBindFramebufferEXT"))) ++ n_failed_functions;
	if(!(glDeleteFramebuffersEXT = (void(GLApi*)(GLsizei n, const GLuint *framebuffers))wglGetProcAddress("glDeleteFramebuffersEXT"))) ++ n_failed_functions;
	if(!(glGenFramebuffersEXT = (void(GLApi*)(GLsizei n, GLuint *framebuffers))wglGetProcAddress("glGenFramebuffersEXT"))) ++ n_failed_functions;
	if(!(glCheckFramebufferStatusEXT = (GLenum(GLApi*)(GLenum target))wglGetProcAddress("glCheckFramebufferStatusEXT"))) ++ n_failed_functions;
	if(!(glFramebufferTexture1DEXT = (void(GLApi*)(GLenum target, GLenum attachment, GLenum textarget, GLuint texture, GLint level))wglGetProcAddress("glFramebufferTexture1DEXT"))) ++ n_failed_functions;
	if(!(glFramebufferTexture2DEXT = (void(GLApi*)(GLenum target, GLenum attachment, GLenum textarget, GLuint texture, GLint level))wglGetProcAddress("glFramebufferTexture2DEXT"))) ++ n_failed_functions;
	if(!(glFramebufferTexture3DEXT = (void(GLApi*)(GLenum target, GLenum attachment, GLenum textarget, GLuint texture, GLint level, GLint zoffset))wglGetProcAddress("glFramebufferTexture3DEXT"))) ++ n_failed_functions;
	if(!(glFramebufferRenderbufferEXT = (void(GLApi*)(GLenum target, GLenum attachment, GLenum renderbuffertarget, GLuint renderbuffer))wglGetProcAddress("glFramebufferRenderbufferEXT"))) ++ n_failed_functions;
	if(!(glGetFramebufferAttachmentParameterivEXT = (void(GLApi*)(GLenum target, GLenum attachment, GLenum pname, GLint *params))wglGetProcAddress("glGetFramebufferAttachmentParameterivEXT"))) ++ n_failed_functions;
	if(!(glGenerateMipmapEXT = (void(GLApi*)(GLenum target))wglGetProcAddress("glGenerateMipmapEXT"))) ++ n_failed_functions;

#endif //__GENERATE_GL_EXT_framebuffer_object__

	return n_failed_functions;
}

/*
 *		--- ext compiled array func pointers ---
 */

#ifdef __GENERATE_GL_EXT_compiled_vertex_array__

void (GLApi *glLockArraysEXT)(GLint, GLsizei) INIT_FUNC_POINTERS;
void (GLApi *glUnlockArraysEXT)(void) INIT_FUNC_POINTERS;

#endif //__GENERATE_GL_EXT_compiled_vertex_array__

/*
 *	static int CGLExtensionHandler::n_GetCompiledArrayEXTFuncPointers()
 *		- gets GL_EXT_compiled_vertex_array function pointers
 *		- returns number of missing functions
 */
int CGLExtensionHandler::n_GetCompiledArrayEXTFuncPointers()
{
	__FuncGuard("CGLExtensionHandler::n_GetCompiledArrayEXTFuncPointers");

	int n_failed_functions = 0;

#ifdef __GENERATE_GL_EXT_compiled_vertex_array__

	if(!(glLockArraysEXT = (void(GLApi *)(GLint, GLsizei))wglGetProcAddress("glLockArraysEXT"))) ++ n_failed_functions;
	if(!(glUnlockArraysEXT = (void(GLApi *)(void))wglGetProcAddress("glUnlockArraysEXT"))) ++ n_failed_functions;

#endif //__GENERATE_GL_EXT_compiled_vertex_array__

	return n_failed_functions;
}

/*
 *		--- ext texture object func pointers --- (20. EXT from 1995/10/03; outdated)
 */

/*GLboolean (GLApi *glAreTexturesResidentEXT)(GLsizei, const GLuint *, GLboolean*) INIT_FUNC_POINTERS;
void (GLApi *glBindTextureEXT)(GLenum, GLuint) INIT_FUNC_POINTERS;
void (GLApi *glDeleteTexturesEXT)(GLsizei, const GLuint*) INIT_FUNC_POINTERS;
void (GLApi *glGenTexturesEXT)(GLsizei, GLuint*) INIT_FUNC_POINTERS;
GLboolean (GLApi *glIsTextureEXT)(GLuint) INIT_FUNC_POINTERS;
void (GLApi *glPrioritizeTexturesEXT)(GLsizei, const GLuint *, const GLclampf*) INIT_FUNC_POINTERS;

/ *
 *	static int CGLExtensionHandler::n_GetTextureObjectEXTFuncPointers()
 *		- gets GL_EXT_texture_object function pointers
 *		- outdated 20th EXT from 1995/10/03; deprecated
 *		- returns number of missing functions
 * /
int CGLExtensionHandler::n_GetTextureObjectEXTFuncPointers()
{
	__FuncGuard("CGLExtensionHandler::n_GetTextureObjectEXTFuncPointers");

	int n_failed_functions = 0;

	if(!(glAreTexturesResidentEXT = (GLboolean(GLApi*)(GLsizei, const GLuint *, GLboolean*))wglGetProcAddress("glAreTexturesResidentEXT"))) ++ n_failed_functions;
	if(!(glBindTextureEXT = (void(GLApi*)(GLenum, GLuint))wglGetProcAddress("glBindTextureEXT"))) ++ n_failed_functions;
	if(!(glDeleteTexturesEXT = (void(GLApi*)(GLsizei, const GLuint*))wglGetProcAddress("glDeleteTexturesEXT"))) ++ n_failed_functions;
	if(!(glGenTexturesEXT = (void(GLApi*)(GLsizei, GLuint*))wglGetProcAddress("glGenTexturesEXT"))) ++ n_failed_functions;
	if(!(glIsTextureEXT = (GLboolean(GLApi*)(GLuint))wglGetProcAddress("glIsTextureEXT"))) ++ n_failed_functions;
	if(!(glPrioritizeTexturesEXT = (void(GLApi*)(GLsizei, const GLuint *, const GLclampf*))wglGetProcAddress("glPrioritizeTexturesEXT"))) ++ n_failed_functions;

	return n_failed_functions;
}*/

/*
 *		--- ext polygon offset func pointers ---
 */

/*void (GLApi *glPolygonOffsetEXT)(GLfloat, GLfloat) INIT_FUNC_POINTERS;

/ *
 *	static int CGLExtensionHandler::n_GetPolygonOffsetEXTFuncPointers()
 *		- gets GL_EXT_polygon_offset function pointers
 *		- returns number of missing functions
 * /
int CGLExtensionHandler::n_GetPolygonOffsetEXTFuncPointers()
{
	__FuncGuard("CGLExtensionHandler::n_GetPolygonOffsetEXTFuncPointers");

	int n_failed_functions = 0;

	if(!(glPolygonOffsetEXT = (void(GLApi*)(GLfloat, GLfloat))wglGetProcAddress("glPolygonOffsetEXT"))) ++ n_failed_functions;

	return n_failed_functions;
}*/

/*
 *		--- arb color buffer float func pointers ---
 */

#ifdef __GENERATE_GL_ARB_color_buffer_float__

void (GLApi *glClampColorARB)(GLenum, GLenum) INIT_FUNC_POINTERS;

#endif //__GENERATE_GL_ARB_color_buffer_float__

/*
 *	static int CGLExtensionHandler::n_GetColorBufferFloatARBFuncPointers()
 *		- gets GL_ARB_color_buffer_float function pointers
 *		- returns number of missing functions
 */
int CGLExtensionHandler::n_GetColorBufferFloatARBFuncPointers()
{
	__FuncGuard("CGLExtensionHandler::n_GetColorBufferFloatARBFuncPointers");

	int n_failed_functions = 0;

#ifdef __GENERATE_GL_ARB_color_buffer_float__

	if(!(glClampColorARB = (void(GLApi*)(GLenum, GLenum))wglGetProcAddress("glClampColorARB"))) ++ n_failed_functions;

#endif //__GENERATE_GL_ARB_color_buffer_float__

	return n_failed_functions;
}

/*
 *		--- ext vertex array func pointers ---
 */

/*void (GLApi *glArrayElementEXT)(GLint) INIT_FUNC_POINTERS;
void (GLApi *glColorPointerEXT)(GLint, GLenum, GLsizei, GLsizei, const GLvoid*) INIT_FUNC_POINTERS;
void (GLApi *glDrawArraysEXT)(GLenum, GLint, GLsizei) INIT_FUNC_POINTERS;
void (GLApi *glEdgeFlagPointerEXT)(GLsizei, GLsizei, const GLboolean*) INIT_FUNC_POINTERS;
void (GLApi *glGetPointervEXT)(GLenum, GLvoid**) INIT_FUNC_POINTERS;
void (GLApi *glIndexPointerEXT)(GLenum, GLsizei, GLsizei, const GLvoid*) INIT_FUNC_POINTERS;
void (GLApi *glNormalPointerEXT)(GLenum, GLsizei, GLsizei, const GLvoid*) INIT_FUNC_POINTERS;
void (GLApi *glTexCoordPointerEXT)(GLint, GLenum, GLsizei, GLsizei, const GLvoid*) INIT_FUNC_POINTERS;
void (GLApi *glVertexPointerEXT)(GLint, GLenum, GLsizei, GLsizei, const GLvoid*) INIT_FUNC_POINTERS;

/ *
 *	static int CGLExtensionHandler::n_GetVertexArrayEXTFuncPointers()
 *		- gets GL_EXT_vertex_array function pointers
 *		- returns number of missing functions
 * /
int CGLExtensionHandler::n_GetVertexArrayEXTFuncPointers()
{
	__FuncGuard("CGLExtensionHandler::n_GetVertexArrayEXTFuncPointers");

	int n_failed_functions = 0;

	if(!(glArrayElementEXT = (void(GLApi*)(GLint))wglGetProcAddress("glArrayElementEXT"))) ++ n_failed_functions;
	if(!(glColorPointerEXT = (void(GLApi*)(GLint, GLenum, GLsizei, GLsizei, const GLvoid*))wglGetProcAddress("glColorPointerEXT"))) ++ n_failed_functions;
	if(!(glDrawArraysEXT = (void(GLApi*)(GLenum, GLint, GLsizei))wglGetProcAddress("glDrawArraysEXT"))) ++ n_failed_functions;
	if(!(glEdgeFlagPointerEXT = (void(GLApi*)(GLsizei, GLsizei, const GLboolean*))wglGetProcAddress("glEdgeFlagPointerEXT"))) ++ n_failed_functions;
	if(!(glGetPointervEXT = (void(GLApi*)(GLenum, GLvoid**))wglGetProcAddress("glGetPointervEXT"))) ++ n_failed_functions;
	if(!(glIndexPointerEXT = (void(GLApi*)(GLenum, GLsizei, GLsizei, const GLvoid*))wglGetProcAddress("glIndexPointerEXT"))) ++ n_failed_functions;
	if(!(glNormalPointerEXT = (void(GLApi*)(GLenum, GLsizei, GLsizei, const GLvoid*))wglGetProcAddress("glNormalPointerEXT"))) ++ n_failed_functions;
	if(!(glTexCoordPointerEXT = (void(GLApi*)(GLint, GLenum, GLsizei, GLsizei, const GLvoid*))wglGetProcAddress("glTexCoordPointerEXT"))) ++ n_failed_functions;
	if(!(glVertexPointerEXT = (void(GLApi*)(GLint, GLenum, GLsizei, GLsizei, const GLvoid*))wglGetProcAddress("glVertexPointerEXT"))) ++ n_failed_functions;

	return n_failed_functions;
}*/

/*
 *		--- GL20 func pointers ---
 */

#ifdef __GENERATE_GL_20__

// shader objects api
void (GLApi *glDeleteShader)(GLuint shader) INIT_FUNC_POINTERS;
void (GLApi *glDetachShader)(GLuint program, GLuint shader) INIT_FUNC_POINTERS;
GLuint (GLApi *glCreateShader)(GLenum type) INIT_FUNC_POINTERS;
void (GLApi *glShaderSource)(GLuint shader, GLsizei count, const GLchar* *string, const GLint *length) INIT_FUNC_POINTERS;
void (GLApi *glCompileShader)(GLuint shader) INIT_FUNC_POINTERS;

// program objects
GLuint (GLApi *glCreateProgram)(void) INIT_FUNC_POINTERS;
void (GLApi *glAttachShader)(GLuint program, GLuint shader) INIT_FUNC_POINTERS;
void (GLApi *glLinkProgram)(GLuint program) INIT_FUNC_POINTERS;
void (GLApi *glUseProgram)(GLuint program) INIT_FUNC_POINTERS;
void (GLApi *glDeleteProgram)(GLuint program) INIT_FUNC_POINTERS;
void (GLApi *glValidateProgram)(GLuint program) INIT_FUNC_POINTERS;

// uniforms
void (GLApi *glUniform1f)(GLint location, GLfloat v0) INIT_FUNC_POINTERS;
void (GLApi *glUniform2f)(GLint location, GLfloat v0, GLfloat v1) INIT_FUNC_POINTERS;
void (GLApi *glUniform3f)(GLint location, GLfloat v0, GLfloat v1, GLfloat v2) INIT_FUNC_POINTERS;
void (GLApi *glUniform4f)(GLint location, GLfloat v0, GLfloat v1, GLfloat v2, GLfloat v3) INIT_FUNC_POINTERS;
void (GLApi *glUniform1i)(GLint location, GLint v0) INIT_FUNC_POINTERS;
void (GLApi *glUniform2i)(GLint location, GLint v0, GLint v1) INIT_FUNC_POINTERS;
void (GLApi *glUniform3i)(GLint location, GLint v0, GLint v1, GLint v2) INIT_FUNC_POINTERS;
void (GLApi *glUniform4i)(GLint location, GLint v0, GLint v1, GLint v2, GLint v3) INIT_FUNC_POINTERS;
void (GLApi *glUniform1fv)(GLint location, GLsizei count, const GLfloat *value) INIT_FUNC_POINTERS;
void (GLApi *glUniform2fv)(GLint location, GLsizei count, const GLfloat *value) INIT_FUNC_POINTERS;
void (GLApi *glUniform3fv)(GLint location, GLsizei count, const GLfloat *value) INIT_FUNC_POINTERS;
void (GLApi *glUniform4fv)(GLint location, GLsizei count, const GLfloat *value) INIT_FUNC_POINTERS;
void (GLApi *glUniform1iv)(GLint location, GLsizei count, const GLint *value) INIT_FUNC_POINTERS;
void (GLApi *glUniform2iv)(GLint location, GLsizei count, const GLint *value) INIT_FUNC_POINTERS;
void (GLApi *glUniform3iv)(GLint location, GLsizei count, const GLint *value) INIT_FUNC_POINTERS;
void (GLApi *glUniform4iv)(GLint location, GLsizei count, const GLint *value) INIT_FUNC_POINTERS;
void (GLApi *glUniformMatrix2fv)(GLint location, GLsizei count, GLboolean transpose, const GLfloat *value) INIT_FUNC_POINTERS;
void (GLApi *glUniformMatrix3fv)(GLint location, GLsizei count, GLboolean transpose, const GLfloat *value) INIT_FUNC_POINTERS;
void (GLApi *glUniformMatrix4fv)(GLint location, GLsizei count, GLboolean transpose, const GLfloat *value) INIT_FUNC_POINTERS;

// attrib locations
void (GLApi *glBindAttribLocation)(GLuint program, GLuint index, const GLchar *name) INIT_FUNC_POINTERS;
GLint (GLApi *glGetAttribLocation)(GLuint program, const GLchar *name) INIT_FUNC_POINTERS;

// vertex attributes
void (GLApi *glVertexAttrib1d)(GLuint index, GLdouble x) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib1dv)(GLuint index, const GLdouble *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib1f)(GLuint index, GLfloat x) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib1fv)(GLuint index, const GLfloat *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib1s)(GLuint index, GLshort x) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib1sv)(GLuint index, const GLshort *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib2d)(GLuint index, GLdouble x, GLdouble y) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib2dv)(GLuint index, const GLdouble *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib2f)(GLuint index, GLfloat x, GLfloat y) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib2fv)(GLuint index, const GLfloat *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib2s)(GLuint index, GLshort x, GLshort y) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib2sv)(GLuint index, const GLshort *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib3d)(GLuint index, GLdouble x, GLdouble y, GLdouble z) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib3dv)(GLuint index, const GLdouble *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib3f)(GLuint index, GLfloat x, GLfloat y, GLfloat z) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib3fv)(GLuint index, const GLfloat *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib3s)(GLuint index, GLshort x, GLshort y, GLshort z) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib3sv)(GLuint index, const GLshort *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4Nbv)(GLuint index, const GLbyte *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4Niv)(GLuint index, const GLint *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4Nsv)(GLuint index, const GLshort *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4Nub)(GLuint index, GLubyte x, GLubyte y, GLubyte z, GLubyte w) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4Nubv)(GLuint index, const GLubyte *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4Nuiv)(GLuint index, const GLuint *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4Nusv)(GLuint index, const GLushort *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4bv)(GLuint index, const GLbyte *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4d)(GLuint index, GLdouble x, GLdouble y, GLdouble z, GLdouble w) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4dv)(GLuint index, const GLdouble *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4f)(GLuint index, GLfloat x, GLfloat y, GLfloat z, GLfloat w) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4fv)(GLuint index, const GLfloat *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4iv)(GLuint index, const GLint *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4s)(GLuint index, GLshort x, GLshort y, GLshort z, GLshort w) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4sv)(GLuint index, const GLshort *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4ubv)(GLuint index, const GLubyte *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4uiv)(GLuint index, const GLuint *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4usv)(GLuint index, const GLushort *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttribPointer)(GLuint index, GLint size, GLenum type, GLboolean normalized, GLsizei stride, const GLvoid *pointer) INIT_FUNC_POINTERS;
void (GLApi *glEnableVertexAttribArray)(GLuint index) INIT_FUNC_POINTERS;
void (GLApi *glDisableVertexAttribArray)(GLuint index) INIT_FUNC_POINTERS;
void (GLApi *glGetVertexAttribdv)(GLuint index, GLenum pname, GLdouble *params) INIT_FUNC_POINTERS;
void (GLApi *glGetVertexAttribfv)(GLuint index, GLenum pname, GLfloat *params) INIT_FUNC_POINTERS;
void (GLApi *glGetVertexAttribiv)(GLuint index, GLenum pname, GLint *params) INIT_FUNC_POINTERS;
void (GLApi *glGetVertexAttribPointerv)(GLuint index, GLenum pname, GLvoid* *pointer) INIT_FUNC_POINTERS;

// queries
GLboolean (GLApi *glIsShader)(GLuint shader) INIT_FUNC_POINTERS;
GLboolean (GLApi *glIsProgram)(GLuint program) INIT_FUNC_POINTERS;
void (GLApi *glGetShaderiv)(GLuint program, GLenum pname, GLint *params) INIT_FUNC_POINTERS;
void (GLApi *glGetProgramiv)(GLuint program, GLenum pname, GLint *params) INIT_FUNC_POINTERS;
void (GLApi *glGetAttachedShaders)(GLuint program, GLsizei maxCount, GLsizei *count, GLuint *shaders) INIT_FUNC_POINTERS;
void (GLApi *glGetShaderInfoLog)(GLuint shader, GLsizei bufSize, GLsizei *length, GLchar *infoLog) INIT_FUNC_POINTERS;
void (GLApi *glGetProgramInfoLog)(GLuint program, GLsizei bufSize, GLsizei *length, GLchar *infoLog) INIT_FUNC_POINTERS;
GLint (GLApi *glGetUniformLocation)(GLuint program, const GLchar *name) INIT_FUNC_POINTERS;
void (GLApi *glGetActiveUniform)(GLuint program, GLuint index, GLsizei bufSize, GLsizei *length, GLsizei *size, GLenum *type, GLchar *name) INIT_FUNC_POINTERS;
void (GLApi *glGetUniformfv)(GLuint program, GLint location, GLfloat *params) INIT_FUNC_POINTERS;
void (GLApi *glGetUniformiv)(GLuint program, GLint location, GLint *params) INIT_FUNC_POINTERS;
void (GLApi *glGetShaderSource)(GLuint shader, GLsizei bufSize, GLsizei *length, GLchar *source) INIT_FUNC_POINTERS;
void (GLApi *glGetActiveAttrib)(GLuint program, GLuint index, GLsizei bufSize, GLsizei *length, GLsizei *size, GLenum *type, GLchar *name) INIT_FUNC_POINTERS;

// multiple draw buffers
void (GLApi *glDrawBuffers)(GLsizei n, const GLenum *bufs) INIT_FUNC_POINTERS;

// two-sided stencil
void (GLApi *glStencilFuncSeparate)(GLenum face, GLenum func, GLint ref, GLuint mask) INIT_FUNC_POINTERS;
void (GLApi *glStencilOpSeparate)(GLenum face, GLenum fail, GLenum zfail, GLenum zpass) INIT_FUNC_POINTERS;
void (GLApi *glStencilMaskSeparate)(GLenum face, GLuint mask) INIT_FUNC_POINTERS;

// separate alpha blending
void (GLApi *glBlendEquationSeparate)(GLenum modeRGB, GLenum modeAlpha) INIT_FUNC_POINTERS;

#endif //__GENERATE_GL_20__

/*
 *	static int CGLExtensionHandler::n_GetGL20FuncPointers()
 *		- gets OpenGL 2.0 core function pointers
 *		- returns number of missing functions
 */
int CGLExtensionHandler::n_GetGL20FuncPointers()
{
	__FuncGuard("CGLExtensionHandler::n_GetGL20FuncPointers");

	int n_failed_functions = 0;

#ifdef __GENERATE_GL_20__

	if(!(glDeleteShader = (void(GLApi*)(GLuint))wglGetProcAddress("glDeleteShader"))) ++ n_failed_functions;
	if(!(glDetachShader = (void(GLApi*)(GLuint, GLuint))wglGetProcAddress("glDetachShader"))) ++ n_failed_functions;
	if(!(glCreateShader = (GLuint(GLApi*)(GLenum))wglGetProcAddress("glCreateShader"))) ++ n_failed_functions;
	if(!(glShaderSource = (void(GLApi*)(GLuint, GLsizei, const GLchar**, const GLint*))wglGetProcAddress("glShaderSource"))) ++ n_failed_functions;
	if(!(glCompileShader = (void(GLApi*)(GLuint))wglGetProcAddress("glCompileShader"))) ++ n_failed_functions;
	if(!(glCreateProgram = (GLuint(GLApi*)(void))wglGetProcAddress("glCreateProgram"))) ++ n_failed_functions;
	if(!(glAttachShader = (void(GLApi*)(GLuint, GLuint shader))wglGetProcAddress("glAttachShader"))) ++ n_failed_functions;
	if(!(glLinkProgram = (void(GLApi*)(GLuint))wglGetProcAddress("glLinkProgram"))) ++ n_failed_functions;
	if(!(glUseProgram = (void(GLApi*)(GLuint))wglGetProcAddress("glUseProgram"))) ++ n_failed_functions;
	if(!(glDeleteProgram = (void(GLApi*)(GLuint))wglGetProcAddress("glDeleteProgram"))) ++ n_failed_functions;
	if(!(glValidateProgram = (void(GLApi*)(GLuint))wglGetProcAddress("glValidateProgram"))) ++ n_failed_functions;
	if(!(glUniform1f = (void(GLApi*)(GLint, GLfloat))wglGetProcAddress("glUniform1f"))) ++ n_failed_functions;
	if(!(glUniform2f = (void(GLApi*)(GLint, GLfloat, GLfloat))wglGetProcAddress("glUniform2f"))) ++ n_failed_functions;
	if(!(glUniform3f = (void(GLApi*)(GLint, GLfloat, GLfloat, GLfloat))wglGetProcAddress("glUniform3f"))) ++ n_failed_functions;
	if(!(glUniform4f = (void(GLApi*)(GLint, GLfloat, GLfloat, GLfloat, GLfloat))wglGetProcAddress("glUniform4f"))) ++ n_failed_functions;
	if(!(glUniform1i = (void(GLApi*)(GLint, GLint))wglGetProcAddress("glUniform1i"))) ++ n_failed_functions;
	if(!(glUniform2i = (void(GLApi*)(GLint, GLint, GLint))wglGetProcAddress("glUniform2i"))) ++ n_failed_functions;
	if(!(glUniform3i = (void(GLApi*)(GLint, GLint, GLint, GLint))wglGetProcAddress("glUniform3i"))) ++ n_failed_functions;
	if(!(glUniform4i = (void(GLApi*)(GLint, GLint, GLint, GLint, GLint))wglGetProcAddress("glUniform4i"))) ++ n_failed_functions;
	if(!(glUniform1fv = (void(GLApi*)(GLint, GLsizei, const GLfloat*))wglGetProcAddress("glUniform1fv"))) ++ n_failed_functions;
	if(!(glUniform2fv = (void(GLApi*)(GLint, GLsizei, const GLfloat*))wglGetProcAddress("glUniform2fv"))) ++ n_failed_functions;
	if(!(glUniform3fv = (void(GLApi*)(GLint, GLsizei, const GLfloat*))wglGetProcAddress("glUniform3fv"))) ++ n_failed_functions;
	if(!(glUniform4fv = (void(GLApi*)(GLint, GLsizei, const GLfloat*))wglGetProcAddress("glUniform4fv"))) ++ n_failed_functions;
	if(!(glUniform1iv = (void(GLApi*)(GLint, GLsizei, const GLint*))wglGetProcAddress("glUniform1iv"))) ++ n_failed_functions;
	if(!(glUniform2iv = (void(GLApi*)(GLint, GLsizei, const GLint*))wglGetProcAddress("glUniform2iv"))) ++ n_failed_functions;
	if(!(glUniform3iv = (void(GLApi*)(GLint, GLsizei, const GLint*))wglGetProcAddress("glUniform3iv"))) ++ n_failed_functions;
	if(!(glUniform4iv = (void(GLApi*)(GLint, GLsizei, const GLint*))wglGetProcAddress("glUniform4iv"))) ++ n_failed_functions;
	if(!(glUniformMatrix2fv = (void(GLApi*)(GLint, GLsizei, GLboolean transpose, const GLfloat*))wglGetProcAddress("glUniformMatrix2fv"))) ++ n_failed_functions;
	if(!(glUniformMatrix3fv = (void(GLApi*)(GLint, GLsizei, GLboolean transpose, const GLfloat*))wglGetProcAddress("glUniformMatrix3fv"))) ++ n_failed_functions;
	if(!(glUniformMatrix4fv = (void(GLApi*)(GLint, GLsizei, GLboolean transpose, const GLfloat*))wglGetProcAddress("glUniformMatrix4fv"))) ++ n_failed_functions;
	if(!(glBindAttribLocation = (void(GLApi*)(GLuint, GLuint, const GLchar *name))wglGetProcAddress("glBindAttribLocation"))) ++ n_failed_functions;
	if(!(glGetAttribLocation = (GLint(GLApi*)(GLuint, const GLchar *name))wglGetProcAddress("glGetAttribLocation"))) ++ n_failed_functions;
	if(!(glVertexAttrib1d = (void(GLApi*)(GLuint, GLdouble))wglGetProcAddress("glVertexAttrib1d"))) ++ n_failed_functions;
	if(!(glVertexAttrib1dv = (void(GLApi*)(GLuint, const GLdouble*))wglGetProcAddress("glVertexAttrib1dv"))) ++ n_failed_functions;
	if(!(glVertexAttrib1f = (void(GLApi*)(GLuint, GLfloat))wglGetProcAddress("glVertexAttrib1f"))) ++ n_failed_functions;
	if(!(glVertexAttrib1fv = (void(GLApi*)(GLuint, const GLfloat*))wglGetProcAddress("glVertexAttrib1fv"))) ++ n_failed_functions;
	if(!(glVertexAttrib1s = (void(GLApi*)(GLuint, GLshort))wglGetProcAddress("glVertexAttrib1s"))) ++ n_failed_functions;
	if(!(glVertexAttrib1sv = (void(GLApi*)(GLuint, const GLshort*))wglGetProcAddress("glVertexAttrib1sv"))) ++ n_failed_functions;
	if(!(glVertexAttrib2d = (void(GLApi*)(GLuint, GLdouble, GLdouble))wglGetProcAddress("glVertexAttrib2d"))) ++ n_failed_functions;
	if(!(glVertexAttrib2dv = (void(GLApi*)(GLuint, const GLdouble*))wglGetProcAddress("glVertexAttrib2dv"))) ++ n_failed_functions;
	if(!(glVertexAttrib2f = (void(GLApi*)(GLuint, GLfloat, GLfloat))wglGetProcAddress("glVertexAttrib2f"))) ++ n_failed_functions;
	if(!(glVertexAttrib2fv = (void(GLApi*)(GLuint, const GLfloat*))wglGetProcAddress("glVertexAttrib2fv"))) ++ n_failed_functions;
	if(!(glVertexAttrib2s = (void(GLApi*)(GLuint, GLshort, GLshort))wglGetProcAddress("glVertexAttrib2s"))) ++ n_failed_functions;
	if(!(glVertexAttrib2sv = (void(GLApi*)(GLuint, const GLshort*))wglGetProcAddress("glVertexAttrib2sv"))) ++ n_failed_functions;
	if(!(glVertexAttrib3d = (void(GLApi*)(GLuint, GLdouble, GLdouble, GLdouble))wglGetProcAddress("glVertexAttrib3d"))) ++ n_failed_functions;
	if(!(glVertexAttrib3dv = (void(GLApi*)(GLuint, const GLdouble*))wglGetProcAddress("glVertexAttrib3dv"))) ++ n_failed_functions;
	if(!(glVertexAttrib3f = (void(GLApi*)(GLuint, GLfloat, GLfloat, GLfloat z))wglGetProcAddress("glVertexAttrib3f"))) ++ n_failed_functions;
	if(!(glVertexAttrib3fv = (void(GLApi*)(GLuint, const GLfloat*))wglGetProcAddress("glVertexAttrib3fv"))) ++ n_failed_functions;
	if(!(glVertexAttrib3s = (void(GLApi*)(GLuint, GLshort, GLshort, GLshort z))wglGetProcAddress("glVertexAttrib3s"))) ++ n_failed_functions;
	if(!(glVertexAttrib3sv = (void(GLApi*)(GLuint, const GLshort*))wglGetProcAddress("glVertexAttrib3sv"))) ++ n_failed_functions;
	if(!(glVertexAttrib4Nbv = (void(GLApi*)(GLuint, const GLbyte*))wglGetProcAddress("glVertexAttrib4Nbv"))) ++ n_failed_functions;
	if(!(glVertexAttrib4Niv = (void(GLApi*)(GLuint, const GLint*))wglGetProcAddress("glVertexAttrib4Niv"))) ++ n_failed_functions;
	if(!(glVertexAttrib4Nsv = (void(GLApi*)(GLuint, const GLshort*))wglGetProcAddress("glVertexAttrib4Nsv"))) ++ n_failed_functions;
	if(!(glVertexAttrib4Nub = (void(GLApi*)(GLuint, GLubyte, GLubyte, GLubyte, GLubyte))wglGetProcAddress("glVertexAttrib4Nub"))) ++ n_failed_functions;
	if(!(glVertexAttrib4Nubv = (void(GLApi*)(GLuint, const GLubyte*))wglGetProcAddress("glVertexAttrib4Nubv"))) ++ n_failed_functions;
	if(!(glVertexAttrib4Nuiv = (void(GLApi*)(GLuint, const GLuint*))wglGetProcAddress("glVertexAttrib4Nuiv"))) ++ n_failed_functions;
	if(!(glVertexAttrib4Nusv = (void(GLApi*)(GLuint, const GLushort*))wglGetProcAddress("glVertexAttrib4Nusv"))) ++ n_failed_functions;
	if(!(glVertexAttrib4bv = (void(GLApi*)(GLuint, const GLbyte*))wglGetProcAddress("glVertexAttrib4bv"))) ++ n_failed_functions;
	if(!(glVertexAttrib4d = (void(GLApi*)(GLuint, GLdouble, GLdouble, GLdouble, GLdouble))wglGetProcAddress("glVertexAttrib4d"))) ++ n_failed_functions;
	if(!(glVertexAttrib4dv = (void(GLApi*)(GLuint, const GLdouble*))wglGetProcAddress("glVertexAttrib4dv"))) ++ n_failed_functions;
	if(!(glVertexAttrib4f = (void(GLApi*)(GLuint, GLfloat, GLfloat, GLfloat, GLfloat))wglGetProcAddress("glVertexAttrib4f"))) ++ n_failed_functions;
	if(!(glVertexAttrib4fv = (void(GLApi*)(GLuint, const GLfloat*))wglGetProcAddress("glVertexAttrib4fv"))) ++ n_failed_functions;
	if(!(glVertexAttrib4iv = (void(GLApi*)(GLuint, const GLint*))wglGetProcAddress("glVertexAttrib4iv"))) ++ n_failed_functions;
	if(!(glVertexAttrib4s = (void(GLApi*)(GLuint, GLshort, GLshort, GLshort, GLshort))wglGetProcAddress("glVertexAttrib4s"))) ++ n_failed_functions;
	if(!(glVertexAttrib4sv = (void(GLApi*)(GLuint, const GLshort*))wglGetProcAddress("glVertexAttrib4sv"))) ++ n_failed_functions;
	if(!(glVertexAttrib4ubv = (void(GLApi*)(GLuint, const GLubyte*))wglGetProcAddress("glVertexAttrib4ubv"))) ++ n_failed_functions;
	if(!(glVertexAttrib4uiv = (void(GLApi*)(GLuint, const GLuint*))wglGetProcAddress("glVertexAttrib4uiv"))) ++ n_failed_functions;
	if(!(glVertexAttrib4usv = (void(GLApi*)(GLuint, const GLushort*))wglGetProcAddress("glVertexAttrib4usv"))) ++ n_failed_functions;
	if(!(glVertexAttribPointer = (void(GLApi*)(GLuint, GLint, GLenum, GLboolean, GLsizei, const GLvoid*))wglGetProcAddress("glVertexAttribPointer"))) ++ n_failed_functions;
	if(!(glEnableVertexAttribArray = (void(GLApi*)(GLuint index))wglGetProcAddress("glEnableVertexAttribArray"))) ++ n_failed_functions;
	if(!(glDisableVertexAttribArray = (void(GLApi*)(GLuint index))wglGetProcAddress("glDisableVertexAttribArray"))) ++ n_failed_functions;
	if(!(glGetVertexAttribdv = (void(GLApi*)(GLuint, GLenum, GLdouble*))wglGetProcAddress("glGetVertexAttribdv"))) ++ n_failed_functions;
	if(!(glGetVertexAttribfv = (void(GLApi*)(GLuint, GLenum, GLfloat*))wglGetProcAddress("glGetVertexAttribfv"))) ++ n_failed_functions;
	if(!(glGetVertexAttribiv = (void(GLApi*)(GLuint, GLenum, GLint*))wglGetProcAddress("glGetVertexAttribiv"))) ++ n_failed_functions;
	if(!(glGetVertexAttribPointerv = (void(GLApi*)(GLuint, GLenum, GLvoid**))wglGetProcAddress("glGetVertexAttribPointerv"))) ++ n_failed_functions;
	if(!(glIsShader = (GLboolean(GLApi*)(GLuint shader))wglGetProcAddress("glIsShader"))) ++ n_failed_functions;
	if(!(glIsProgram = (GLboolean(GLApi*)(GLuint program))wglGetProcAddress("glIsProgram"))) ++ n_failed_functions;
	if(!(glGetShaderiv = (void(GLApi*)(GLuint, GLenum, GLint*))wglGetProcAddress("glGetShaderiv"))) ++ n_failed_functions;
	if(!(glGetProgramiv = (void(GLApi*)(GLuint, GLenum, GLint*))wglGetProcAddress("glGetProgramiv"))) ++ n_failed_functions;
	if(!(glGetAttachedShaders = (void(GLApi*)(GLuint, GLsizei maxCount, GLsizei *count, GLuint*))wglGetProcAddress("glGetAttachedShaders"))) ++ n_failed_functions;
	if(!(glGetShaderInfoLog = (void(GLApi*)(GLuint shader, GLsizei bufSize, GLsizei*, GLchar*))wglGetProcAddress("glGetShaderInfoLog"))) ++ n_failed_functions;
	if(!(glGetProgramInfoLog = (void(GLApi*)(GLuint, GLsizei bufSize, GLsizei*, GLchar*))wglGetProcAddress("glGetProgramInfoLog"))) ++ n_failed_functions;
	if(!(glGetUniformLocation = (GLint(GLApi*)(GLuint, const GLchar *name))wglGetProcAddress("glGetUniformLocation"))) ++ n_failed_functions;
	if(!(glGetActiveUniform = (void(GLApi*)(GLuint, GLuint, GLsizei bufSize, GLsizei*, GLsizei*, GLenum*, GLchar*))wglGetProcAddress("glGetActiveUniform"))) ++ n_failed_functions;
	if(!(glGetUniformfv = (void(GLApi*)(GLuint, GLint, GLfloat*))wglGetProcAddress("glGetUniformfv"))) ++ n_failed_functions;
	if(!(glGetUniformiv = (void(GLApi*)(GLuint, GLint, GLint*))wglGetProcAddress("glGetUniformiv"))) ++ n_failed_functions;
	if(!(glGetShaderSource = (void(GLApi*)(GLuint, GLsizei bufSize, GLsizei*, GLchar*))wglGetProcAddress("glGetShaderSource"))) ++ n_failed_functions;
	if(!(glGetActiveAttrib = (void(GLApi*)(GLuint, GLuint, GLsizei bufSize, GLsizei*, GLsizei*, GLenum*, GLchar*))wglGetProcAddress("glGetActiveAttrib"))) ++ n_failed_functions;
	if(!(glDrawBuffers = (void(GLApi*)(GLsizei, const GLenum*))wglGetProcAddress("glDrawBuffers"))) ++ n_failed_functions;
	if(!(glStencilFuncSeparate = (void(GLApi*)(GLenum, GLenum, GLint, GLuint))wglGetProcAddress("glStencilFuncSeparate"))) ++ n_failed_functions;
	if(!(glStencilOpSeparate = (void(GLApi*)(GLenum, GLenum, GLenum, GLenum))wglGetProcAddress("glStencilOpSeparate"))) ++ n_failed_functions;
	if(!(glStencilMaskSeparate = (void(GLApi*)(GLenum, GLuint))wglGetProcAddress("glStencilMaskSeparate"))) ++ n_failed_functions;
	if(!(glBlendEquationSeparate = (void(GLApi*)(GLenum, GLenum))wglGetProcAddress("glBlendEquationSeparate"))) ++ n_failed_functions;

#endif //__GENERATE_GL_20__

	return n_failed_functions;
}

/*
 *				--- GL_ARB_vertex_program ---
 */

#ifdef __GENERATE_GL_ARB_vertex_program__

void (GLApi *glVertexAttrib1sARB)(GLuint index, short x) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib1fARB)(GLuint index, float x) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib1dARB)(GLuint index, double x) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib2sARB)(GLuint index, short x, short y) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib2fARB)(GLuint index, float x, float y) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib2dARB)(GLuint index, double x, double y) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib3sARB)(GLuint index, short x, short y, short z) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib3fARB)(GLuint index, float x, float y, float z) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib3dARB)(GLuint index, double x, double y, double z) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4sARB)(GLuint index, short x, short y, short z, short w) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4fARB)(GLuint index, float x, float y, float z, float w) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4dARB)(GLuint index, double x, double y, double z, double w) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4NubARB)(GLuint index, GLubyte x, GLubyte y, GLubyte z, GLubyte w) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib1svARB)(GLuint index, const short *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib1fvARB)(GLuint index, const float *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib1dvARB)(GLuint index, const double *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib2svARB)(GLuint index, const short *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib2fvARB)(GLuint index, const float *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib2dvARB)(GLuint index, const double *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib3svARB)(GLuint index, const short *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib3fvARB)(GLuint index, const float *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib3dvARB)(GLuint index, const double *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4bvARB)(GLuint index, const GLbyte *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4svARB)(GLuint index, const short *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4ivARB)(GLuint index, const int *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4ubvARB)(GLuint index, const GLubyte *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4usvARB)(GLuint index, const GLushort *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4uivARB)(GLuint index, const GLuint *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4fvARB)(GLuint index, const float *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4dvARB)(GLuint index, const double *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4NbvARB)(GLuint index, const GLbyte *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4NsvARB)(GLuint index, const short *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4NivARB)(GLuint index, const int *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4NubvARB)(GLuint index, const GLubyte *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4NusvARB)(GLuint index, const GLushort *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4NuivARB)(GLuint index, const GLuint *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttribPointerARB)(GLuint index, int size, GLenum type, GLboolean normalized, GLsizei stride, const void *pointer) INIT_FUNC_POINTERS;
void (GLApi *glEnableVertexAttribArrayARB)(GLuint index) INIT_FUNC_POINTERS;
void (GLApi *glDisableVertexAttribArrayARB)(GLuint index) INIT_FUNC_POINTERS;
void (GLApi *glProgramStringARB)(GLenum target, GLenum format, GLsizei len, const void *string) INIT_FUNC_POINTERS; 
void (GLApi *glBindProgramARB)(GLenum target, GLuint program) INIT_FUNC_POINTERS;
void (GLApi *glDeleteProgramsARB)(GLsizei n, const GLuint *programs) INIT_FUNC_POINTERS;
void (GLApi *glGenProgramsARB)(GLsizei n, GLuint *programs) INIT_FUNC_POINTERS;
void (GLApi *glProgramEnvParameter4dARB)(GLenum target, GLuint index, double x, double y, double z, double w) INIT_FUNC_POINTERS;
void (GLApi *glProgramEnvParameter4dvARB)(GLenum target, GLuint index, const double *params) INIT_FUNC_POINTERS;
void (GLApi *glProgramEnvParameter4fARB)(GLenum target, GLuint index, float x, float y, float z, float w) INIT_FUNC_POINTERS;
void (GLApi *glProgramEnvParameter4fvARB)(GLenum target, GLuint index, const float *params) INIT_FUNC_POINTERS;
void (GLApi *glProgramLocalParameter4dARB)(GLenum target, GLuint index, double x, double y, double z, double w) INIT_FUNC_POINTERS;
void (GLApi *glProgramLocalParameter4dvARB)(GLenum target, GLuint index, const double *params) INIT_FUNC_POINTERS;
void (GLApi *glProgramLocalParameter4fARB)(GLenum target, GLuint index, float x, float y, float z, float w) INIT_FUNC_POINTERS;
void (GLApi *glProgramLocalParameter4fvARB)(GLenum target, GLuint index, const float *params) INIT_FUNC_POINTERS;
void (GLApi *glGetProgramEnvParameterdvARB)(GLenum target, GLuint index, double *params) INIT_FUNC_POINTERS;
void (GLApi *glGetProgramEnvParameterfvARB)(GLenum target, GLuint index, float *params) INIT_FUNC_POINTERS;
void (GLApi *glGetProgramLocalParameterdvARB)(GLenum target, GLuint index, double *params) INIT_FUNC_POINTERS;
void (GLApi *glGetProgramLocalParameterfvARB)(GLenum target, GLuint index, float *params) INIT_FUNC_POINTERS;
void (GLApi *glGetProgramivARB)(GLenum target, GLenum pname, int *params) INIT_FUNC_POINTERS;
void (GLApi *glGetProgramStringARB)(GLenum target, GLenum pname, void *string) INIT_FUNC_POINTERS;
void (GLApi *glGetVertexAttribdvARB)(GLuint index, GLenum pname, double *params) INIT_FUNC_POINTERS;
void (GLApi *glGetVertexAttribfvARB)(GLuint index, GLenum pname, float *params) INIT_FUNC_POINTERS;
void (GLApi *glGetVertexAttribivARB)(GLuint index, GLenum pname, int *params) INIT_FUNC_POINTERS;
void (GLApi *glGetVertexAttribPointervARB)(GLuint index, GLenum pname, void **pointer) INIT_FUNC_POINTERS;
GLboolean (GLApi *glIsProgramARB)(GLuint program) INIT_FUNC_POINTERS;

#endif //__GENERATE_GL_ARB_vertex_program__

/*
 *	static int CGLExtensionHandler::n_GetVertexProgramARBFuncPointers()
 *		- gets GL_ARB_vertex_program function pointers
 *		- returns number of missing functions
 */
int CGLExtensionHandler::n_GetVertexProgramARBFuncPointers()
{
	__FuncGuard("CGLExtensionHandler::n_GetVertexProgramARBFuncPointers");

	int n_failed_functions = 0;

#ifdef __GENERATE_GL_ARB_vertex_program__

	if(!(glVertexAttrib1sARB = (void(GLApi*)(GLuint, short))wglGetProcAddress("glVertexAttrib1sARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib1fARB = (void(GLApi*)(GLuint, float))wglGetProcAddress("glVertexAttrib1fARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib1dARB = (void(GLApi*)(GLuint, double))wglGetProcAddress("glVertexAttrib1dARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib2sARB = (void(GLApi*)(GLuint, short, short))wglGetProcAddress("glVertexAttrib2sARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib2fARB = (void(GLApi*)(GLuint, float, float))wglGetProcAddress("glVertexAttrib2fARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib2dARB = (void(GLApi*)(GLuint, double, double))wglGetProcAddress("glVertexAttrib2dARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib3sARB = (void(GLApi*)(GLuint, short, short, short))wglGetProcAddress("glVertexAttrib3sARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib3fARB = (void(GLApi*)(GLuint, float, float, float))wglGetProcAddress("glVertexAttrib3fARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib3dARB = (void(GLApi*)(GLuint, double, double, double))wglGetProcAddress("glVertexAttrib3dARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib4sARB = (void(GLApi*)(GLuint, short, short, short, short))wglGetProcAddress("glVertexAttrib4sARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib4fARB = (void(GLApi*)(GLuint, float, float, float, float))wglGetProcAddress("glVertexAttrib4fARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib4dARB = (void(GLApi*)(GLuint, double, double, double, double))wglGetProcAddress("glVertexAttrib4dARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib4NubARB = (void(GLApi*)(GLuint, GLubyte, GLubyte, GLubyte, GLubyte))wglGetProcAddress("glVertexAttrib4NubARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib1svARB = (void(GLApi*)(GLuint, const short*))wglGetProcAddress("glVertexAttrib1svARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib1fvARB = (void(GLApi*)(GLuint, const float*))wglGetProcAddress("glVertexAttrib1fvARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib1dvARB = (void(GLApi*)(GLuint, const double*))wglGetProcAddress("glVertexAttrib1dvARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib2svARB = (void(GLApi*)(GLuint, const short*))wglGetProcAddress("glVertexAttrib2svARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib2fvARB = (void(GLApi*)(GLuint, const float*))wglGetProcAddress("glVertexAttrib2fvARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib2dvARB = (void(GLApi*)(GLuint, const double*))wglGetProcAddress("glVertexAttrib2dvARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib3svARB = (void(GLApi*)(GLuint, const short*))wglGetProcAddress("glVertexAttrib3svARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib3fvARB = (void(GLApi*)(GLuint, const float*))wglGetProcAddress("glVertexAttrib3fvARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib3dvARB = (void(GLApi*)(GLuint, const double*))wglGetProcAddress("glVertexAttrib3dvARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib4bvARB = (void(GLApi*)(GLuint, const GLbyte*))wglGetProcAddress("glVertexAttrib4bvARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib4svARB = (void(GLApi*)(GLuint, const short*))wglGetProcAddress("glVertexAttrib4svARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib4ivARB = (void(GLApi*)(GLuint, const int*))wglGetProcAddress("glVertexAttrib4ivARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib4ubvARB = (void(GLApi*)(GLuint, const GLubyte*))wglGetProcAddress("glVertexAttrib4ubvARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib4usvARB = (void(GLApi*)(GLuint, const GLushort*))wglGetProcAddress("glVertexAttrib4usvARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib4uivARB = (void(GLApi*)(GLuint, const GLuint*))wglGetProcAddress("glVertexAttrib4uivARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib4fvARB = (void(GLApi*)(GLuint, const float*))wglGetProcAddress("glVertexAttrib4fvARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib4dvARB = (void(GLApi*)(GLuint, const double*))wglGetProcAddress("glVertexAttrib4dvARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib4NbvARB = (void(GLApi*)(GLuint, const GLbyte*))wglGetProcAddress("glVertexAttrib4NbvARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib4NsvARB = (void(GLApi*)(GLuint, const short*))wglGetProcAddress("glVertexAttrib4NsvARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib4NivARB = (void(GLApi*)(GLuint, const int*))wglGetProcAddress("glVertexAttrib4NivARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib4NubvARB = (void(GLApi*)(GLuint, const GLubyte*))wglGetProcAddress("glVertexAttrib4NubvARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib4NusvARB = (void(GLApi*)(GLuint, const GLushort*))wglGetProcAddress("glVertexAttrib4NusvARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib4NuivARB = (void(GLApi*)(GLuint, const GLuint*))wglGetProcAddress("glVertexAttrib4NuivARB"))) ++ n_failed_functions;
	if(!(glVertexAttribPointerARB = (void(GLApi*)(GLuint, int size, GLenum, GLboolean, GLsizei stride, const void*))wglGetProcAddress("glVertexAttribPointerARB"))) ++ n_failed_functions;
	if(!(glEnableVertexAttribArrayARB = (void(GLApi*)(GLuint))wglGetProcAddress("glEnableVertexAttribArrayARB"))) ++ n_failed_functions;
	if(!(glDisableVertexAttribArrayARB = (void(GLApi*)(GLuint))wglGetProcAddress("glDisableVertexAttribArrayARB"))) ++ n_failed_functions;
	if(!(glProgramStringARB = (void(GLApi*)(GLenum, GLenum, GLsizei, const void*))wglGetProcAddress("glProgramStringARB"))) ++ n_failed_functions; 
	if(!(glBindProgramARB = (void(GLApi*)(GLenum, GLuint program))wglGetProcAddress("glBindProgramARB"))) ++ n_failed_functions;
	if(!(glDeleteProgramsARB = (void(GLApi*)(GLsizei, const GLuint *programs))wglGetProcAddress("glDeleteProgramsARB"))) ++ n_failed_functions;
	if(!(glGenProgramsARB = (void(GLApi*)(GLsizei, GLuint *programs))wglGetProcAddress("glGenProgramsARB"))) ++ n_failed_functions;
	if(!(glProgramEnvParameter4dARB = (void(GLApi*)(GLenum, GLuint, double, double, double, double))wglGetProcAddress("glProgramEnvParameter4dARB"))) ++ n_failed_functions;
	if(!(glProgramEnvParameter4dvARB = (void(GLApi*)(GLenum, GLuint, const double*))wglGetProcAddress("glProgramEnvParameter4dvARB"))) ++ n_failed_functions;
	if(!(glProgramEnvParameter4fARB = (void(GLApi*)(GLenum, GLuint, float, float, float, float))wglGetProcAddress("glProgramEnvParameter4fARB"))) ++ n_failed_functions;
	if(!(glProgramEnvParameter4fvARB = (void(GLApi*)(GLenum, GLuint, const float*))wglGetProcAddress("glProgramEnvParameter4fvARB"))) ++ n_failed_functions;
	if(!(glProgramLocalParameter4dARB = (void(GLApi*)(GLenum, GLuint, double, double, double, double))wglGetProcAddress("glProgramLocalParameter4dARB"))) ++ n_failed_functions;
	if(!(glProgramLocalParameter4dvARB = (void(GLApi*)(GLenum, GLuint, const double*))wglGetProcAddress("glProgramLocalParameter4dvARB"))) ++ n_failed_functions;
	if(!(glProgramLocalParameter4fARB = (void(GLApi*)(GLenum, GLuint, float, float, float, float))wglGetProcAddress("glProgramLocalParameter4fARB"))) ++ n_failed_functions;
	if(!(glProgramLocalParameter4fvARB = (void(GLApi*)(GLenum, GLuint, const float*))wglGetProcAddress("glProgramLocalParameter4fvARB"))) ++ n_failed_functions;
	if(!(glGetProgramEnvParameterdvARB = (void(GLApi*)(GLenum, GLuint, double*))wglGetProcAddress("glGetProgramEnvParameterdvARB"))) ++ n_failed_functions;
	if(!(glGetProgramEnvParameterfvARB = (void(GLApi*)(GLenum, GLuint, float*))wglGetProcAddress("glGetProgramEnvParameterfvARB"))) ++ n_failed_functions;
	if(!(glGetProgramLocalParameterdvARB = (void(GLApi*)(GLenum, GLuint, double*))wglGetProcAddress("glGetProgramLocalParameterdvARB"))) ++ n_failed_functions;
	if(!(glGetProgramLocalParameterfvARB = (void(GLApi*)(GLenum, GLuint, float*))wglGetProcAddress("glGetProgramLocalParameterfvARB"))) ++ n_failed_functions;
	if(!(glGetProgramivARB = (void(GLApi*)(GLenum, GLenum, int*))wglGetProcAddress("glGetProgramivARB"))) ++ n_failed_functions;
	if(!(glGetProgramStringARB = (void(GLApi*)(GLenum, GLenum, void*))wglGetProcAddress("glGetProgramStringARB"))) ++ n_failed_functions;
	if(!(glGetVertexAttribdvARB = (void(GLApi*)(GLuint, GLenum, double*))wglGetProcAddress("glGetVertexAttribdvARB"))) ++ n_failed_functions;
	if(!(glGetVertexAttribfvARB = (void(GLApi*)(GLuint, GLenum, float*))wglGetProcAddress("glGetVertexAttribfvARB"))) ++ n_failed_functions;
	if(!(glGetVertexAttribivARB = (void(GLApi*)(GLuint, GLenum, int*))wglGetProcAddress("glGetVertexAttribivARB"))) ++ n_failed_functions;
	if(!(glGetVertexAttribPointervARB = (void(GLApi*)(GLuint, GLenum, void**))wglGetProcAddress("glGetVertexAttribPointervARB"))) ++ n_failed_functions;
	if(!(glIsProgramARB = (GLboolean(GLApi*)(GLuint program))wglGetProcAddress("glIsProgramARB"))) ++ n_failed_functions;

#endif //__GENERATE_GL_ARB_vertex_program__

	return n_failed_functions;
}

/*
 *				--- GL_ARB_fragment_program ---
 */

/*
void (GLApi *glProgramStringARB)(GLenum target, GLenum format, GLsizei len, const void *string) INIT_FUNC_POINTERS; 
void (GLApi *glBindProgramARB)(GLenum target, GLuint program) INIT_FUNC_POINTERS;
void (GLApi *glDeleteProgramsARB)(GLsizei n, const GLuint *programs) INIT_FUNC_POINTERS;
void (GLApi *glGenProgramsARB)(GLsizei n, GLuint *programs) INIT_FUNC_POINTERS;
void (GLApi *glProgramEnvParameter4dARB)(GLenum target, GLuint index, double x, double y, double z, double w) INIT_FUNC_POINTERS;
void (GLApi *glProgramEnvParameter4dvARB)(GLenum target, GLuint index, const double *params) INIT_FUNC_POINTERS;
void (GLApi *glProgramEnvParameter4fARB)(GLenum target, GLuint index, float x, float y, float z, float w) INIT_FUNC_POINTERS;
void (GLApi *glProgramEnvParameter4fvARB)(GLenum target, GLuint index, const float *params) INIT_FUNC_POINTERS;
void (GLApi *glProgramLocalParameter4dARB)(GLenum target, GLuint index, double x, double y, double z, double w) INIT_FUNC_POINTERS;
void (GLApi *glProgramLocalParameter4dvARB)(GLenum target, GLuint index, const double *params) INIT_FUNC_POINTERS;
void (GLApi *glProgramLocalParameter4fARB)(GLenum target, GLuint index, float x, float y, float z, float w) INIT_FUNC_POINTERS;
void (GLApi *glProgramLocalParameter4fvARB)(GLenum target, GLuint index, const float *params) INIT_FUNC_POINTERS;
void (GLApi *glGetProgramEnvParameterdvARB)(GLenum target, GLuint index, double *params) INIT_FUNC_POINTERS;
void (GLApi *glGetProgramEnvParameterfvARB)(GLenum target, GLuint index, float *params) INIT_FUNC_POINTERS;
void (GLApi *glGetProgramLocalParameterdvARB)(GLenum target, GLuint index, double *params) INIT_FUNC_POINTERS;
void (GLApi *glGetProgramLocalParameterfvARB)(GLenum target, GLuint index, float *params) INIT_FUNC_POINTERS;
void (GLApi *glGetProgramivARB)(GLenum target, GLenum pname, int *params) INIT_FUNC_POINTERS;
void (GLApi *glGetProgramStringARB)(GLenum target, GLenum pname, void *string) INIT_FUNC_POINTERS;
GLboolean (GLApi *glIsProgramARB)(GLuint program) INIT_FUNC_POINTERS;
*/
// declared in vertex shader

/*
 *	static int CGLExtensionHandler::n_GetFragmentProgramARBFuncPointers()
 *		- gets GL_ARB_fragment_program function pointers
 *		- returns number of missing functions
 */
int CGLExtensionHandler::n_GetFragmentProgramARBFuncPointers()
{
	__FuncGuard("CGLExtensionHandler::n_GetFragmentProgramARBFuncPointers");

	int n_failed_functions = 0;

#ifdef __GENERATE_GL_ARB_fragment_program__

	if(!(glProgramStringARB = (void(GLApi*)(GLenum, GLenum, GLsizei, const void*))wglGetProcAddress("glProgramStringARB"))) ++ n_failed_functions; 
	if(!(glBindProgramARB = (void(GLApi*)(GLenum, GLuint program))wglGetProcAddress("glBindProgramARB"))) ++ n_failed_functions;
	if(!(glDeleteProgramsARB = (void(GLApi*)(GLsizei, const GLuint *programs))wglGetProcAddress("glDeleteProgramsARB"))) ++ n_failed_functions;
	if(!(glGenProgramsARB = (void(GLApi*)(GLsizei, GLuint *programs))wglGetProcAddress("glGenProgramsARB"))) ++ n_failed_functions;
	if(!(glProgramEnvParameter4dARB = (void(GLApi*)(GLenum, GLuint, double, double, double, double))wglGetProcAddress("glProgramEnvParameter4dARB"))) ++ n_failed_functions;
	if(!(glProgramEnvParameter4dvARB = (void(GLApi*)(GLenum, GLuint, const double*))wglGetProcAddress("glProgramEnvParameter4dvARB"))) ++ n_failed_functions;
	if(!(glProgramEnvParameter4fARB = (void(GLApi*)(GLenum, GLuint, float, float, float, float))wglGetProcAddress("glProgramEnvParameter4fARB"))) ++ n_failed_functions;
	if(!(glProgramEnvParameter4fvARB = (void(GLApi*)(GLenum, GLuint, const float*))wglGetProcAddress("glProgramEnvParameter4fvARB"))) ++ n_failed_functions;
	if(!(glProgramLocalParameter4dARB = (void(GLApi*)(GLenum, GLuint, double, double, double, double))wglGetProcAddress("glProgramLocalParameter4dARB"))) ++ n_failed_functions;
	if(!(glProgramLocalParameter4dvARB = (void(GLApi*)(GLenum, GLuint, const double*))wglGetProcAddress("glProgramLocalParameter4dvARB"))) ++ n_failed_functions;
	if(!(glProgramLocalParameter4fARB = (void(GLApi*)(GLenum, GLuint, float, float, float, float))wglGetProcAddress("glProgramLocalParameter4fARB"))) ++ n_failed_functions;
	if(!(glProgramLocalParameter4fvARB = (void(GLApi*)(GLenum, GLuint, const float*))wglGetProcAddress("glProgramLocalParameter4fvARB"))) ++ n_failed_functions;
	if(!(glGetProgramEnvParameterdvARB = (void(GLApi*)(GLenum, GLuint, double*))wglGetProcAddress("glGetProgramEnvParameterdvARB"))) ++ n_failed_functions;
	if(!(glGetProgramEnvParameterfvARB = (void(GLApi*)(GLenum, GLuint, float*))wglGetProcAddress("glGetProgramEnvParameterfvARB"))) ++ n_failed_functions;
	if(!(glGetProgramLocalParameterdvARB = (void(GLApi*)(GLenum, GLuint, double*))wglGetProcAddress("glGetProgramLocalParameterdvARB"))) ++ n_failed_functions;
	if(!(glGetProgramLocalParameterfvARB = (void(GLApi*)(GLenum, GLuint, float*))wglGetProcAddress("glGetProgramLocalParameterfvARB"))) ++ n_failed_functions;
	if(!(glGetProgramivARB = (void(GLApi*)(GLenum, GLenum, int*))wglGetProcAddress("glGetProgramivARB"))) ++ n_failed_functions;
	if(!(glGetProgramStringARB = (void(GLApi*)(GLenum, GLenum, void*))wglGetProcAddress("glGetProgramStringARB"))) ++ n_failed_functions;
	if(!(glIsProgramARB = (GLboolean(GLApi*)(GLuint program))wglGetProcAddress("glIsProgramARB"))) ++ n_failed_functions;

#endif //__GENERATE_GL_ARB_fragment_program__

	return n_failed_functions;
}

/*
 *				--- GL_ARB_vertex_shader ---
 */

#ifdef __GENERATE_GL_ARB_vertex_program__

/*void (GLApi *glVertexAttrib1fARB)(GLuint index, float v0) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib1sARB)(GLuint index, short v0) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib1dARB)(GLuint index, double v0) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib2fARB)(GLuint index, float v0, float v1) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib2sARB)(GLuint index, short v0, short v1) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib2dARB)(GLuint index, double v0, double v1) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib3fARB)(GLuint index, float v0, float v1, float v2) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib3sARB)(GLuint index, short v0, short v1, short v2) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib3dARB)(GLuint index, double v0, double v1, double v2) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4fARB)(GLuint index, float v0, float v1, float v2, float v3) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4sARB)(GLuint index, short v0, short v1, short v2, short v3) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4dARB)(GLuint index, double v0, double v1, double v2, double v3) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4NubARB)(GLuint index, GLubyte x, GLubyte y, GLubyte z, GLubyte w) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib1fvARB)(GLuint index, const float *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib1svARB)(GLuint index, const short *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib1dvARB)(GLuint index, const double *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib2fvARB)(GLuint index, const float *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib2svARB)(GLuint index, const short *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib2dvARB)(GLuint index, const double *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib3fvARB)(GLuint index, const float *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib3svARB)(GLuint index, const short *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib3dvARB)(GLuint index, const double *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4fvARB)(GLuint index, const float *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4svARB)(GLuint index, const short *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4dvARB)(GLuint index, const double *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4ivARB)(GLuint index, const int *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4bvARB)(GLuint index, const GLbyte *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4ubvARB)(GLuint index, const GLubyte *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4usvARB)(GLuint index, const GLushort *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4uivARB)(GLuint index, const GLuint *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4NbvARB)(GLuint index, const GLbyte *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4NsvARB)(GLuint index, const short *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4NivARB)(GLuint index, const int *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4NubvARB)(GLuint index, const GLubyte *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4NusvARB)(GLuint index, const GLushort *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttrib4NuivARB)(GLuint index, const GLuint *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttribPointerARB)(GLuint index, int size, GLenum type, GLboolean normalized, GLsizei stride, const void *pointer) INIT_FUNC_POINTERS;
void (GLApi *glEnableVertexAttribArrayARB)(GLuint index) INIT_FUNC_POINTERS;
void (GLApi *glDisableVertexAttribArrayARB)(GLuint index) INIT_FUNC_POINTERS;*/
void (GLApi *glBindAttribLocationARB)(GLhandleARB programObj, GLuint index, const GLcharARB *name) INIT_FUNC_POINTERS;
void (GLApi *glGetActiveAttribARB)(GLhandleARB programObj, GLuint index, GLsizei maxLength, GLsizei *length, int *size, GLenum *type, GLcharARB *name) INIT_FUNC_POINTERS;
int (GLApi *glGetAttribLocationARB)(GLhandleARB programObj, const GLcharARB *name) INIT_FUNC_POINTERS;
/*void (GLApi *glGetVertexAttribdvARB)(GLuint index, GLenum pname, double *params) INIT_FUNC_POINTERS;
void (GLApi *glGetVertexAttribfvARB)(GLuint index, GLenum pname, float *params) INIT_FUNC_POINTERS;
void (GLApi *glGetVertexAttribivARB)(GLuint index, GLenum pname, int *params) INIT_FUNC_POINTERS;
void (GLApi *glGetVertexAttribPointervARB)(GLuint index, GLenum pname, void **pointer) INIT_FUNC_POINTERS;*/
// some functions declared in vertex program

#endif //__GENERATE_GL_ARB_vertex_program__

/*
 *	static int CGLExtensionHandler::n_GetVertexShaderARBFuncPointers()
 *		- gets GL_ARB_vertex_shader function pointers
 *		- returns number of missing functions
 */
int CGLExtensionHandler::n_GetVertexShaderARBFuncPointers()
{
	__FuncGuard("CGLExtensionHandler::n_GetVertexShaderARBFuncPointers");

	int n_failed_functions = 0;

#ifdef __GENERATE_GL_ARB_vertex_program__

	if(!(glVertexAttrib1fARB = (void(GLApi*)(GLuint, float))wglGetProcAddress("glVertexAttrib1fARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib1sARB = (void(GLApi*)(GLuint, short))wglGetProcAddress("glVertexAttrib1sARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib1dARB = (void(GLApi*)(GLuint, double))wglGetProcAddress("glVertexAttrib1dARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib2fARB = (void(GLApi*)(GLuint, float, float))wglGetProcAddress("glVertexAttrib2fARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib2sARB = (void(GLApi*)(GLuint, short, short))wglGetProcAddress("glVertexAttrib2sARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib2dARB = (void(GLApi*)(GLuint, double, double))wglGetProcAddress("glVertexAttrib2dARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib3fARB = (void(GLApi*)(GLuint, float, float, float))wglGetProcAddress("glVertexAttrib3fARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib3sARB = (void(GLApi*)(GLuint, short, short, short))wglGetProcAddress("glVertexAttrib3sARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib3dARB = (void(GLApi*)(GLuint, double, double, double))wglGetProcAddress("glVertexAttrib3dARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib4fARB = (void(GLApi*)(GLuint, float, float, float, float))wglGetProcAddress("glVertexAttrib4fARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib4sARB = (void(GLApi*)(GLuint, short, short, short, short))wglGetProcAddress("glVertexAttrib4sARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib4dARB = (void(GLApi*)(GLuint, double, double, double, double))wglGetProcAddress("glVertexAttrib4dARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib4NubARB = (void(GLApi*)(GLuint, GLubyte, GLubyte, GLubyte, GLubyte))wglGetProcAddress("glVertexAttrib4NubARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib1fvARB = (void(GLApi*)(GLuint, const float*))wglGetProcAddress("glVertexAttrib1fvARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib1svARB = (void(GLApi*)(GLuint, const short*))wglGetProcAddress("glVertexAttrib1svARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib1dvARB = (void(GLApi*)(GLuint, const double*))wglGetProcAddress("glVertexAttrib1dvARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib2fvARB = (void(GLApi*)(GLuint, const float*))wglGetProcAddress("glVertexAttrib2fvARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib2svARB = (void(GLApi*)(GLuint, const short*))wglGetProcAddress("glVertexAttrib2svARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib2dvARB = (void(GLApi*)(GLuint, const double*))wglGetProcAddress("glVertexAttrib2dvARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib3fvARB = (void(GLApi*)(GLuint, const float*))wglGetProcAddress("glVertexAttrib3fvARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib3svARB = (void(GLApi*)(GLuint, const short*))wglGetProcAddress("glVertexAttrib3svARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib3dvARB = (void(GLApi*)(GLuint, const double*))wglGetProcAddress("glVertexAttrib3dvARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib4fvARB = (void(GLApi*)(GLuint, const float*))wglGetProcAddress("glVertexAttrib4fvARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib4svARB = (void(GLApi*)(GLuint, const short*))wglGetProcAddress("glVertexAttrib4svARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib4dvARB = (void(GLApi*)(GLuint, const double*))wglGetProcAddress("glVertexAttrib4dvARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib4ivARB = (void(GLApi*)(GLuint, const int*))wglGetProcAddress("glVertexAttrib4ivARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib4bvARB = (void(GLApi*)(GLuint, const GLbyte*))wglGetProcAddress("glVertexAttrib4bvARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib4ubvARB = (void(GLApi*)(GLuint, const GLubyte*))wglGetProcAddress("glVertexAttrib4ubvARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib4usvARB = (void(GLApi*)(GLuint, const GLushort*))wglGetProcAddress("glVertexAttrib4usvARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib4uivARB = (void(GLApi*)(GLuint, const GLuint*))wglGetProcAddress("glVertexAttrib4uivARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib4NbvARB = (void(GLApi*)(GLuint, const GLbyte*))wglGetProcAddress("glVertexAttrib4NbvARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib4NsvARB = (void(GLApi*)(GLuint, const short*))wglGetProcAddress("glVertexAttrib4NsvARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib4NivARB = (void(GLApi*)(GLuint, const int*))wglGetProcAddress("glVertexAttrib4NivARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib4NubvARB = (void(GLApi*)(GLuint, const GLubyte*))wglGetProcAddress("glVertexAttrib4NubvARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib4NusvARB = (void(GLApi*)(GLuint, const GLushort*))wglGetProcAddress("glVertexAttrib4NusvARB"))) ++ n_failed_functions;
	if(!(glVertexAttrib4NuivARB = (void(GLApi*)(GLuint, const GLuint*))wglGetProcAddress("glVertexAttrib4NuivARB"))) ++ n_failed_functions;
	if(!(glVertexAttribPointerARB = (void(GLApi*)(GLuint, int size, GLenum, GLboolean, GLsizei, const void*))wglGetProcAddress("glVertexAttribPointerARB"))) ++ n_failed_functions;
	if(!(glEnableVertexAttribArrayARB = (void(GLApi*)(GLuint))wglGetProcAddress("glEnableVertexAttribArrayARB"))) ++ n_failed_functions;
	if(!(glDisableVertexAttribArrayARB = (void(GLApi*)(GLuint))wglGetProcAddress("glDisableVertexAttribArrayARB"))) ++ n_failed_functions;
	if(!(glBindAttribLocationARB = (void(GLApi*)(GLhandleARB, GLuint, const GLcharARB*))wglGetProcAddress("glBindAttribLocationARB"))) ++ n_failed_functions;
	if(!(glGetActiveAttribARB = (void(GLApi*)(GLhandleARB, GLuint, GLsizei, GLsizei*, int*, GLenum*, GLcharARB*))wglGetProcAddress("glGetActiveAttribARB"))) ++ n_failed_functions;
	if(!(glGetAttribLocationARB = (int(GLApi*)(GLhandleARB, const GLcharARB*))wglGetProcAddress("glGetAttribLocationARB"))) ++ n_failed_functions;
	if(!(glGetVertexAttribdvARB = (void(GLApi*)(GLuint, GLenum, double*))wglGetProcAddress("glGetVertexAttribdvARB"))) ++ n_failed_functions;
	if(!(glGetVertexAttribfvARB = (void(GLApi*)(GLuint, GLenum, float*))wglGetProcAddress("glGetVertexAttribfvARB"))) ++ n_failed_functions;
	if(!(glGetVertexAttribivARB = (void(GLApi*)(GLuint, GLenum, int*))wglGetProcAddress("glGetVertexAttribivARB"))) ++ n_failed_functions;
	if(!(glGetVertexAttribPointervARB = (void(GLApi*)(GLuint, GLenum, void**))wglGetProcAddress("glGetVertexAttribPointervARB"))) ++ n_failed_functions;

#endif //__GENERATE_GL_ARB_vertex_program__

	return n_failed_functions;
}

/*
 *				--- GL_ARB_fragment_shader ---
 */

/*
 *	static int CGLExtensionHandler::n_GetFragmentShaderARBFuncPointers()
 *		- gets GL_ARB_fragment_shader function pointers
 *		- returns number of missing functions
 */
int CGLExtensionHandler::n_GetFragmentShaderARBFuncPointers()
{
	__FuncGuard("CGLExtensionHandler::n_GetFragmentShaderARBFuncPointers");

	return 0;
}

/*
 *				--- GL_ARB_shader_object ---
 */

#ifdef __GENERATE_GL_ARB_shader_object__

void (GLApi *glDeleteObjectARB)(GLhandleARB obj) INIT_FUNC_POINTERS;
GLhandleARB (GLApi *glGetHandleARB)(GLenum pname) INIT_FUNC_POINTERS;
void (GLApi *glDetachObjectARB)(GLhandleARB containerObj, GLhandleARB attachedObj) INIT_FUNC_POINTERS;
GLhandleARB (GLApi *glCreateShaderObjectARB)(GLenum shaderType) INIT_FUNC_POINTERS;
void (GLApi *glShaderSourceARB)(GLhandleARB shaderObj, GLsizei count, const GLcharARB **string, const int *length) INIT_FUNC_POINTERS;
void (GLApi *glCompileShaderARB)(GLhandleARB shaderObj) INIT_FUNC_POINTERS;
GLhandleARB (GLApi *glCreateProgramObjectARB)(void) INIT_FUNC_POINTERS;
void (GLApi *glAttachObjectARB)(GLhandleARB containerObj, GLhandleARB obj) INIT_FUNC_POINTERS;
void (GLApi *glLinkProgramARB)(GLhandleARB programObj) INIT_FUNC_POINTERS;
void (GLApi *glUseProgramObjectARB)(GLhandleARB programObj) INIT_FUNC_POINTERS;
void (GLApi *glValidateProgramARB)(GLhandleARB programObj) INIT_FUNC_POINTERS;
void (GLApi *glUniform1fARB)(int location, float v0) INIT_FUNC_POINTERS;
void (GLApi *glUniform2fARB)(int location, float v0, float v1) INIT_FUNC_POINTERS;
void (GLApi *glUniform3fARB)(int location, float v0, float v1, float v2) INIT_FUNC_POINTERS;
void (GLApi *glUniform4fARB)(int location, float v0, float v1, float v2, float v3) INIT_FUNC_POINTERS;
void (GLApi *glUniform1iARB)(int location, int v0) INIT_FUNC_POINTERS;
void (GLApi *glUniform2iARB)(int location, int v0, int v1) INIT_FUNC_POINTERS;
void (GLApi *glUniform3iARB)(int location, int v0, int v1, int v2) INIT_FUNC_POINTERS;
void (GLApi *glUniform4iARB)(int location, int v0, int v1, int v2, int v3) INIT_FUNC_POINTERS;
void (GLApi *glUniform1fvARB)(int location, GLsizei count, const float *value) INIT_FUNC_POINTERS;
void (GLApi *glUniform2fvARB)(int location, GLsizei count, const float *value) INIT_FUNC_POINTERS;
void (GLApi *glUniform3fvARB)(int location, GLsizei count, const float *value) INIT_FUNC_POINTERS;
void (GLApi *glUniform4fvARB)(int location, GLsizei count, const float *value) INIT_FUNC_POINTERS;
void (GLApi *glUniform1ivARB)(int location, GLsizei count, const int *value) INIT_FUNC_POINTERS;
void (GLApi *glUniform2ivARB)(int location, GLsizei count, const int *value) INIT_FUNC_POINTERS;
void (GLApi *glUniform3ivARB)(int location, GLsizei count, const int *value) INIT_FUNC_POINTERS;
void (GLApi *glUniform4ivARB)(int location, GLsizei count, const int *value) INIT_FUNC_POINTERS;
void (GLApi *glUniformMatrix2fvARB)(int location, GLsizei count, GLboolean transpose, const float *value) INIT_FUNC_POINTERS;
void (GLApi *glUniformMatrix3fvARB)(int location, GLsizei count, GLboolean transpose, const float *value) INIT_FUNC_POINTERS;
void (GLApi *glUniformMatrix4fvARB)(int location, GLsizei count, GLboolean transpose, const float *value) INIT_FUNC_POINTERS;
void (GLApi *glGetObjectParameterfvARB)(GLhandleARB obj, GLenum pname, float *params) INIT_FUNC_POINTERS;
void (GLApi *glGetObjectParameterivARB)(GLhandleARB obj, GLenum pname, int *params) INIT_FUNC_POINTERS;
void (GLApi *glGetInfoLogARB)(GLhandleARB obj, GLsizei maxLength, GLsizei *length, GLcharARB *infoLog) INIT_FUNC_POINTERS;
void (GLApi *glGetAttachedObjectsARB)(GLhandleARB containerObj, GLsizei maxCount, GLsizei *count, GLhandleARB *obj) INIT_FUNC_POINTERS;
int (GLApi *glGetUniformLocationARB)(GLhandleARB programObj, const GLcharARB *name) INIT_FUNC_POINTERS;
void (GLApi *glGetActiveUniformARB)(GLhandleARB programObj, GLuint index, GLsizei maxLength, GLsizei *length, int *size, GLenum *type, GLcharARB *name) INIT_FUNC_POINTERS;
void (GLApi *glGetUniformfvARB)(GLhandleARB programObj, int location, float *params) INIT_FUNC_POINTERS;
void (GLApi *glGetUniformivARB)(GLhandleARB programObj, int location, int *params) INIT_FUNC_POINTERS;
void (GLApi *glGetShaderSourceARB)(GLhandleARB obj, GLsizei maxLength, GLsizei *length, GLcharARB *source) INIT_FUNC_POINTERS;

#endif //__GENERATE_GL_ARB_shader_object__

/*
 *	static int CGLExtensionHandler::n_GetShaderObjectARBFuncPointers()
 *		- gets GL_ARB_shader_object function pointers
 *		- returns number of missing functions
 */
int CGLExtensionHandler::n_GetShaderObjectARBFuncPointers()
{
	__FuncGuard("CGLExtensionHandler::n_GetShaderObjectARBFuncPointers");

	int n_failed_functions = 0;

#ifdef __GENERATE_GL_ARB_shader_object__

	if(!(glDeleteObjectARB = (void(GLApi*)(GLhandleARB))wglGetProcAddress("glDeleteObjectARB"))) ++ n_failed_functions;
	if(!(glGetHandleARB = (GLhandleARB(GLApi*)(GLenum))wglGetProcAddress("glGetHandleARB"))) ++ n_failed_functions;
	if(!(glDetachObjectARB = (void(GLApi*)(GLhandleARB, GLhandleARB))wglGetProcAddress("glDetachObjectARB"))) ++ n_failed_functions;
	if(!(glCreateShaderObjectARB = (GLhandleARB(GLApi*)(GLenum))wglGetProcAddress("glCreateShaderObjectARB"))) ++ n_failed_functions;
	if(!(glShaderSourceARB = (void(GLApi*)(GLhandleARB, GLsizei, const GLcharARB**, const int*))wglGetProcAddress("glShaderSourceARB"))) ++ n_failed_functions;
	if(!(glCompileShaderARB = (void(GLApi*)(GLhandleARB))wglGetProcAddress("glCompileShaderARB"))) ++ n_failed_functions;
	if(!(glCreateProgramObjectARB = (GLhandleARB(GLApi*)(void))wglGetProcAddress("glCreateProgramObjectARB"))) ++ n_failed_functions;
	if(!(glAttachObjectARB = (void(GLApi*)(GLhandleARB, GLhandleARB))wglGetProcAddress("glAttachObjectARB"))) ++ n_failed_functions;
	if(!(glLinkProgramARB = (void(GLApi*)(GLhandleARB))wglGetProcAddress("glLinkProgramARB"))) ++ n_failed_functions;
	if(!(glUseProgramObjectARB = (void(GLApi*)(GLhandleARB))wglGetProcAddress("glUseProgramObjectARB"))) ++ n_failed_functions;
	if(!(glValidateProgramARB = (void(GLApi*)(GLhandleARB))wglGetProcAddress("glValidateProgramARB"))) ++ n_failed_functions;
	if(!(glUniform1fARB = (void(GLApi*)(int, float))wglGetProcAddress("glUniform1fARB"))) ++ n_failed_functions;
	if(!(glUniform2fARB = (void(GLApi*)(int, float, float))wglGetProcAddress("glUniform2fARB"))) ++ n_failed_functions;
	if(!(glUniform3fARB = (void(GLApi*)(int, float, float, float))wglGetProcAddress("glUniform3fARB"))) ++ n_failed_functions;
	if(!(glUniform4fARB = (void(GLApi*)(int, float, float, float, float))wglGetProcAddress("glUniform4fARB"))) ++ n_failed_functions;
	if(!(glUniform1iARB = (void(GLApi*)(int, int))wglGetProcAddress("glUniform1iARB"))) ++ n_failed_functions;
	if(!(glUniform2iARB = (void(GLApi*)(int, int, int))wglGetProcAddress("glUniform2iARB"))) ++ n_failed_functions;
	if(!(glUniform3iARB = (void(GLApi*)(int, int, int, int))wglGetProcAddress("glUniform3iARB"))) ++ n_failed_functions;
	if(!(glUniform4iARB = (void(GLApi*)(int, int, int, int, int))wglGetProcAddress("glUniform4iARB"))) ++ n_failed_functions;
	if(!(glUniform1fvARB = (void(GLApi*)(int, GLsizei, const float*))wglGetProcAddress("glUniform1fvARB"))) ++ n_failed_functions;
	if(!(glUniform2fvARB = (void(GLApi*)(int, GLsizei, const float*))wglGetProcAddress("glUniform2fvARB"))) ++ n_failed_functions;
	if(!(glUniform3fvARB = (void(GLApi*)(int, GLsizei, const float*))wglGetProcAddress("glUniform3fvARB"))) ++ n_failed_functions;
	if(!(glUniform4fvARB = (void(GLApi*)(int, GLsizei, const float*))wglGetProcAddress("glUniform4fvARB"))) ++ n_failed_functions;
	if(!(glUniform1ivARB = (void(GLApi*)(int, GLsizei, const int*))wglGetProcAddress("glUniform1ivARB"))) ++ n_failed_functions;
	if(!(glUniform2ivARB = (void(GLApi*)(int, GLsizei, const int*))wglGetProcAddress("glUniform2ivARB"))) ++ n_failed_functions;
	if(!(glUniform3ivARB = (void(GLApi*)(int, GLsizei, const int*))wglGetProcAddress("glUniform3ivARB"))) ++ n_failed_functions;
	if(!(glUniform4ivARB = (void(GLApi*)(int, GLsizei, const int*))wglGetProcAddress("glUniform4ivARB"))) ++ n_failed_functions;
	if(!(glUniformMatrix2fvARB = (void(GLApi*)(int, GLsizei, GLboolean, const float*))wglGetProcAddress("glUniformMatrix2fvARB"))) ++ n_failed_functions;
	if(!(glUniformMatrix3fvARB = (void(GLApi*)(int, GLsizei, GLboolean, const float*))wglGetProcAddress("glUniformMatrix3fvARB"))) ++ n_failed_functions;
	if(!(glUniformMatrix4fvARB = (void(GLApi*)(int, GLsizei, GLboolean, const float*))wglGetProcAddress("glUniformMatrix4fvARB"))) ++ n_failed_functions;
	if(!(glGetObjectParameterfvARB = (void(GLApi*)(GLhandleARB, GLenum, float*))wglGetProcAddress("glGetObjectParameterfvARB"))) ++ n_failed_functions;
	if(!(glGetObjectParameterivARB = (void(GLApi*)(GLhandleARB, GLenum, int*))wglGetProcAddress("glGetObjectParameterivARB"))) ++ n_failed_functions;
	if(!(glGetInfoLogARB = (void(GLApi*)(GLhandleARB, GLsizei, GLsizei*, GLcharARB*))wglGetProcAddress("glGetInfoLogARB"))) ++ n_failed_functions;
	if(!(glGetAttachedObjectsARB = (void(GLApi*)(GLhandleARB, GLsizei, GLsizei *count, GLhandleARB*))wglGetProcAddress("glGetAttachedObjectsARB"))) ++ n_failed_functions;
	if(!(glGetUniformLocationARB = (int(GLApi*)(GLhandleARB, const GLcharARB*))wglGetProcAddress("glGetUniformLocationARB"))) ++ n_failed_functions;
	if(!(glGetActiveUniformARB = (void(GLApi*)(GLhandleARB, GLuint, GLsizei, GLsizei*, int*, GLenum*, GLcharARB*))wglGetProcAddress("glGetActiveUniformARB"))) ++ n_failed_functions;
	if(!(glGetUniformfvARB = (void(GLApi*)(GLhandleARB, int, float*))wglGetProcAddress("glGetUniformfvARB"))) ++ n_failed_functions;
	if(!(glGetUniformivARB = (void(GLApi*)(GLhandleARB, int, int*))wglGetProcAddress("glGetUniformivARB"))) ++ n_failed_functions;
	if(!(glGetShaderSourceARB = (void(GLApi*)(GLhandleARB, GLsizei, GLsizei*, GLcharARB*))wglGetProcAddress("glGetShaderSourceARB"))) ++ n_failed_functions;

#endif //__GENERATE_GL_ARB_shader_object__

	return n_failed_functions;
}

/*
 *				--- GL_ARB_vertex_buffer_object ---
 */

#ifdef __GENERATE_GL_ARB_vertex_buffer_object__

void (GLApi *glBindBufferARB)(GLenum target, GLuint buffer) INIT_FUNC_POINTERS;
void (GLApi *glDeleteBuffersARB)(GLsizei n, const GLuint *buffers) INIT_FUNC_POINTERS;
void (GLApi *glGenBuffersARB)(GLsizei n, GLuint *buffers) INIT_FUNC_POINTERS;
GLboolean (GLApi *glIsBufferARB)(GLuint buffer) INIT_FUNC_POINTERS;
void (GLApi *glBufferDataARB)(GLenum target, GLsizeiptrARB size, const void *data, GLenum usage) INIT_FUNC_POINTERS;
void (GLApi *glBufferSubDataARB)(GLenum target, GLintptrARB offset, GLsizeiptrARB size, const void *data) INIT_FUNC_POINTERS;
void (GLApi *glGetBufferSubDataARB)(GLenum target, GLintptrARB offset, GLsizeiptrARB size, void *data) INIT_FUNC_POINTERS;
void *(GLApi *glMapBufferARB)(GLenum target, GLenum access) INIT_FUNC_POINTERS;
GLboolean (GLApi *glUnmapBufferARB)(GLenum target) INIT_FUNC_POINTERS;
void (GLApi *glGetBufferParameterivARB)(GLenum target, GLenum pname, int *params) INIT_FUNC_POINTERS;
void (GLApi *glGetBufferPointervARB)(GLenum target, GLenum pname, void **params) INIT_FUNC_POINTERS;

#endif //__GENERATE_GL_ARB_vertex_buffer_object__

/*
 *	static int CGLExtensionHandler::n_GetVertexBufferObjectARBFuncPointers()
 *		- gets GL_ARB_vertex_buffer_object function pointers
 *		- returns number of missing functions
 */
int CGLExtensionHandler::n_GetVertexBufferObjectARBFuncPointers()
{
	__FuncGuard("CGLExtensionHandler::n_GetVertexBufferObjectARBFuncPointers");

	int n_failed_functions = 0;

#ifdef __GENERATE_GL_ARB_vertex_buffer_object__

	if(!(glBindBufferARB = (void(GLApi*)(GLenum target, GLuint buffer))wglGetProcAddress("glBindBufferARB"))) ++ n_failed_functions;
	if(!(glDeleteBuffersARB = (void(GLApi*)(GLsizei n, const GLuint *buffers))wglGetProcAddress("glDeleteBuffersARB"))) ++ n_failed_functions;
	if(!(glGenBuffersARB = (void(GLApi*)(GLsizei n, GLuint *buffers))wglGetProcAddress("glGenBuffersARB"))) ++ n_failed_functions;
	if(!(glIsBufferARB = (GLboolean(GLApi*)(GLuint buffer))wglGetProcAddress("glIsBufferARB"))) ++ n_failed_functions;
	if(!(glBufferDataARB = (void(GLApi*)(GLenum target, GLsizeiptrARB size, const void *data, GLenum usage))wglGetProcAddress("glBufferDataARB"))) ++ n_failed_functions;
	if(!(glBufferSubDataARB = (void(GLApi*)(GLenum target, GLintptrARB offset, GLsizeiptrARB size, const void *data))wglGetProcAddress("glBufferSubDataARB"))) ++ n_failed_functions;
	if(!(glGetBufferSubDataARB = (void(GLApi*)(GLenum target, GLintptrARB offset, GLsizeiptrARB size, void *data))wglGetProcAddress("glGetBufferSubDataARB"))) ++ n_failed_functions;
	if(!(glMapBufferARB = (void*(GLApi*)(GLenum target, GLenum access))wglGetProcAddress("glMapBufferARB"))) ++ n_failed_functions;
	if(!(glUnmapBufferARB = (GLboolean(GLApi*)(GLenum target))wglGetProcAddress("glUnmapBufferARB"))) ++ n_failed_functions;
	if(!(glGetBufferParameterivARB = (void(GLApi*)(GLenum target, GLenum pname, int *params))wglGetProcAddress("glGetBufferParameterivARB"))) ++ n_failed_functions;
	if(!(glGetBufferPointervARB = (void(GLApi*)(GLenum target, GLenum pname, void **params))wglGetProcAddress("glGetBufferPointervARB"))) ++ n_failed_functions;

#endif //__GENERATE_GL_ARB_vertex_buffer_object__

	return n_failed_functions;
}

/*
 *								--- GL_NV_fence ---
 */

#ifdef __GENERATE_GL_NV_fence__

void (GLApi *glGenFencesNV)(GLsizei n, GLuint *fences) INIT_FUNC_POINTERS;
void (GLApi *glDeleteFencesNV)(GLsizei n, const GLuint *fences) INIT_FUNC_POINTERS;
void (GLApi *glSetFenceNV)(GLuint fence, GLenum condition) INIT_FUNC_POINTERS;
GLboolean (GLApi *glTestFenceNV)(GLuint fence) INIT_FUNC_POINTERS;
void (GLApi *glFinishFenceNV)(GLuint fence) INIT_FUNC_POINTERS;
GLboolean (GLApi *glIsFenceNV)(GLuint fence) INIT_FUNC_POINTERS;
void (GLApi *glGetFenceivNV)(GLuint fence, GLenum pname, GLint *params) INIT_FUNC_POINTERS;

#endif //__GENERATE_GL_NV_fence__

/*
 *	static int CGLExtensionHandler::n_GetFenceNVFuncPointers()
 *		- gets GL_NV_fence function pointers
 *		- returns number of missing functions
 */
int CGLExtensionHandler::n_GetFenceNVFuncPointers()
{
	__FuncGuard("CGLExtensionHandler::n_GetFenceNVFuncPointers");

	int n_failed_functions = 0;

#ifdef __GENERATE_GL_NV_fence__

	if(!(glGenFencesNV = (void(GLApi*)(GLsizei n, GLuint *fences))wglGetProcAddress("glGenFencesNV"))) n_failed_functions ++;
	if(!(glDeleteFencesNV = (void(GLApi*)(GLsizei n, const GLuint *fences))wglGetProcAddress("glDeleteFencesNV"))) n_failed_functions ++;
	if(!(glSetFenceNV = (void(GLApi*)(GLuint fence, GLenum condition))wglGetProcAddress("glSetFenceNV"))) n_failed_functions ++;
	if(!(glTestFenceNV = (GLboolean(GLApi*)(GLuint fence))wglGetProcAddress("glTestFenceNV"))) n_failed_functions ++;
	if(!(glFinishFenceNV = (void(GLApi*)(GLuint fence))wglGetProcAddress("glFinishFenceNV"))) n_failed_functions ++;
	if(!(glIsFenceNV = (GLboolean(GLApi*)(GLuint fence))wglGetProcAddress("glIsFenceNV"))) n_failed_functions ++;
	if(!(glGetFenceivNV = (void(GLApi*)(GLuint fence, GLenum pname, GLint *params))wglGetProcAddress("glGetFenceivNV"))) n_failed_functions ++;

#endif //__GENERATE_GL_NV_fence__

	return n_failed_functions;
}

/*
 *								--- GL_ARB_occlusion_query ---
 */

#ifdef __GENERATE_GL_ARB_occlusion_query__

void (GLApi *glGenQueriesARB)(GLsizei n, GLuint *ids) INIT_FUNC_POINTERS;
void (GLApi *glDeleteQueriesARB)(GLsizei n, const GLuint *ids) INIT_FUNC_POINTERS;
GLboolean (GLApi *glIsQueryARB)(GLuint id) INIT_FUNC_POINTERS;
void (GLApi *glBeginQueryARB)(GLenum target, GLuint id) INIT_FUNC_POINTERS;
void (GLApi *glEndQueryARB)(GLenum target) INIT_FUNC_POINTERS;
void (GLApi *glGetQueryivARB)(GLenum target, GLenum pname, GLint *params) INIT_FUNC_POINTERS;
void (GLApi *glGetQueryObjectivARB)(GLuint id, GLenum pname, GLint *params) INIT_FUNC_POINTERS;
void (GLApi *glGetQueryObjectuivARB)(GLuint id, GLenum pname, GLuint *params) INIT_FUNC_POINTERS;

#endif //__GENERATE_GL_ARB_occlusion_query__

/*
 *	static int CGLExtensionHandler::n_GetOcclusionQueryARBFuncPointers()
 *		- gets GL_ARB_occlusion_query function pointers
 *		- returns number of missing functions
 */
int CGLExtensionHandler::n_GetOcclusionQueryARBFuncPointers()
{
	__FuncGuard("CGLExtensionHandler::n_GetOcclusionQueryARBFuncPointers");

	int n_failed_functions = 0;

#ifdef __GENERATE_GL_ARB_occlusion_query__

	if(!(glGenQueriesARB = (void (GLApi*)(GLsizei n, GLuint *ids))wglGetProcAddress("glGenQueriesARB"))) ++ n_failed_functions;
	if(!(glDeleteQueriesARB = (void (GLApi*)(GLsizei n, const GLuint *ids))wglGetProcAddress("glDeleteQueriesARB"))) ++ n_failed_functions;
	if(!(glIsQueryARB = (GLboolean (GLApi*)(GLuint id))wglGetProcAddress("glIsQueryARB"))) ++ n_failed_functions;
	if(!(glBeginQueryARB = (void (GLApi*)(GLenum target, GLuint id))wglGetProcAddress("glBeginQueryARB"))) ++ n_failed_functions;
	if(!(glEndQueryARB = (void (GLApi*)(GLenum target))wglGetProcAddress("glEndQueryARB"))) ++ n_failed_functions;
	if(!(glGetQueryivARB = (void (GLApi*)(GLenum target, GLenum pname, GLint *params))wglGetProcAddress("glGetQueryivARB"))) ++ n_failed_functions;
	if(!(glGetQueryObjectivARB = (void (GLApi*)(GLuint id, GLenum pname, GLint *params))wglGetProcAddress("glGetQueryObjectivARB"))) ++ n_failed_functions;
	if(!(glGetQueryObjectuivARB = (void (GLApi*)(GLuint id, GLenum pname, GLuint *params))wglGetProcAddress("glGetQueryObjectuivARB"))) ++ n_failed_functions;

#endif //__GENERATE_GL_ARB_occlusion_query__

	return n_failed_functions;
}

/*
 *								--- OpenGL 2.1 ---
 */

#ifdef __GENERATE_GL_21__

void (GLApi *glUniformMatrix2x3fv)(GLint location, GLsizei count, GLboolean transpose, const GLfloat *value) INIT_FUNC_POINTERS;
void (GLApi *glUniformMatrix3x2fv)(GLint location, GLsizei count, GLboolean transpose, const GLfloat *value) INIT_FUNC_POINTERS;
void (GLApi *glUniformMatrix2x4fv)(GLint location, GLsizei count, GLboolean transpose, const GLfloat *value) INIT_FUNC_POINTERS;
void (GLApi *glUniformMatrix4x2fv)(GLint location, GLsizei count, GLboolean transpose, const GLfloat *value) INIT_FUNC_POINTERS;
void (GLApi *glUniformMatrix3x4fv)(GLint location, GLsizei count, GLboolean transpose, const GLfloat *value) INIT_FUNC_POINTERS;
void (GLApi *glUniformMatrix4x3fv)(GLint location, GLsizei count, GLboolean transpose, const GLfloat *value) INIT_FUNC_POINTERS;

#endif //__GENERATE_GL_21__

/*
 *	static int CGLExtensionHandler::n_GetGL21FuncPointers()
 *		- gets OpenGL 2.1 core function pointers
 *		- returns number of missing functions
 */
int CGLExtensionHandler::n_GetGL21FuncPointers()
{
	__FuncGuard("CGLExtensionHandler::n_GetGL21FuncPointers");

	int n_failed_functions = 0;

#ifdef __GENERATE_GL_21__

	if(!(glUniformMatrix2x3fv = (void(GLApi*)(GLint location, GLsizei count, GLboolean transpose, const GLfloat *value))wglGetProcAddress("glUniformMatrix2x3fv"))) ++ n_failed_functions;
	if(!(glUniformMatrix3x2fv = (void(GLApi*)(GLint location, GLsizei count, GLboolean transpose, const GLfloat *value))wglGetProcAddress("glUniformMatrix3x2fv"))) ++ n_failed_functions;
	if(!(glUniformMatrix2x4fv = (void(GLApi*)(GLint location, GLsizei count, GLboolean transpose, const GLfloat *value))wglGetProcAddress("glUniformMatrix2x4fv"))) ++ n_failed_functions;
	if(!(glUniformMatrix4x2fv = (void(GLApi*)(GLint location, GLsizei count, GLboolean transpose, const GLfloat *value))wglGetProcAddress("glUniformMatrix4x2fv"))) ++ n_failed_functions;
	if(!(glUniformMatrix3x4fv = (void(GLApi*)(GLint location, GLsizei count, GLboolean transpose, const GLfloat *value))wglGetProcAddress("glUniformMatrix3x4fv"))) ++ n_failed_functions;
	if(!(glUniformMatrix4x3fv = (void(GLApi*)(GLint location, GLsizei count, GLboolean transpose, const GLfloat *value))wglGetProcAddress("glUniformMatrix4x3fv"))) ++ n_failed_functions;

#endif //__GENERATE_GL_21__

	return n_failed_functions;
}

/*
 *								--- GL_EXT_timer_query ---
 */

#ifdef __GENERATE_GL_EXT_timer_query__

void (GLApi *glGetQueryObjecti64vEXT)(GLuint id, GLenum pname, GLint64EXT *params) INIT_FUNC_POINTERS;
void (GLApi *glGetQueryObjectui64vEXT)(GLuint id, GLenum pname, GLuint64EXT *params) INIT_FUNC_POINTERS;

#endif //__GENERATE_GL_EXT_timer_query__

/*
 *	static int CGLExtensionHandler::n_GetTimerQueryEXTFuncPointers()
 *		- gets GL_EXT_timer_query function pointers
 *		- returns number of missing functions
 */
int CGLExtensionHandler::n_GetTimerQueryEXTFuncPointers()
{
	__FuncGuard("CGLExtensionHandler::n_GetTimerQueryEXTFuncPointers");

	int n_failed_functions = 0;

#ifdef __GENERATE_GL_EXT_timer_query__

	if(!(glGetQueryObjecti64vEXT = (void(GLApi*)(GLuint id, GLenum pname, GLint64EXT *params))wglGetProcAddress("glGetQueryObjecti64vEXT"))) ++ n_failed_functions;
	if(!(glGetQueryObjectui64vEXT = (void(GLApi*)(GLuint id, GLenum pname, GLuint64EXT *params))wglGetProcAddress("glGetQueryObjectui64vEXT"))) ++ n_failed_functions;

#endif //__GENERATE_GL_EXT_timer_query__

	return n_failed_functions;
}

/*
 *								--- GL_EXT_draw_buffers2 ---
 */

#ifdef __GENERATE_GL_EXT_draw_buffers2__

void (GLApi *glColorMaskIndexedEXT)(GLuint buf, GLboolean r, GLboolean g, GLboolean b, GLboolean a) INIT_FUNC_POINTERS;
void (GLApi *glGetGLbooleanIndexedvEXT)(GLenum value, GLuint index, GLboolean *data) INIT_FUNC_POINTERS;
void (GLApi *glGetIntegerIndexedvEXT)(GLenum value, GLuint index, GLint *data) INIT_FUNC_POINTERS;
void (GLApi *glEnableIndexedEXT)(GLenum target, GLuint index) INIT_FUNC_POINTERS;
void (GLApi *glDisableIndexedEXT)(GLenum target, GLuint index) INIT_FUNC_POINTERS;
GLboolean (GLApi *glIsEnabledIndexedEXT)(GLenum target, GLuint index) INIT_FUNC_POINTERS;

#endif //__GENERATE_GL_EXT_draw_buffers2__

/*
 *	static int CGLExtensionHandler::n_GetDrawBuffers2EXTFuncPointers()
 *		- gets GL_EXT_draw_buffers2 function pointers
 *		- returns number of missing functions
 */
int CGLExtensionHandler::n_GetDrawBuffers2EXTFuncPointers()
{
	__FuncGuard("CGLExtensionHandler::n_GetDrawBuffers2EXTFuncPointers");

	int n_failed_functions = 0;

#ifdef __GENERATE_GL_EXT_draw_buffers2__

	if(!(glColorMaskIndexedEXT = (void(GLApi*)(GLuint buf, GLboolean r, GLboolean g, GLboolean b, GLboolean a))wglGetProcAddress("glColorMaskIndexedEXT"))) ++ n_failed_functions;
	if(!(glGetGLbooleanIndexedvEXT = (void(GLApi*)(GLenum value, GLuint index, GLboolean *data))wglGetProcAddress("glGetGLbooleanIndexedvEXT"))) ++ n_failed_functions;
	if(!(glGetIntegerIndexedvEXT = (void(GLApi*)(GLenum value, GLuint index, GLint *data))wglGetProcAddress("glGetIntegerIndexedvEXT"))) ++ n_failed_functions;
	if(!(glEnableIndexedEXT = (void(GLApi*)(GLenum target, GLuint index))wglGetProcAddress("glEnableIndexedEXT"))) ++ n_failed_functions;
	if(!(glDisableIndexedEXT = (void(GLApi*)(GLenum target, GLuint index))wglGetProcAddress("glDisableIndexedEXT"))) ++ n_failed_functions;
	if(!(glIsEnabledIndexedEXT = (GLboolean(GLApi*)(GLenum target, GLuint index))wglGetProcAddress("glIsEnabledIndexedEXT"))) ++ n_failed_functions;

#endif //__GENERATE_GL_EXT_draw_buffers2__

	return n_failed_functions;
}

/*
 *								--- GL_EXT_texture_integer ---
 */

#ifdef __GENERATE_GL_EXT_texture_integer__

void (GLApi *glClearColorIiEXT)(GLint r, GLint g, GLint b, GLint a) INIT_FUNC_POINTERS;
void (GLApi *glClearColorIuiEXT)(GLuint r, GLuint g, GLuint b, GLuint a) INIT_FUNC_POINTERS;
void (GLApi *glTexParameterIivEXT)(GLenum target, GLenum pname, GLint *params) INIT_FUNC_POINTERS;
void (GLApi *glTexParameterIuivEXT)(GLenum target, GLenum pname, GLuint *params) INIT_FUNC_POINTERS;
void (GLApi *glGetTexParameterIivEXT)(GLenum target, GLenum pname, GLint *params) INIT_FUNC_POINTERS;
void (GLApi *glGetTexParameterIiuvEXT)(GLenum target, GLenum pname, GLuint *params) INIT_FUNC_POINTERS;

#endif //__GENERATE_GL_EXT_texture_integer__

/*
 *	static int CGLExtensionHandler::n_GetTextureIntegerEXTFuncPointers()
 *		- gets GL_EXT_texture_integer function pointers
 *		- returns number of missing functions
 */
int CGLExtensionHandler::n_GetTextureIntegerEXTFuncPointers()
{
	__FuncGuard("CGLExtensionHandler::n_GetTextureIntegerEXTFuncPointers");

	int n_failed_functions = 0;

#ifdef __GENERATE_GL_EXT_texture_integer__

	if(!(glClearColorIiEXT = (void(GLApi*)(GLint r, GLint g, GLint b, GLint a))wglGetProcAddress("glClearColorIiEXT"))) ++ n_failed_functions;
	if(!(glClearColorIuiEXT = (void(GLApi*)(GLuint r, GLuint g, GLuint b, GLuint a))wglGetProcAddress("glClearColorIuiEXT"))) ++ n_failed_functions;
	if(!(glTexParameterIivEXT = (void(GLApi*)(GLenum target, GLenum pname, GLint *params))wglGetProcAddress("glTexParameterIivEXT"))) ++ n_failed_functions;
	if(!(glTexParameterIuivEXT = (void(GLApi*)(GLenum target, GLenum pname, GLuint *params))wglGetProcAddress("glTexParameterIuivEXT"))) ++ n_failed_functions;
	if(!(glGetTexParameterIivEXT = (void(GLApi*)(GLenum target, GLenum pname, GLint *params))wglGetProcAddress("glGetTexParameterIivEXT"))) ++ n_failed_functions;
	if(!(glGetTexParameterIiuvEXT = (void(GLApi*)(GLenum target, GLenum pname, GLuint *params))wglGetProcAddress("glGetTexParameterIiuvEXT"))) ++ n_failed_functions;

#endif //__GENERATE_GL_EXT_texture_integer__

	return n_failed_functions;
}

/*
 *								--- GL_EXT_gpu_shader4 ---
 */

#ifdef __GENERATE_GL_EXT_gpu_shader4__

void (GLApi *glVertexAttribI1iEXT)(GLuint index, GLint x) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttribI2iEXT)(GLuint index, GLint x, GLint y) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttribI3iEXT)(GLuint index, GLint x, GLint y, GLint z) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttribI4iEXT)(GLuint index, GLint x, GLint y, GLint z, GLint w) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttribI1uiEXT)(GLuint index, GLuint x) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttribI2uiEXT)(GLuint index, GLuint x, GLuint y) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttribI3uiEXT)(GLuint index, GLuint x, GLuint y, GLuint z) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttribI4uiEXT)(GLuint index, GLuint x, GLuint y, GLuint z, GLuint w) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttribI1ivEXT)(GLuint index, const GLint *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttribI2ivEXT)(GLuint index, const GLint *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttribI3ivEXT)(GLuint index, const GLint *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttribI4ivEXT)(GLuint index, const GLint *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttribI1uivEXT)(GLuint index, const GLuint *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttribI2uivEXT)(GLuint index, const GLuint *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttribI3uivEXT)(GLuint index, const GLuint *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttribI4uivEXT)(GLuint index, const GLuint *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttribI4bvEXT)(GLuint index, const GLbyte *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttribI4svEXT)(GLuint index, const GLshort *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttribI4ubvEXT)(GLuint index, const GLubyte *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttribI4usvEXT)(GLuint index, const GLushort *v) INIT_FUNC_POINTERS;
void (GLApi *glVertexAttribIPointerEXT)(GLuint index, GLint size, GLenum type, GLsizei stride, const void *pointer) INIT_FUNC_POINTERS;
void (GLApi *glGetVertexAttribIivEXT)(GLuint index, GLenum pname, GLint *params) INIT_FUNC_POINTERS;
void (GLApi *glGetVertexAttribIuivEXT)(GLuint index, GLenum pname, GLuint *params) INIT_FUNC_POINTERS;
void (GLApi *glUniform1uiEXT)(GLint location, GLuint v0) INIT_FUNC_POINTERS;
void (GLApi *glUniform2uiEXT)(GLint location, GLuint v0, GLuint v1) INIT_FUNC_POINTERS;
void (GLApi *glUniform3uiEXT)(GLint location, GLuint v0, GLuint v1, GLuint v2) INIT_FUNC_POINTERS;
void (GLApi *glUniform4uiEXT)(GLint location, GLuint v0, GLuint v1, GLuint v2, GLuint v3) INIT_FUNC_POINTERS;
void (GLApi *glUniform1uivEXT)(GLint location, GLsizei count, const GLuint *value) INIT_FUNC_POINTERS;
void (GLApi *glUniform2uivEXT)(GLint location, GLsizei count, const GLuint *value) INIT_FUNC_POINTERS;
void (GLApi *glUniform3uivEXT)(GLint location, GLsizei count, const GLuint *value) INIT_FUNC_POINTERS;
void (GLApi *glUniform4uivEXT)(GLint location, GLsizei count, const GLuint *value) INIT_FUNC_POINTERS;
void (GLApi *glGetUniformuivEXT)(GLuint program, GLint location, GLuint *params) INIT_FUNC_POINTERS;
void (GLApi *glBindFragDataLocationEXT)(GLuint program, GLuint colorNumber, const char *name) INIT_FUNC_POINTERS;
int (GLApi *glGetFragDataLocationEXT)(GLuint program, const char *name) INIT_FUNC_POINTERS;

#endif //__GENERATE_GL_EXT_gpu_shader4__

/*
 *	static int CGLExtensionHandler::n_GetGPUShader4EXTFuncPointers()
 *		- gets GL_EXT_gpu_shader4 function pointers
 *		- returns number of missing functions
 */
int CGLExtensionHandler::n_GetGPUShader4EXTFuncPointers()
{
	__FuncGuard("CGLExtensionHandler::n_GetGPUShader4EXTFuncPointers");

	int n_failed_functions = 0;

#ifdef __GENERATE_GL_EXT_gpu_shader4__

	if(!(glVertexAttribI1iEXT = (void(GLApi*)(GLuint index, GLint x))wglGetProcAddress("glVertexAttribI1iEXT"))) ++ n_failed_functions;
	if(!(glVertexAttribI2iEXT = (void(GLApi*)(GLuint index, GLint x, GLint y))wglGetProcAddress("glVertexAttribI2iEXT"))) ++ n_failed_functions;
	if(!(glVertexAttribI3iEXT = (void(GLApi*)(GLuint index, GLint x, GLint y, GLint z))wglGetProcAddress("glVertexAttribI3iEXT"))) ++ n_failed_functions;
	if(!(glVertexAttribI4iEXT = (void(GLApi*)(GLuint index, GLint x, GLint y, GLint z, GLint w))wglGetProcAddress("glVertexAttribI4iEXT"))) ++ n_failed_functions;
	if(!(glVertexAttribI1uiEXT = (void(GLApi*)(GLuint index, GLuint x))wglGetProcAddress("glVertexAttribI1uiEXT"))) ++ n_failed_functions;
	if(!(glVertexAttribI2uiEXT = (void(GLApi*)(GLuint index, GLuint x, GLuint y))wglGetProcAddress("glVertexAttribI2uiEXT"))) ++ n_failed_functions;
	if(!(glVertexAttribI3uiEXT = (void(GLApi*)(GLuint index, GLuint x, GLuint y, GLuint z))wglGetProcAddress("glVertexAttribI3uiEXT"))) ++ n_failed_functions;
	if(!(glVertexAttribI4uiEXT = (void(GLApi*)(GLuint index, GLuint x, GLuint y, GLuint z, GLuint w))wglGetProcAddress("glVertexAttribI4uiEXT"))) ++ n_failed_functions;
	if(!(glVertexAttribI1ivEXT = (void(GLApi*)(GLuint index, const GLint *v))wglGetProcAddress("glVertexAttribI1ivEXT"))) ++ n_failed_functions;
	if(!(glVertexAttribI2ivEXT = (void(GLApi*)(GLuint index, const GLint *v))wglGetProcAddress("glVertexAttribI2ivEXT"))) ++ n_failed_functions;
	if(!(glVertexAttribI3ivEXT = (void(GLApi*)(GLuint index, const GLint *v))wglGetProcAddress("glVertexAttribI3ivEXT"))) ++ n_failed_functions;
	if(!(glVertexAttribI4ivEXT = (void(GLApi*)(GLuint index, const GLint *v))wglGetProcAddress("glVertexAttribI4ivEXT"))) ++ n_failed_functions;
	if(!(glVertexAttribI1uivEXT = (void(GLApi*)(GLuint index, const GLuint *v))wglGetProcAddress("glVertexAttribI1uivEXT"))) ++ n_failed_functions;
	if(!(glVertexAttribI2uivEXT = (void(GLApi*)(GLuint index, const GLuint *v))wglGetProcAddress("glVertexAttribI2uivEXT"))) ++ n_failed_functions;
	if(!(glVertexAttribI3uivEXT = (void(GLApi*)(GLuint index, const GLuint *v))wglGetProcAddress("glVertexAttribI3uivEXT"))) ++ n_failed_functions;
	if(!(glVertexAttribI4uivEXT = (void(GLApi*)(GLuint index, const GLuint *v))wglGetProcAddress("glVertexAttribI4uivEXT"))) ++ n_failed_functions;
	if(!(glVertexAttribI4bvEXT = (void(GLApi*)(GLuint index, const GLbyte *v))wglGetProcAddress("glVertexAttribI4bvEXT"))) ++ n_failed_functions;
	if(!(glVertexAttribI4svEXT = (void(GLApi*)(GLuint index, const GLshort *v))wglGetProcAddress("glVertexAttribI4svEXT"))) ++ n_failed_functions;
	if(!(glVertexAttribI4ubvEXT = (void(GLApi*)(GLuint index, const GLubyte *v))wglGetProcAddress("glVertexAttribI4ubvEXT"))) ++ n_failed_functions;
	if(!(glVertexAttribI4usvEXT = (void(GLApi*)(GLuint index, const GLushort *v))wglGetProcAddress("glVertexAttribI4usvEXT"))) ++ n_failed_functions;
	if(!(glVertexAttribIPointerEXT = (void(GLApi*)(GLuint index, GLint size, GLenum type, GLsizei stride, const void *pointer))wglGetProcAddress("glVertexAttribIPointerEXT"))) ++ n_failed_functions;
	if(!(glGetVertexAttribIivEXT = (void(GLApi*)(GLuint index, GLenum pname, GLint *params))wglGetProcAddress("glGetVertexAttribIivEXT"))) ++ n_failed_functions;
	if(!(glGetVertexAttribIuivEXT = (void(GLApi*)(GLuint index, GLenum pname, GLuint *params))wglGetProcAddress("glGetVertexAttribIuivEXT"))) ++ n_failed_functions;
	if(!(glUniform1uiEXT = (void(GLApi*)(GLint location, GLuint v0))wglGetProcAddress("glUniform1uiEXT"))) ++ n_failed_functions;
	if(!(glUniform2uiEXT = (void(GLApi*)(GLint location, GLuint v0, GLuint v1))wglGetProcAddress("glUniform2uiEXT"))) ++ n_failed_functions;
	if(!(glUniform3uiEXT = (void(GLApi*)(GLint location, GLuint v0, GLuint v1, GLuint v2))wglGetProcAddress("glUniform3uiEXT"))) ++ n_failed_functions;
	if(!(glUniform4uiEXT = (void(GLApi*)(GLint location, GLuint v0, GLuint v1, GLuint v2, GLuint v3))wglGetProcAddress("glUniform4uiEXT"))) ++ n_failed_functions;
	if(!(glUniform1uivEXT = (void(GLApi*)(GLint location, GLsizei count, const GLuint *value))wglGetProcAddress("glUniform1uivEXT"))) ++ n_failed_functions;
	if(!(glUniform2uivEXT = (void(GLApi*)(GLint location, GLsizei count, const GLuint *value))wglGetProcAddress("glUniform2uivEXT"))) ++ n_failed_functions;
	if(!(glUniform3uivEXT = (void(GLApi*)(GLint location, GLsizei count, const GLuint *value))wglGetProcAddress("glUniform3uivEXT"))) ++ n_failed_functions;
	if(!(glUniform4uivEXT = (void(GLApi*)(GLint location, GLsizei count, const GLuint *value))wglGetProcAddress("glUniform4uivEXT"))) ++ n_failed_functions;
	if(!(glGetUniformuivEXT = (void(GLApi*)(GLuint program, GLint location, GLuint *params))wglGetProcAddress("glGetUniformuivEXT"))) ++ n_failed_functions;
	if(!(glBindFragDataLocationEXT = (void(GLApi*)(GLuint program, GLuint colorNumber, const char *name))wglGetProcAddress("glBindFragDataLocationEXT"))) ++ n_failed_functions;
	if(!(glGetFragDataLocationEXT = (int(GLApi*)(GLuint program, const char *name))wglGetProcAddress("glGetFragDataLocationEXT"))) ++ n_failed_functions;

#endif //__GENERATE_GL_EXT_gpu_shader4__

	return n_failed_functions;
}

/*
 *								--- GL_EXT_gpu_program_parameters ---
 */

#ifdef __GENERATE_GL_EXT_gpu_program_parameters__

void (GLApi *glProgramEnvParameters4fvEXT)(GLenum target, GLuint index, GLsizei count, const GLfloat *params) INIT_FUNC_POINTERS;
void (GLApi *glProgramLocalParameters4fvEXT)(GLenum target, GLuint index, GLsizei count, const GLfloat *params) INIT_FUNC_POINTERS;

#endif //__GENERATE_GL_EXT_gpu_program_parameters__

/*
 *	static int CGLExtensionHandler::n_GetGPUProgramParametersEXTFuncPointers()
 *		- gets GL_EXT_gpu_program_parameters function pointers
 *		- returns number of missing functions
 */
int CGLExtensionHandler::n_GetGPUProgramParametersEXTFuncPointers()
{
	__FuncGuard("CGLExtensionHandler::n_GetGPUShader4EXTFuncPointers");

	int n_failed_functions = 0;

#ifdef __GENERATE_GL_EXT_gpu_program_parameters__

	if(!(glProgramEnvParameters4fvEXT = (void(GLApi*)(GLenum target, GLuint index, GLsizei count, const GLfloat *params))wglGetProcAddress("glProgramEnvParameters4fvEXT"))) ++ n_failed_functions;
	if(!(glProgramLocalParameters4fvEXT = (void(GLApi*)(GLenum target, GLuint index, GLsizei count, const GLfloat *params))wglGetProcAddress("glProgramLocalParameters4fvEXT"))) ++ n_failed_functions;

#endif //__GENERATE_GL_EXT_gpu_program_parameters__

	return n_failed_functions;
}

/*
 *								--- GL_EXT_bindable_uniform ---
 */

#ifdef __GENERATE_GL_EXT_bindable_uniform__

void (GLApi *glUniformBufferEXT)(GLuint program, GLint location, GLuint buffer) INIT_FUNC_POINTERS;
int (GLApi *glGetUniformBufferSizeEXT)(GLuint program, GLint location) INIT_FUNC_POINTERS;
GLintptr (GLApi *glGetUniformOffsetEXT)(GLuint program, GLint location) INIT_FUNC_POINTERS;

#endif //__GENERATE_GL_EXT_bindable_uniform__

/*
 *	static int CGLExtensionHandler::n_GetBindableUniformEXTFuncPointers()
 *		- gets GL_EXT_bindable_uniform function pointers
 *		- returns number of missing functions
 */
int CGLExtensionHandler::n_GetBindableUniformEXTFuncPointers()
{
	__FuncGuard("CGLExtensionHandler::n_GetBindableUniformEXTFuncPointers");

	int n_failed_functions = 0;

#ifdef __GENERATE_GL_EXT_bindable_uniform__

	if(!(glUniformBufferEXT = (void(GLApi*)(GLuint program, GLint location, GLuint buffer))wglGetProcAddress("glUniformBufferEXT"))) ++ n_failed_functions;
	if(!(glGetUniformBufferSizeEXT = (int(GLApi*)(GLuint program, GLint location))wglGetProcAddress("glGetUniformBufferSizeEXT"))) ++ n_failed_functions;
	if(!(glGetUniformOffsetEXT = (GLintptr(GLApi*)(GLuint program, GLint location))wglGetProcAddress("glGetUniformOffsetEXT"))) ++ n_failed_functions;

#endif //__GENERATE_GL_EXT_bindable_uniform__

	return n_failed_functions;
}

/*
 *								--- GL_EXT_geometry_shader4 ---
 */

#ifdef __GENERATE_GL_EXT_geometry_shader4__

void (GLApi *glProgramParameteriEXT)(GLuint program, GLenum pname, GLint value) INIT_FUNC_POINTERS;
void (GLApi *glFramebufferTextureEXT)(GLenum target, GLenum attachment, GLuint texture, GLint level) INIT_FUNC_POINTERS;
void (GLApi *glFramebufferTextureLayerEXT)(GLenum target, GLenum attachment, GLuint texture, GLint level, GLint layer) INIT_FUNC_POINTERS;
void (GLApi *glFramebufferTextureFaceEXT)(GLenum target, GLenum attachment, GLuint texture, GLint level, GLenum face) INIT_FUNC_POINTERS;

#endif //__GENERATE_GL_EXT_geometry_shader4__

/*
 *	static int CGLExtensionHandler::n_GetGeometryShader4EXTFuncPointers()
 *		- gets GL_EXT_geometry_shader4 function pointers
 *		- returns number of missing functions
 */
int CGLExtensionHandler::n_GetGeometryShader4EXTFuncPointers()
{
	__FuncGuard("CGLExtensionHandler::n_GetGeometryShader4EXTFuncPointers");

	int n_failed_functions = 0;

#ifdef __GENERATE_GL_EXT_geometry_shader4__

	if(!(glProgramParameteriEXT = (void (GLApi *)(GLuint program, GLenum pname, GLint value))wglGetProcAddress("glProgramParameteriEXT"))) ++ n_failed_functions;
	if(!(glFramebufferTextureEXT = (void (GLApi *)(GLenum target, GLenum attachment, GLuint texture, GLint level))wglGetProcAddress("glFramebufferTextureEXT"))) ++ n_failed_functions;
	if(!(glFramebufferTextureLayerEXT = (void (GLApi *)(GLenum target, GLenum attachment, GLuint texture, GLint level, GLint layer))wglGetProcAddress("glFramebufferTextureLayerEXT"))) ++ n_failed_functions;
	if(!(glFramebufferTextureFaceEXT = (void (GLApi *)(GLenum target, GLenum attachment, GLuint texture, GLint level, GLenum face))wglGetProcAddress("glFramebufferTextureFaceEXT"))) ++ n_failed_functions;

#endif //__GENERATE_GL_EXT_geometry_shader4__

	return n_failed_functions;
}

/*
 *								--- GL_EXT_draw_instanced ---
 */

#ifdef __GENERATE_GL_EXT_draw_instanced__

void (GLApi *glDrawArraysInstancedEXT)(GLenum mode, GLint first, GLsizei count, GLsizei primcount) INIT_FUNC_POINTERS;
void (GLApi *glDrawElementsInstancedEXT)(GLenum mode, GLsizei count, GLenum type, const void *indices, GLsizei primcount) INIT_FUNC_POINTERS;

#endif //__GENERATE_GL_EXT_draw_instanced__

/*
 *	static int CGLExtensionHandler::n_GetDrawInstancedEXTFuncPointers()
 *		- gets GL_EXT_draw_instanced function pointers
 *		- returns number of missing functions
 */
int CGLExtensionHandler::n_GetDrawInstancedEXTFuncPointers()
{
	__FuncGuard("CGLExtensionHandler::n_GetDrawInstancedEXTFuncPointers");

	int n_failed_functions = 0;

#ifdef __GENERATE_GL_EXT_draw_instanced__

	if(!(glDrawArraysInstancedEXT = (void(GLApi*)(GLenum mode, GLint first, GLsizei count, GLsizei primcount))wglGetProcAddress("glDrawArraysInstancedEXT"))) ++ n_failed_functions;
	if(!(glDrawElementsInstancedEXT = (void(GLApi*)(GLenum mode, GLsizei count, GLenum type, const void *indices, GLsizei primcount))wglGetProcAddress("glDrawElementsInstancedEXT"))) ++ n_failed_functions;

#endif //__GENERATE_GL_EXT_draw_instanced__

	return n_failed_functions;
}

/*
 *								--- GL_EXT_texture_array ---
 */

/*void (GLApi *glFramebufferTextureLayerEXT)(GLenum target, GLenum attachment,
	GLuint texture, GLint level, GLint layer) INIT_FUNC_POINTERS;*/
// GL_EXT_geometry_shader4

/*
 *	static int CGLExtensionHandler::n_GetTextureArrayEXTFuncPointers()
 *		- gets GL_EXT_texture_array function pointers
 *		- note this extension overlaps with GL_EXT_geometry_shader4
 *		- returns number of missing functions
 */
int CGLExtensionHandler::n_GetTextureArrayEXTFuncPointers()
{
	__FuncGuard("CGLExtensionHandler::n_GetTextureArrayEXTFuncPointers");

	int n_failed_functions = 0;

#ifdef __GENERATE_GL_EXT_texture_array__

	if(!(glFramebufferTextureLayerEXT = (void(GLApi*)(GLenum target, GLenum attachment, GLuint texture, GLint level, GLint layer))wglGetProcAddress("glFramebufferTextureLayerEXT"))) ++ n_failed_functions;

#endif //__GENERATE_GL_EXT_texture_array__

	return n_failed_functions;
}

/*
 *								--- GL_EXT_texture_object ---
 */

#ifdef __GENERATE_GL_EXT_texture_buffer_object__

void (GLApi *glTexBufferEXT)(GLenum target, GLenum internalformat, GLuint buffer) INIT_FUNC_POINTERS;

#endif //__GENERATE_GL_EXT_texture_buffer_object__

/*
 *	static int CGLExtensionHandler::n_GetTextureObjectEXTFuncPointers()
 *		- gets GL_EXT_texture_object function pointers
 *		- returns number of missing functions
 */
int CGLExtensionHandler::n_GetTextureObjectEXTFuncPointers()
{
	__FuncGuard("CGLExtensionHandler::n_GetTextureObjectEXTFuncPointers");

	int n_failed_functions = 0;

#ifdef __GENERATE_GL_EXT_texture_buffer_object__

	if(!(glTexBufferEXT = (void(GLApi*)(GLenum target, GLenum internalformat, GLuint buffer))wglGetProcAddress("glTexBufferEXT"))) ++ n_failed_functions;

#endif //__GENERATE_GL_EXT_texture_buffer_object__

	return n_failed_functions;
}

/*
 *								--- GL_EXT_blend_minmax ---
 */

#ifdef __GENERATE_GL_EXT_blend_minmax__

void (GLApi *glBlendEquationEXT)(GLenum mode) INIT_FUNC_POINTERS;

#endif //__GENERATE_GL_EXT_blend_minmax__

/*
 *	static int CGLExtensionHandler::n_GetBlendMinMaxEXTFuncPointers()
 *		- gets GL_EXT_blend_minmax function pointers
 *		- returns number of missing functions
 */
int CGLExtensionHandler::n_GetBlendMinMaxEXTFuncPointers()
{
	__FuncGuard("CGLExtensionHandler::n_GetBlendMinMaxEXTFuncPointers");

	int n_failed_functions = 0;

#ifdef __GENERATE_GL_EXT_blend_minmax__

	if(!(glBlendEquationEXT = (void(GLApi*)(GLenum mode))wglGetProcAddress("glBlendEquationEXT"))) ++ n_failed_functions;

#endif //__GENERATE_GL_EXT_blend_minmax__

	return n_failed_functions;
}

/*
 *								=== ~CGLExtensionHandler ===
 */
